;; MANEJADOR DE INTERFAZ PARA MANTA
;; Por Hernan Aponte (1995)
;; Modificaciones por Vladimir Tamara (1998)

; PROCEDIMIENTO PRINCIPAL
(defun manta (archivo)
  "Corre 'manta' redireccionando su entrada y salida estandar,
   verificando antes que el archivo de configuracion exista."
  (interactive "sConfiguration file for ManTa: ")
  (cond ((string-equal archivo "")
	 (manta-mode "manta.ini"))
	((file-exists-p archivo)
	 (manta-mode archivo))
	(t (message "Cannot open %s" archivo))))

;DEFINICION DE VARIABLES GLOBALES

;key-map para comandos manta
(defvar manta-mode-map nil)

;buffers de trabajo para la ejecucion de manta
(defvar menu nil)
(defvar resultados nil)
(defvar manta-salida nil)
(defvar buffer1 nil)
(defvar buffer2 nil)

;variables para guardar el status de manta
(defvar tipo-actual "?")
(defvar repr-actual "?")

;configuracion previa a la ejecucion de manta
(defvar configuracion-inicial nil)

;FUNCIONES DEL MANEJADOR DE INTERFAZ
(if manta-mode-map
    nil
  (setq manta-mode-map (make-sparse-keymap))
  (define-key manta-mode-map "\C-m" 'seleccionar-opcion)
  (define-key manta-mode-map "\C-cs" 'mostrar-status))


(defun manta-mode (arch-config)
  "Salva el buffer actual, determina el modo y corre manta"
  (setq configuracion-inicial (current-window-configuration))
  (setq major-mode 'manta-mode)
  (setq mode-name "manta-menu")
  (correr-manta arch-config)
)



(defun correr-manta (arch-config)
  "Alista los buffers y windows necesarios para correr ManTa, inicia el 
proceso 'manta', e inicia la interaccion con este subproceso"
  (inicializar)
  (start-process "manta" manta-salida shell-file-name
		 "-c"  (concat "./manta -i " arch-config))
;VTP. 11/feb/1998 Decia (concat "exec " "manta -i" arch-config)
  (global-unset-key "\C-cm")
  (procesar-objeto)
)


(defun iniciar-ventanas ()
  "Divide la pantalla horizontalmente en dos ventanas, y deja como ventana
seleccionada la inferior."
  (delete-other-windows)
  (setq superior (selected-window))
  (setq inferior (split-window superior))
  (set-window-buffer inferior resultados))


(defun inicializar ()
  "Genera tres buffers (menu,resultados,manta-salida) y dos ventanas (superior,
inferior). Estos son los buffers y windows necesarios para la interaccion con
ManTa."
  (setq menu (generate-new-buffer "menu"))
  (setq resultados (generate-new-buffer "resultados"))
  (setq manta-salida (generate-new-buffer "manta-salida"))
  (iniciar-ventanas)
)



(defun procesar-objeto ()
  "Recibe y procesa cualquier mensaje del subproceso 'manta'. Cada mensaje 
codifica un objeto mediante una secuencia de lineas; la primera es un
identificador del objeto, la ultima es un indicador de fin de objeto, 
las demas son el objeto mismo."
  (reciba-objeto)
  (let ((men (ident-objeto)))
    (cond ((string= men "abrir-menu")
	   (abrir-menu))
	  ((string= men "mostrar-directorio")
	   (mostrar-directorio))
	  ((string= men "mostrar-mensaje")
	   (mostrar-mensaje))
	  ((string= men "mostrar-pregunta")
	   (mostrar-pregunta))
	  ((string= men "mostrar-archivo")
	   (mostrar-archivo))
	  ((string= men "pedir-nombre")
	   (pedir-nombre))
	  ((string= men "compilar-prototipo")
	   (compilar-prototipo))
	  ((string= men "ejecutar-prototipo")
	   (ejecutar-prototipo))
	  ((string= men "actualizar-status")
	   (actualizar-status))
	  ((string= men "terminar")
	   (terminar))
	  (t
	   (message "objeto desconocido: <%s>" men)))))


(defun terminar ()
  "Mata los buffers usados por ManTa y deja como buffer actual, el que habia
antes de la ejecucion de ManTa"
  (set-window-configuration configuracion-inicial)
  (global-set-key "\C-cm" 'manta)
  (kill-buffer menu)
  (kill-buffer resultados)
  (kill-buffer manta-salida)
  (message "Process manta finished"))

(defun reciba-objeto ()
  "Borra el buffer <manta-salida> y luego espera hasta recibir un objeto 
enviado por el subproceso 'manta'. El string 'fin-objeto' es el indicador de
final de envio"
  (let ((centinela t))
    (set-buffer manta-salida)
    (setq buffer-read-only nil)
    (erase-buffer)
    (accept-process-output "manta" 0 0)
;VTP. 11/feb/1998 Decia (accept-process-output "manta")
    (while centinela
      (goto-char (point-max))
      (beginning-of-line)
;      (setq centinela nil)
      (if (word-search-forward "fin-objeto" nil t)
       (setq centinela nil)
	(accept-process-output "manta" 0 0)
;VTP. 11/feb/1998 Decia (accept-process-output "manta")
))
))



(defun ident-objeto ()
  "Obtiene el identificador de un objeto enviado por ManTa. El identificador
es un string que aparece en la primera linea del envio"
  (set-buffer manta-salida)
  (goto-char (point-min))
  (end-of-line)
  (buffer-substring (point-min) (point)))



(defun tomar-objeto ()
  "retorna los limites de un objeto enviado por ManTa: el inicio y
el final, sobre el buffer <manta-salida>. Deja este buffer como buffer
actual"
  (set-buffer manta-salida)
  (setq buffer-read-only nil)
  (goto-char (point-min))
  (goto-line 2)
  (let ((inicio (point)))
    (goto-char (point-max))
    (beginning-of-line)
    (cons inicio (1- (point)))))


(defun abrir-menu ()
  "Presenta en el buffer 'menu' el menu que envio ManTa."
  (set-buffer menu)
  (setq buffer-read-only nil)
  (let ((limites (tomar-objeto)))
    (goto-char (car limites))
    (end-of-line)
    (let ((cursor (string-to-int (buffer-substring (car limites) (point)))))
      (goto-char (car limites))
      (forward-line 1)
      (copy-to-buffer menu (point) (cdr limites))
      (set-buffer menu)
      (goto-char (point-min))
      (forward-line (+ 1 cursor))
      (setq buffer-read-only t)
      (iniciar-ventanas)
      (set-window-buffer superior menu)
      (use-local-map manta-mode-map))))


(defun pedir-nombre ()
  "Pide al usuario un dato en el minibuffer, presentandole primero una cadena
que envio ManTa. Envia este dato y deja a 'superior' como ventana actual."
  (let ((limites (tomar-objeto)))
    (let ((respuesta (lea-nombre
		      (buffer-substring (car limites) (cdr limites)))))
      (envie respuesta)
      (select-window superior)
      (procesar-objeto))))


(defun mostrar-pregunta ()
  "Pasa al minibuffer una pregunta enviada por ManTa, lee ahi mismo la 
respuesta dada por el usuario, y se la envia a ManTa"
  (let ((limites (tomar-objeto)))
    (let ((resp (lea-nombre (buffer-substring (car limites)
					      (cdr limites)))))
      (envie "siga")
      (envie resp)
      (select-window superior)
      (procesar-objeto))))


(defun mostrar-mensaje ()
  "Pasa al buffer 'resultados' un mensaje enviado por ManTa, y lo despliega
en la ventana 'inferior'"
  (set-buffer resultados)
  (setq buffer-read-only nil)
  (let ((limites (tomar-objeto)))
    (copy-to-buffer resultados (car limites) (cdr limites))
    (set-buffer resultados)
    (setq buffer-read-only t)
    (iniciar-ventanas)
;    (set-window-buffer inferior resultados)
    (select-window superior)
    (envie "siga")
    (procesar-objeto)))


(defun mostrar-directorio ()
  "Pasa al buffer 'resultados' un directorio enviado por ManTa, y lo despliega
en la ventana 'inferior'"
  (set-buffer resultados)
  (setq buffer-read-only nil)
  (let ((limites (tomar-objeto)))
    (copy-to-buffer resultados (car limites) (cdr limites))
    (erase-buffer)
    (set-buffer resultados)
    (setq buffer-read-only t)
    (iniciar-ventanas)
;    (set-window-buffer inferior resultados)
    (select-window superior)
    (envie "siga")
    (procesar-objeto)))
    
(defun mostrar-archivo ()
  "Atiende un pedido de edicion, hecho por ManTa"
  (let ((limites (tomar-objeto)))
    (goto-char (car limites))
    (end-of-line)
    (let ((num-archs (buffer-substring (car limites) (point))))
      (cond ((string= num-archs "1")
	     (goto-char (car limites))
	     (forward-line 1)
	     (editar-un-archivo))
	    ((string= num-archs "2")           ; SE EDITAN DOS ARCHIVOS 
	     (goto-char (car limites))
	     (forward-line 1)
	     (editar-dos-archivos))
	    (t (message "numero de archivos desconocido: <%s>" num-archs))))))


(defun tomar-trio-edicion ()
  "Toma del buffer 'manta-salida' el modo de edicion, el nombre, y linea actual
de un archivo a editar."
  (let ((inicio (point)))
    (end-of-line)
    (let ((modo (buffer-substring inicio (point))))
      (beginning-of-line)
      (forward-line 1)
      (setq inicio (point))
      (end-of-line)
      (let ((linea (buffer-substring inicio (point))))
	(beginning-of-line)
	(forward-line 1)
	(setq inicio (point))
	(end-of-line)
	(let ((nombre (buffer-substring inicio (point))))
	  (beginning-of-line)
	  (forward-line 1)
	  (cons modo (cons nombre linea)))))))

	
(defun editar-un-archivo ()
  "Hace la edicion de un archivo."
  (let ((trio (tomar-trio-edicion)))
    (cond ((string= (car trio) "0")          ; ARCHIVO NO MODIFICABLE
;	   (set-buffer menu)
;	   (use-local-map nil)               ;inhibe los menus
	   (delete-other-windows)
	   (view-file (car (cdr trio)))
	   (iniciar-ventanas)
	   (envie "siga")
	   (procesar-objeto))
	  ((string= (car trio) "1")           ; ARCHIVO MODIFICABLE
;	   (set-buffer menu)
;	   (use-local-map nil)                ;inhibe los menus
	   (find-file (car (cdr trio)))
	   (goto-line (string-to-int (cdr (cdr trio))))
	   (delete-other-windows)
	   (message "Type C-c t for save and quit")
	   (local-set-key "\C-ct" 'fin-edicion))
	  (t
	   (message "modo de edicion desconocido : <%s>" modo)))))


(defun editar-dos-archivos ()
  "Hace la edicion de dos archivos." 
  (let ((trio1 (tomar-trio-edicion))
	(trio2 (tomar-trio-edicion)))
    (setq buffer1 (find-file-noselect (car (cdr trio1))))
    (setq buffer2 (find-file-noselect (car (cdr trio2))))
    (set-buffer buffer1)
    (if (string= (car trio1) "0")
	(setq buffer-read-only t)
      (setq buffer-read-only nil)
      (goto-line (string-to-int (cdr (cdr trio1)))))
    (set-buffer buffer2)
    (if (string= (car trio2) "0")
	(setq buffer-read-only t)
      (setq buffer-read-only nil)
      (goto-line (string-to-int (cdr (cdr trio2)))))
;    (set-buffer menu)
;    (use-local-map nil)                   ;inhibe los menus
    (delete-other-windows)
    (setq superior (selected-window))
    (setq inferior (split-window superior))
    (set-window-buffer superior buffer1)
    (set-window-buffer inferior buffer2)
    (select-window inferior)
    (message "Type C-c t for save and quit")
    (local-set-key "\C-ct" 'fin-edicion-2w)))



(defun fin-edicion ()
  "Recibe control cuando el usuario termina la edicion de un archivo,
mediante el comando C-c t."  
  (interactive)
  (basic-save-buffer)
  (local-unset-key "\C-ct")
  (kill-buffer (current-buffer))
  (iniciar-ventanas)
  (envie "siga")
  (procesar-objeto))


(defun fin-edicion-2w ()
  "Recibe control cuando el usuario termina la edicion de dos archivos,
mediante el comando C-c t."
    (interactive)
    (set-buffer buffer1)
    (if (not buffer-read-only)
	(basic-save-buffer))
    (set-buffer buffer2)
    (if (not buffer-read-only)
	(basic-save-buffer))
    (kill-buffer buffer1)
    (kill-buffer buffer2)
    (setq buffer1 nil)
    (setq buffer2 nil)
    (iniciar-ventanas)
    (envie "siga")
    (procesar-objeto))


(defun compilar-prototipo ()
  "Procesar el pedido de compilacion hecho por 'manta'."
  (let ((limites (tomar-objeto)))
    (goto-char (car limites))
    (end-of-line)
    (let ((directorio (buffer-substring (car limites) (point))))
      (goto-char (car limites))
      (forward-line 1)
      (let ((inicio (point)))
	(end-of-line)
	(let ((make-f (buffer-substring inicio (point))))
	  (goto-char (car limites))
	  (forward-line 2)
	  (setq inicio (point))
	  (end-of-line)
	  (let ((info (buffer-substring inicio (point))))
	    (dired directorio)
	    (compile (concat "make -f " make-f))
	    (find-file (concat "../" info))
;	    (set-buffer menu)
;	    (use-local-map nil)
	    (message "Type C-c t for quit")
	    (local-set-key "\C-ct" 'fin-compilacion)))))))

(defun fin-compilacion ()
  "Recibe control cuando el usuario termina la compilacion de un prototipo,
mediante el comando C-c t."
  (interactive)
  (local-unset-key "\C-ct")
  (if (get-buffer "temp2")
      (kill-buffer "temp2"))
  (iniciar-ventanas)
  (envie "siga")
  (procesar-objeto))


(defun ejecutar-prototipo ()
  "Procesar la solicitud de ejecutar un prototipo."
  (let ((limites (tomar-objeto)))
    (let ((comando (buffer-substring (car limites) (cdr limites))))
      (delete-other-windows)
      (shell)
      (insert comando)
;      (set-buffer menu)
;      (use-local-map nil)
      (message "Type C-c t for quit")
      (local-set-key "\C-ct" 'fin-shell-prot))))

(defun fin-shell-prot ()
  "Recibe control cuando el usuario termina la ejecucion de un prototipo,
mediante el comando C-c t."
  (interactive)
  (local-unset-key "\C-ct")
  (iniciar-ventanas)
  (kill-buffer "*shell*")
  (envie "siga")
  (procesar-objeto))


(defun actualizar-status ()
  (let ((limites (tomar-objeto)))
    (goto-char (car limites))
    (end-of-line)
    (let ((variable (buffer-substring (car limites) (point))))
      (goto-char (car limites))
      (forward-line 1)
      (let ((inicio (point)))
	(end-of-line)
	(let ((valor (buffer-substring inicio (point))))
	  (cond ((string= variable "tipo")
		 (setq tipo-actual valor))
		((string= variable "representacion")
		 (setq repr-actual valor))
		(t (message "variable de status desconocida <%s>" variable))))))
    (envie "siga")
    (procesar-objeto)))

		 
(defun seleccionar-opcion ()
  "Recibe control cuando el usuario elige una opcion (con la tecla Return).
Toma la opcion elegida y se la envia a ManTa."
  (interactive)
  (beginning-of-line)
  (if (not (= (point) (point-min)))
      (let ((actual (point)))
	(end-of-line)
	(let ((opcion (buffer-substring actual (point))))
	  (envie opcion)
	  (procesar-objeto)))))


(defun mostrar-status ()
  "Recibe control cuando el usuario digita 'C-cs', y muestra el status de
ManTa"
  (interactive)
  (message "ADT : %s      REP : %s" tipo-actual repr-actual))


(defun envie (cadena)
  "Envia al proceso manta el mensaje en <cadena>"
  (send-string (get-process "manta") (concat cadena "\n")))


(defun lea-nombre (cadena)
  "Pide un dato en el minibuffer, despues de escribir alli <cadena>"
  (read-from-minibuffer cadena nil nil nil))

