/* TADS.C							*/
/* Contiene el manejador principal del sistema			*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include "definic.h"
#include "declara.h"
#include "os_dep.h"
#include "interf.h"
#include "apimanta.h"

/**************************************************************************
* main(int argc,char **argv)                                              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Manejar el nivel superior de interaccion MANTA-usuario.    *
* DESCRIPCION: Despliega el menu principal de MANTA e invoca los modulos  *
*        de acuerdo a las selecciones del usuario hasta que este desee    *
*        desee terminar.                                                  *
*        -p : Presentacion                                                *
*        -i : Archivo de iniciacion                                       *
* PRE!:  { TRUE }                                                         *
* POST!: { Se ejecutaron las opciones del menu principal de MANTA de      *
*          acuedo a las selecciones del usuario.                       }  *
* LLAMA A: 								  *
*  void ini_term void			void ini_kbd_vt220		  *
*  *LINEA_PANT desp_presentacion   	int  lea_tecla			  *
*  void borr_presentacion		void cargue_tads_primitivos       * 
*  int  menu				void manejar_tipos                * 
*  void manejar_representaciones   	void manejar_teoremas             * 
*  void manejar_prototipos   		                                  *
*  void borre_pant			void fin_term			  *
*  void fin_kbd_vt220                					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 25-Ene-90 *
* MODIFICADA: Hernan Aponte                                               *
* VTP. 20/Dic/96.  Antes invocaba una funcin para pedir nombre de usuario*
*      esto no es portable y se puso en compilacion condicional           *
*      Hacia llamadas al sistema para borrar archivo (rm) e invocar touch *
*      se reemplazaron por invocaciones a las funciones unlink y touch    *
**************************************************************************/
int
main(int argc,char **argv)
{
  
  int   opcion;
  int i,error;
  char configFile[200];
  char workDir[1000];
  
  workDir[0]='\0';
  strcpy(configFile,"manta.ini\0");
  for (i=1,error=0;!error && i<argc && argv[i][0]=='-';i++)
    {
      switch (argv[i][1]) 
	{
	case 'd': 
	  if (i+1<argc)
	    { 
	      strcpy(workDir,argv[i+1]); 
	      transArch(workDir);
	      if (chdir(workDir)!=0) {
		printf("Could'nt change to directory %s\n",workDir);
		error=1;
	      }
	      i++;
	    }
	  else
	    { error=1; }
	  break;
	case 'i': 
	  if (i+1<argc)
	    { 
	      strcpy(configFile,argv[i+1]); 
	      transArch(configFile);  
	      i++;
	    }
	  else
	    { error=1; }
	  break;
	default:
	  error=1;
	  break;
	}
    }

  if (error) 
    {
      printf("Syntax is: \n"
	     " manta [-d <dir>] [-i <confFile>]\n"
	     "<dir> is the working directory\n"
	     "<confFile> is the configuration file (by default manta.ini)\n");
      return 1;
    }

  if (iniMantaAPI(configFile)<0)
    {
      printf("Initialization Failed (sure you have configuration file %s ?)\n",
	     configFile);
      return 1;
    }

  iniciar_menus();
  
  if (!flag_emacs)
    desp_presentacion();
  
  opcion = admon_menu(menus_manta[MANTA]);
  while ( opcion != NO_OPCION &&
	  opcion != menus_manta[MANTA]->num_op - 2) {
    switch (opcion) {
    case 0:
      manejar_tipos();
      break;
    case 1:
      manejar_teoremas();
      break;
    case 2:
      manejar_prototipos();
      break;
    case 4:
      escriba_mensaje(151);   /* Mensaje de ayuda */
      break;
    default:
      escriba_mensaje(146);   /* Opcion Desconocida */
      break;       
    }
    opcion = admon_menu(menus_manta[MANTA]);
  } 

  salve_todo_tad();
  /*** VTP. Decia system("touch ./basura~; touch ./#basura"); ***/
  /*** VTP. Decia  system("rm -f ./ *~; rm -f ./#*"); sin
       el espacio entre / y el *. ***/
  touch("./basura~");
  touch("./#basura");
  unlink("./basura~");
  unlink("./#basura");
  if (symtable!=NULL)
    {
      freeSymbols(&symtable);
    }
  finMantaAPI();
  if (flag_emacs)
    printf("terminar\nfin-objeto");
  return 0;
}


