/**
 * PROYECTO MANTA
 * DERECHOS RESERVADOS A LA UNIVERSIDAD DE LOS ANDES  (c) 1985-1999
 *
 * API PARA EL KERNEL DE MANTA
 * Descripcion: Contiene funciones para accesar el kernel de ManTa
 * @author   Vladimir Tamara Patino
 * @version 2.0
 * Santafe de Bogota D.C. (COLOMBIA)  1997-1999
 **/

#include "definic.h"


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   Del API para el API
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */


/**
 * Inicializa API de ManTa
 * Inicializa ManTa empleando archivo de configuracion nom.
 *       Los codigos de retorno son:
 *       0 Exito
 *       -1 No pueden retornarse mensajes, memoria insuficiente
 *       -2 Archivo de configuracion incorrecto
 *       -3 Problemas al cargar tabla de simbolos
 *       -4 Problemas al inicializar motor de reescritura
 * @param nom Nombre del archivo de configuracion
 **/
int iniMantaAPI(char *nom);

/**
 * Finaliza API de ManTa.
 * Debe ser invocado despues de haber inicializado el API con iniMantaAPI
 **/
int finMantaAPI();



/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Nombre
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */


/**
 * Crea un nodo nodo_nombre inicializado con 0
 * @param n En este parametro retorna el nuevo nodo (la funcion lozaliza mem);
 * @param merr Buffer para mensajes de error
 * @return 0 sii no hay error
 **/
int nuevoNodoNombre(nodo_nombre **n,char *merr);


/**
 * Retorna el nombre de un nodo_nombre
 * @param n nodo del cual retornar nombre
 * @param nom espacio ya localizado donde se retornar el nombre
 **/
void nombreNodoNombre(nodo_nombre *n, char *nom);


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD LNombre (Lista de Nombres)
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */


/**
 * Inicializa la lista que recibe como una lista vacia.
 * @param ln Cabeza de la lista de nombres por inicializar
 **/
int inicLNombre(nodo_nombre **ln);


/**
 * Saca una copia de una lista de nombres
 * @param n Cabeza de la lista por copiar 
 * @param nf Sera la cabeza de la nueva lista (esta funcion localiza memoria);
 * @param merr Buffer para mensajes de error
 **/
int copiaLNombre(nodo_nombre **n,nodo_nombre *nf,char *merr);


extern const int ADIC_ADD,  /* Agrega siempre */
                 ADIC_REP;  /* Si ya existe reemplaza */

/**
 * Agrega en orden alfabetico el nombre nuevo, Reemplaza
 * o agrega segun el valor remp (ADIC_REP o ADIC_ADD);
 * @param ln Cabeza de la lista de nombres
 * @param nuevo Nuevo nodo por agregar
 * @param remp Forma de agregar el nodo
 *             ADIC_ADD  si existe otro del mismo nombre agrega el nuevo antes
 *             ADIC_REP si existe otro con el mismo nombre lo reemplaza
 *                     (libera la memoria del otro)
 **/
void adicElemLNombre(nodo_nombre **ln,nodo_nombre *nuevo, int remp);




/**
 * Retorna la cantidad de nombres en la lista
 * @param ln Cabeza de la lista de nombres
 **/
int cantElemLNombre(nodo_nombre *ln);


/**
 * Retorna un apuntador al noodo_nombre cuyo nombre es nom
 * Si no existe retorna NULL
 * @param n cabeza de la lista de nombres
 * @param nom Nombre buscado
 **/
nodo_nombre *buscaNombreLNombre(nodo_nombre *n,char *nom);

/**
 * Retorna el nombre del i-esimo nombre de la lista n
 * @param n Cabeza de la lista
 * @param i Indice del cual sacar nombre.  Debe ser menor que la cantidad
 *          de elementos de la lista
 * @param nom En este parametro deja el nombre del i-esimo nodo
 **/
void nombreElemLNombre(nodo_nombre *n,int i, char* nom);


/**
 * Reemplaza toda ocurrencia del nombre old por new
 * @param n Cabeza de la lista
 * @param old Nombre antiguo
 * @param new Nombre nuevo
 **/
void reemplazaLNombre(nodo_nombre *n,char *old,char *new);


/**
 * Libera una lista de nombres
 * @param n cabeza de la lista 
 **/
void elimLNombre(nodo_nombre **n);



/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Dominio
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */


/**
 * Crea un nodo nuevo para un dominio
 * @param n Sera el nuevo nodo (memoria localizada por esta funcion)
 * @param merr Buffer para mensajes de error
 **/
int nuevoNodoDominio(nodo_dom **n,char *merr);



/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD LDominio (Lista de Dominios)
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Reemplaza toda ocurrencia del nombre old por new
 * @param n Cabeza de la lista
 * @param old Nombre antiguo
 * @param new Nombre nuevo
 **/
void reemplazaLDominio(nodo_dom *n,char *old,char *new);


/**
 * Crea una lista de dominios nueva copiando otra existente
 * @param n cabeza de la lista existente
 * @param nf Sera la cabeza de la nueva lista (localizada por esta funcion)
 * @para merr Buffer para manejar mensajes de error
 **/
int copiaLDominio(nodo_dom **n,nodo_dom *nf,char *merr);


/**
 * Libera una lista de dominios
 * @param n cabeza de la lista 
 **/
void elimLDominio(nodo_dom **n);


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Generadora
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Crea un nodo nuevo para una generadora
 * @param n Sera el nuevo nodo (memoria localizada por esta funcion)
 * @param merr Buffer para mensajes de error
 **/
int nuevoNodoGeneradora(nodo_generadora **n,char *merr);


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD LGeneradora
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Crea una lista de generadoras nueva copiando otra existente
 * @param n cabeza de la lista existente
 * @param nf Sera la cabeza de la nueva lista (localizada por esta funcion)
 * @para merr Buffer para manejar mensajes de error
 **/
int copiaLGeneradora(nodo_generadora **n,nodo_generadora *nf,char *merr);



/**
 * Libera una lista de generadoras
 * @param n cabeza de la lista 
 **/
void elimLGeneradora(nodo_generadora **n);


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Expresion
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Crea un nodo nuevo para una expresin
 * @param n Sera el nuevo nodo (memoria localizada por esta funcion)
 * @param merr Buffer para mensajes de error
 **/
int nuevoNodoExpresion(nodo_expresion **n,char *merr);


/**
 * Crea una expresin nueva copiando otra existente
 * @param n expresin existente
 * @param nf Sera la nueva expresin (localizada por esta funcion)
 * @para merr Buffer para manejar mensajes de error
 **/
int copiaNodoExpresion(nodo_expresion **n,nodo_expresion *nf,char *merr);



/**
 * Libera una expresion
 * @param n cabeza de la lista 
 **/
void elimNodoExpresion(nodo_expresion **n);


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Argumento
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Crea un nodo nuevo para un argumento
 * @param n Sera el nuevo nodo (memoria localizada por esta funcion)
 * @param merr Buffer para mensajes de error
 **/
int nuevoNodoArgumento(nodo_argum **n,char *merr);

/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD LArgumento
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */


/**
 * Crea una lista de argumentos nueva copiando otra existente
 * @param n cabeza de la lista existente
 * @param nf Sera la cabeza de la nueva lista (localizada por esta funcion)
 * @para merr Buffer para manejar mensajes de error
 **/
int copiaLArgumento(nodo_argum **n,nodo_argum *nf,char *merr);

/**
 * Libera una lista de argumentos
 * @param n cabeza de la lista 
 **/
void elimLArgumento(nodo_argum **n);


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Igualdad
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Crea un nodo nuevo para una igualdad
 * @param n Sera el nuevo nodo (memoria localizada por esta funcion)
 * @param merr Buffer para mensajes de error
 **/
int nuevoNodoIgualdad(nodo_igualdad **n,char *merr);

/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD LIgualdad
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Crea una lista de igualdades nueva copiando otra existente
 * @param n cabeza de la lista existente
 * @param nf Sera la cabeza de la nueva lista (localizada por esta funcion)
 * @para merr Buffer para manejar mensajes de error
 **/
int copiaLIgualdad(nodo_igualdad **n,nodo_igualdad *nf,char *merr);

/**
 * Libera una lista de igualdades
 * @param n cabeza de la lista 
 **/
void elimLIgualdad(nodo_igualdad **n);


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Selectora
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Crea un nodo nuevo para una selectora
 * @param n Sera el nuevo nodo (memoria localizada por esta funcion)
 * @param merr Buffer para mensajes de error
 **/
int nuevoNodoSelectora(nodo_select **n,char *merr);


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD LSelectora
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Crea una lista de selectoras nueva copiando otra existente
 * @param n cabeza de la lista existente
 * @param nf Sera la cabeza de la nueva lista (localizada por esta funcion)
 * @para merr Buffer para manejar mensajes de error
 **/
int copiaLSelectora(nodo_select **n,nodo_select *nf,char *merr);


/**
 * Busca una selectora por nombre en una lista
 * @param sel Cabeza de la lista
 * @paam nombre Nombre de la selectora buscada
 * @return Un apuntado al nodo con la selectora o NULL si no esta
 **/
nodo_select *buscaNombreLSelectora(nodo_select *sel,char *nombre);


/**
 * Busca una selectora por el nombre de la fuente en una lista
 * @param sel Cabeza de la lista
 * @paam nombre Nombre de la fuente
 * @return Un apuntado al nodo con la selectora o NULL si no esta
 **/
nodo_select *buscaFuenteLSelectora(nodo_select *sel,char *nombre_fte);


/**
 * Libera una lista de selectoras
 * @param n cabeza de la lista 
 **/
void elimLSelectora(nodo_select **n);


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Tipo
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */


/**
 * Crea un nuevo nodo de un tipo
 * @param p Apuntara al nuevo tipo (localizado por esta funcion)
 * @param merr Buffer de errores
 * @return 0 si y solo si no hay error
 **/
int nuevoNodoTipo(nodo_tad **p,char *merr);


/**
 * Retorna el nombre del TAD t
 * Si t es NULL retorna un nombre vacio
 * @param t Tad del cual se obtendra el nombre
 * @param largo Indica si se debe retornar el nombre largo
 * @param nom En este buffer queda el nombre (debe venir localizado)
 **/
void nombreNodoTipo(nodo_tad *t, int largo, char *nom);


/**
 * Retorna la cantidad de parametros del tad que recibe
 * @param t tad por examinar
 * @return La cantidad de parametros del tipo
 **/
int numParamT(nodo_tad *t);


/**
 * Retorna un parametro del tad recibido
 * @param t Tad por examinar
 * @param i Numero de Parametro por devolver
 * @param nom En este buffer deja el nombre (debe venir localizado)
 **/
void nombreParamT(nodo_tad *t,int i,char *nom);


/**
 * Indica si el tipo es inservible
 * @param t El tipo por revisar
 * @return Verdadero si el tipo es inservible
 **/
int tipoMalo(nodo_tad *t,char *merr);


/**
 * Indica si un tipo es basico
 * @param t1 Tipo por revisar
 * @rturn verdadero si y solo si es un tipo bsico
 **/
int esTipoBasico(nodo_tad *t1);


/**
 * Indica si un tipo es extensin (sin generadoras ni parametros)
 * @param t1 Tipo por revisar
 * @param merr Buffer para mensajes de error
 * @return 1 si es extension
 *         0 y un mensaje de error en el buffer si no es
 **/
int esTipoExtension(nodo_tad *t1,char *merr);



/**
 * Indica si un tipo es una representacion de otro
 * @param r Tipo por verificar
 * @param merr Buffer con mensajes de error
 * @return 1 si es representacion
 *         0 si no lo es y un mensaje en el buffer merr
 **/
int esRepresentacion(nodo_tad *r,char *merr);


/**
 * Elimina de memoria el TAD que recibe
 * OJO: Si es parte de una lista deja el resto de la lista, y solo
 *      elimina el nodo solicitado
 * @param nt TAD por eliminar
 **/
void elimNodoTipo(nodo_tad **nt);


/**
 * Crea una copia de un tipo
 * @param dest En este quedara la copia (memoria localizada por esta func)
 * @param fte  Tad fuente
 * @param merr Buffer para mensajes de error
 * @param Retorna 0 si y solo si no hay errores
 **/
int copiaNodoTipo(nodo_tad **dest,nodo_tad *fte,char *merr);


/**
 * Carga un tipo del archivo bytecode abierto que recibe 
 * @param fp Archivo abierto para lectura
 * @param nt En este sitio dejara el tipo cargado (localizado por esta func.)
 * @param merr Buffer para mensajes de error
 * @return 0 si y solo si no hay errores
 **/
int cargaTipo(FILE *fp,nodo_tad **nt,char *merr);



/**
 * Salva un tipo en un archivo en formato bytecode
 * @param fp Archivo abierto para escritura
 * @param nt TAD por escribir
 * @return 0 si y solo si no hay errores
 **/
int salvaTipo(FILE *fp,nodo_tad *nt);


/**
 * Carga del archivo tipo texto abierto que recibe un TAD
 * (hace analisis sintactico) y llena con este el tad que recibe
 * @param fp Archivo tipo texto abierto
 * @param tad En este sitio dejara el tipo cargado (localizado por esta func.)
 * @param merr Buffer para mensajes de error
 * @return 0 si y solo si no hay errores
 **/
int cargaTipoTexto(FILE *fp, nodo_tad **tad,char *merr);


/**
 * Crea una archivo con la representacion del TAD tad en el
 * lenguaje de ManTa y la pone en el archivo abierto fp.
 * @param fp Archivo tipo texto abierto para escritura
 * @param tad Tad que se escribira
 **/
void salvaTipoTexto(FILE *fp,nodo_tad *tad);


/**** PARA DEPURAR ***/
void muestraTipo(nodo_tad *tad);



/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD LTipo
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */


/** 
 * Inicializa el ambiente vacio
 **/
int inicLTipo(nodo_tad **L);


/**
 * Agrega el elemento nuevo al final de la lista, si ya habia
 * uno con el nombre, el comportamiento depende del parametro remp.
 * @param tad  Cabeza de la lista de Tads
 * @param nuevo Tad por Agregar
 * @param remp  Forma de agregar:
 *              ADIC_ADD  si existe otro del mismo nombre agrega el nuevo antes
 *              ADIC_REP si existe otro con el mismo nombre lo reemplaza
 *                       (libera la memoria del otro)
 * @param merr  Buffer con mensajes de error
 **/
int adicElemLTipo(nodo_tad **tad,nodo_tad *nuevo, int remp,char *merr);


/**
 * Retorna la cantidad de Tipos en la lista iniciada con tad
 * @param tad Cabeza de la lista de TADs
 * @return Cantidad de elementos de la lista
 **/
int cantElemLTipo(nodo_tad *tad);


/**
 * Retorna un apuntador al TAD cuyo nombre es nomtad
 * @return apuntado al TAD o NULL si no existe
 **/
nodo_tad *buscaNombreLTipo(nodo_tad *tad,char *nomtad);


/**
 * Retorna el nombre del i-esimo Tipo de la lista tad
 * @param tad Cabeza de la lista de tads
 * @param i Indice del elemento
 * @param nom Apuntar al nombre del elemento que est en la pos. i
 **/
void nombreElemLTipo(nodo_tad *tad,int i, char* nom);



/**
 * Si existe un tipo con el nombre recibido lo elimina
 * @param tad Cabeza de la lista de TADs
 * @param nom Nombre del tipo por eliminar
 **/
void elimElemLTipo(nodo_tad **tad,char *nom);


/**** PARA DEPURAR ***/

/**
 * Muestra una lista de tipos en pantalla
 * @param t Cabeza de la lista 
 **/
void muestraLTipo(nodo_tad *t);


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Tabla de Simbolos.  Extension al TAD LTipos
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/* 
   Indica si el nombre de un tipo es valido
   nuevo  es verdadero si el tipo es nuevo 
          (en tal caso el nombre no puede estar definido)
*/
int nombreTipoValido(char *nom,int nuevo,char *merr);


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Ambiente.  Extension a TAD LTipos y Tabla de Simbolos
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */


/**
 * Adiciona un tipo al ambiente y actualiza la tabla de simbolos
 * @param tad Cabeza de la lista de tads que conforman el ambiente
 * @param symtabl Cabeza de la tabla de simbolos
 * @param nuevo Nuevo TAD por adicionar
 * @param merr Buffer para mensajes de error
 * @return 0 si y solo si no hay error
 **/
int adicTipoAmbiente(nodo_tad **tad,nodo_tad **symtable, nodo_tad *nuevo,
		     char *merr);


/**
 *  Carga un tipo en caso de que no este en el ambiente
 * @param tadAmb Cabeza del ambiente
 * @param nom    Nombre del tipo por cargar 
 * @param merr   Buffera para mensajes de error
 **/
nodo_tad* tipoAmbiente(nodo_tad **tadAmb,char *nom,char *merr);

     
/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Prototipos en C.  Extension al TAD Tipo
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

extern const int CP_ENC,
  CP_CON,
  CP_SEL,
  CP_INT,
  CP_MAKE,  
  CP_MAIN,
  CP_TODO;

/**
 * Genera los archivos del prototipo en C del TAD que
 * recibe, los archivo generados se especifican en que
 * con las constantes CP_CON,CP_SEL,CP_INT,CP_MAKE,CP_MAIN o CP_TODO
 * @para t Tad por generar
 * @param que Constante que indica que generar 
 * @return 0 si y solo si no hay error
 **/
int generaCProt(nodo_tad *t,int que);


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Generacion Tipos.  Extension al TAD Tipo
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Crea un nuevo nodo de un tipo, por instanciacin
 * @param nombre 	Nombre del nuevo Tipo
 * @param p		Tad por instanciar
 * @param nomparam      Nombre del parametro de p por instanciar
 * @param p_inst	Tad con el cual instanciar
 * @param nuevo         Nuevo TAD (localiza memoria)
 * @param merr          Buffer para mensajes de error
 * @return Retorna 0 si y solo si no hay errores
 **/
int nuevoInstanciacionT(char *nombre,nodo_tad *p, char *nom_param,
                        nodo_tad *p_inst, nodo_tad **nuevo,char *merr);


/**
 * Crea un nuevo nodo de un tipo, por copia de otro
 * @param nom 	    Nombre del nuevo Tipo
 * @param fuente    Nodo del Tad Fuente
 * @param nuevo     Nuevo TAD (localiza memoria)
 * @param merr      Buffer para mensajes de error
 * @return Retorna 0 si y solo si no hay errores
 **/
int nuevoCopiaT(char *nom, nodo_tad *fuente, nodo_tad **nuevo,char *merr);



/**
 * Crea un nuevo nodo de un tipo como producto cartesiano de 2
 * @param  nom 	     Nombre del nuevo Tipo
 * @param  t1        Primer Tipo para el producto
 * @param  t2        Segundo Tipo para el producto
 * @param  nuevo     Nuevo TAD (localiza memoria)
 * @param merr      Buffer para mensajes de error
 * @return Retorna 0 si y solo si no hay errores
 **/
int nuevoProdT(char *nom, nodo_tad *t1, nodo_tad *t2, 
	       nodo_tad **nuevo ,char *merr);


/**
 * Genera lados izquierdos de los axiomas para las selectoras de un 
 * tipo por restriccin
 * @param sel     Nodo de selectora
 * @param tr      Tipo que se esta restringiendo
 * @param li_str  Buffer donde retorna el lado izquierdo del axioma 
 *                (ya localizado)
 * @param trvar   Buffer para retornar una lista de variables del tipo que 
 *                se restringe separadas por | (ya localizado)
 **/
void genereAxiomaRestriccionIzquierdo(nodo_select *sel,
				      char *tr,
				      char *li_str,
				      char *trvar);

/**
 * Genera lados derechos de los axiomas para las selectoras de un 
 * tipo por restriccin
 * @param sel     Nodo de selectora 
 * @param trvar   Lista de variables del tipo que se restringe separadas por |
 * @param npred   Nombre de la funcion de restriccion
 * @param li_str  Lado izquierdo lleno
 * @param ld_str  Buffer para guardar el lado derecho calculado (ya localizado)
 **/
void genereAxiomaRestriccionDerecho(nodo_select *sel,
				    char *trvar,
				    char *npred,
				    char *li_str,
				    char *ld_str);
/**
 * Genera los axiomas para la selectora sel, sabiendo que
 * es una selectora de un tipo que restringe otro
 * @param tr    Es el tad que se restringe
 * @param sel   La selectora a la que se le crean (cambian) axiomas
 * @param npred Es el nombre del predicado de restriccion
 * @param merr  Buffer para errores
 * @return 0 si y solo no hay error
 **/
int genereAxiomaRestriccion(char *tr,nodo_select *sel,char *npred, char *merr);


/**
 * Cambia todos los parametros de la selectora sel a inductivos o no 
 * segun el parametro ind
 * @param sel  Selectora cuyo dominio se cambiara
 * @param ind  Valor que se pondra al campo induccion de todos 
 *             los elementos del dominio
 **/
void cambiaParametrosInduccion(nodo_select *sel, short ind);


/**
 * Crea un nuevo nodo de un tipo como restriccion de otro por
 * un predicado
 * @param nom 	    Nombre del nuevo Tipo
 * @param tf        Tad por restringir
 * @param npred     Nombre del Predicado
 * @param pred      Predicado para restriccion
 * @param fp        Archivo abierto para escritura donde deja la
 *                  definicion parcial del TAD
 * @param nuevo     Apuntara a un nuevo tad (definido parcialmente)
 * @param merr      Buffer para mensajes de error
 * @return 0 si y solo si no hay error
 **/
int nuevoResT(char *nom, nodo_tad *tf, char *npred, char *pred, 
              FILE *fp,nodo_tad **nuevo,char *merr);


/**
 * Crea un nuevo nodo de un tipo como suma directa de los dos
 * tipos que recibe
 * @param nom 	    Nombre del nuevo Tipo
 * @param t1        Primer TAD por sumar
 * @param t2        Segundo TAD por sumar
 * @param fp        Archivo abierto para escritura donde deja la
 *                  definicion parcial del TAD
 * @param nuevo     Apuntara a un nuevo tad (definido parcialmente)
 * @param merr      Buffer para mensajes de error
 * @return 0 si y solo si no hay error
 **/
int nuevoSumT(char *nom, nodo_tad *t1, nodo_tad *t2,
              FILE *fp,nodo_tad **nuevo,char *merr);


/**
 * Crea un nuevo nodo de un tipo por completacion con un elmento
 * @param nom 	    Nombre del nuevo Tipo
 * @param tf        Tad por restringir
 * @param ne        Nombre del nuevo elemento
 * @param fp        Archivo abierto para escritura donde deja la
 *                  definicion parcial del TAD
 * @param nuevo     Apuntara a un nuevo tad (definido parcialmente)
 * @param merr      Buffer para mensajes de error
 * @return 0 si y solo si no hay error
 **/
int nuevoComT(char *nom, nodo_tad *tf, char *ne,
              FILE *fp,nodo_tad **nuevo,char *merr);



/**
 * Crea un nuevo nodo de un tipo como variacion de dos existentes
 * @param nom 	    Nombre del nuevo Tipo
 * @param t1        Primer TAD por sumar
 * @param t2        Segundo TAD por sumar
 * @param fp        Archivo abierto para escritura donde deja la
 *                  definicion parcial del TAD
 * @param nuevo     Apuntara a un nuevo tad (definido parcialmente)
 * @param merr      Buffer para mensajes de error
 * @return 0 si y solo si no hay error
 **/
int nuevoVarT(char *nom, nodo_tad *t1, nodo_tad *t2,
              FILE *fp,nodo_tad **nuevo,char *merr);


/**
 * Crea un nuevo nodo de un tipo inductivo
 * @param nom 	Nombre del nuevo Tipo
 * @param fp        Archivo abierto para escritura donde deja la
 *                 definicion parcial del TAD
 * @param nuevo     Apuntara a un nuevo tad (definido parcialmente)
 * @param merr      Buffer para mensajes de error
 * @return 0 si y solo si no hay error
 **/
int nuevoIndT(char *nom, FILE *fp,nodo_tad **nuevo,char *merr);

/**
 * Las generadoras del tipo representado se copian como selectoras
 * en un tipo nuevo cambiando los nombres (agregando como prefijo 
 * el nombre de la representacion.)
 * Tambien cambiamos en todo dominio y codominio las ocurrencias 
 *  del tad representado con el tad que representa.
 *  Ningun dominio es inductivo en principio
 * @param rep Representacion que se construye
 * @param gen Generadoras del tad que se representa
 * @param antTipo nombre del TAD que se representa
 * @param nuevoTipo Nombre del TAD con el que se representa
 * @param merr Buffer para mensajes de error
 * @return codigo de error o 0 si no hay problemas
 **/
int generadorasRep(nodo_tad *rep,nodo_generadora *gen,
		   char *antTipo,char *nuevoTipo,char *merr);



/**
 * Un conjunto de selectoras sel se copia  como selectoras del nuevo tipo rep
 * cambiando los nombres (agregando como prefijo el nombre de la 
 * representacion.)
 * Tambien cambiamos en todo dominio y codominio las ocurrencias 
 * del tad antTipo por nuevoTipo.  Ningun dominio es inductivo en principio
 * @param rep TAD al que se agregan selectoras
 * @param s Nuevas selectoras
 * @param antTipo Nombre del tipo antiguo
 * @param nuevoTipo Nombre del nuevo tipo
 * @param merr Buffer para mensajes de error
 * @return Codigo de error o 0 si no hay problemas
 **/
int selectorasRep(nodo_tad *rep,nodo_select *s,
		   char *antTipo,char *nuevoTipo,char *merr);

/**
 * Crea un TAD extensin para representar t1 sobre t2
 * @param nom Nombre del nuevo tipo
 * @param t1 TAD que se representa
 * @param t2 TAD con el que se representa
 * @param nuevo Apuntara al nuevo tad (localizado por esta funcion)
 * @param merr Buffer para mensajes de error
 * @return Codigo de error o 0 si no hay problemas
 **/
int creaTADRep(char *nom,nodo_tad *t1,nodo_tad *t2,
	       nodo_tad **nuevo,char *merr);


/**
 * Crea la definicion parcial de una representacion del TAD t1 en el TAD t2
 * @param nom Nombre del nuevo tipo
 * @param t1 TAD que se representa
 * @param t2 TAD con el que se representa
 * @param fp Archivo abierto para escritura donde dejara un TAD por completar
 * @param nuevo Apuntara al nuevo TAD
 * @param merr Buffer para mensajes de error
 **/
int nuevaRep(char *nom,nodo_tad *t1, nodo_tad *t2,
	     FILE *fp,nodo_tad **nuevo,char *merr);

/**
 * Cambia toda ocurrencia de una selectora fuente 
 * por la asociada en nsel
 * @param expr Expresion donde se hara el cambio
 * @param nsel Selectoras con las que se remplazarn las antiguas
 * @param oldtype nombre del tipo anterior
 * @param newtype nombre del nuevo tipo
 **/
void cambiaSelectorasExpresion(nodo_expresion *expr,nodo_select *nsel,
			       char *oldtype, char *newtype);


/**
 * Analiza cambios a una representacion (despues de intervencion del
 * usuario) y la actualiza con tales cambios
 * @param rep Representacion original
 * @param fp Archivo abierto para lectura con cambios del usuario
 * @param merr Buffer para mensajes de error
 * @return Codigo de error o 0 si no hay problemas
 **/
int cambiosRep(nodo_tad **rep,FILE *fp,char *merr);


/**
 * Crea una nueva representacion por composicion (r2 o r1)
 * @param nom Nombre de la nueva representacion
 * @param r1 Representacion por componer
 * @param r2 Representacion por componer
 * @param nuevo Apuntara a la nueva representacion (localizada por esta fun)
 * @param merr Buffer para mensajes de error
 * @return 0 si y solo no hay problemas
 **/
int componeRep(char *nom,nodo_tad *r1, nodo_tad *r2,
	       nodo_tad **nuevo,char *merr);


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Entrada
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Libera una entrada
 * @param n cabeza de la lista 
 **/
void elimNodoEntrada(nodo_entrada **ne);


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Tabla
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Libera una Tabla
 * @param n cabeza de la lista 
 **/
void elimLTabla(Ptr_tabla *t);


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   Tad Motor de Reescritura
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */


/**
 * Normaliza la expresin que recibe
 * @param expr Expresion que se normaliza
 * @param merr Buffer para mensajes de error
 * @return 
 **/
int normalizaExpresion(nodo_expresion **expr,char *merr);



/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Teoremas
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */


/**
 *  Crea las conjeturas necesarias para probar una representacion
 * @param fp Archivo abierto donde dejara las conjeturas
 * @param rep Representacion por probar
 * @param merr Buffer con mensajes de error
 * @return Codigo de error o 0 si no hay problemas
 **/
int conjeturasRep(FILE *fp,nodo_tad *rep,char *merr);


/**
 * Intenta probar las conjeturas que hay en un archivo
 * @param fp     Archivo con conjeturas
 * @param nprob  Numero de conjeturas probadas
 * @param prob   Apuntador a las conjeturas probadas
 * @param nnoprob Numero de conjeturas no probadas
 * @param noprob Apuntador a las conjeturas no probadas
 * @param merr   Buffer para mensajes de error
 * @return Codigo de error o 0 si no hay problemas
 **/
int probarConjeturas(FILE *fp,int *nprob,nodo_demost **prob,
                     int *nnoprob,nodo_demost **noprob,char *merr);


/**
 * En un archivo escribe demostraciones de teoremas probados
 * @param fp     Archivo en el cual escribe
 * @param prob   Apuntador a los teoremas probados
 * @return Codigo de error o 0 si no hay problemas
 **/
int escribirPrueba(FILE *fp,nodo_demost *prob);


/**
 * En un archivo escribe el intento de prueba de una conjetura
 * @param fp     Archivo en el cual escribe
 * @param noprob Apuntador a las conjeturas que no logro probar
 * @return Codigo de error o 0 si no hay problemas
 **/
int escribirIntentoPrueba(FILE *fp,nodo_demost *noprob);

/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   Utilidades
   @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Dice si el nombre que recibe es sintacticamente valido para un TAD 
 * o nombre de funcion
 * @param nom Nombre por revisar
 * @return Verdadero si y solo es un nombre valido
 **/
int esNombreValido(char *nom);


/**
 * Compara dos cadenas sin tener en cuenta capitalizacion.
 * Este es un reemplaza para strcasecmp que no es ANSI
 * @param a Cadena  por comparar
 * @param b Cadena  por comparar
 * @return Un positivo si a>b, un negativoo si a<b y 0 si son iguales
 **/
int strcmp_nocase(char *a,char *b);

/** 
 * Retorna una copia de la cadena que recibe.
 * Este es un remplazo para strdup que no es ANSI
 * @param c Cadena por duplicar
 * @return Un apuntador a un duplicado o NULL si no logra localizar memoria
 **/
char *duplicastr(char *c); 


