 /* PROTOTIPOS.C                                                 */
/* Arhivo de manejo de Prototipos.                              */

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include "definic.h"
#include "os_dep.h"

#include <assert.h>		/* VTP. 21/4/97 */

/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* j-tamara 29/May/1997 */


/************************************************************************
* void manejar_prototipos()                                             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Controla el proceso de administracion de prototipos.    *
* DESCRIPCION:  Despliega el menu de opciones de prototipos e invoca la *
*       funcion correspondiente a la seleccion del usuario.             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz                                    FECHA: 20 Feb 91 *
************************************************************************/

void
manejar_prototipos ()
{
  int opcion;

  opcion = admon_menu (menus_manta[PROTOTIPOS]);
  while (opcion != NO_OPCION &&
	 opcion != menus_manta[PROTOTIPOS]->num_op - 2)
    {
      switch (opcion)
	{
	case 0:		/* DIRECTORIO */
	  escriba_dir (direct[7], "h");
	  break;
	case 1:		/* CREACION AUTOMATICA */
	  nuevo_prototipo (tad_actual, 1);
	  break;
/* VTP. 8-Dec-1998	case 2:		 CREACION MANUAL
	  nuevo_prototipo (tad_actual, 0);
	  break; */
	case 2:		/* EDITAR */
	  editar_prot (tad_actual);
	  break;
	case 3:		/* COMPILAR */
	  compilar (tad_actual);
	  break;
	case 4:		/* EJECUTAR */
	  ejecutar (tad_actual);
	  break;
	case 6:		/* AYUDA */
	  escriba_mensaje (155);
	  break;
	default:
	  escriba_mensaje (146);	/* opc. desconocida */
	  break;
	}
      opcion = admon_menu (menus_manta[PROTOTIPOS]);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void nuevo_prototipo(nodo_tad *t, int autom)                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Desplegar el menu que permite crear un prototipo del tad   *
*              que entra como parametro. Si auto==1 significa creacion    *
*              automatica, si no es creacion manual.                      *
* !OJO!: Si el tipo es primitivo no hace nada.                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Cesar Munoz                                     FECHA: 15-Ene-91 *
**************************************************************************/

void
nuevo_prototipo (nodo_tad * t, int autom)
{
/*----------------------------------------------------------------------*/
  int opcion;

  if ((t == NULL) || tipo_primitivo (t->nombre))
    {
      escriba_mensaje (124);	/* 124:Los tipos primitivos no se generan */
      return;
    }
  do
    {
      opcion = admon_menu (menus_manta[NUEVO_PROTOTIPO]);
      switch (opcion)
	{
	case 0:		/* GENERA TODO */
	  generar_all_prot (t, autom);
	  break;
	case 1:		/* GENERA CONSTRUCTORAS */
	  generar_constructoras (t, autom);
	  break;
	case 2:		/* GENERA SELECTORAS */
	  generar_selectoras (t, autom);
	  break;
	case 3:		/* GENERA ENCABEZADOS */
	  crear_headers (t);
	  break;
	case 4:		/* GENERA FUNC. DE INTERFAZ */
	  crear_interfaz (t, autom);
	  break;
	case 5:		/* SALIR */
	  break;
	case 6:		/* AYUDA */
	  if (autom == 1)
	    escriba_mensaje (162);
	  else
	    escriba_mensaje (164);
	  break;
	case NO_OPCION:
	  break;
	default:
	  escriba_mensaje (146);
	  break;
	}
    }
  while (opcion > 5);
  if (opcion >= 0 && opcion <= 4)
    escriba_mensaje (133);	/* 133:Prototipos OK! */
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void generar_all_prot(nodo_tad *t,int autom)                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar todos los archivos que para implantar el tad t.    *
*              Si autom == 1, se generan automaticamente. Si no,solo se   *
*              generan las plantillas.                                    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Cesar Munoz                                     FECHA: 15-Ene-91 *
**************************************************************************/

void
generar_all_prot (nodo_tad * t, int autom)
{
/*----------------------------------------------------------------------*/

  crear_headers (t);		/* genera el archivo <nombre>.h */
  crear_defs (t, autom);	/* genera ell archivo <nombre>_de.h */
  if (autom == 1)
    crear_funciones (t, 1, 1);
  /* genera los archivos <nombre>_co.c y <nombre>.c */
  else
    crear_funciones (t, 2, 2);
  crear_interfaz (t, autom);	/* genera el archivo <nombre>_inter.c */
  generar_main (t);		/* genera el archivo <nombre>_ma.c */
  generar_make (t);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void generar_constructoras(nodo_tad *t,int autom)                       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar todos los archivos que implantan las constructoras.*
*              Si autom == 1, se generan automaticamente. Si no,solo se   *
*              generan las plantillas.                                    *
*              Las selectoras no se generan.                              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Cesar Munoz                                     FECHA: 15-Ene-91 *
**************************************************************************/

void
generar_constructoras (nodo_tad * t, int autom)
{
/*----------------------------------------------------------------------*/

  crear_defs (t, autom);
  if (autom == 1)
    crear_funciones (t, 1, 0);
  else
    crear_funciones (t, 2, 0);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void generar_selectoras(nodo_tad *t,int autom)                          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar todos los archivos que implantan las selectoras.   *
*              Si autom == 1, se generan automaticamente. Si no,solo se   *
*              generan las plantillas.                                    *
*              Las constructoras no se generan.                           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Cesar Munoz                                     FECHA: 15-Ene-91 *
**************************************************************************/

void
generar_selectoras (nodo_tad * t, int autom)
{
/*----------------------------------------------------------------------*/
  if (autom == 1)
    crear_funciones (t, 0, 1);
  else
    crear_funciones (t, 0, 2);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void generar_main(nodo_tad *t)                                           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar automaticamente un programa main, que permite      *
*              probar el tad t.                                           *
* !OJO!: No se implantan los TADs BOOL, NAT.                              *
*        Se supone que los archivos de los tad involucrados han sido      *
*        creados.                                                         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Cesar Munoz                                     FECHA: 15-Ene-91 *
**************************************************************************/

void
generar_main (nodo_tad * t)
{
/*----------------------------------------------------------------------*/

  if ((t == NULL) || tipo_primitivo (t->nombre))
    return;
  crear_main (t);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void generar_make(nodo_tad *t)                                          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar automaticamente un archivo que sirve de entrada    *
*              a la aplicacion makefile. Permite compilar los archivos    *
*              y ejecutar el proyecto.                                    *
* !OJO!: No se implantan los TADs BOOL, NAT.                              *
*        Se supone que los archivos de los tad involucrados han sido      *
*        creados.                                                         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Cesar Munoz                                     FECHA: 15-Ene-91 *
**************************************************************************/

void
generar_make (nodo_tad * t)
{
/*----------------------------------------------------------------------*/

  if ((t == NULL) || tipo_primitivo (t->nombre))
    return;
  crear_make (t);
/*----------------------------------------------------------------------*/
}



/**************************************************************************
* void editar_prot_aux(texpr nombre)                                      *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Editar el archivo de prototipos <nombre> si existe, o avi- *
*              sar que no ha sido generado                                *
* AUTOR: Hernan Aponte                                                    *
**************************************************************************/

void
editar_prot_aux (texpr nombre)
{
/*----------------------------------------------------------------------*/
  /* Alvaro
     texpr   comando;
   */
  trio_edicion edicion[MAX_EDICION];

  if (existe_archivo (nombre, direct[7]))
    {
      edicion[0].modo = 1;
      edicion[0].linea = 1;
      sprintf (edicion[0].nombre, "%s/%s", direct[7], nombre);
      llamar_al_editor (1, edicion, "");
      return;
    }
  else
    {
      escriba_mensaje (128);	/*Este tad no tiene prototipos */
      return;
    }
/*----------------------------------------------------------------------*/
}


/**************************************************************************
* void editar_prot(nodo_tad *t)                                           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Permite editar algun archivo de los prototipos.            *
* AUTOR: Hernan Aponte                                                    *
* MODIFICACIONES:                                                         *
* VTP 13/Ene/97                                                           *
*	Antes borraba un archivo empleando system("rm ... ahora           *
*	se usa unlink (portable para DOS y UNIX)                          *
**************************************************************************/

void
editar_prot (nodo_tad * t)
{
/*----------------------------------------------------------------------*/
  texpr nombre, nombreh, nombrel;
/* Alvaro
   texpr       comando;
   texpr       nom;
   texpr       ape;
 */
  int opcion;
  trio_edicion edicion[MAX_EDICION];

  if ((t == NULL) || tipo_primitivo (t->nombre))
    {
      escriba_mensaje (126);	/* 126:los tipos primitivos no se generan */
      return;
    }

  opcion = admon_menu (menus_manta[PROT_EDICION]);
  while (opcion != NO_OPCION &&
	 opcion != menus_manta[PROT_EDICION]->num_op - 2)
    {
      switch (opcion)
	{
	case 0:		/* ESTRUCTURAS DE DATOS */
	  sprintf (nombre, "%s_de.h", t->nombre);
	  editar_prot_aux (nombre);
	  break;
	case 1:		/* CONSTRUCTORAS */
	  sprintf (nombre, "%s_co.inc", t->nombre);
	  editar_prot_aux (nombre);
	  break;
	case 2:		/* SELECTORAS */
	  sprintf (nombre, "%s.c", t->nombre);
	  sprintf (nombreh, "%s.h", t->nombre);
	  if (existe_archivo (nombre, direct[7]) &&
	      existe_archivo (nombreh, direct[7]))
	    {
	      sprintf (nombrel, "%s/%s", direct[7], nombreh);
	      editar_2w (nombrel, "");
	      edicion[0].modo = 0;
	      edicion[0].linea = 1;
	      sprintf (edicion[0].nombre, "%s/%s", direct[7], nombreh);
	      edicion[1].modo = 1;
	      edicion[1].linea = 1;
	      sprintf (edicion[1].nombre, "%s/%s", direct[7], nombre);
	      llamar_al_editor (2, edicion, "temp");
/*** VTP. Decia system("rm -f temp"); ***/
	      unlink ("temp");
	    }
	  else if (existe_archivo (nombre, direct[7]))
	    {
	      edicion[0].modo = 1;
	      edicion[0].linea = 1;
	      sprintf (edicion[0].nombre, "%s/%s", direct[7], nombre);
	      llamar_al_editor (1, edicion, "");
	    }
	  else if (existe_archivo (nombreh, direct[7]))
	    {
	      edicion[0].modo = 1;
	      edicion[0].linea = 1;
	      sprintf (edicion[0].nombre, "%s/%s", direct[7], nombreh);
	      llamar_al_editor (1, edicion, "");
	    }
	  else
	    {
	      escriba_mensaje (128);	/* 128:Este tad no tiene prototipos */
	    }
	  break;
	case 3:		/* FUNC. DE INTERFAZ */
	  sprintf (nombre, "%s_in.c", t->nombre);
	  sprintf (nombreh, "%s_in.h", t->nombre);
	  if (existe_archivo (nombre, direct[7]) &&
	      existe_archivo (nombreh, direct[7]))
	    {
	      sprintf (nombrel, "%s/%s", direct[7], nombreh);
	      editar_2w (nombrel, "");
	      edicion[0].modo = 0;
	      edicion[0].linea = 1;
	      sprintf (edicion[0].nombre, "%s/%s", direct[7], nombreh);
	      edicion[1].modo = 1;
	      edicion[1].linea = 1;
	      sprintf (edicion[1].nombre, "%s/%s", direct[7], nombre);
	      llamar_al_editor (2, edicion, "temp");
/*** VTP. Decia: system("rm -f temp"); ***/
	      unlink ("temp");
	    }
	  else if (existe_archivo (nombre, direct[7]))
	    {
	      edicion[0].modo = 1;
	      edicion[0].linea = 1;
	      sprintf (edicion[0].nombre, "%s/%s", direct[7], nombre);
	      llamar_al_editor (1, edicion, "");
	    }
	  else if (existe_archivo (nombreh, direct[7]))
	    {
	      edicion[0].modo = 1;
	      edicion[0].linea = 1;
	      sprintf (edicion[0].nombre, "%s/%s", direct[7], nombreh);
	      llamar_al_editor (1, edicion, "");
	    }
	  else
	    {
	      escriba_mensaje (128);	/* 128:este tad no tiene prototipos */
	    }
	  break;
	case 4:		/* PROGRAMA DE PRUEBA */
	  sprintf (nombre, "%s_ma.c", t->nombre);
	  editar_prot_aux (nombre);
	  break;
	case 5:		/* MAKEFILE */
	  sprintf (nombre, "%s.mak", t->nombre);
	  editar_prot_aux (nombre);
	  break;
	case 7:		/* AYUDA */
	  escriba_mensaje (163);
	  break;
	default:
	  escriba_mensaje (146);
	  break;
	}
      opcion = admon_menu (menus_manta[PROT_EDICION]);
    }
/* OJO NO SE HA MODIFICADO ESTO PARA VERIFICAR QUE MODIFICO */
/*    sprintf(comando,"%s/%s",direct[7],nombre);
   strcpy(nom,nombre);
   parta_expresion(nom,ape,'.');
   if (strcmp(ape,"c") && arch_modificado(comando)) {
   sprintf(comando,"touch %s/%s*.c",direct[7],t->nombre);
   system(comando);
   sprintf(comando,"rm %s/%s*.o",direct[7],t->nombre);
   system(comando);
   } */
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void compilar(nodo_tad *t)                                              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Compilar y encadenar todos los archivos que se generaron   *
*              para el tad t.                                             *
* !OJO!: No se implantan los TADs BOOL, NAT.                              *
*        Se supone que los archivos de los tad involucrados han sido      *
*        creados.                                                         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Cesar Munoz                                     FECHA: 15-Ene-91 *
**************************************************************************/

void
compilar (nodo_tad * t)
{
/*----------------------------------------------------------------------*/

  if ((t == NULL) || tipo_primitivo (t->nombre))
    {
      escriba_mensaje (127);	/* 127:los tipos primitivos no se compilan */
      return;
    }
  /* VTP. 8/Mar/1998 se quito  generar_make (t);
          y se agrego al generar todas las fuentes */
  llamar_al_compilador (t);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void llamar_al_compilador(nodo_tad *t)                                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Compilar y encadenar todos los archivos que se generaron   *
*              para el tad t.                                             *
* !OJO!: No se implantan los TADs BOOL, NAT.                              *
*        Se supone que los archivos de los tad involucrados han sido      *
*        creados.                                                         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: HERNAN APONTE                                   FECHA: 15-NOV-94 *
* MODIFICACIONES:                                                         *
* VTP 13/Ene/97                                                           *
*	Antes borraba un archivo empleando system("rm ... ahora           *
*	se usa unlink (portable para DOS y UNIX)                          *
*       Por otra parte para la compilacion fuera de EMACS, creaba un      *
*       script en UNIX que se cambiaba de directorio y ejecutaba make     *
*       En vez del script se emplean las funciones getcwd, chdir y system *
**************************************************************************/

void
llamar_al_compilador (nodo_tad * t)
{
/*----------------------------------------------------------------------*/
  FILE *ejecutable;
  texpr comando;
  char inidir[255];
/*** VTP. 18/Mar/1997.  Directorio de trabajo ***/
  cad6 siga;

  if (flag_emacs)
    {
      ejecutable = fopen ("temp2", "w");
      FileOpen(ejecutable,"temp2");             /* j-tamara 29/May/1997 */
      fprintf (ejecutable, mensajes[121], t->nombre);
      fprintf (ejecutable, mensajes[122]);
      fclose (ejecutable);
      printf ("compilar-prototipo\n%s\n", direct[7]);
      printf ("%s.mak\ntemp2\n", t->nombre);
      printf ("fin-objeto");
      lea_cadena (siga);
/*** VTP. Decia system("rm -f temp2"); ***/
      unlink ("temp2");
      return;
    }
  if (!strcmp (editor[0], "emacs"))
    {
      ejecutable = fopen ("temp2", "w");
      FileOpen(ejecutable,"temp2");             /* j-tamara 29/May/1997 */
      fprintf (ejecutable, mensajes[121], t->nombre);
      fprintf (ejecutable, mensajes[122]);
      fprintf (ejecutable, mensajes[123]);
      fclose (ejecutable);
      ejecutable = fopen ("temp", "w");
      FileOpen(ejecutable,"temp");              /* j-tamara 29/May/1997 */
      fprintf (ejecutable, "(dired \"%s\")\n", direct[7]);
      fprintf (ejecutable, "(compile \"make -f %s.mak\")\n", t->nombre);
      fprintf (ejecutable, "(find-file \"../temp2\")\n");
      fclose (ejecutable);
      sprintf (comando, "%s -l temp", editor[0]);
      system (comando);
    }
  else
    {
/*** VTP. 14/Ene/97 El metodo empleado para compilar no es portable.  Decia:
      ejecutable = fopen("temp","w");
      fprintf(ejecutable,"cd %s\n",direct[7]);
      fprintf(ejecutable,"make -f %s.mak\n", t->nombre);
      fclose(ejecutable);
      system("chmod 755 temp");
      system("temp"); ***/
      getcwd (inidir, 255);
      chdir (direct[7]);
/*      sprintf(comando,"cd D_PROT\n",direct[7]);
      transArch(comando);
      printf(comando);
      system (comando); */
      sprintf (comando, "make -f %s.mak", t->nombre);
      printf(comando);
      system (comando);
/*** VTP. OK ***/
      chdir (inidir);
    }

/*** VTP. Decia: system("rm -f temp"); ***/
  unlink ("temp");
/*** VTP. Decia: system("rm -f temp2"); ***/
  unlink ("temp2");
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void ejecutar(nodo_tad *t)                                              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Ejecutar el prototipo.                                     *
**************************************************************************/

void
ejecutar (nodo_tad * t)
{
/*----------------------------------------------------------------------*/
  texpr comando;
  cad6 siga;

  if ((t == NULL) || tipo_primitivo (t->nombre))
    {
      escriba_mensaje (127);
      return;
    }
  if (flag_emacs)
    {
      printf ("ejecutar-prototipo\n");
      printf ("%s/%s\n", direct[7], t->nombre);
      printf ("fin-objeto");
      lea_cadena (siga);
    }
  else
    {
      sprintf (comando, "%s/%s", direct[7], t->nombre);
      transArch (comando);
      system (comando);
    }
/*----------------------------------------------------------------------*/
}
