 /************************************************************************
* PROYECTO: MANTA          MODULO: anal_teoremas FECHA: 8-Feb-1991      *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
*    PROPOSITO GLOBAL DE LAS FUNCIONES IMPLEMENTADAS EN ESTE MODULO     *
*                                                                       *
* Hacer el analisis sintactico de las conjeturas, y las demostraciones  *
* de conjeturas.                                                        *
*************************************************************************
* AUTOR DE ESTE MODULO: Cesar A. Munoz H.                               *
************************************************************************/

	/*------------------------------------------------------+
        |          Encabezados de librerias estandares          |
        +------------------------------------------------------*/

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include "definic.h"
#include "apimanta.h"
#include "os_dep.h"

/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* v-tamara 8/May/1997 */


/**************************************************************************
* void determine_tipo_teo(cad20 tipo_eq)                                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determina le tipo del siguiente token.                     *
* PARAMETROS:  							          *
*              cad20 tipo_eq      Para retornar el tipo del token.        *
* V.GLOBALES: texpr  token        Contiene el ultimo token leido.         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz                                      FECHA: 19-Jun-91 *
**************************************************************************/

void
determine_tipo_teo (cad20 tipo_eq)
{
/*----------------------------------------------------------------------*/

  assert (token != NULL && tipo_eq != NULL);	/* VTP. 8/May/1997 */

  if (!strcmp (token, claves[8]))
    strcpy (tipo_eq, "");
  else if (!strcmp (token, claves[12]))
    strcpy (tipo_eq, "BOOL");
  else if (!es_generadora_algun_tipo (token, tipo_eq))
    if (!es_selectora_algun_tipo (token, tipo_eq))
      strcpy (tipo_eq, "");

/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short analice_conj(FILE *fp, texpr mens_error,texpr str_conjetura,      *
*                   nodo_expresion **exp)                                 *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Analizar sintacticamente un lado de una conjetura.         *
* PARAMETROS:  							          *
*   FILE        *fp             Ap. al archivo que contiene las conjeturas*
*   texpr       mens_error      Comentario acerca de un eventual error.   *
*   texpr       str_conjetura   Segmento ya analizado del axioma actual.  *
*   nodo_expresion **exp        Ap. a la estructura que se construye a    *
*			        medida que se "parsea" la expresion.      *
* V.GLOBALES: texpr  token      Contiene el ultimo token leido.           *
* RESULTADO:  short             0, Si se detecto algun error. 1, Sino.    *
* PRE!:  {fp apunt. al archivo de conjeturas}                             *
* POST!: {analice_conj = 0, Si la expresion leida contiene algun error    *
*            sintactico, mens_error contiene una explicacion acerca del   *
*            error detectado, o   		  			  *
*         analice_conj = 1, Si la expresion leida es correcta,            *
*	     str_conjetura = "////<Expresion>" , (*e) apunta a la         *
*	     estructura parseada de <Expresion>                           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz                                      FECHA: 19-Jun-91 *
**************************************************************************/

short
analice_conj (FILE * fp, texpr mens_error, texpr str_conjetura,
	      nodo_expresion ** exp)
{
/*----------------------------------------------------------------------*/
  short error;
  nodo_nombre *dom_generadora = NULL;
  nodo_dom *dom_select = NULL;

  assert (token != NULL);	/* VTP. 8/May/1997 */

  error = 0;
  (*exp) = (nodo_expresion *) malloc (sizeof (nodo_expresion));
  HayMem (*exp, sizeof (nodo_expresion));
  strcpy ((*exp)->nombre, token);
  (*exp)->argumentos = NULL;
  determine_tipo_teo ((*exp)->tipo);
  strcpy (mens_error, "");
  strcpy (str_conjetura, token);
  if (!strcmp (token, ""))
    {
      error = 1;
      sprintf (mens_error, mensajes[114], mensajes[28]);
    }
  else if (!es_identificador (token) && strcmp (token, claves[8]) &&
	   strcmp (token, claves[11]) && strcmp (token, claves[12]))
    {
      error = 1;
      sprintf (mens_error, mensajes[36], mensajes[28], token);
    }
  else if (!strcmp (token, claves[11]))
    {
      strcpy ((*exp)->clase, token);
    }
  else if (!strcmp (token, claves[8]))
    {
      strcpy ((*exp)->clase, token);
      strcat (token, "(");
      strcat (str_conjetura, "(");
      error = (short) (!analice_if (fp, mens_error, str_conjetura, &((*exp)->argumentos),
				    NULL, (*exp)->tipo));
    }
  else if ((!strcmp (token, claves[12])) &&
	   (!strcmp ((*exp)->tipo, "BOOL")))
    {
      strcpy ((*exp)->clase, token);
      error = (short) (!analice_equal (fp, mens_error, str_conjetura,
				       &((*exp)->argumentos), NULL));
    }
  else if (!strcmp ((*exp)->tipo, ""))
    {
      strcpy ((*exp)->clase, "VAR");
    }
  else if (es_inicial_tipo_dom (token, (*exp)->tipo, &dom_generadora))
    {
      strcpy ((*exp)->clase, "INIC");
      if (dom_generadora != NULL)
	{
	  error = (short) (!analice_argum_generadora (fp, mens_error, str_conjetura,
			      &((*exp)->argumentos), dom_generadora, NULL));
	}
    }
  else if (es_constructora_tipo_dom (token, (*exp)->tipo, &dom_generadora))
    {
      strcpy ((*exp)->clase, "CONSTR");
      error = (short) (!analice_argum_generadora (fp, mens_error, str_conjetura,
			      &((*exp)->argumentos), dom_generadora, NULL));
    }
  else if (es_selectora_tipo_dom (token, (*exp)->tipo, &dom_select))
    {
      strcpy ((*exp)->clase, "SELECT");
      error = (short) (!analice_argum_select (fp, mens_error, str_conjetura,
				  &((*exp)->argumentos), dom_select, NULL));
    }
  else
    {
      switch (tad_ed->tipo)
	{
	case NORMAL:
	case PROD_CARTESIANO:
	  error = 1;
	  break;
	case RESTRICCION:
	  if (!strcmp (tad_ed->info_tipo->nombre, (*exp)->tipo))
	    {
	      error = (short) (!analice_expr_der (fp, mens_error, str_conjetura, exp,
						  NULL, tad_ed->nombre));
	    }
	  else
	    {
	      error = 1;
	    }
	  break;
	case COMPLETACION:
	  error = (short) !(analice_expr_der (fp, mens_error, str_conjetura,
				     exp, NULL, tad_ed->info_tipo->nombre));
	  break;
	case NT_EXTENSION:
/*** VTP. 18/Mar/1997 ***/
	case VARIACION:
	  error = 1;
	  /*******
            error = !(analice_expr_der(fp,mens_error,str_conjetura,exp,
				NULL,tad_ed->info_tipo->nombre)
		      ||
            	      analice_expr_der(fp,mens_error,str_conjetura,exp,
		      		NULL,tad_ed->info_tipo->sig->nombre));
	    *********/
	  break;
	default:
	  error = 1;
	  break;
	}
      if (error)
	{
	  sprintf (mens_error, mensajes[35], token, (*exp)->tipo);
	}
    }
  return ((short) !error);
/*----------------------------------------------------------------------*/
}

/************************************************************************
* nodo_argum *analice_lista_exp(FILE *fp, texpr mens_error)             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO: Generar una lista de expresiones para normalizar           *
* DESCRIPCION:  Analiza sistacticamente el archivo, y produce una lista *
*       con todas las expresiones.Si el archivo tiene errores           *
*       sintacticos retorna NULL.                  			*
* PARAMETROS:   fp      *FILE       Archivo con las expresiones         *
*               texpr   mens_error  Variable para retornar posibles     *
*                                   errores sintacticos.		*
************************************************************************/

nodo_argum *
analice_lista_exp (FILE * fp, texpr mens_error)
{
/*--------------------------------------------------------------------*/

  nodo_argum *ptr_arg, *ult_arg, *arg;
  short error;
  texpr str_teorema;
  nodo_expresion *exp = NULL;

  assert (mens_error != NULL);	/* VTP. 8/May/1997 */

  error = 0;
  num_ed = 1;
  ptr_arg = NULL;
  ult_arg = NULL;
  strcpy (mens_error, "");
  deme_token (fp);
  while ((strcmp (token, "")) && (!error))
    {
      arg = crear_argumento (NULL);
      if (!error && analice_conj (fp, mens_error, str_teorema, &(arg->expr)))
	{
	  if (ptr_arg == NULL)
	    ptr_arg = arg;
	  else
	    ult_arg->sig = arg;
	  ult_arg = arg;
	  deme_token (fp);
	}
      else
	error = 1;
    }
  if (!error)
    return (ptr_arg);
  else
    {
      exp = crear_expresion ("", "", "");
      exp->argumentos = ptr_arg;
      liberar_expresion (&exp);
      return (NULL);
    }

/*--------------------------------------------------------------------*/
}

/************************************************************************
* nodo_igualdad *analice_conjeturas(FILE *fp, texpr mens_error)         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO: Generar una lista de conjeturas del archivo apuntado por fp*
* DESCRIPCION:  Analiza sistacticamente el archivo, y produce una lista *
*       con todas las conjeturas. Si el archivo tiene errores           *
*       sintacticos retorna NULL.                  			*
* PARAMETROS:   texpr   mens_error  Variable para retornar posibles     *
*                                   errores sintacticos.		*
* PRE: fp apunta a un archivo de conjeturas.                            *
* POST: analice_conjeturas = <j1,..,jn>, lista de conjeturas  or        *
*       analice_conjeturas = NULL, el archivo tiene errores. 		*
************************************************************************/

nodo_igualdad *
analice_conjeturas (FILE * fp, texpr mens_error)
{
/*--------------------------------------------------------------------*/

  nodo_igualdad *ptr_conjetura = NULL, *ult_conjetura, *ig;
  short error;
  texpr str_teorema;

  assert (mens_error != NULL);	/* VTP. 8/May/1997 */
  error = 0;
  num_ed = 1;
  ptr_conjetura = NULL;
  ult_conjetura = NULL;
  strcpy (mens_error, "");
  deme_token (fp);
  while ((strcmp (token, "")) && (!error))
    {
      ig = crear_igualdad (NULL, NULL);
      if (!error && analice_conj (fp, mens_error, str_teorema,
				  &(ig->lado_izq)))
	{
	  deme_token (fp);
	  error = es_error_esperaba ("=", token, mens_error);
	  if (!error)
	    {
	      strcpy (ig->li_str, str_teorema);
	      deme_token (fp);
	      if (analice_conj (fp, mens_error, str_teorema,
				&(ig->lado_der)))
		{
		  strcpy (ig->ld_str, str_teorema);
		  if (!strcmp (ig->lado_izq->tipo, ""))
		    strcpy (ig->lado_izq->tipo, ig->lado_der->tipo);
		  if (ptr_conjetura == NULL)
		    ptr_conjetura = ig;
		  else
		    ult_conjetura->sig = ig
		      ;
		  ult_conjetura = ig;
		  deme_token (fp);
		}
	      else
		error = 1;
	    }
	}
      else
	{
	  error = 1;
	}
    }
  if (!error)
    return (ptr_conjetura);
  else
    {
      liberar_conjeturas (&ptr_conjetura);
      return (NULL);
    }

/*--------------------------------------------------------------------*/
}

/************************************************************************
* nodo_igualdad *crear_igualdad(nodo_expresion *e1,nodo_expresion *e2)  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Crea una igualdad con e1 y e2.                          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio 91  *
************************************************************************/

nodo_igualdad *
crear_igualdad (nodo_expresion * e1, nodo_expresion * e2)
{
/*-------------------------------------------------------------------*/
  nodo_igualdad *ig;

  ig = (nodo_igualdad *) malloc (sizeof (nodo_igualdad));
  HayMem (ig, sizeof (nodo_igualdad));
  ig->lado_izq = e1;
  ig->lado_der = e2;
  if (e1 != NULL)
    construya_exp (e1, ig->li_str,"");
  else
    strcpy (ig->li_str, "");
  if (e2 != NULL)
    construya_exp (e2, ig->ld_str,"");
  else
    strcpy (ig->ld_str, "");
  ig->sig = NULL;
  return (ig);

/*--------------------------------------------------------------------*/
}

/************************************************************************
* nodo_demost *crear_demost()                                           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Crear un nodo de demostracion.                          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio 91  *
************************************************************************/

nodo_demost *
crear_demost ()
{
/*-------------------------------------------------------------------*/
  nodo_demost *d;

  d = (nodo_demost *) malloc (sizeof (nodo_demost));
  HayMem (d, sizeof (nodo_demost));
  d->his_li = NULL;
  d->ult_li = NULL;
  d->his_ld = NULL;
  d->ult_ld = NULL;
  d->sig = NULL;
  return (d);

/*--------------------------------------------------------------------*/
}

/************************************************************************
* nodo_expresion *crear_expresion(cad12 nombre, cad20 tipo,             *
*                                cad6 clase)                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Crea una expresion de nombre,tipo y clase indicado.     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio 91  *
************************************************************************/

nodo_expresion *
crear_expresion (cad12 nombre, cad20 tipo,
		 cad6 clase)
{
/*-------------------------------------------------------------------*/
  nodo_expresion *exp;

  assert (nombre != NULL && tipo != NULL && clase != NULL);	/* VTP. 8/May/1997 */
  exp = (nodo_expresion *) malloc (sizeof (nodo_expresion));
  HayMem (exp, sizeof (nodo_expresion));
  strcpy (exp->nombre, nombre);
  strcpy (exp->tipo, tipo);
  strcpy (exp->clase, clase);
  exp->argumentos = NULL;
  return (exp);

/*--------------------------------------------------------------------*/
}

/************************************************************************
* nodo_historia *crear_historia(nodo_expresion *expr, nodo_igualdad *r, *
*                               cad12 nivel)                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Crea un nodo de histora con la expr,r y nivel           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio 91  *
************************************************************************/

nodo_historia *
crear_historia (nodo_expresion * expr, nodo_igualdad * r,
		cad12 nivel)
{
/*-------------------------------------------------------------------*/
  nodo_historia *his;

  his = (nodo_historia *) malloc (sizeof (nodo_historia));
  HayMem (his, sizeof (nodo_historia));
  /* VTP .
    Mejor hacer copia de la expresion y la regla ?
     Originalmente decia
    his->expr = expr;
    his->regla = r;
  */
  his->expr=copia_expresion(expr);
  if (r==NULL)
    {
      his->regla=NULL;
    }
  else
    {
      his->regla=(nodo_igualdad *)malloc(sizeof(nodo_igualdad));
      HayMem(his->regla,sizeof(nodo_igualdad));
      strcpy(his->regla->li_str,r->li_str);
      strcpy(his->regla->ld_str,r->ld_str);
      his->regla->lado_izq=copia_expresion(r->lado_izq);
      his->regla->lado_der=copia_expresion(r->lado_der);
      his->sig = NULL;
    }
  strcpy (his->nivel, nivel);

  return (his);
/*--------------------------------------------------------------------*/
}

/************************************************************************
*void sustituciones_rep(Ptr_tabla s,nodo_expresion *e,nodo_represent *r)*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO: Crear las sustitucion s, que cambia variables de la        *
*            representacion objeto,a funciones de la representacion     *
*            fuente, en la expresion e.                                 *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio 91  *
************************************************************************/
/*
void
sustituciones_rep (Ptr_tabla s, nodo_expresion * e, nodo_represent * r)
{
  nodo_argum *arg;
  nodo_expresion *funcion, *var_objeto;

  assert (r != NULL);		
  if (e == NULL)
    return;
  if ((!strcmp (e->clase, "VAR")) && (!strcmp (e->tipo, r->tad_fuente)))
    {
      var_objeto = crear_expresion (e->nombre, r->tad_objeto, "VAR");
      strcat (var_objeto->nombre, "_");
      strcat (var_objeto->nombre, r->tad_objeto);
      funcion = crear_expresion (r->nom_funcion, r->tad_fuente, "F_REP");
      funcion->argumentos = crear_argumento (var_objeto);
      agr (s, e->nombre, funcion);
    }
  else if (strcmp (e->clase, "VAR"))
    for (arg = e->argumentos; arg != NULL; arg = arg->sig)
      sustituciones_rep (s, arg->expr, r);
}
*/
/************************************************************************
*  nodo_igualdad *generar_conj_rep(nodo_select *sel,nodo_represent *r)  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Generar una lista de conjeturas con los axiomas de las  *
*               selectoras sel, para verificar la correccion de la      *
*               representacion r.                                       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio 91  *
************************************************************************/

/*
nodo_igualdad *
generar_conj_rep (nodo_select * sel, nodo_represent * r)
{

  nodo_igualdad *lista, *nuevo, *cola;
  Ptr_tabla sust;
  short aplico;
  nodo_igualdad *ax;

  for (lista = NULL, cola = NULL; sel != NULL; sel = sel->sig)
    for (ax = sel->axiomas; ax != NULL; ax = ax->sig)
      {

 Cada axioma se debe modificar, cambiar una variable x a una  f(x'),
     x es de tipo tad_objeto y x' es de tipo tad fuente.              --
	sust = eps ();
	sustituciones_rep (sust, ax->lado_izq, r);
	nuevo = crear_igualdad (apl (ax->lado_izq, sust, 1, &aplico),
				apl (ax->lado_der, sust, 1, &aplico));
	if (lista == NULL)
	  lista = nuevo;
	else
	  cola->sig = nuevo;
	cola = nuevo;
      }
  return lista;

}
*/
/************************************************************************
* void arme_generadora(nodo_expresion **expr,nodo_generadora *i,        *
*                      cad12 tipo, cad6 clase, cad50 nombre)            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Generar una expresion en expr, que corresponda a una    *
*               instancia de la generadora i.                           *
*               Si nombre <> NULL, ese nombre lo debe llevar la primera *
*               variable del mismo tipo que se genera.                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio 91  *
************************************************************************/

void
arme_generadora (nodo_expresion ** expr, nodo_generadora * i,
		 cad12 tipo, cad6 clase, cad50 nombre)
{
/*-------------------------------------------------------------------*/
  nodo_nombre *n;
  int pos;
  texpr str_arg;
  nodo_argum *arg, *ult;
  int es_primera;

  assert (expr != NULL && i != NULL);	/* VTP. 8/May/1997 */
  assert (tipo != NULL && clase != NULL && nombre != NULL);	/* VTP. 8/May/1997 */
  (*expr) = crear_expresion (i->nombre, tipo, clase);
  ult = NULL;
  es_primera = 1;
  for (n = i->dominio, pos = 1; n != NULL; n = n->sig, pos++)
    {
      sprintf (str_arg, "%s_%d", n->nombre, pos);
      arg = crear_argumento (NULL);
      if (!strcmp (tipo, n->nombre) && es_primera && strcmp (nombre, ""))
	{
	  es_primera = 0;
	  arg->expr = crear_expresion (nombre, n->nombre, "CONST");
	}
      else
	arg->expr = crear_expresion (str_arg, n->nombre, "VAR");
      if (ult == NULL)
	(*expr)->argumentos = arg;
      else
	ult->sig = arg;
      ult = arg;
    }
/*--------------------------------------------------------------------*/
}

/************************************************************************
* nodo_expresion  *buscar_if(nodo_expresion *expr)                      *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO: Buscar en la expresion expr una expresion de la forma      *
* IF EQUAL ...., si la encuentra retorna la posicion encontrada,        *
* si no retorna NULL.                                                   *
************************************************************************/

nodo_expresion *
buscar_if (nodo_expresion * expr)
{
/*-------------------------------------------------------------------*/
  nodo_argum *arg;
  nodo_expresion *e;

  assert (expr != NULL && expr->clase != NULL);		/* VTP. 8/May/1997 */
  if (!strcmp_nocase (expr->clase, "IF") &&
      !strcmp_nocase (expr->argumentos->expr->clase, "EQUAL"))
    return expr;
  for (e = NULL, arg = expr->argumentos; arg != NULL && e == NULL;
       arg = arg->sig)
    {
      e = buscar_if (arg->expr);
    }
  return (e);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* int  probar_casos(nodo_demost *dem, nodo_expresion *li,               *
*                      nodo_expresion *ld, cad12 nivel)                 *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO: Hacer la prueba por casos de las dos expresiones.          *
* PRE :                                                                 *
* dem = Historia de la demostracion.                                    *
* li = expresion izquierda que se debe probar.                          *
* ld = expresion derecha que se debe probar.                            *
* POST : probar_casos == 1 , ssi pudo hacer demostracion.               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio  91 *
************************************************************************/

int
probar_casos (nodo_demost * dem, nodo_expresion * li,
	      nodo_expresion * ld, cad12 nivel)
{
/*-------------------------------------------------------------------*/
  int demostro;
  nodo_expresion *exp;
  nodo_expresion *exp2;
  nodo_expresion *ed, *ei;
  Ptr_tabla unif;
  nodo_historia *hi_li, *hi_ld, *hi_ult_li, *hi_ult_ld;

  demostro = 0;
  ei = copia_exp (li);
  ed = copia_exp (ld);
  exp = buscar_if (ei);
  if (exp == NULL)
    exp = buscar_if (ed);
  if (exp == NULL)
    return (demostro);

  exp2 = exp->argumentos->expr;
  unif = umg (exp2->argumentos->expr, exp2->argumentos->sig->expr);
  if (unif == NULL)
    return (demostro);
  adicionar_nivel (dem, nivel, 'E');
  hi_li = crear_historia (aplr (ei, unif, 1), NULL, nivel);
  hi_ld = crear_historia (aplr (ed, unif, 1), NULL, nivel);
  fnlo (hi_li->expr, &(hi_li->sig), &(hi_ult_li));
  fnlo (hi_ld->expr, &(hi_ld->sig), &(hi_ult_ld));
  dem->ult_li->sig = hi_li->sig;
  dem->ult_ld->sig = hi_ld->sig;
  dem->ult_li = hi_ult_li;
  dem->ult_ld = hi_ult_ld;
  if (equal (hi_ult_li->expr, hi_ult_ld->expr))
    {
      adicionar_nivel (dem, nivel, 'F');
      strcpy (exp2->clase, "INIC");
      strcpy (exp2->tipo, "BOOL");
      strcpy (exp2->nombre, "FALSE");
      exp2->argumentos = NULL;
      hi_li = crear_historia (ei, NULL, nivel);
      hi_ld = crear_historia (ed, NULL, nivel);
      fnlo (hi_li->expr, &(hi_li->sig), &(hi_ult_li));
      fnlo (hi_ld->expr, &(hi_ld->sig), &(hi_ult_ld));
      dem->ult_li->sig = hi_li->sig;
      dem->ult_ld->sig = hi_ld->sig;
      dem->ult_li = hi_ult_li;
      dem->ult_ld = hi_ult_ld;
      if (equal (hi_ult_li->expr, hi_ult_ld->expr))
	{
	  adicionar_nivel (dem, nivel, 'G');
	  demostro = 1;
	}
      else
	adicionar_nivel (dem, nivel, 'H');
    }
  else
    adicionar_nivel (dem, nivel, 'H');
  return (demostro);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* int  probar_semantica(nodo_demost *dem, nodo_expresion *li,           *
*                      nodo_expresion *ld, cad12 nivel)                 *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO: Hacer la prueba semantica de dos expresiones               *
* Este procedimiento se usa cuando ya no hay mas variables de           *
* induccion. Trata de demostrar con la igualdad del tipo.               *
* PRE :                                                                 *
* dem = Historia de la demostracion.                                    *
* li = expresion izquierda que se debe probar.                          *
* ld = expresion derecha que se debe probar.                            *
* POST : probar_semantica == 1 , ssi pudo hacer demostracion.           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio  91 *
************************************************************************/

int
probar_semantica (nodo_demost * dem, nodo_expresion * li,
		  nodo_expresion * ld, cad12 nivel)
{
/*-------------------------------------------------------------------*/
  cad50 nombre;
  int demostro;
  nodo_tad *t;
  nodo_expresion *exp;
  nodo_historia *hi_li, *hi_ld, *hi_ult_li = NULL, *hi_ult_ld = NULL;
  nodo_argum *arg;

  assert (li != NULL && ld != NULL);	/* VTP. 8/May/1997 */
  demostro = 0;
  t = existe_tad (li->tipo);
  if ((t != NULL) && strcmp (ld->tipo, "")
      && !strcmp (li->tipo, ld->tipo))
    {
      sprintf (nombre, "EQ_%s", li->tipo);
      if (es_selectora (nombre, t->selectoras))
	{
/*-- Se definio la igualdad en el tipo. --*/
	  arg = crear_argumento (li);
	  arg->sig = crear_argumento (ld);
	  exp = crear_expresion (nombre, "BOOL", "SELECT");
	  exp->argumentos = arg;
	  hi_li = crear_historia (exp, NULL, nivel);
	  strcat (hi_li->nivel, "3");
	  exp = crear_expresion ("TRUE", "BOOL", "INIC");
	  hi_ld = crear_historia (exp, NULL, hi_li->nivel);
	  fnlo (hi_li->expr, &(hi_li->sig), &(hi_ult_li));
	  fnlo (hi_ld->expr, &(hi_ld->sig), &(hi_ult_ld));
	  dem->ult_li->sig = hi_li;
	  dem->ult_ld->sig = hi_ld;
	  dem->ult_li = hi_ult_li;
	  dem->ult_ld = hi_ult_ld;
	  if (equal (hi_ult_li->expr, hi_ult_ld->expr))
	    {
	      adicionar_nivel (dem, nivel, '6');
	      demostro = 1;
	    }
	  else
	    {
	      adicionar_nivel (dem, nivel, '7');
	      demostro = probar_casos (dem, hi_ult_li->expr,
				       hi_ult_ld->expr, nivel);
	    }
	}
    }
  return (demostro);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* int probar_inicial(nodo_tad *t, nodo_expresion *var, nodo_demost *dem,*
*                   nodo_expresion *li, nodo_expresion *ld, cad12 nivel)*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO: Demostrar la conjetura con todas las instancias iniciales  *
* posibles.                                                             *
* PRE : t = tad sobre el cual se hace la demostracion.                  *
* var = variable semantica nueva.                                       *
* dem = Historia de la demostracion.                                    *
* li = expresion izquierda que se debe probar.                          *
* ld = expresion derecha que se debe probar.                            *
* POST : probar_inicial == 1 , ssi pudo hacer demostracion para todas   *
* las iniciales.                                                        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                FECHA: Febrero 92 *
************************************************************************/

int
probar_inicial (nodo_tad * t, nodo_expresion * var, nodo_demost * dem,
		nodo_expresion * li, nodo_expresion * ld, cad12 nivel)

{
/*-------------------------------------------------------------------*/
  nodo_generadora *i;
  int demostro;
  nodo_expresion *exp = NULL;
  nodo_historia *hi_li, *hi_ld, *hi_ult_li = NULL, *hi_ult_ld = NULL;
  short aplico;
  Ptr_tabla sust;

  assert (t != NULL && var != NULL);	/* VTP. 8/May/1997 */
  for (demostro = 1, i = t->iniciales;
       (demostro) && (i != NULL); i = i->sig)
    {
      arme_generadora (&exp, i, t->nombre, "INIC", "");
      sust = eps ();
      agr (sust, var->nombre, exp);
      hi_li = crear_historia (var, NULL, nivel);
      strcat (hi_li->nivel, "1");
      hi_ld = crear_historia (exp, NULL, hi_li->nivel);
      fnlo (apl (li, sust, 1, &aplico), &(hi_li->sig), &(hi_ult_li));
      fnlo (apl (ld, sust, 1, &aplico), &(hi_ld->sig), &(hi_ult_ld));
      dem->ult_li->sig = hi_li;
      dem->ult_ld->sig = hi_ld;
      dem->ult_li = hi_ult_li;
      dem->ult_ld = hi_ult_ld;
      elimLTabla(&sust);      
      if (!equal (hi_ult_li->expr, hi_ult_ld->expr))
	demostro = probar_induccion (dem, hi_ult_li->expr,
				     hi_ult_ld->expr,
				     hi_ld->nivel);
      else
	demostro = 1;
      if (demostro)
	adicionar_nivel (dem, nivel, '8');
      else
	adicionar_nivel (dem, nivel, 'A');
    }
  return (demostro);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* int  probar_hipotesis( nodo_expresion *li_hip,nodo_expresion *ld_hip, *
*                       nodo_expresion *var,nodo_expresion *exp,        *
*                       nodo_demost *dem, nodo_expresion *li,           *
*                       nodo_expresion *ld,Ptr_tabla sust, cad12 nivel) *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO: Demostrar la conjetura con todas las constructuras         *
* usando la hipotesis de induccion. Se puede orientar li_hip->ld_hip    *
* PRE : t = tad sobre el cual se hace la demostracion.                  *
* var = variable semantica nueva.                                       *
* dem = Historia de la demostracion.                                    *
* li = expresion izquierda que se debe probar.                          *
* ld = expresion derecha que se debe probar.                            *
* POST : probar_hipotesis == 1 , ssi pudo hacer demostracion para       *
* todas las constructoras, con la hipotesis.                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                FECHA: Febrero 92 *
  MODIFICADA POR: Vladimir Tamara
  Antes agregaba las hipotesis a un axioma de una selectora
  cualquiera del TAD tad_actual
  lista de axiomas,
************************************************************************/

int
probar_hipotesis (nodo_expresion * li_hip, nodo_expresion * ld_hip,
		  nodo_expresion * var, nodo_expresion * exp,
		  nodo_demost * dem, nodo_expresion * li,
		  nodo_expresion * ld, Ptr_tabla sust, cad12 nivel)
{
/*-------------------------------------------------------------------*/
  nodo_demost *dem_aux;
  nodo_historia *hi_li, *hi_ld, *hi_ult_li = NULL, *hi_ult_ld = NULL;
  short aplico;
  int demostro;
  nodo_igualdad *hipot;

  assert(li_hip!=NULL);
  assert(ld_hip!=NULL);
  assert(li!=NULL);
  assert(ld!=NULL);
  assert(dem!=NULL);

  /*  VTP. 30/Nov/98 axiom = tad_actual->selectoras->axiomas; */
  hipot = crear_igualdad (li_hip, ld_hip);
  hipot->sig=especiales;
/*  VTP. 30/Nov/98 hipot->sig = axiom; */
  especiales=hipot;
/*  tad_actual->selectoras->axiomas = hipot; */
  hi_li = crear_historia (var, hipot, nivel);
  strcat (hi_li->nivel, "2");
  hi_ld = crear_historia (exp, NULL, hi_li->nivel);
  fnlo (apl (li, sust, 1, &aplico), &(hi_li->sig), &(hi_ult_li));
  fnlo (apl (ld, sust, 1, &aplico), &(hi_ld->sig), &(hi_ult_ld));
  dem_aux = crear_demost ();
  dem_aux->his_li = hi_li;
  dem_aux->ult_li = hi_ult_li;
  dem_aux->his_ld = hi_ld;
  dem_aux->ult_ld = hi_ult_ld;
  if (!equal (hi_ult_li->expr, hi_ult_ld->expr))
    demostro = probar_induccion (dem_aux, hi_ult_li->expr,
				 hi_ult_ld->expr, hi_ld->nivel);
  else
    demostro = 1;
  dem->ult_li->sig = dem_aux->his_li;
  dem->ult_ld->sig = dem_aux->his_ld;
  dem->ult_li = dem_aux->ult_li;
  dem->ult_ld = dem_aux->ult_ld;
  dem_aux->his_li = NULL;
  dem_aux->ult_li = NULL;
  dem_aux->his_ld = NULL;
  dem_aux->ult_ld = NULL;
  dem_aux->sig = NULL;
  if (demostro)
    adicionar_nivel (dem, nivel, '9');
  else
    adicionar_nivel (dem, nivel, 'B');
  /* VTP. 28/Nov/98 tad_actual->selectoras->axiomas = axiom;
     Adicionadas las siguientes lineas */
  especiales=especiales->sig;
  hipot->sig=NULL;
  /* No se liberan izquierdo y derecho porque son li_hip y
     ld_hip, que no deben destruirse */
  hipot->lado_izq=NULL;
  hipot->lado_der=NULL;
  elimLIgualdad(&hipot);

  return (demostro);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* int  probar_constructora( nodo_tad *t, nodo_expresion *var,           *
*                          nodo_demost *dem, nodo_expresion *li,        *
*                          nodo_expresion *ld, cad12 nivel)             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO: Demostrar la conjetura con todas las constructuras         *
* posibles. Para la demostracion supone la hipotesis de induccion       *
* PRE : t = tad sobre el cual se hace la demostracion.                  *
* var = variable semantica nueva.                                       *
* dem = Historia de la demostracion.                                    *
* li = expresion izquierda que se debe probar.                          *
* ld = expresion derecha que se debe probar.                            *
* POST : probar_constructora == 1 , ssi pudo hacer demostracion para    *
* todas las constructoras.                                              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                FECHA: Febrero 92 *
************************************************************************/

int
probar_constructora (nodo_tad * t, nodo_expresion * var,
		     nodo_demost * dem, nodo_expresion * li,
		     nodo_expresion * ld, cad12 nivel)
{
/*-------------------------------------------------------------------*/
  cad50 nombre;
  int demostro;
  nodo_generadora *i;
  Ptr_tabla sust;
  nodo_expresion *li_hip, *ld_hip;
  nodo_expresion *exp = NULL;
  short aplico;
  nodo_demost *dem_aux;

  assert (var != NULL && t != NULL);	/* VTP. 8/May/1997 */
  sprintf (nombre, "_%s", var->nombre);
  demostro = 1;
  for (i = t->constructoras; (demostro) && (i != NULL); i = i->sig)
    {
      demostro = 0;
      sust = eps ();
      exp = crear_expresion (nombre, t->nombre, "CONST");
      agr (sust, var->nombre, exp);
      dem_aux = crear_demost ();
      fnlo (apl (li, sust, 1, &aplico), &(dem_aux->his_li), &(dem_aux->ult_li));
      fnlo (apl (ld, sust, 1, &aplico), &(dem_aux->his_ld), &(dem_aux->ult_ld));
      li_hip = dem_aux->ult_li->expr;
      ld_hip = dem_aux->ult_ld->expr;
      elimLTabla(&sust);
      arme_generadora (&exp, i, t->nombre, "CONSTR", nombre);
      sust = eps ();
      agr (sust, var->nombre, exp);
      if ((long_expr (li_hip) >= long_expr (ld_hip)) &&
	  (!aplica (li_hip, ld_hip)))
	demostro = probar_hipotesis (li_hip, ld_hip, var, exp, dem, li, ld,
				     sust, nivel);
      if (!demostro &&
	  (long_expr (ld_hip) >= long_expr (li_hip)) &&
	  (!aplica (ld_hip, li_hip)))
	demostro = probar_hipotesis (ld_hip, li_hip, var, exp, dem, li, ld,
				     sust, nivel);
      elimLTabla(&sust);
    }
  return (demostro);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* int  probar_induccion(nodo_demost *dem,                               *
*                      nodo_expresion *li, nodo_expresion *ld,          *
*                      cad12 nivel)                                     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Demostrar por induccion las formas normales de dem.     *
* PRE: Nivel = <a1..an>, Estado en el que esta la demostracion          *
* dem = historia de la demostracion.                                    *
* li = expresion izquierda que se debe probar.                          *
* ld = expresion derecha que se debe probar.                            *
* DESCRIPCION:  - Prueba las iniciales. -Hace hipotesis. -Usa           *
*                 reescritura.                                          *
* POST : probar_induccion == 1 , ssi pudo hacer demostracion.           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio  91 *
************************************************************************/

int
probar_induccion (nodo_demost * dem,
		  nodo_expresion * li, nodo_expresion * ld,
		  cad12 nivel)
{
/*-------------------------------------------------------------------*/
  nodo_tad *t;
  nodo_argum *arg;
  nodo_expresion *var;
  int demostro;

  demostro = 0;
  arg = buscar_variable_ind (li, ld);
  if (arg != NULL)
    for (; (!demostro) && (arg != NULL); arg = arg->sig)
      {
	var = arg->expr;
	t = existe_tad (var->tipo);
	if (t != NULL)
	  {
	    adicionar_nivel (dem, nivel, 'C');
	    if (probar_inicial (t, var, dem, li, ld, nivel)) {
	      demostro = probar_constructora (t, var, dem, li, ld, nivel);
	    }
	    adicionar_nivel (dem, nivel, 'D');
	  }
      }
  else
    demostro = probar_semantica (dem, li, ld, nivel);
  elimLArgumento(&arg);
  return (demostro);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* void normalizar_lista(nodo_argum **arg,nodo_igualdad **conj)          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Normalizar la lista de expresiones que vienen en arg,   *
*               retorna la lista **conj de igualdades. La parte         *
*               izquierda es la expresion, la derecha es la normalizada *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Abril  91 *
************************************************************************/

void
normalizar_lista (nodo_argum ** arg, nodo_igualdad ** conj)
{
/*--------------------------------------------------------------------*/
  nodo_argum *p, *q;		/* Nodo de la lista que se normaliza      */
  nodo_igualdad *ig,		/* Donde se coloca la normalizacion actual */
   *ult_ig;			/* Ultima normalizacion                   */
  nodo_demost *dem;		/* Nodo proxima demostracion.             */

  assert (conj != NULL);	/* VTP. 8/May/1997 */
  (*conj) = NULL;
  ult_ig = NULL;
  for (p = (*arg); p != NULL; p = q)
    {
      dem = crear_demost ();
      fnlo (p->expr, &(dem->his_li), &(dem->ult_li));
      q = p->sig;
      liberar_expresion (&p->expr);
      p->sig = NULL;		/* VTP. 8/May/1997 */
      free (p);
      ig = crear_igualdad (copia_exp(dem->his_li->expr),
			   copia_exp(dem->ult_li->expr));
      if (*conj == NULL)
	(*conj) = ig;
      else
	ult_ig->sig = ig;
      ult_ig = ig;
      liberar_demost(&dem); /* VTP. 25/MAY/1998 */
    }

/*--------------------------------------------------------------------*/
}

/************************************************************************
* void probar_conjeturas(int *ciertas, int *falsas,                     *
*                        nodo_igualdad **conj)                          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Demostrar conjeturas y actualizar las listas de         *
*               conjeturas probadas y conjeturas falsas.                *
* DESCRIPCION:  - Usa reescritura. - Si falla, usa induccion sobre las  *
*       formas normales. - Usa prueba semantica                         *
* PARAMETROS: int ciertas : Numero de conjeturas ciertas.               *
*             int falsas  : Numero de conjeturas falsas.                *
*             nodo_igualdad *conj : Conjeturas que se deben probar.     *
* V. GLOBALES: proved y unproved.                                       *
* PRE : conj = <c1...cm>, lista de conjeturas.                          *
*       proved = <P> , Lista de conjeturas ciertas.                     *
*       unproved = <U>, Lista de conjeturas no probadas.                *
* POST: conj = <ck,..,cl>, donde ck..cl son conjeturas no demostradas   *
*                          como ciertas.			        *
*       unproved = <U,ck,..,cl>                                         *
*       proved = <P,ci,..,cj>, donde ci,..,cj son conjeturas ciertas.   *
*       ciertas = |<ci,..,cj>| y falsas = |<ck,..,cl>|                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Abril  91 *
************************************************************************/

void
probar_conjeturas (int *ciertas, int *falsas,
		   nodo_igualdad ** conj)
{
/*-------------------------------------------------------------------*/
  nodo_igualdad *c;		/* Conjetura que se demuestra.           */
  nodo_igualdad *ant_c;		/* Conjetura anterior a la actual.       */
  nodo_demost *dem;		/* Nodo proxima demostracion.            */
  int si_demostro;		/* Variable que indica se pudo hacer
				   una demostracion */
  nodo_expresion *li, *ld;	/* Lado izquierdo y derecho que se debe
				   demostrar.                            */

  assert (ciertas != NULL && falsas != NULL && conj != NULL);	/* VTP. 8/May/1997 */
  (*ciertas) = 0;
  (*falsas) = 0;
  for (c = (*conj), ant_c = NULL; c != NULL;)
    {
      dem = crear_demost ();
      fnlo (c->lado_izq, &(dem->his_li), &(dem->ult_li));
      fnlo (c->lado_der, &(dem->his_ld), &(dem->ult_ld));
      li = dem->ult_li->expr;
      ld = dem->ult_ld->expr;
      if (equal (li, ld))
/*-- Demostro por reescritura --*/
	si_demostro = 1;
      else
/*-- Debe usar induccion --*/
	si_demostro = probar_induccion (dem, li, ld, "");
      if (si_demostro)
	{
	  adicionar_nivel (dem, "", '4');
	  if (ant_c == NULL)
	    (*conj) = c->sig;
	  else
	    ant_c->sig = c->sig;
	  liberar_expresion (&(c->lado_izq));
	  liberar_expresion (&(c->lado_der));
	  c->sig = NULL;	/* VTP. 8/May/1997 */
	  free (c);
	  if (proved == NULL)
	    proved = dem;
	  else
	    ult_proved->sig = dem;
	  ult_proved = dem;
	  if (ant_c != NULL)
	    c = ant_c->sig;
	  else
	    c = (*conj);
	  (*ciertas)++;
	}
      else
	{
	  adicionar_nivel (dem, "", '5');
	  ant_c = c;
	  c = c->sig;
	  (*falsas)++;
	  if (unproved == NULL)
	    unproved = dem;
	  else
	    ult_unproved->sig = dem;
	  ult_unproved = dem;
	}
    }
/*--------------------------------------------------------------------*/
}

/************************************************************************
* void liberar_demost(nodo_demost **dem)                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Libera la memoria de dem.                               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio  91 *
************************************************************************/

void
liberar_demost (nodo_demost ** dem)
{
/*--------------------------------------------------------------------*/
  nodo_demost *d;

  while ((*dem) != NULL)
    {
      d = (*dem)->sig;
      liberar_historia (&((*dem)->his_li));
      liberar_historia (&((*dem)->his_ld));
      (*dem)->sig = NULL;	/* VTP. 8/May/1997 */
      free (*dem);
      (*dem) = d;
    }
  (*dem) = NULL;
/*--------------------------------------------------------------------*/
}

/************************************************************************
int crear_historia_vacia(nodo_historia **h,char *merr)
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
 PROPOSITO:    Crea un nodo de historia vacio
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
 AUTOR: V. Tamara
 FECHA: Junio  98
************************************************************************/
int crear_historia_vacia(nodo_historia **h,char *merr)
  {
    assert(h!=NULL);
    *h=(nodo_historia *)malloc(sizeof(nodo_historia));
    if (*h==NULL)
      {
        strcat(merr,mensajes[166]);
        strcat(merr,"\n");
        return -1;
      }
    (*h)->expr=NULL;
    (*h)->regla=NULL;
    (*h)->nivel[0]='\0';
    (*h)->sig=NULL;
    return 0;
  }

/************************************************************************
copiar_historia (nodo_historia *s,nodo_historia ** d,char *merr)
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
 PROPOSITO:    Saca una copia de la historia s y la deja en d
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
 AUTOR: V. Tamara
 FECHA: Junio  98
************************************************************************/
int
copiar_historia (nodo_historia ** d,nodo_historia *s,char *merr)
{
  assert(d!=NULL);
  assert(merr!=NULL);

  *d=NULL;
  if (s==NULL)
    { return 0; }
  if (crear_historia_vacia(d,merr)!=0)
    {
      return -1;
    }
  if (copiaNodoExpresion(&((*d)->expr),s->expr,merr)!=0)
    {
      liberar_historia(d);
      *d=NULL;
      return -1;
    }
  if (copiaLIgualdad(&((*d)->regla),s->regla,merr)!=0)
    {
      liberar_historia(d);
      *d=NULL;
      return -1;
    }
  strcpy((*d)->nivel,s->nivel);
  if (copiar_historia(&((*d)->sig),s->sig,merr)!=0)
    {
      liberar_historia(d);
      *d=NULL;
      return -1;
    }
  return 0;
} 


/************************************************************************
  int copiar_demost(nodo_demost *s, nodo_demost **d,char *merr)
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
 PROPOSITO:    Saca una copia de la demostracion s y la deja en d
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
 AUTOR: V. Tamara
 FECHA: Junio  98
************************************************************************/

int
copiar_demost (nodo_demost ** d,nodo_demost *s,char *merr)
{
/*--------------------------------------------------------------------*/

  nodo_historia *ah;

  assert(d!=NULL);
  assert(merr!=NULL);
  *d=NULL;
  if (s==NULL)
    { return 0; }
  *d=crear_demost();
  if (*d==NULL)
    {
      strcat(merr,mensajes[166]);
      strcat(merr,"\n");
      return -1;
    }
  if (copiar_historia(&((*d)->his_li),s->his_li,merr)!=0)
    {
      liberar_demost(d);
      *d=NULL;
      return -1;
    }
  /* Ponemos ult_li al final de la historia del lado izquierdo */
  for ((*d)->ult_li=NULL,ah=(*d)->his_li;
       ah!=NULL;
       (*d)->ult_li=ah,ah=ah->sig);
  if (copiar_historia(&((*d)->his_ld),s->his_ld,merr)!=0)
    {
      liberar_demost(d);
      *d=NULL;
      return -1;
    }
  /* Ponemos ult_ld al final de la historia del lado derecho */
  for ((*d)->ult_ld=NULL,ah=(*d)->his_ld;
       ah!=NULL;
       (*d)->ult_ld=ah,ah=ah->sig);

  if (copiar_demost(&((*d)->sig),s->sig,merr)!=0)
    {
      liberar_demost(d);
      *d=NULL;
      return -1;
    }
  return 0;
}


/************************************************************************
* void liberar_conjeturas(nodo_igualdad **conj)                         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Libera la memoria de conj.                              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio  91 *
************************************************************************/

void
liberar_conjeturas (nodo_igualdad ** conj)
{
/*--------------------------------------------------------------------*/
  nodo_igualdad *c;

  while ((*conj) != NULL)
    {
      c = (*conj)->sig;
      liberar_expresion (&((*conj)->lado_izq));
      liberar_expresion (&((*conj)->lado_der));
      (*conj)->sig = NULL;	/*VTP. 8/may/1997 */
      free (*conj);
      (*conj) = c;
    }
  (*conj) = NULL;
/*--------------------------------------------------------------------*/
}

/************************************************************************
* void liberar_historia(nodo_historia **his)                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Libera la memoria de his.                               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio  91 *
************************************************************************/

void
liberar_historia (nodo_historia ** his)
{
/*--------------------------------------------------------------------*/
  nodo_historia *h;

  while ((*his) != NULL)
    {
      h = (*his)->sig;
      liberar_expresion (&((*his)->expr));
      free (*his);
      (*his) = h;
    }
  (*his) = NULL;
/*--------------------------------------------------------------------*/
}

/************************************************************************
* int es_parametro_induccion(nodo_expresion *expr,int pos)              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Indica si el parametro pos de expr es de induccion.     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio  91 *
************************************************************************/

int
es_parametro_induccion (nodo_expresion * expr, int pos)
{
/*--------------------------------------------------------------------*/
  nodo_dom *d;
  nodo_select *s;
  nodo_tad *t;

  if (expr == NULL || expr->argumentos == NULL)
    return (0);
  s = NULL;
  for (t = tad; (t != NULL);)
    {
      if (s == NULL)
	s = t->selectoras;
      if (!strcmp (s->nombre, expr->nombre))
	t = NULL;
      else
	{
	  s = s->sig;
	  if (s == NULL)
	    t = t->sig;
	}
    }
  if (s == NULL)
    return (0);
  for (d = s->dominio; (d != NULL) && (pos > 1); d = d->sig, pos--);
  if (d == NULL)
    return (0);
  return (d->induccion);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* nodo_argum *buscar_variable_ind(nodo_expresion *expr_li,              *
*                                 nodo_expresion *expr_ld)              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Encontrar todas las posibles variables de induccion     *
* PARAMETROS :                                          		*
*    nodo_expresion *expr_li : Expresion lado izquierdo.                *
*    nodo_expresion *expr_ld : Expresion lado derecho.                  *
* POS : buscar_varible_ind = Lista de variables de induccion de las     *
*       expresion expr_li,expr_ld                                       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio  91 *
************************************************************************/

nodo_argum *
buscar_variable_ind (nodo_expresion * expr_li,
		     nodo_expresion * expr_ld)
{
/*--------------------------------------------------------------------*/
  nodo_argum *lis_arg = NULL;

  lis_arg = NULL;
  buscar_variable (&lis_arg, expr_li, NULL, 0);
  buscar_variable (&lis_arg, expr_ld, NULL, 0);
  return (lis_arg);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* void *buscar_variable(nodo_argum *lis,                                *
*                       nodo_expresion *expr,                           *
*                       nodo_expresion *padre, int pos)                 *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Encontrar todas las variables de induccion de una       *
*               expresion.                                              *
* PARAMETROS :                                          		*
*    nodo_argum     *lis  : Lista de varibles encontradas.              *
*    nodo_expresion *expr : Expresion .                                 *
*    nodo_expresion *padre: Expresion de la cual expr es descendiente   *
*    int            pos   : Posicion que ocupa expr dentro de padre     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio  91 *
************************************************************************/

void
buscar_variable (nodo_argum ** lis,
		 nodo_expresion * expr,
		 nodo_expresion * padre, int pos)
{
/*--------------------------------------------------------------------*/
  int i;
  nodo_argum *arg;
  nodo_expresion *var;

  assert (lis != NULL);		/* VTP. 8/May/1997 */
  if (expr == NULL || !strcmp (expr->tipo, ""))
    return;
  if (!strcmp (expr->clase, "VAR"))
    {
      if ((padre != NULL) && strcmp (padre->clase, "CONSTR") &&
	  strcmp (padre->clase, "INIC") &&
	  es_parametro_induccion (padre, pos))
	insertar_expresion (lis, expr);
      else
	return;
    }
  else
    {
      var = NULL;
      for (i = 1, arg = expr->argumentos; arg != NULL && var == NULL;
	   arg = arg->sig, i++)
	buscar_variable (lis, arg->expr, expr, i);
    }
/*--------------------------------------------------------------------*/
}

/************************************************************************
* void adicionar_nivel (nodo_demost *dem, cad20 nivel, char c)          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO: Adicionar un par de historias a dem, cuyos niveles tienen  *
* el valor nivel+c.                                                     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio  91 *
************************************************************************/

void
adicionar_nivel (nodo_demost * dem, cad20 nivel, char c)
{
/*--------------------------------------------------------------------*/
  cad20 nuevo_nivel;

  assert (nivel != NULL && dem != NULL);	/* VTP. 8/May/1997 */
  sprintf (nuevo_nivel, "%s%c", nivel, c);
  dem->ult_ld->sig = crear_historia (NULL, NULL, nuevo_nivel);
  dem->ult_li->sig = crear_historia (NULL, NULL, nuevo_nivel);
  dem->ult_ld = dem->ult_ld->sig;
  dem->ult_li = dem->ult_li->sig;
/*--------------------------------------------------------------------*/
}
