 /************************************************************************
* PROYECTO: MANTA        MODULO: sustituc.c      FECHA: 10 Nov 1990     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
*    PROPOSITO GLOBAL DE LAS FUNCIONES IMPLEMENTADAS EN ESTE MODULO     *
*                                                                       *
* TAD de sustituciones, las sustituciones se representaran por medio    *
* de una tabla, en la cual el selector es el nombre de la variable      *
* y el item es el termino  						*
*************************************************************************
* AUTOR DE ESTE MODULO:  C.A. Munoz                                     *
************************************************************************/

	/*------------------------------------------------------+
        |          Encabezados de librerias estandares          |
        +------------------------------------------------------*/


#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include "definic.h"
#include "apimanta.h"

/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* v-tamara 8/May/1997 */

/************************************************************************
* nodo_expresion *apl(nodo_expresion *expr,Ptr_tabla sust,		*
*		    short copia, short *aplico)                         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Aplicar una sustitucion a una expresion .               	*
* DESCRIPCION : Si la expresion es variable, hace directamente la       *
*               sustitucion, de lo contrario aplica a cada argumento    *
*		 de la expresion. En aplico retorna si pudo aplicar la  *
*		 sustitucion. Si el parametro copia = 0 , la expresion  *
*		 es modificada o detruida. Si copia = 1, saca una copia *
*               de la expresion						*
* PARAMETROS : 								*
*   nodo_expresion *expr Expresion sobre para aplicar la sustitucion.   *
*   Ptr_tabla      sust  Sustitucion que se va a aplicar 		*
*   short          copia Indica si debe sacar copia o no		*
*   short 	    *aplico Retorna 1 si pudo aplicar la sustitucion    *
* PRE! : sust, expr = E							*
* POST! : apl = expr@sust(aplicar la sustitucion a expr) y		*
*         (copia = 1 y apl aplica sobre una copia de expr) o		*
*         (copia = 0 y E es destruida)					*
*	   (aplico =1 y apl <> expr ) o (aplico = 0 y apl = expr )	*
* LLAMA A:								*
*   short es_variable_exp  	Indica si una expresion es variable 	*
*   nodo_entrada   *loctab     Localiza una entrada de la tabla		*	
*   nodo_expresion *copia_exp  Copia una expresion			*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 14-Nov-1990	*
************************************************************************/

nodo_expresion *
apl (nodo_expresion * expr, Ptr_tabla sust,
     short copia, short *aplico)

{
/*-------------------------------------------------------------------*/
  nodo_expresion *expaux;
  nodo_entrada *ent;
  nodo_argum *arg;
  short aplico_exp,		/* Aplico en la expresion */
    aplico_arg;			/* Aplico en un argumento */

  assert (aplico != NULL);	/* VTP. 8/May/1997 */
  aplico_exp = 0;
  if (expr == NULL)
    {
      *aplico = 0;
      return (expr);
    }
  if (es_variable_exp (expr))
    {
      ent = loctab (sust, (expr)->nombre);
      if (ent != NULL)
	{
	  aplico_exp = 1;
	  expaux = copia_exp (ent->item);
	}
      else if (copia)
	expaux = copia_exp (expr);
      else
	expaux = expr;
    }
  else
    {
      if (copia)
	expaux = copia_exp (expr);
      else
	expaux = expr;
      for (arg = expaux->argumentos; arg != NULL; arg = arg->sig)
	{
	  arg->expr = apl (arg->expr, sust, 0, &aplico_arg);
	  /* VTP. The argument copia can be false because if it was true
	     now we are working with a copy (expaux) and we must change 
	     that copy */
	  aplico_exp = (short)(aplico_exp || aplico_arg);
	}
    }
  *aplico = aplico_exp;
  return (expaux);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* nodo_expresion *aplr(nodo_expresion *expr,Ptr_tabla sust,		*
*		    short copia )                         		*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Aplicar una sustitucion a una expresion, mientras se 	*
*	       pueda.               					*
* DESCRIPCION : Llama la funcion apl (aplicar) hasta que no hayan mas   *
*		 sustituciones posibles					*
* PARAMETROS : 								*
*   nodo_expresion *expr Expresion sobre para aplicar la sustitucion.   *
*   Ptr_tabla      sust  Sustitucion que se va a aplicar 		*
*   short          copia Indica si debe sacar copia o no		*
* PRE! : sust, expr = E							*
* POST! : aplr = expr@sust* (aplicar la sustitucion a expr mientras se  *
*	   pueda ) y (copia = 1 y aplr aplica sobre una copia de expr)  *
*         o (copia = 0 y E es destruida)				*
* LLAMA A:								*
*   nodo_expresion *apl Aplica una sustitucion	a una expresion		*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 14-Nov-1990	*
************************************************************************/

nodo_expresion *
aplr (nodo_expresion * expr, Ptr_tabla sust,
      short copia)

{
/*-------------------------------------------------------------------*/
  short aplico;
  nodo_expresion *expaux;

  aplico = 0;
  expaux = apl (expr, sust, copia, &aplico);
  while (aplico)
    expaux = apl (expaux, sust, 0, &aplico);
  return (expaux);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* void agr(Ptr_tabla sust,cad12 var,nodo_expresion *term)		*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Agregar una pareja variable -> termino a una sustitucion  *
* DESCRIPCION : Invoca la funcion que agrega una entrada a una tabla    *
* PARAMETROS : 								*
*   Ptr_tabla sust       Sustitucion donde se agrega la pareja         	*
*   cad12     var        Nombre de la variable que se agrega a sust	*
*   nodo_expresion *term Termino que se agrega a sust			*
* PRE : sust --> [<"SUST",Null>,<v1,t1>,..,<vn,tn>],var ,term		*
* POST : sust --> [<"SUST",Null>,<var,term>,<v1,t1>,..,<vn,tn>],	*
* LLAMA A:								*
*   void adjtab    Adiciona una entrada a una tabla		        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 14-Nov-1990	*
************************************************************************/

void
agr (Ptr_tabla sust, cad12 var, nodo_expresion * term)

{
/*-------------------------------------------------------------------*/

  if (loctab (sust, var) == NULL)
    adjtab (sust, var, term);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* nodo_expresion *copia_exp(nodo_expresion *expr)			*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Hacer la copia de una expresion.                          *
* DESCRIPCION : Recorre recursivamente la funcion, sacando una copia    *
* PARAMETROS : 								*
*   nodo_expresion   *expr   Ap. a la expresion				*
* PRE! : expr es un ap. a una expresion 				*
* POST!: copia_exp  es una copia de expr                    		*
* LLAMA A:								*
*   short es_variable_exp   Indica si una expresion es una variable     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 14-Nov-1990	*
************************************************************************/

nodo_expresion *
copia_exp (nodo_expresion * expr)

{
/*-------------------------------------------------------------------*/
  nodo_expresion *expaux;
  nodo_argum *arg,		/* recorrido de los argumentos de origen */
   *argaux,			/* argumento que se esta creando         */
   *ult;			/* ultimo argumento creado               */

  if (expr == NULL)
    return (expr);
  expaux = (nodo_expresion *) malloc (sizeof (nodo_expresion));
  if (expaux==NULL)
    {
      HayMem (expaux, sizeof (nodo_expresion));	/* VTP. 8/May/1997 */
    }
  strcpy (expaux->nombre, expr->nombre);
  strcpy (expaux->tipo, expr->tipo);
  strcpy (expaux->clase, expr->clase);
  expaux->argumentos = NULL;
  ult = NULL;
  if (!es_variable_exp (expr))
    {
      for (arg = expr->argumentos; arg != NULL; arg = arg->sig)
	{
	  argaux = (nodo_argum *) malloc (sizeof (nodo_argum));
	  HayMem (argaux, sizeof (nodo_argum));		/* VTP. 8/May/1997 */
	  argaux->sig = NULL;
	  if (ult == NULL)
	    expaux->argumentos = argaux;
	  else
	    ult->sig = argaux;
	  ult = argaux;
	  argaux->expr = copia_exp (arg->expr);
	}
    }
  return (expaux);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* Ptr_tabla eps()							 *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Hacer una sustitucion vacia.                               *
* DESCRIPCION : Invoca la funcion que crea una tabla.	                 *
* PARAMETROS : 						         	 *
* PRE! : TRUE 								 *
* POST!: eps --> [<"SUST",Null>]                    			 *
* LLAMA A:								 *
*   Ptr_tabla vactab    Retorna una tabla vacia		        	 *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 14-Nov-1990	 *
*************************************************************************/

Ptr_tabla
eps ()

{
/*-------------------------------------------------------------------*/
  return (vactab ("SUST"));
/*--------------------------------------------------------------------*/
}

/************************************************************************
*  short es_funtor_exp ( nodo_expresion *expr )				*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Indicar si la la expresion que entra como parametro es   	*
*	       una funcion, en cuyo caso retorna la aridad.		*
* DESCRIPCION : Examina si la la clase de la expresiones es diferente  	*
*             	 de "VAR", en este caso recorre la lista de argumentos  *
*		 y retorna la aridad.					*
* PARAMETROS : 								*
*   nodo_expresion   *expr   Ap. a la expresion				*
* PRE! : expr es un ap. a una expresion 				*
* POST!: es_functor_exp == n , n >= 0 y expr es un functor o		*
*	  es_functor_exp < 0 y expr no es functor			*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 14-Nov-1990	*
************************************************************************/

short
es_funtor_exp (nodo_expresion * expr)

{
/*-------------------------------------------------------------------*/
  short num_arg;
  nodo_argum *a;

  num_arg = 0;
  if ((expr != NULL) && (strcmp (expr->clase, "VAR")))
    for (a = expr->argumentos; a != NULL; a = a->sig, ++num_arg);
  else
    --num_arg;
  return (num_arg);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* short es_variable_exp ( nodo_expresion *expr )			*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Indicar si la la expresion que entra como parametro es   	*
*	       variable.						*
* DESCRIPCION : Examina si la la clase de la expresiones "VAR".		*
* PARAMETROS : 								*
*   nodo_expresion   *expr   Ap. a la expresion				*
* PRE! : expr es un ap. a una expresion 				*
* POST!: es_variable_exp ssi. la clase de expr es "VAR"			*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 14-Nov-1990	*
************************************************************************/

short
es_variable_exp (nodo_expresion * expr)

{
/*-------------------------------------------------------------------*/
  short es_var;

  es_var = 0;
  if (expr != NULL && !strcmp (expr->clase, "VAR"))
    es_var = 1;
  return (es_var);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* short equal( nodo_expresion *expr1, nodo_expresion *expr2 )		*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Indicar si dos expresiones son iguales			*
* DESCRIPCION : Compara recursivamente los funtores y sus argumentos    *
* PARAMETROS : 					        		*
*   nodo_expresion *expr1,*expr2 Expresiones que se deben comparar 	*
* PRE! : expr1 , expr2							*
* POST! : equal <=> expr1 = expr2					*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 14-Nov-1990	*
************************************************************************/

short
equal (nodo_expresion * expr1, nodo_expresion * expr2)

{
/*-------------------------------------------------------------------*/
  short igual;
  nodo_argum *a1, *a2;

  igual = 1;
  if ((expr1 == NULL) && (expr2 == NULL))
    return (igual);
  if (!strcmp (expr1->nombre, expr2->nombre))
    for (a1 = expr1->argumentos, a2 = expr2->argumentos;
	 (a1 != NULL) && (a2 != NULL) && igual; a1 = a1->sig, a2 = a2->sig)
      igual = equal (a1->expr, a2->expr);
  else
    igual = 0;
  return (igual);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* short match( nodo_expresion *expr1, nodo_expresion *expr2,		*
*	Ptr_tabla unif )                                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Adicionar a unif, la sustitucion del  umg1 de expr1	*
*	       y expr2. 						*
* DESCRIPCION : Sustituir variables por terminos y hacer recursion      *
*               sobre los parametros de las funciones. En caso de que	*
*		 no exista un UMG1 retorna 0.				*
*              expr1 es el patron, expr2 es el objetivo                 *
* PARAMETROS : 						        	*
*   nodo_expresion *expr1,*expr2 Expresiones que se deben 1-unificar 	*
*   Ptr_tabla      unif   Se guarda la sustitucion del UMG1 		*
* PRE! : expr1 , expr2, unif es una sustitucion 		        *
* POST! : match = 0 y expr1 no 1-unifica con expr2 o			*
*	   match  =1 y unif contiene el UMG1 de expr1 y expr2		*
* LLAMA A:								*
*     short *es_variable_exp    Indica si una expresion es variable  	*
*     void           agr        Agrega una entrada a una sustitucion 	*
*     short es_variable_tipo    Indica si una variable exite en         *
*				 una expresion con un tipo dado 	*
*     nodo_expresion *copia_exp Copia una expresion                     *
*     nodo_entrada   *loctab    Localiza una entrada en una tabla	*
*     short  equal		 Indica si dos expresiones son iguales  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 14-Nov-1990	*
************************************************************************/

short
match (nodo_expresion * expr1, nodo_expresion * expr2, Ptr_tabla unif)

{
/*-------------------------------------------------------------------*/
  nodo_entrada *a;
  nodo_argum *arg1, *arg2;
  short si_unif;

  /* si son de tipos diferentes, no pueden unificar */
  if (expr1 == NULL || expr2 == NULL ||
      (strcmp (expr1->tipo, "") && strcmp (expr1->tipo, expr2->tipo)))
    si_unif = 0;
  else if (es_variable_exp (expr1))
    {
      a = loctab (unif, expr1->nombre);
      if (a != NULL)
	si_unif = equal (a->item, expr2);
      else
	{
	  agr (unif, duplicastr(expr1->nombre), copia_exp (expr2));
	  si_unif = 1;
	}
    }
  else if (es_variable_exp (expr2))
    si_unif = 0;
  else if (!strcmp (expr1->nombre, expr2->nombre))
    {
      for (si_unif = 1, arg1 = expr1->argumentos, arg2 = expr2->argumentos;
	   (arg1 != NULL) && (arg2 != NULL)
	   && si_unif; arg1 = arg1->sig, arg2 = arg2->sig)
	si_unif = match (arg1->expr, arg2->expr, unif);
      if (arg1 != NULL || arg2 != NULL)
	si_unif = 0;
    }
  else
    si_unif = 0;
  return (si_unif);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* short umgaux( nodo_expresion *expr1, nodo_expresion *expr2,		*
*	Ptr_tabla unif )                                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Adicionar a unif, la sustitucion del  umg de expr1	*
*	       y expr2. 						*
* DESCRIPCION : Sustituir variables por terminos y hacer recursion      *
*               sobre los parametros de las funciones. En caso de que	*
*		 no exista un UMG retorna 0.				*
* PARAMETROS : 						        	*
*   nodo_expresion *expr1,*expr2 Expresiones que se deben unificar 	*
*   Ptr_tabla      unif   Se guarda la sustitucion del UMG 		*
* PRE! : expr1 , expr2, unif es una sustitucion 		        *
* POST! : umgaux = 0 y expr1 no unifica con expr2 o			*
*	   umgaux =1 y unif contiene el UMG de expr1 y expr2		*
* LLAMA A:								*
*     short *es_variable_exp    Indica si una expresion  es variable    *
*     void           agr        Agrega una entrada a una sustitucion 	*
*     nodo_expresion *copia_exp Copia una expresion                     *
*     nodo_entrada   *loctab    Localiza una entrada en una tabla	*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 14-Nov-1990	*
************************************************************************/

short
umgaux (nodo_expresion * expr1, nodo_expresion * expr2, Ptr_tabla unif)

{
/*-------------------------------------------------------------------*/
  nodo_entrada *a;
  nodo_expresion *expaux;
  nodo_argum *arg1, *arg2;
  short si_unif;

  /* si son de tipos diferentes, no pueden unificar */
  if (expr1 == NULL || expr2 == NULL || strcmp (expr1->tipo, expr2->tipo))
    return (0);
  if (es_variable_exp (expr1))
    {
      a = loctab (unif, expr1->nombre);
      if (a != NULL)
	return (umgaux (a->item, expr2, unif));
      else
	{
	  expaux = aplr (expr2, unif, 1);
	  if (es_variable_exp (expaux) && equal (expr1, expaux))
	    return (1);
	  else if (es_variable_tipo (expr1->nombre, expaux, expr1->tipo))
	    return (0);
	  else
	    {
	      agr (unif, expr1->nombre, expaux);
	      return (1);
	    }
	}
    }
  else if (es_variable_exp (expr2))
    return (umgaux (expr2, expr1, unif));
  else if (!strcmp (expr1->nombre, expr2->nombre))
    {
      for (si_unif = 1, arg1 = expr1->argumentos, arg2 = expr2->argumentos;
	   (arg1 != NULL) && (arg2 != NULL)
	   && si_unif; arg1 = arg1->sig, arg2 = arg2->sig)
	si_unif = umgaux (arg1->expr, arg2->expr, unif);
      if (arg1 == NULL && arg2 == NULL)
	return (si_unif);
      else
	return (0);
    }
  else
    return (0);
/*--------------------------------------------------------------------*/
}


/************************************************************************
* Ptr_tabla umg( nodo_expresion *expr1, nodo_expresion *expr2 )		*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Encontrar el UMG de expr1 y expr2	        		*
* DESCRIPCION : Llama la funcion umgaux con la sustitucion vacia	*
* PARAMETROS : 								*
*   nodo_expresion *expr1,*expr2 Expresiones que se deben unificar 	*
* PRE! : expr1 , expr2							*
* POST! : umg = NULL y expr1 no unifica con expr2 o			*
*	   umg es el UMG de expr1 y expr2				*
* LLAMA A:								*
*    Ptr_tabla eps  Crea una sustitucion vacia				*
*    short esvactab Indica si la tabla de sustitucion es vacia		*
*    short umgaxu   Acumula el unificador de dos expresione		*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 14-Nov-1990	*
************************************************************************/

Ptr_tabla
umg (nodo_expresion * expr1, nodo_expresion * expr2)

{
/*-------------------------------------------------------------------*/
  Ptr_tabla unif;

  unif=eps();
  if (unif==NULL || umgaux (expr1, expr2, unif) == 0)
    { 
      elimLTabla(&unif);
    }
  return (unif);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* Ptr_tabla umg1( nodo_expresion *expr1, nodo_expresion *expr2 )	*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Encontrar el UMG1 de expr1 y expr2			*
* DESCRIPCION : Llama la funcion match con la sustitucion vacia   	*
* PARAMETROS : 						        	*
*   nodo_expresion *expr1,*expr2 Expresiones que se deben 1-unificar 	*
* PRE! : expr1 , expr2							*
* POST! : umg1 = NULL y expr1 no 1-unifica con expr2 o			*
*	   umg1 es el UMG1 de expr1 y expr2				*
* LLAMA A:								*
*    Ptr_tabla eps  Crea una sustitucion vacia				*
*    short esvactab Indica si la tabla de sustitucion es vacia		*
*    short match    Acumula el 1-unificador de dos expresione		*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 14-Nov-1990	*
************************************************************************/

Ptr_tabla
umg1 (nodo_expresion * expr1, nodo_expresion * expr2)

{
/*-------------------------------------------------------------------*/
  Ptr_tabla unif = NULL;
/* Alvaro
   nodo_tabla *a;
 */

  unif = eps ();
  if (match (expr1, expr2, unif) == 0)
    {
      elimLTabla(&unif);
      unif=NULL;
    }
  return unif;
/*--------------------------------------------------------------------*/
}
