 /************************************************************************
* PROYECTO: MANTA        MODULO: tabla.c         FECHA: 10 Nov 1991     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
*    PROPOSITO GLOBAL DE LAS FUNCIONES IMPLEMENTADAS EN ESTE MODULO     *
*                                                                       *
* Implementacion del TAD TABLA. Una tabla es una lista de entradas.     *
* Una entrada tiene un selector y un contenido.                         *
*************************************************************************
* AUTOR DE ESTE MODULO: C.A. Munoz                                      *
************************************************************************/


#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include "definic.h"

/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* v-tamara 8/May/1997 */

/************************************************************************/
/*  void adjtab( Ptr_tabla tabla, cad12 sel, nodo_expresion *it )       */
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/* PROPOSITO : Adicionar una pareja <sel,it> a la tabla.                */
/* DESCRIPCION : La pareja se inserta al comienzo de la tabla.          */
/* PARAMETROS :                                                         */
/*   Ptr_tabla tabla      Sustitucion donde se agrega la pareja         */
/*   cad12     sel        Selector                                      */
/*   nodo_expresion *it   Item                                          */
/* PRE!:  tabla --> [<ident,Null><Sel1,It1>..<Seln,Itn>],<sel,it>       */
/* POST!: tabla --> [<Ident,Null>,<Sel,It>,<Sel1,It1>...<Seln,Itn>]     */
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/* AUTOR : Cesar Augusto Munoz Hurtado          FECHA : 14-Nov-1990     */
/************************************************************************/

void
adjtab (Ptr_tabla tabla, cad12 sel, nodo_expresion * it)

{
  nodo_entrada *nodo_ent;
  nodo_tabla *nodo_tab;

  assert (sel != NULL && tabla != NULL);	/* VTP. 8/May/1997 */
  nodo_ent = (nodo_entrada *) malloc (sizeof (nodo_entrada));
  HayMem (nodo_ent, sizeof (nodo_entrada));
  strcpy (nodo_ent->selector, sel);
  nodo_ent->item = copia_expresion(it);
  nodo_tab = (nodo_tabla *) malloc (sizeof (nodo_tabla));
  HayMem (nodo_tab, sizeof (nodo_tabla));
  nodo_tab->entrada = nodo_ent;
  nodo_tab->sig = tabla->sig;
  tabla->sig = nodo_tab;
}

/************************************************************************/
/* void  elimtab( Ptr_tabla tabla,cad12 sel )                           */
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/* PROPOSITO : Elimina la entrada en la tabla con el selector sel, si no */
/*             existe retorna la tabla sin ningun cambio.               */
/* DESCRIPCION : Usa loctabizq y elimina la entrada correspondiente.    */
/* PARAMETROS :                                                         */
/*   Ptr_tabla tabla      Tabla donde se busca el selector              */
/*   cad12     sel        Selector de la entrada que se desea eliminar  */
/* PRE!: tabla --> [<Ident,Null><sel1,it1>..<seln,itn>], sel            */
/* POST! sel = seli y tabla --> [...<seli-1,iti-1>,<seli+1,iti+1>...]   */
/*       sel <> seli para todo i y                                      */
/*       tabla --> [<Ident,Null><sel1,it1>..<seln,itn>]                 */
/* LLAMA A :                                                            */
/*    Nodo_tabla *loctabizq Localiza una entrada en la tabla            */
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/* AUTOR : Cesar Augusto Munoz Hurtado          FECHA : 14-Nov-1990     */
/************************************************************************/

void
elimtab (Ptr_tabla tabla, cad12 sel)

{
  nodo_tabla *a, *aux;

  assert (sel != NULL);		/* VTP. 8/May/1997 */
  a = loctabizq (tabla, sel);
  if (a != NULL)
    {
      aux = a->sig;
      if (aux != NULL)
	{
	  a->sig = a->sig->sig;
	  free (aux->entrada);
	  aux->sig = NULL;
	  free (aux);
	}
    }
}

/************************************************************************
* void  destruyatab( Ptr_tabla *tabla)                                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Destruye la tabla que entra como parametro.               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 14-Nov-1990	*
************************************************************************/

void
destruyatab (Ptr_tabla * tabla)
{
/*--------------------------------------------------------------------*/
  nodo_tabla *a;

  while ((*tabla) != NULL)
    {
      a = (*tabla);
      (*tabla) = a->sig;
      free (a->entrada);
      a->sig = NULL;
      free (a);
    }
  (*tabla) = NULL;
/*--------------------------------------------------------------------*/
}

/************************************************************************/
/* short esvactab( Ptr_tabla tabla)                                     */
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/* PROPOSITO : Indicar si una tabla es vacia.                           */
/* DESCRIPCION : Una tabla es vacia, si solo tiene la entrada para el   */
/*               identificador .                                        */
/* PARAMETROS :                                                         */
/*     Ptr_tabla tabla                                                  */
/* PRE! : tabla                                                         */
/* POST! :  esvactab = 1  <==> tabla --> [<ident,Null>]                 */
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/* AUTOR : Cesar Augusto Munoz Hurtado          FECHA : 14-Nov-1990     */
/************************************************************************/

short
esvactab (Ptr_tabla tabla)
{
  assert (tabla != NULL);	/* VTP. 8/May/1997 */
  return ((short) (tabla->sig == NULL));
}

/************************************************************************/
/* nodo_tabla  *loctab( Ptr_tabla tabla, cad12 sel )                    */
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/* PROPOSITO : Localiza la entrada de la tabla que contiene el selector */
/* DESCRIPCION : Usa loctabizq.                                         */
/* PARAMETROS :                                                         */
/*   Ptr_tabla tabla      Tabla donde se busca el selector              */
/*   cad12     sel        Selector                                      */
/* PRE!: tabla --> [<Ident,Null><sel1,it1>..<seln,itn>], sel            */
/* POST! sel = seli y loctab = <seli,iti> o                             */
/*       sel <> seli para todo i y loctab = Null                        */
/* LLAMA A :                                                            */
/*    Nodo_tabla *loctabizq Localiza una entrada en la tabla            */
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/* AUTOR : Cesar Augusto Munoz Hurtado          FECHA : 14-Nov-1990     */
/************************************************************************/

nodo_entrada *
loctab (Ptr_tabla tabla, cad12 sel)

{
  nodo_tabla *a;

  if (tabla == NULL || esvactab (tabla))
    return (NULL);
  a = loctabizq (tabla, sel);
  if (a != NULL)
    return (a->sig->entrada);
  return (NULL);
}

/************************************************************************/
/* nodo_tabla  *loctabizq( Ptr_tabla tabla, cad12 sel )                 */
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/* PROPOSITO : Localiza el nodo de la tabla que esta a la izquierda de  */
/*             aquel que contiene el selector.                          */
/* DESCRIPCION : Recorre al tabla y si no encuentra el selector         */
/*               retorna Null.                                          */
/* PARAMETROS :                                                         */
/*   Ptr_tabla tabla      Tabla donde se busca el selector              */
/*   cad12     sel        Selector                                      */
/* PRE!: tabla --> [<Ident,Null><sel1,it1>..<seln,itn>], sel            */
/* POST! sel = seli y loctabizq = *<seli-1,iti-1> o                     */
/*       sel <> seli para todo i y loctabizq = Null                     */
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/* AUTOR : Cesar Augusto Munoz Hurtado          FECHA : 14-Nov-1990     */
/************************************************************************/

nodo_tabla *
loctabizq (Ptr_tabla tabla, cad12 sel)

{
  nodo_tabla *a;

  if ((tabla == NULL) || esvactab (tabla))
    a = NULL;
  else
    for (a = tabla;
	 ((a->sig) != NULL)
	 && (strcmp (a->sig->entrada->selector, sel));
	 a = a->sig);
  if ((a->sig) != NULL)
    return (a);
  else
    return (NULL);
}

/************************************************************************/
/* Ptr_tabla vactab( cad12 ident)                                       */
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/* PROPOSITO : Crear una tabla vacia.                                   */
/* DESCRIPCION : Reserva espacio para la cabeza de la tabla.            */
/* PARAMETROS :                                                         */
/*   cad12 ident  Identificador de la tabla.                            */
/* PRE! : ident es una cadena de caracteres                             */
/* POST! :  vactab  --> [<ident,Null>]                                  */
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/* AUTOR : Cesar Augusto Munoz Hurtado          FECHA : 14-Nov-1990     */
/************************************************************************/

Ptr_tabla
vactab (cad12 ident)

{
  Ptr_tabla a;

  assert (ident != NULL);	/* VTP. 8/May/1997 */
  a = (Ptr_tabla) malloc (sizeof (nodo_tabla));
  HayMem (a, sizeof (nodo_tabla));
  a->sig = NULL;
  a->entrada = (nodo_entrada *) malloc (sizeof (nodo_entrada));
  HayMem (a->entrada, sizeof (nodo_entrada));
  strcpy (a->entrada->selector, ident);
  a->entrada->item = NULL;
  return (a);
}
