 
/* ANAL_SIGN.C                                                           */
/* Analiza sintacticamente el archivo "signa_ed.tad" que contiene la */
/* definicion de un TAD. y la almacena en una estructura de tipo nodo_tad */
/* apuntada por tad_ed.                                                  */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include "definic.h"

/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* v-tamara 28/May/1997 */


/**************************************************************************
* short analice_constructora(FILE *fp,texpr mens_error,			  *
*     			     nodo_generadora **n)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Analizar sintacticamente la funcionalidad de una construc- *
*              tora.                                                      *
* DESCRIPCION: Analiza los siguientes tokens del archivo verificando que  *
*        describan la funcionalidad de una constructora del tad_ed.       *
* PARAMETROS:  							 	  *
*   FILE            *fp        	  Ap. al archivo que contiene la signatura*
*   texpr           mens_error    Comentario acerca de un eventual error. *
*   nodo_generadora **n		  Ap. a la lista que contiene el dominio  *
*				  de la constructora analizada.		  *
* V.GLOBALES: 								  *
*   texpr  	    token      	  Contiene el ultimo token leido.         *
*   nodo_tad 	    *tad_ed    	  Ap. a la estructura que contiene la in- *
*				  formacion analizada hasta el momento de *
*				  la signatura contenida en el archivo    *
*				  apunt. por fp.		  	  *
* RESULTADO:  short               0, Si se detecto algun error. 1, Sino.  *
* PRE!:  {fp apunt. al archivo de axiomas.				} *
* POST!: {analice_constructora = 0, Si la funcionalidad de la constructora*
*            tiene algun error sintactico y mens_error contiene una expli-*
*            cacion acerca del error detectado, o                         *
*         analice_constructora = 1, Sino. (*n) apunta a la lista que con- *
*	     tiene el dominio de la constructora analizada.		} *
* !OJO!: El codominio de la constructora debe ser el tad_ed.              *
* LLAMA A: void   deme_token      	 short  analice_prod_cjtos        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
*************************************************************************/

short
analice_constructora (FILE * fp, texpr mens_error, nodo_generadora ** n)
{
/*----------------------------------------------------------------------*/

  short error;
  nodo_nombre *c=NULL;                  /* v-tamara 28/May/1997 */
  cad12 tadname;
  int aparece;

  assert(tad_ed!=NULL);                 /* v-tamara 28/May/1997 */
  error = 0;
  if (es_error_esperaba ("", token, mens_error))
    return (0);

  /* VTP. 4/May/1998 Good names */
  if (existsGenerator(tad_ed->iniciales,token))
    {
      sprintf(mens_error,mensajes[180],token,tad_ed->nombre);
      return 0;
    }
  if (existsGenerator(tad_ed->constructoras,token))
    {
      sprintf(mens_error,mensajes[180],token,tad_ed->nombre);
      return 0;
    }
  /* No se prohibe que una constructora tenga el nombre 
     del tipo */
  if (existsSymbol(symtable,token,tadname) &&
      strcmp(tad_ed->nombre,tadname)!=0)    
    {
      sprintf(mens_error,mensajes[180],token,tadname);
      return 0;
    }
  if (existsName(tad_ed->parametros,token))
    {
      sprintf(mens_error,mensajes[181],token,tad_ed->nombre);
      return 0;
    }
  if (strncmp(token,"EQ_",3)==0)
    { /* No puede llamarse EQ_X */
      sprintf(mens_error,mensajes[190],token);
      return 0;
    }

  (*n) = (nodo_generadora *) malloc (sizeof (nodo_generadora));
  HayMem(*n,sizeof(nodo_generadora));   /* v-tamara 28/May/1997 */
  strcpy ((*n)->nombre, token);
  strcpy ((*n)->nombre_fte, token);
  (*n)->sig = NULL;
  c = NULL;
  deme_token (fp);
  if (es_error_esperaba (":", token, mens_error))
    return (0);
  deme_token (fp);
  if (analice_prod_cjtos (tad_ed->nombre,fp, mens_error, &c, 1,&aparece))
    {
      if (aparece==0)
	{
	  sprintf(mens_error,mensajes[191],tad_ed->nombre,(*n)->nombre);
	  error=1;
	}
      else
	{
	  (*n)->dominio = c;
	  deme_token (fp);
	  if (strcmp (token, tad_ed->nombre))
	    {
	      error = 1;
	      strcpy (mens_error, mensajes[43]);
	    }
	}
    }
  else
    {
      error = 1;
    }
      
  return (short)(!error);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short analice_constructoras(FILE *fp,texpr mens_error)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Analizar sintacticamente la funcionalidad de todas las     *
*        constructoras de la signatura.                                   *
* DESCRIPCION: Analiza los siguientes tokens del archivo verificando que  *
*        d>escriban nombres y funcionalidades de constructoras del tad_ed. *
* PARAMETROS:  							          *	
*   FILE            *fp        	  Ap. al archivo que contiene la signatura*
*   texpr           mens_error    Comentario acerca de un eventual error. *
* V.GLOBALES:                                                             *
*   texpr  	    token      	  Contiene el ultimo token leido.         *
*   nodo_tad 	    *tad_ed    	  Ap. a la estructura que contiene la in- *
*				  formacion analizada hasta el momento de *
*				  la signatura contenida en el archivo    *
*				  apunt. por fp.		  	  *
* RESULTADO:  short               0, Si se detecto algun error. 1, Sino.  *
* PRE!:  {fp apunt. al archivo de axiomas.				} *
* POST!: {analice_constructoras = 0, Si la funcionalidad de las construc- *
*   	     toras tiene algun error sintactico y mens_error contiene una *
*	     explicacion acerca del error detectado, o                    *
*         analice_constructoras = 1, Sino. tad_ed contiene la informacion *
*            de las constructoras analizadas.                           } *
* LLAMA A: void   deme_token      	                                  *
*          short  analice_constructora                                    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

short
analice_constructoras (FILE * fp, texpr mens_error)
{
/*----------------------------------------------------------------------*/

  short error;
  nodo_generadora *n=NULL,              /* v-tamara 28/May/1997 */
                  *ult_construct;

  assert(tad_ed!=NULL);                 /* v-tamara 28/May/1997 */
  error = 0;
  tad_ed->constructoras = NULL;
  ult_construct = NULL;
  while (strcmp (token, claves[3]) && (!error))
    {
      if (analice_constructora (fp, mens_error, &n))
	{
	  if (tad_ed->constructoras == NULL)
	    tad_ed->constructoras = n;
	  else
	    ult_construct->sig = n;
	  ult_construct = n;
	  deme_token (fp);
	}
      else
	{
	  error = 1;
	}
    }
  deme_token (fp);
  return (short)(!error);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short analice_inicial(FILE *fp, texpr mens_error,nodo_generadora **n)   *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Analizar sintacticamente la funcionalidad de una inicial.  *
* DESCRIPCION: Analiza los siguientes tokens del archivo verificando que  *
*        describan la funcionalidad de una inicial del tad_ed.       	  *
* PARAMETROS:  							          *	
*   FILE            *fp        	  Ap. al archivo que contiene la signatura*
*   texpr           mens_error    Comentario acerca de un eventual error. *
*   nodo_generadora **n		  Ap. a la lista que contiene el dominio  *
*				  de la inicial analizada.		  *
* V.GLOBALES: 								  *
*   texpr  	    token      	  Contiene el ultimo token leido.         *
*   nodo_tad 	    *tad_ed    	  Ap. a la estructura que contiene la in- *
*				  formacion analizada hasta el momento de *
*				  la signatura contenida en el archivo    *
*				  apunt. por fp.		  	  *
* RESULTADO:  short               0, Si se detecto algun error. 1, Sino.  *
* PRE!:  {fp apunt. al archivo de axiomas.				} *
* POST!: {analice_inicial = 0, Si la funcionalidad de la inicial tiene    *
*	     algun error sintactico y mens_error contiene una explicacion *
*	     acerca del error detectado, o                         	  *
*         analice_inicial = 1, Sino. (*n) apunta a la lista que contiene  *
*	     el dominio de la inicial analizada.			} *
* !OJO!: El codominio de la inicial debe ser el tad_ed.              	  *
* LLAMA A: void   deme_token      	 short  analice_prod_cjtos        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

short
analice_inicial (FILE * fp, texpr mens_error, nodo_generadora ** n)
{
/*----------------------------------------------------------------------*/

  short error;
  nodo_nombre *d=NULL;          /* v-tamara 28/May/1997 */
  cad12 tadname;
  int aparece;

  if (es_error_esperaba ("", token, mens_error))
    return (0);
  /* VTP. 4/May/1998 Good names */
  if (existsGenerator(tad_ed->iniciales,token))
    {
      sprintf(mens_error,mensajes[180],token,tad_ed->nombre);
      printf("Entro por el primero\n");
      return 0;
    }
  if (existsSymbol(symtable,token,tadname) &&
      strcmp(tad_ed->nombre,tadname)!=0)
    {
      sprintf(mens_error,mensajes[180],token,tadname);
      printf("Entro por el segundo\n");
      return 0;
    }
  if (existsName(tad_ed->parametros,token))
    {
      sprintf(mens_error,mensajes[181],token,tad_ed->nombre);
      return 0;
    }
  if (strncmp(token,"EQ_",3)==0)
    { /* No puede llamarse EQ_X */
      sprintf(mens_error,mensajes[190],token);
      return 0;
    }
      
  (*n) = (nodo_generadora *) malloc (sizeof (nodo_generadora));
  HayMem(*n,sizeof (nodo_generadora));  /* v-tamara 28/May/1997 */
  strcpy ((*n)->nombre, token);
  strcpy ((*n)->nombre_fte, token);
  (*n)->sig = NULL;
  deme_token (fp);
  if (es_error_esperaba (":", token, mens_error))
    return (0);
  deme_token (fp);
  error = 0;
  if (analice_prod_cjtos (tad_ed->nombre,fp, mens_error, &d, 1,&aparece))
    {
      if (aparece>0)
	{
	  sprintf(mens_error,mensajes[192],tad_ed->nombre,(*n)->nombre);
	  error=1;
	}
      else
	{
	  (*n)->dominio = d;
	  deme_token (fp);
	  if (strcmp (token, tad_ed->nombre))
	    {
	      error = 1;
	      strcpy (mens_error, mensajes[42]);
	    }
	}
    }
  else
    {
      error = 1;
    }
  return (short)(!error);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short analice_iniciales(FILE *fp,texpr mens_error)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Analizar sintacticamente la funcionalidad de todas las     *
*        iniciales de la signatura.                                   	  *
* DESCRIPCION: Analiza los siguientes tokens del archivo verificando que  *
*        describan nombres y funcionalidades de iniciales del tad_ed. 	  *
* PARAMETROS:  							          *	
*   FILE            *fp        	  Ap. al archivo que contiene la signatura*
*   texpr           mens_error    Comentario acerca de un eventual error. *
* V.GLOBALES:                                                             *
*   texpr  	    token      	  Contiene el ultimo token leido.         *
*   nodo_tad 	    *tad_ed    	  Ap. a la estructura que contiene la in- *
*				  formacion analizada hasta el momento de *
*				  la signatura contenida en el archivo    *
*				  apunt. por fp.		  	  *
* RESULTADO:  short               0, Si se detecto algun error. 1, Sino.  *
* PRE!:  {fp apunt. al archivo de axiomas.				} *
* POST!: {analice_iniciales = 0, Si la funcionalidad de las iniciales     *
*   	     tiene algun error sintactico y mens_error contiene una expli-*
*	     cacion acerca del error detectado, o                    	  *
*         analice_iniciales = 1, Sino. tad_ed contiene la informacion 	  *
*            de las iniciales analizadas.                               } *
* LLAMA A: void   deme_token      	                                  *
*          short  analice_inicial                                         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

short
analice_iniciales (FILE * fp, texpr mens_error)
{
/*----------------------------------------------------------------------*/

  short error;
  nodo_generadora *n=NULL,              /* v-tamara 28/May/1997 */
                  *ult_inic;
  assert(tad_ed!=NULL);
  tad_ed->iniciales = NULL;
  if (es_error_esperaba (claves[1], token, mens_error))
    return (0);
  deme_token (fp);
  ult_inic = NULL;
  error = 0;
  while (strcmp (token, claves[2]) && (!error))
    {
      if (analice_inicial (fp, mens_error, &n))
	{
	  if (tad_ed->iniciales == NULL)
	    tad_ed->iniciales = n;
	  else
	    ult_inic->sig = n;
	  ult_inic = n;
	  deme_token (fp);
	}
      else
	error = 1;
    }
  /*  VTP. Se quito la siguiente porcion de codigo que verificaba
      que el TAD tuviese iniciales
      if (!error && ult_inic == NULL)
      {
      error = 1;
      strcpy (mens_error, mensajes[106]);
      }
      else*/
  deme_token (fp);
  return (short)(!error);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short analice_nombre(FILE *fp,texpr mens_error)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Analizar sintacticamente la declaracion del nombre y de los*
*	 parametros del tad.                                              *
* DESCRIPCION: Verifica que los siguientes tokens del archivo correspondan*
*        a la declaracion del nombre y de los parametros del tad.         *
* PARAMETROS:                                                             *
*   FILE            *fp        	  Ap. al archivo que contiene la signatura*
*   texpr           mens_error    Comentario acerca de un eventual error. *
* V.GLOBALES:                                                             *
*   texpr           token  	  Contiene el ultimo token leido.         *
*   nodo_tad 	    *tad_ed    	  Ap. a la estructura que contiene la in- *
*				  formacion analizada hasta el momento de *
*				  la signatura contenida en el archivo    *
*				  apunt. por fp.		  	  *
* RESULTADO:  short               0, Si se detecto algun error. 1, Sino.  *
* PRE!:  {fp apunt. al archivo de axiomas.				} *
* POST!: {analice_nombres   = 0, Si la definicion del nombre y de los     *
*   	     parametros del tad tiene algun error sintactico, y mens_error*
*	     contiene una explicacion acerca del error detectado, o       *
*         analice_nombres   = 1, Sino. tad_ed contiene el nombre y los	  *
*            parametros analizados.                                     } *
* LLAMA A: void   deme_token      	                                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/


short
analice_nombre (FILE * fp, texpr mens_error)
{
/*----------------------------------------------------------------------*/
  nodo_nombre *n, *ult_param;
  cad12 tadname;

  if (es_error_esperaba (claves[0], token, mens_error))
    return (0);
  deme_token (fp);
  if (es_error_esperaba ("", token, mens_error))
    return (0);

  if (existsSymbol(symtable,token,tadname) && 
      strcmp(tadname,token)!=0)
    {
      sprintf(mens_error,mensajes[180],token,tadname);
      return 0;
    }
  if (existsSymbolParameter(symtable,token,tadname) && 
      strcmp(tadname,token)!=0)
    {
      sprintf(mens_error,mensajes[181],token,tadname);
      return 0;
    }
  if (strcmp(token,"EQ")==0)
    {
      sprintf(mens_error,mensajes[190]);
      return 0;
    }

  strcpy (tad_ed->nombre, token);
  tad_ed->parametros = NULL;
  deme_token (fp);
  if (es_error_esperaba ("[", token, mens_error))
    return (0);
  deme_token (fp);
  ult_param = NULL;
  while (strcmp (token, "]"))
    {
      if (es_error_esperaba ("", token, mens_error))
	return (0);
      /* VTP. 4/May/1998.  Good names */
      if (strcmp(token,tad_ed->nombre)==0)
	{
	  sprintf(mens_error,mensajes[180],token,tad_ed->nombre);
	  return 0;
	}
      if (existsName(tad_ed->parametros,token))
	{
	  sprintf(mens_error,mensajes[181],token,tad_ed->nombre);
	  return 0;
	}
      if (existsSymbol(symtable,token,tadname))
	{
	  sprintf(mens_error,mensajes[180],token,tadname);
	  return 0;
	}


      n = (nodo_nombre *) malloc (sizeof (nodo_nombre));
      HayMem(n,sizeof (nodo_nombre));            /* v-tamara 28/May/1997 */
      strcpy (n->nombre, token);
      n->sig = NULL;
      deme_token (fp);
      if (tad_ed->parametros == NULL)
	tad_ed->parametros = n;
      else
	ult_param->sig = n;
      ult_param = n;
      if (strcmp (token, "]"))
	{
	  if (es_error_esperaba (",", token, mens_error))
	    return (0);
	  deme_token (fp);
	}
    }
  deme_token (fp);
  /* VTP. 2/MAY/1998. Now the grammar don't supports = xxx in:
     ADT name [parameters] = xxx */
  return (1);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short analice_prod_cjtos(FILE *fp,texpr mens_error,nodo_nombre **d,     *
*			   short tad_ed_valido)				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Analizar un producto cruz de conjuntos.                    *
* DESCRIPCION: Analiza los siguientes tokens del archivo (hasta encontrar *
*        "->" lo que indica el fin del producto de conjuntos del dominio) *
*	 y verifica que todos ellos sean conjuntos conocidos.  		  *
*	 Esta funcion se usa para analizar el dominio de constructoras y  *
*	 selectoras.							  *
* PARAMETROS:  							          *
*   FILE            *fp        	  Ap. al archivo que contiene la signatura*
*   texpr           mens_error    Comentario acerca de un eventual error. *
*   nodo_nombre     **d		  Ap. a la lista que contiene el producto *
*				  de conjuntos analizados.         	  *
* V.GLOBALES: 								  *
*   texpr  	    token      	  Contiene el ultimo token leido.         *
*   nodo_tad 	    *tad_ed    	  Ap. a la estructura que contiene la in- *
*				  formacion analizada hasta el momento de *
*				  la signatura contenida en el archivo    *
*				  apunt. por fp.		  	  *
* RESULTADO:  short               0, Si se detecto algun error. 1, Sino.  *
* PRE!:  {fp apunt. al archivo de axiomas.				} *
* POST!: {analice_prod_cjtos   = 0, Si el producto de conjuntos contiene  *
*            alguno "desconocido", y mens_error contiene una explicacion  *
*	     acerca del error detectado, o                         	  *
*         analice_prod_cjtos   = 1, Sino. (*d) apunta a la lista conjuntos*
*	     que constituyen el producto cruz analizado.   		} *
* !OJO!: El separador 'x' o 'X' entre conjuntos es opcional.              *
* LLAMA A: void   deme_token      	 short  es_cjto_conocido          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
MODIFICACIONES: 
Retorna el numero de veces que nomtad aparece en el dominio, en caso
de que nomtad y aparece no sean NULL
**************************************************************************/

short
analice_prod_cjtos (char *nomtad,FILE * fp, texpr mens_error, nodo_nombre ** d,
		    short tad_ed_valido,int *aparece)
{
/*----------------------------------------------------------------------*/

  nodo_nombre *n;
  nodo_nombre *ult_cjto = NULL;         /* v-tamara 28/May/1997 */
  short error;
  int param,gen; 

  error = 0;
  (*d) = NULL;
  if (aparece!=NULL)
    {
      *aparece=0;
    }
  while ((strcmp (token, "->")) && (!error))
    {
      if (es_error_esperaba ("", token, mens_error))
	return (0);
      if (es_cjto_conocido (token, tad_ed_valido,&param,&gen))
	{
	  if (nomtad!=NULL && aparece!=NULL && strcmp(token,nomtad)==0)
	    {
	      (*aparece)++;
	    }
	  if (param && strcmp(token,tad_ed->nombre)!=0)
	    {
	      sprintf(mens_error,mensajes[182],token);
	      error=1;
	    }
	  else if (!gen && strcmp(token,tad_ed->nombre)!=0)
	    {
	      sprintf(mens_error,mensajes[184],token);
	      error=1;
	    }
	  else
	    {
	      n = (nodo_nombre *) malloc (sizeof (nodo_nombre));
	      HayMem(n,sizeof (nodo_nombre));               /* v-tamara 28/May/1997 */
	      strcpy (n->nombre, token);
	      n->sig = NULL;
	      if ((*d) == NULL)
		(*d) = n;
	      else
		ult_cjto->sig = n
		  ;
	      ult_cjto = n;
	      deme_token (fp);
	      if (strcmp (token, "->"))
		if (!(error = es_error_esperaba ("*", token, mens_error)))
		  deme_token (fp);
	    }
	}
      else
	{
	  error = 1;
	  sprintf (mens_error, mensajes[29], token);
	}
    }
  return (short)(!error);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short analice_selectora(FILE *fp, texpr mens_error,nodo_select **n)	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Analizar sintacticamente la funcionalidad de una selectora.*
* DESCRIPCION: Analiza los siguientes tokens del archivo verificando que  *
*        describan la funcionalidad de una selectora del tad_ed.       	  *
* PARAMETROS:  							          *
*   FILE            *fp        	  Ap. al archivo que contiene la signatura*
*   texpr           mens_error    Comentario acerca de un eventual error. *
*   nodo_select     **n		  Ap. a la lista que contiene el dominio  *
*				  de la selectora analizada.		  *
* V.GLOBALES: 								  *
*   texpr  	    token      	  Contiene el ultimo token leido.         *
*   nodo_tad 	    *tad_ed    	  Ap. a la estructura que contiene la in- *
*				  formacion analizada hasta el momento de *
*				  la signatura contenida en el archivo    *
*				  apunt. por fp.		  	  *
* RESULTADO:  short               0, Si se detecto algun error. 1, Sino.  *
* PRE!:  {fp apunt. al archivo de axiomas.				} *
* POST!: {analice_selectora = 0, Si la funcionalidad de la selectora	  *
*            tiene algun error sintactico y mens_error contiene una expli-*
*            cacion acerca del error detectado, o                         *
*         analice_selectora = 1, Sino. (*n) apunta a la lista que contiene*
*	  el dominio de la selectora analizada.				} *
* LLAMA A: void   deme_token      	 short  es_cjto_conocido          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

short
analice_selectora (FILE * fp, texpr mens_error, nodo_select ** n)
{
/*----------------------------------------------------------------------*/

  short error;
/*** Alvaro
    nodo_nombre	*c;
 **/
  nodo_dom *d, *ult_dom;
  cad12 tadname,eqstr;
  int param,gen;

  error = 0;
  if (es_error_esperaba ("", token, mens_error))
    return (0);

  /* VTP. 4/May/1998 Good names */
  if (existsGenerator(tad_ed->iniciales,token))
    {
      sprintf(mens_error,mensajes[180],token,tad_ed->nombre);
      return 0;
    }
  if (existsGenerator(tad_ed->constructoras,token))
    {
      sprintf(mens_error,mensajes[180],token,tad_ed->nombre);
      return 0;
    }
  if (existsSelector(tad_ed->selectoras,token))
    {
      sprintf(mens_error,mensajes[180],token,tad_ed->nombre);
      return 0;
    }
  if (existsSymbol(symtable,token,tadname) &&
      strcmp(tad_ed->nombre,tadname)!=0)
    {
      sprintf(mens_error,mensajes[180],token,tadname);
      return 0;
    }
  if (existsName(tad_ed->parametros,token))
    {
      sprintf(mens_error,mensajes[181],token,tad_ed->nombre);
      return 0;
    }
  if (strncmp(token,"EQ_",3)==0)
    { /* Solo puede llamarse EQ_X donde X es el nombre del TAD */
      sprintf(eqstr,"EQ_%s",tad_ed->nombre);
      if (strcmp(token,eqstr)!=0)
	{ 
	  sprintf(mens_error,mensajes[190],token);
	  return 0;
	}
    }
      
  (*n) = (nodo_select *) malloc (sizeof (nodo_select));
  HayMem(*n,sizeof (nodo_select));              /* v-tamara 28/May/1997 */
  strcpy ((*n)->nombre, token);
  strcpy ((*n)->nombre_fte, token);
  (*n)->sig = NULL;
  (*n)->axiomas = NULL;
  deme_token (fp);
  if (!strcmp (token, "("))
    {
      deme_token (fp);
      if (es_error_esperaba (claves[13], token, mens_error))
	return (0);
      (*n)->caracterizadora = 1;
      deme_token (fp);
      error = es_error_esperaba (")", token, mens_error);
      if (error)
	return (0);
      deme_token (fp);
    }
  else
    {
      (*n)->caracterizadora = 0;
    }
  if (es_error_esperaba (":", token, mens_error))
    return (0);
  deme_token (fp);
  (*n)->dominio = NULL;
  ult_dom = NULL;
  while (strcmp (token, "->") && (!error))
    {
      if (es_error_esperaba ("", token, mens_error))
	return (0);
      if (es_cjto_conocido (token, 1,&param,&gen))
	{
	  if (param && strcmp(token,tad_ed->nombre)!=0)
	    {
	      sprintf(mens_error,mensajes[182],token);
	      return 0;
	    }
	  else if (!gen)
	    {
	      sprintf(mens_error,mensajes[184],token);
	      return 0;
	    }
	  else
	    {
	      d = (nodo_dom *) malloc (sizeof (nodo_dom));
	      HayMem(d,sizeof (nodo_dom));  /* v-tamara 28/May/1997 */
	      strcpy (d->nombre, token);
	      d->sig = NULL;
	      if ((*n)->dominio == NULL)
		(*n)->dominio = d;
	      else
		ult_dom->sig = d;
	      ult_dom = d;
	      deme_token (fp);
	      if (!strcmp (token, "("))
		{
		  deme_token (fp);
		  if (es_error_esperaba (claves[14], token, mens_error))
		    return (0);
		  d->induccion = 1;
		  deme_token (fp);
		  error = es_error_esperaba (")", token, mens_error);
		  if (error)
		    return (0);
		  deme_token (fp);
		}
	      else
		{
		  d->induccion = 0;
		}
	      if (strcmp (token, "->"))
		{
		  if (es_error_esperaba ("*", token, mens_error))
		    return (0);
		  else
		    deme_token (fp);
		}
	    }
	}
      else
	{
	  error = 1;
	  sprintf (mens_error, mensajes[29], token);
	}
    }
  if (error)
    return (0);
  deme_token (fp);
  if (es_error_esperaba ("", token, mens_error))
    return (0);
  if (es_cjto_conocido (token, 1,&param,&gen))
    {
      if (param && strcmp(token,tad_ed->nombre)!=0)
	{
	  sprintf(mens_error,mensajes[182],token);
	  return 0;
	}
      else if (!gen)
	{
	  sprintf(mens_error,mensajes[184],token);
	  return 0;
	}
      else
	{
	  strcpy ((*n)->codominio, token);
	}
    }
  else
    {
      error = 1;
      sprintf (mens_error, mensajes[31], token);
    }
  return ((char)!error);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short analice_selectoras(FILE *fp,texpr mens_error, int todo)
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Analizar sintacticamente la funcionalidad de todas las     *
*        selectoras de la signatura.                                   	  *
* DESCRIPCION: Analiza los siguientes tokens del archivo verificando que  *
*        describan nombres y funcionalidades de selectoras del tad_ed. 	  *
* PARAMETROS:  							          *
*   FILE            *fp        	  Ap. al archivo que contiene la signatura*
*   texpr           mens_error    Comentario acerca de un eventual error. *
* V.GLOBALES:                                                             *
*   texpr  	    token      	  Contiene el ultimo token leido.         *
*   nodo_tad 	    *tad_ed    	  Ap. a la estructura que contiene la in- *
*				  formacion analizada hasta el momento de *
*				  la signatura contenida en el archivo    *
*				  apunt. por fp.		  	  *
* RESULTADO:  short               0, Si se detecto algun error. 1, Sino.  *
* PRE!:  {fp apunt. al archivo de axiomas.				} *
* POST!: {analice_selectoras = 0, Si la funcionalidad de las selectoras   *
*   	     tiene algun error sintactico y mens_error contiene una 	  *
*	     explicacion acerca del error detectado, o                    *
*         analice_selectoras = 1, Sino. tad_ed contiene la informacion 	  *
*            de las selectoras analizadas.                           	} *
* LLAMA A: void   deme_token      	                                  *
*          short  analice_selectora                                       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
* MODIFICACIONES:
* VTP. 1/Jul/1997 Se agrego parametro todo para indicar que se esta
*      analizando un archivo con la definicion completa de un TAD (incluye
*      axiomas)
**************************************************************************/

short
analice_selectoras (FILE * fp, texpr mens_error, int todo)
{ 
/*----------------------------------------------------------------------*/

  short error;
  nodo_select *n=NULL,          /* v-tamara 28/May/1997 */
              *ult_select,
              *eq;
  cad12 eqstr;

  assert(tad_ed!=NULL);         /* v-tamara 28/May/1997 */
  error = 0;
  tad_ed->selectoras = NULL;
  ult_select = NULL;
  /* VTP. 1/Jul/1997. Agregado para que este analizador tambien
                      sirva para revisar un archivo con una def. que incluya
                      axiomas. */
  while (!((todo && !strcmp (token, claves[4]))
           ||(!todo && feof(fp))
           || error))
/* La condicion original era ((!feof (fp)) && (!error)) */
    {
      if (analice_selectora (fp, mens_error, &n))
	{
	  if (tad_ed->selectoras == NULL)
	    tad_ed->selectoras = n;
	  else
	    ult_select->sig = n;
	  ult_select = n;
	  deme_token (fp);
	}
      else
	{
	  error = 1;
	}
    }
  /* VTP. 5/May/1998 If an ADT has generators it has to have 
    equality */      
  sprintf(eqstr,"EQ_%s",tad_ed->nombre);
  for (eq=tad_ed->selectoras;
       eq!=NULL && strcmp(eq->nombre,eqstr)!=0;
       eq=eq->sig);
  if (!error && (tad_ed->iniciales!=NULL || tad_ed->constructoras!=NULL))
    {
      if (eq==NULL)
	{
	  sprintf(mens_error,mensajes[185],eqstr,tad_ed->nombre);
	  error=1;
	  goto exit;
	}
      if (strcmp(eq->codominio,"BOOL")!=0 ||
	  eq->dominio==NULL ||
	  strcmp(eq->dominio->nombre,tad_ed->nombre)!=0 ||
	  eq->dominio->sig==NULL ||
	  strcmp(eq->dominio->sig->nombre,tad_ed->nombre)!=0)
	{
	  sprintf(mens_error,mensajes[186],
		  eqstr,tad_ed->nombre,tad_ed->nombre);
	  error=1;
	  goto exit;
	}
      eq->caracterizadora=1;
      eq->dominio->induccion=1;
      eq->dominio->sig->induccion=1;
    }
  else if (!error && eq!=NULL)
    {
      /* If type doesn't have generators, it can't have equality */
      sprintf(mens_error,mensajes[187],eqstr);
      error=1;
      goto exit;
    }

exit:
  return (short)(!error);
/*----------------------------------------------------------------------*/
}


/**************************************************************************
* short analizador_sintactico(texpr mens_error)				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Analizar sintacticamente la signatura de un tad almacenada *
*	 en el archivo "signa_ed.tad".                                *
* DESCRIPCION: Verifica que los tokens del archivo correspondan a la defi-*
*        nicion de la signatura de un tad. 				  *
*	 Se lee la informacion del archivo y se va analizando sintactica- *
*	 mente. Mientras que se correcta se va almacenando en una estruc- *
*    	 tura de tipo nodo_tad apuntada por tad_ed.			  *
* PARAMETROS:                                                             *
*   texpr           mens_error    Comentario acerca de un eventual error. *
* V.GLOBALES:                                                             *
*   texpr           token  	  Contiene el ultimo token leido.         *
*   nodo_tad 	    *tad_ed    	  Ap. a la estructura que contiene la in- *
*				  formacion analizada hasta el momento de *
*				  la signatura contenida en el archivo.   *
* RESULTADO:  short               0, Si se detecto algun error. 1, Sino.  *
* PRE!:  {fp apunt. al archivo de axiomas.				} *
* POST!: {analizador_sintactico = 0, Si la definicion de la signatura  	  *
*   	     del tad tiene algun error sintactico, y mens_error contiene  *
*	     una explicacion acerca del error detectado, o       	  *
*         analizador_sintactico = 1, Sino. tad_ed contiene la informacion *
*            de la signatura leida del archivo.                         } *
* LLAMA A: short  analice_nombre 	 short   analice_iniciales        *
*          short  constructoras          short   selectoras               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

short
analizador_sintactico (texpr mens_error)
{
/*----------------------------------------------------------------------*/

  FILE *fp;
  short correcto;

  num_ed = 1;
  correcto = 0;
  strcpy (mens_error, "");
  fp = fopen ("signa_ed.tad", "r");
  FileOpen(fp,"signa_ed.tad");     /* v-tamara 28/May/1997 */
  deme_token (fp);
  if (analice_nombre (fp, mens_error))
    {
      if (analice_iniciales (fp, mens_error))
	{
	  if (analice_constructoras (fp, mens_error))
	    {
	      if (analice_selectoras (fp, mens_error,0))
		{
		  correcto = 1;
		}
	    }
	}
    }
  fclose (fp);
  return (correcto);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void deme_token(FILE *fp)  						  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Leer el siguiente token del archivo apuntado por fp.       *
* DESCRIPCION: Lee los siguientes caracteres del archivo hasta completar  *
*        un token "admisible" descartando los separadores.                *
* PARAMETROS:  							          *	
*   FILE            *fp        	  Ap. al archivo que contiene la signatura*
* V.GLOBALES:                                                             *
*   texpr  	    token      	  Contiene el ultimo token leido.         *
* PRE!:  {fp apunt. al archivo de tokens AT.				} *
* POST!: {token = TK ultimo token leido del archivo AT                  } *
* LLAMA A: short  es_separador           short   es_puntuacion            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
deme_token (FILE * fp)
{
/*----------------------------------------------------------------------*/

  int c;
/*** VTP. 7/Mar/1997 Era tipo char, pero se usaba como int */
  int i;
  int terminar;
  int posibleComentario;

  strcpy (token, "");
  if (feof (fp))
    return;
  while (((c = getc (fp)) != EOF) && (es_separador ((char)c)))
    if (c == '\n')
      num_ed++;
  if (c == EOF)
    return;
  /* Capturar todos los comentarios que haya */
  posibleComentario=1;
  while (posibleComentario && c=='/')
    {
      /*-- Analiza si es un comentario  */
      c = getc (fp);
      if (c == '\n')
	num_ed++;
      if (c == '*')
	{
	  c = getc (fp);
	  for (terminar = 0; !terminar && c != EOF;)
	    {
	      if (c == '\n')
		num_ed++;
	      if (c == '*')
		{
		  c = getc (fp);
		  if (c == '\n')
		    num_ed++;
		  if (c != EOF && c == '/')
		    {
		      terminar = 1;
		      c = getc (fp);
		    }
		}
	      else
		c = getc (fp);
	    }
	}
      else
	{
	  posibleComentario=0;
	  ungetc (c, fp);
	}
      while ((c != EOF) && (es_separador ((char)c)))
	{
	  if (c == '\n')
	    num_ed++;
	  c = getc (fp);
	}
      if (c == EOF)
	return;
    } 
  i = 0;
  token[i++] =(char) c;
  if (isalpha (c))
    {
      c = getc (fp);
      if (c == '\n')
	num_ed++;
      while ((c != EOF) && (isalnum (c) || c == '_'))
	{
	  token[i++] = (char)c;
	  c = getc (fp);
	}
      ungetc (c, fp);
    }
  else if (c == '-')
    {
      c = getc (fp);
      if (c == '>')
	token[i++] = (char)c;
      else
	ungetc (c, fp);
    }
  token[i] = '\0';
/*----------------------------------------------------------------------*/
}

/**************************************************************************
es_cjto_conocido (texpr cjto, short tad_ed_valido,
		  int *parametros,int *generadoras)
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si <cjto> es un conjunto conocido.              *
* DESCRIPCION: Determina si <cjto> es el nombre del tad_ed (en caso de que*
*	 tad_ed_valido <> 0) o el nombre de uno de los tads existentes en *
*	 el ambiente o en menoria secundaria.                  		  *
* PARAMETROS:                                                             *
*   texpr           cjto          Nombre del "supuesto" conjunto.         *
*   short	    tad_ed_valido 1, Si <cjto> puede ser el nombre del    *
*				     tad_ed.				  *
*				  0, Sino.				  *
    int *          parametros     Si es no nulo retorna 0 sii el tipo buscado
                                  no tiene parametros
    int *          generadoras    Si es no nulo retorna 0 sii el tipo buscado
                                  no tiene generadoras

* V.GLOBALES:                                                             *
*   texpr  	    token      	  Contiene el ultimo token leido.         *
*   nodo_tad 	    *tad_ed    	  Ap. a la estructura que contiene la in- *
*				  formacion analizada hasta el momento de *
*				  la signatura contenida en el archivo    *
*				  apunt. por fp.		  	  *
* RESULTADO:  short               1, Si <cjto> es un conjunto conocido.   *
*                                 0, Sino.                                *
* PRE!:  { cjto = CT, tad_ed_valido = TEV = 0/1.			  *
*          tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2,					  *
*	   D_ADT = <a1.adt,...,am.adt>, m >= 0		        	} *
* POST!: { es_cjto_conocido = 1 , Si CT = Nombre del tad_ed y TEV = 1, o  *
*                                    CT = tk.nombre para algun 1<=k<=n, o *
*                                    CT = ak para algun 1<=k<=m.     	  *
*          es_cjto_conocido = 0 , Sino                                  } *
* LLAMA A: short  es_parametro                                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
MODIFICADO POR: Vladimir Tamara 
5/May/1998.  Tambien retorna en un flag si el tipo tiene parametros o no
	     Retorna otro flag indicando si tiene generadoras o no
	     Flag que venga en NULL, flag que no se llena
**************************************************************************/

short
es_cjto_conocido (texpr cjto, short tad_ed_valido,
		  int *parametros,int *generadoras)
{
/*----------------------------------------------------------------------*/

  nodo_tad	*p;

  if (es_parametro(token,tad_ed->parametros))
    {
      if (parametros!=NULL)
	{
	  *parametros=0;
	}
      if (generadoras!=NULL)
	{
	  *generadoras=1;
	}
      return 1;
    }
  if (tad_ed_valido && !strcmp (cjto, tad_ed->nombre))
    {
      p=tad_ed;
    }
  else 
    {
      p=existe_tad (cjto);
    }
  if (parametros!=NULL)
    {
      *parametros=(p!=NULL && p->parametros!=NULL);
    }
  if (generadoras!=NULL)
    {
      *generadoras=(p!=NULL && 
		    (p->iniciales!=NULL || p->constructoras!=NULL));
    }
  return (short)(p!=NULL);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short es_identificador(texpr token)                           	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determina si token es un token de identificador.       	  *
* RESULTADO:  short  1, Si token empieza por letra y continua con         *
*             caracteres alfanumerico o '_'. Y ademas no es ninguna       *
*             palabra clave.                                              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Cesar Munoz                                     FECHA: 11-Mar-92 *
**************************************************************************/

short
es_identificador (texpr token)
{
/*----------------------------------------------------------------------*/
  short es_ident;
  int i;
  int c;
/*** VTP. 7/Mar/1997. Era un char, pero se emplea como int */

  es_ident = 1;
  i = 0;
  c = token[i];
  if (!isalpha (c))
    es_ident = 0;
  for (i++, c = token[i]; es_ident && c != '\0'; i++, c = token[i])
    if (!isalnum (c) && (c != '_'))
      es_ident = 0;
  for (i = 0; es_ident && i < MAX_CLAVES; i++)
    {
      if (!strcmp (claves[i], token))
	es_ident = 0;
    }
  return (es_ident);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short es_error_esperaba(texpr cad1, texpr cad2, texpr mens_error)       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determina si cad1 == cad2. Si no imprime en mens_error se  *
*              copia el mensaje de error.                                 *
*              cad1 es la cadena esperada.(si cad1 == "", espera un       *
*                                          idenficador )                  *
*              cad2 es la cadena recibida.                                *
* RESULTADO:  es_error_esperaba == 1, si hubo error. mens_error tiene el  *
*                                     mensaje del error.                  *
*             es_error_esperaba == 0, si no.                              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Cesar Munoz                                     FECHA: 11-Mar-92 *
**************************************************************************/

short
es_error_esperaba (texpr cad1, texpr cad2, texpr mens_error)
{
/*----------------------------------------------------------------------*/
  short error;

  error = 0;
  if (!strcmp (cad2, "") && !strcmp (cad1, ""))
    {
      error = 1;
      sprintf (mens_error, mensajes[114], mensajes[28]);
    }
  else if (!strcmp (cad2, "") && strcmp (cad1, ""))
    {
      error = 1;
      sprintf (mens_error, mensajes[114], cad1);
    }
  else if (!strcmp (cad1, ""))
    {
      if (!es_identificador (cad2))
	{
	  error = 1;
	  sprintf (mens_error, mensajes[36], mensajes[28], cad2);
	}
    }
  else if (strcmp (cad1, cad2))
    {
      error = 1;
      sprintf (mens_error, mensajes[36], cad1, cad2);
    }
  return (error);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short es_separador(char c)						  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si c es un separador.				  *
* DESCRIPCION: Determina si c es uno de los simbolos de separacion que    *
* 	 pueden aparecer en el archivo de tokens.			  *
* PARAMETROS: char   c              Caracter que se quiere saber si es o  *
*                                   no un simbolo de separacion.          *
* RESULTADO:  short                 1, Si c es uno de los posibles simbo- *
*                                      los de separacion.                 *
*                                   0, Sino.                              *
* PRE!:  { c = C                                                        } *
* POST!: { es_separador  = 1, Si c pertenece al conjunto:		  *
*	      { ' ','\n','\t'} U         				  *
*	      { y | y es un caracter no imprimible.} 			} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

short
es_separador (char c)
{
/*----------------------------------------------------------------------*/

  if ((c == ' ') || (c == '\n') || (c == '\t') ||
      (!isprint ((int) c)))
    return (1);
  else
    return (0)
      ;
/*----------------------------------------------------------------------*/
}
