

/* AXIOMAS.C                                                    */
/* Arhivo de manejo de axiomas                                  */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h> 
#include "os_dep.h"		/* Agregada por VTP. 7/Mar/97 */
#include "definic.h"
#include "apimanta.h"
#include <assert.h>		/* VTP. 7/May/1997 */

/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* v-tamara 7/May/1997 */


/**************************************************************************
* void arregle_dom_inicial(nodo_generadora *inic,nodo_argum *arg,  	  *
*			   nodo_expresion *cab_ld_ax,char componente) 	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Construir los argumentos de una inicial de un producto Car-*
*	       tesiano a partir de los argumentos de una componente de la *
*	       inicial.							  *
* DESCRIPCION: Reemplaza en el arbol sintactico de una expresion los argu-*
*        mentos de una inicial INI de un tad Ti por los args. de una ini- *
*	 cial del producto Cartesiano explicito (de Ti con otro tad Tj)   *
*	 que tiene como componente a INI. Ademas, hace el mismo reeplazo  *
*	 en todas las ocurrencias de la inicial en la expresion apunt. por*
*	 <cab_ld_ax>. 							  *
* PARAMETROS:                                                             *
*   nodo_generadora	*inic		Ap. a la inicial compuesta.	  *
*   nodo_argum 		*arg  		Ap. a los args. de la inicial que *
*					deben ser reemplazados.		  *
*   nodo_expresion 	*cab_ld_ax 	Ap. al lado derecho del axioma en *
*					transformacion.			  *
*   char	    	componente	'1' Si i = 1			  *
*    					'2' Si i = 2			  *
* PRE!:  { inic ap. a una inicial compuesta <INIC1>-<INIC2>,              *
*	   componente = i , 1 <= i <= 2					  *
*	   arg ap. a una estructura con los argumentos de <INICi>	  *
*	   cab_ld_ax ap. a una expresion EXP en la que posiblemente apare *
*	   ce <INICi>.							} *
* POST!: { arg. ap. a una estructura con los argumentos de inic.          *
*          cab_ld_ax ap. REEMP(EXP) que es una expresion igual a EXP excep*
*	   que la ocurrencias de <INICi> y sus argumentos han sido reempla*
*	   zadas por inic y los argumentos de inic.          		} *
* LLAMA A:   void construya_dom_generad					  *
*	void renombre_var_inic		void reemplace_dom_inicial	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
arregle_dom_inicial (nodo_generadora * inic, nodo_argum * arg,
		     nodo_expresion * cab_ld_ax, char componente)
{
/*----------------------------------------------------------------------*/

  nodo_argum *cab_dom = NULL;
  char cont;

  assert (arg != NULL && inic != NULL);		/* VTP. 7/May/1997 */
  cont = '1';
  construya_dom_generad (inic, &cab_dom, &cont);
  if (cab_dom != NULL)
    {
      renombre_var_inic (arg->expr->argumentos, cab_dom, cab_ld_ax, componente);
      arg->expr->argumentos = cab_dom;
      reemplace_dom_inicial (inic->nombre, cab_ld_ax, cab_dom);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short busque_li_axioma(texpr li_axioma,nodo_igualdad *cabeza,		  *
*		   	 texpr ld_axioma)				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Buscar <li_axiomas> como lado izq. de los axiomas de la    *
*              apuntada por cabeza.                                       *
* DESCRIPCION: Construye el arbol sintactico de la expresion almacenada en*
*        <li_axioma> y busca una expresion isomorfa entre los lados izqs. *
*        de la lista de axiomas apuntados por <cabeza>. Si lo encuentra,re*
*        torna en <ld_axioma> la expresion del lado derecho correspondient*
* PARAMETROS:                                                             *
*   texpr		li_axioma	Cadena que contiene la expresion  *
*					que se va a buscar.		  *
*   nodo_igualdad	*cabeza		Ap. a la lista de axiomas.        *
*   texpr		ld_axioma	Cadena que contiene la expresion  *
*					del lado der. correspondiente.	  *
* RESULTADO:  short	1,Si se encontro un lado izq. isomorfo a <li_axio>*
*                   	0,Sino.						  *
* PRE!:  { li_axioma = EXP-I , cabeza -> <a1,...,an>, n >= 0            } *
* POST!: { busque_li_axioma = 1, Si existe un i, 1 <= i <= n  tq.         *
*                  EXP-I es isomorfa (esto es, igual savo renombramiento  *
*                  de variables) a  ai.lado_izq y ld_axioma = ai.ld_str   *
*          busque_li_axioma = 0, Sino                                   } *
* LLAMA A: 								  *
*	void	construya_subestruct 	void copie_lista_expresion        *
*	short expr_isomorfas 		void construya_exp
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

short
busque_li_axioma (texpr li_axioma, nodo_igualdad * cabeza, texpr ld_axioma)
{
/*----------------------------------------------------------------------*/

  short encontro;
  nodo_expresion *exp = NULL, *expr_der = NULL;

  assert (cabeza != NULL);	/* VTP. 7/May/1997 */
  construya_subestruct (li_axioma, &exp);
  for (encontro = 0; (cabeza != NULL) && (!encontro); cabeza = cabeza->sig)
    {
      copie_lista_expresion (cabeza->lado_der, &expr_der, "", NULL,"");
      if (expr_isomorfas (exp, cabeza->lado_izq, expr_der))
	{
	  encontro = 1;
	  construya_exp (expr_der, ld_axioma,"");
	}
    }
  return (encontro);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short cargue_axiomas(nodo_select *s)					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Leer, analizar y almacenar los axiomas de s.               *
* DESCRIPCION: Lee los tokens del archivo "axiom_ed.tad" que deben corres*
*        ponder a axiomas (bien formados) de la selectora s. Si encuentra *
*        errores sintacticos o semanticos (esto es, que los axiomas no    *
*        sean totales), informa al usuario el error y permite que edite   *
*        nuevamente el archivo que contiene los axiomas.                  *
* PARAMETROS:                                                             *
*   nodo_select	*s			Ap. a la selectora cuyos axiomas  *
*					son analizados.  		  *
* V.GLOBALES:                                                             *
*   short	desea_ayuda_tuplas_comp	1, Si hay un error semantico y el *
*					   usuario desea ayuda en la gene-*
*					   cion de lados izqs.		  *
*					0, Sino.			  *
* RESULTADO:  short	1, Si los axiomas leidos son correctos sintactica *
*                          y semanticamente.                              *
*			0, Sino.					  *
* PRE!:  { "axiom_ed.tad" = [a1,...,an], n >= 0.                       } *
* POST!: { cargue_axiomas = 1, Si a1,...,an son un conjunto total de axio-*
*		mas bien formados para la selectora s			  *
*          cargue_axiomas = 0, Sino.                                    } *
* LLAMA A: 								  *
*   short  analice_axiomas 		short desea_corregir_edicion	  *
*   void   ayuda_tuplas_complet_comp 			                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

short
cargue_axiomas (nodo_select * s)
{
  /*----------------------------------------------------------------------*/
  
  short correcto, tipo_error;
  nodo_igualdad *cab_axiomas = NULL;
  texpr mens_error;
  int corregir; 

  /*** Alvaro
		comando;
   ***/
  FILE *fp;
  /*** Alvaro
  cad6          resp;
  ***/
  trio_edicion edicion[MAX_EDICION];

  assert (s != NULL);		/* VTP. 7/May/1997 */
  
  correcto=analice_axiomas (mens_error, &cab_axiomas, 
			    s->nombre, &tipo_error);
  corregir=0;
  if (!correcto)
    {
      corregir=desea_corregir_edicion(tipo_error,mens_error);
    }
  while (corregir)
    {
      edicion[0].modo = 0;
      edicion[0].linea = 1;
      strcpy (edicion[0].nombre, "signa_ed.tad");
      edicion[1].modo = 1;
      edicion[1].linea = num_ed;
      strcpy (edicion[1].nombre, "axiom_ed.tad");
      if (tipo_error!=2 && desea_ayuda_tuplas_comp)
	{			/* Tuplas no cumplen definicion completa */
          fp = fopen ("axiom_ed.tad", "a");
          FileOpen (fp, "axioma_ed.tad");
          ayuda_tuplas_complet_comp (fp, s);
          fclose (fp);
        }
      llamar_al_editor (2, edicion, "temp");
      correcto=analice_axiomas (mens_error, &cab_axiomas, 
				s->nombre, &tipo_error);
      corregir=0;
      if (!correcto)
	{
	  corregir=desea_corregir_edicion(tipo_error,mens_error);
	}
    }
  if (correcto)
    {
      s->axiomas = cab_axiomas;
    }
  return (correcto);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void editar_2w(char f_nombre[], char nombre_sel[])                      *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:  Crear un archivo con un programa para invocar emacs, de tal *
*             forma que se crean dos ventanas de edicion.                 *
*             En una de las cuales se carga el archivo f_nombre, y se     *
*             situa sobre la palabra nombre_sel.                          *
**************************************************************************/

void
editar_2w (char f_nombre[], char nombre_sel[])
{
/*----------------------------------------------------------------------*/
  texpr nombre;
  FILE *f_emacs;

  assert (nombre_sel != NULL && f_nombre != NULL);	/* VTP. 7/May/1997 */
  strcpy (nombre, "temp");
  f_emacs = fopen (nombre, "w");
  FileOpen (f_emacs, nombre);
  fprintf (f_emacs, "(split-window-vertically)\n");
  fprintf (f_emacs, "(find-file \"%s\")\n", f_nombre);
  if (strcmp (nombre_sel, ""))
    fprintf (f_emacs, "(search-forward \"%s\")\n", nombre_sel);
  fprintf (f_emacs, "(other-window 1)\n");
  fclose (f_emacs);
/*----------------------------------------------------------------------*/
}


/**************************************************************************
* short construya_axiomas(nodo_select *s,short restriccion,		  *
*			  cad50 pred_restr,cad12 nombre_tad_r, FILE *fp2)
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Construir los axiomas de la selectora s.                   *
* DESCRIPCION: Genera automaticamente los lados izqs. de los axiomas de s *
*        usando lema de forma normal en todos los argumentos del TDI que  *
*	 sean de induccion, pide que al usuario los lados ders. e invoca a*
*	 la funcion que se encarga de "cargarlos" del archivo. 		  *
* PARAMETROS:                                                             *
*   nodo_select *s		Ap. a la selectora a la cual se le constru*
*				yen los axiomas.			  *
*   short 	restriccion	1,Si el tad editado es de tipo RESTRICCION*
*				0,Sino.					  *
*   cad50 	pred_restr      Nombre del predicado de restriccion, Si el*
*				tad es de tipo RESTRICCION.		  *
*   cad12 	nombre_tad_r    Nombre del tad padre, Si el tad es de tipo*
*				RESTRICCION.				  *
* RESULTADO:  short		1, Si la construccion de los axiomas de s *
*				   fue exitosa.				  *
*				0, Sino.				  *
* PRE!:  { restriccion = R, pred_restr = P, nombre_tad_r = N            } *
* POST!: { construya_axiomas = 1, Si s.axiomas = <a1,..,an> axiomas	  *
*				  correctos				  *
*          construya_axiomas = 0, Sino.                                 } *
* MODIFICACIONES:
* VTP 7/Mar/97
	Antes borraba un archivo empleando system("rm ... ahora
	se usa unlink (portable para DOS y UNIX)
VTP. 6/May/1998
        Last parameter added to avoid user interaction and to write
	in open file fp2 (if is not NULL)
**************************************************************************/


short
construya_axiomas (nodo_select * s, short restriccion, cad50 pred_restr,
		   cad12 nombre_tad_r, FILE *fp2)
{
/*----------------------------------------------------------------------*/

  short correcto;
  FILE *fp;
  texpr li_axioma, li_aux, nd, comando;
  int i;
/*** Alvaro
    cad6          resp;
  ***/
  trio_edicion edicion[MAX_EDICION];

  assert (s != NULL && pred_restr != NULL && nombre_tad_r != NULL);	/* VTP. 7/May/1997 */
    /*-- Creara un archivo con un programa Lisp-emacs que abre dos ventanas,
      en una de las cuales carga la signatura. En el archivo "temp" queda
      un programa emacs que despliega "signature..." en una ventana --*/
  correcto=1;
  if (fp2==NULL)
    {
      esc_arch_edicion_signatura (tad_ed);
      editar_2w ("signa_ed.tad", s->nombre);
      fp = fopen ("axiom_ed.tad", "w");
      FileOpen (fp, "axiom_ed.tad");
      sprintf (comando, mensajes[26], s->nombre);
      fprintf (fp, "\n/* %s */\n\n", comando);
    }
  else
    {
      fp=fp2;
    }
  /* Selectors without parameters don't need ( */
  if (s->dominio==NULL)
     { li_axioma[0]='\0'; }
  else 
    { strcpy (li_axioma, "("); }
  if (restriccion)
    {
      strcpy (nd, s->nombre);
      for (i = strlen (nd) - 1; i > 0 && nd[i] != '_'; i--);
      nd[i] = '\0';
      genere_axiomas_restr (fp, s->nombre, nd, s->dominio, li_axioma, "",
			    pred_restr,
			    nombre_tad_r, '1', s->axiomas);
    }
  else
    {
      sprintf (li_aux, "%s%s", s->nombre, li_axioma);
      strcpy (li_axioma, li_aux);
      genere_axiomas (fp, s->dominio, li_axioma, '1', s->axiomas);
    }
  if (fp2==NULL)
    {
      fclose (fp);
      edicion[0].modo = 0;
      edicion[0].linea = 1;
      strcpy (edicion[0].nombre, "signa_ed.tad");
      edicion[1].modo = 1;
      edicion[1].linea = 4;
      strcpy (edicion[1].nombre, "axiom_ed.tad");
      llamar_al_editor (2, edicion, "temp");
      correcto = cargue_axiomas (s);
      /***VTP. 7/Mar/1997 Decia: system("rm -f axiom_ed.tad");  ***/
      unlink ("axiom_ed.tad");
      /***VTP. 7/Mar/1997 Decia: system("rm -f temp");  ***/
      unlink ("temp");
    }
  return (correcto);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short construya_axs_formas_normales(nodo_tad *tad_generad,		  *
*				      nodo_select *s)		  	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Construir los axiomas de la selectora s.                   *
* DESCRIPCION: Genera automaticamente los lados izqs. de los axiomas de s *
*        usando en lema de forma normal en solo el primer argumento del   *
*	 TDI que sea de induccion, pide que al usuario los lados ders. e  *
*	 invoca a la funcion que se encarga de "cargarlos" del archivo.   *
* PARAMETROS:                                                             *
*   nodo_tad	*tad_generad    Ap. al tad que sera usado para generar las*
*				formas normales.			  *
*   nodo_select *s		Ap. a la selectora a la cual se le constru*
*				yen los axiomas.			  *
* RESULTADO:  short		1, Si la construccion de los axiomas de s *
*				   fue exitosa.				  *
*				0, Sino.				  *
* PRE!:  { tad_generad-> T, s-> S.                                      } *
* POST!: { construya_axs_formas_normales = 1, Si S.axiomas = <a1,..,an>   *
*					      axiomas correctos	 	  *
*          construya_axs_formas_normales = 0, Sino.                     } *
* LLAMA A:	void	genere_plantilla_selectora		 	  *
*	void 	genere_axs_tdi 			                	  *
*       short 	arch_modificado 		short 	cargue_axiomas	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
 AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
 MODIFICACIONES:
 VTP 7/Mar/1997
	Antes borraba archivos empleando system("rm ...") ahora
	se usa unlink (portable para DOS y UNIX)
 VTP 13/Jul/1997
    Ahora puede recibir un parametro mas que es un archivo donde
    se debe escribir
**************************************************************************/

short
construya_axs_formas_normales (nodo_tad * tad_generad, nodo_select * s,
                               FILE *fp2)
{
/*----------------------------------------------------------------------*/

  FILE *fp;
  short correcto;
  nodo_expresion *plant_li_ax;
  char cont;
  trio_edicion edicion[MAX_EDICION];

  assert (editor[0] != NULL);	/* VTP. 7/May/1997 */
  /* Se construyen los axiomas de una selectora con un solo argumento
     usando las formas normales del tad_generad */

  if (fp2==NULL)
    {
      fp = fopen ("axiom_ed.tad", "w");
      FileOpen (fp, "axiom_ed.tad");
    }
  else
    {
      fp=fp2;
    }

  cont = '1';
  genere_plantilla_selectora (s, &plant_li_ax, &cont);
  genere_axs_tdi (fp, plant_li_ax, plant_li_ax->argumentos,
		  tad_generad->iniciales, cont,"");
  genere_axs_tdi (fp, plant_li_ax, plant_li_ax->argumentos,
		  tad_generad->constructoras, cont,"");
  if (fp2==NULL)
    {
      fclose (fp);
      edicion[0].modo = 0;
      edicion[0].linea = 1;
      strcpy (edicion[0].nombre, "signa_ed.tad");
      edicion[1].modo = 1;
      edicion[1].linea = num_ed;
      strcpy (edicion[1].nombre, "axiom_ed.tad");
      llamar_al_editor (2, edicion, "temp");
      /*  VTP. 22/Jun/1999 Invocaba al editor de forma chanda
	  sprintf (comando, "%s axiom_ed.tad", editor[0]);
	  system (comando); */
      if (arch_modificado ("axiom_ed.tad"))
        {
          correcto = cargue_axiomas (s);
        }
      else
        {
          correcto = 0;
        }
    /*** VTP. 7/Mar/1997. Decia: system("rm -f axiom_ed.tad");  */
      unlink ("axiom_ed.tad");
    }
  return (correcto);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void ed_arch_axiomas(FILE *fp,nodo_igualdad *cab_igs)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Escribir una lista de ecuaciones en un archivo.            *
* DESCRIPCION: Escribe las ecuaciones (axiomas de un tad o programas de   *
*	una represent.) de la lista apunt. por cab_igs formateadas en el  *
*	archivo fp.	  						  *
* PARAMETROS:                                                             *
*    FILE		*fp		Ap. al archivo donde se escriben  *
*					las ecuaciones.			  *
*    nodo_igualdad	*cab_igs        Ap. a la cabeza de la lista de    *
*					ecuaciones a ser escritas.	  *
* PRE!:  { fp -> [///], cab_igs -> < e1,...,en > ,  n >= 0.             } *
* POST!: { fp -> [///e1 ... en]                                         } *
* LLAMA A: void   formatee_ld_axioma                                      *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
ed_arch_axiomas (FILE * fp, nodo_igualdad * cab_igs)
{
/*----------------------------------------------------------------------*/

  nodo_igualdad *cabeza;
  texpr fm_ld_str;

  assert (fp != NULL);		/* VTP. 7/May/1997 */
  for (cabeza = cab_igs; cabeza != NULL; cabeza = cabeza->sig)
    {
      strcpy (fm_ld_str, "");
      formatee_ld_axioma (cabeza->lado_der, fm_ld_str,
			  strlen (cabeza->li_str) + 4);
      fprintf (fp, "%s = %s\n", cabeza->li_str, fm_ld_str);
    }
  fprintf (fp, "\n");
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void ed_arch_axiomas_ops(FILE *fp,nodo_select *cab_ops)  	          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Escribir los axiomas de un conjunto de selectoras en un    *
*	archivo.                                                   	  *
* DESCRIPCION: Escribe los axiomas de las operaciones (selectoras un tad u*
*	operaciones auxs. de una represent.) de la lista apun. por cab_ops*
*	formateadas en el archivo fp.					  *
* PARAMETROS:                                                             *
*    FILE		*fp		Ap. al archivo donde se escriben  *
*					las ecuaciones.			  *
*    nodo_select  	*cab_ops        Ap. a la cabeza de la lista de ope*
*					raciones cuyas ecuaciones deben   *
*					ser escritas.	  		  *
* PRE!:  { fp -> [///], cab_ops -> < op1,...,opn > ,  n >= 0.  y para todo*
*	    i, 0 <= i <= n: opi.axiomas -> <ai1,...,aimi>,  mi >= 0, 	} *
* POST!: { fp -> [///a11...a1m1.......an1...anmn] 			} *
* LLAMA A: void ed_arch_axiomas						  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
ed_arch_axiomas_ops (FILE * fp, nodo_select * cab_ops)
{
/*----------------------------------------------------------------------*/

  nodo_select *s;


  for (s = cab_ops; s != NULL; s = s->sig)
    {
      ed_arch_axiomas (fp, s->axiomas);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short edite_axiomas()							  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Construir los axiomas de todas las selectoras de un tad.   *
* DESCRIPCION: Contruye los axiomas de cada una de las selectoras del tad *
*        en edicion apuntado por tad_ed.                                  *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_ed    	Apunt. al tad que esta siendo editado.    *
* RESULTADO:  short		1, Si la construccion de los axiomas fue  *
*				   exitosa.				  *
*				0, Sino.				  *
* PRE!:  { tad_ed-> selectoras = <s1,...,sn> ,  n >= 0.                 } *
* POST!: { edite_axiomas = 1, Si para todo i, 1 <= i <= n, 		  *
*			     si->axiomas es una lista de axiomas correctos*
*          edite_axiomas = 0, Sino.                                     } *
* !OJO!: Tiene en cuenta que el tad editado sea o no de tipo RESTRICCION. *
* LLAMA A: short construya_axiomas					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

short
edite_axiomas ()
{
/*----------------------------------------------------------------------*/

  nodo_select *s;
  short correcto, es_restr;
  cad50 pred_restr;
 /*** Alvaro
  cad6          siga;
  ****/

  correcto = 1;
  if (tad_ed->tipo == RESTRICCION)
    {
      es_restr = 1;
      strcpy (pred_restr, tad_ed->info_tipo->sig->nombre);
    }
  else
    {
      es_restr = 0;
      strcpy (pred_restr, "");
    }
  for (s = tad_ed->selectoras; (s != NULL) && correcto; s = s->sig)
    correcto = construya_axiomas (s, es_restr, pred_restr, 
				  tad_ed->nombre,NULL);
  return (correcto);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short edite_axiomas_selectora(nodo_tad *p,nodo_select *s)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Editar los axiomas existentes de una selectora.            *
* DESCRIPCION: Escribe en el archivo de edicion "axiom_ed.tad" los axio- *
*        mas de la selectora apunt. por s del tad apunt. por p y edita el *
*	 archivo con vi. Si p tiene status modificable, permite que estos *
*	 axiomas sean cambiados, caso en el cual los analiza sintactica y *
*	 semanticamente.         					  *
* PARAMETROS:                                                             *
*   nodo_tad	*p	Ap. al tad del cual se edita un subcjto de axiomas*
*   nodo_select	*s      Ap. a la selectora de p a la cual se le editan los*
*			axiomas.					  *
* RESULTADO:  short	1, Si la edicion de axiomas fue exitosa.          *
*			0, Sino.					  *
* PRE!:  { p-> selectoras = <...,s,...>, s->axiomas = <a1,...,an>, n >= 0}*
* POST!: { edite_axiomas_selectora = 1, Si s->axiomas = <b1,...,bm>, m>=0 *
*		tq. b1,...,bn forman un cjto correcto de axiomas para s y *
*		si p->status = 1 entonces m = n y ai = bi, para i= 1,...,n*
*          edite_axiomas_selectora = 0, Sino.                           } *
* LLAMA A:								  *
*   void  formatee_ld_axioma 	                        		  *
*   short arch_modifarch_modific	short cargue_axiomas		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
* MODIFICACIONES:
* VTP 7/Mar/1997
*	Antes borraba un archivo empleando system("rm ... ahora
*	se usa unlink (portable para DOS y UNIX)
**************************************************************************/

short
edite_axiomas_selectora (nodo_tad * p, nodo_select * s)
{
/*----------------------------------------------------------------------*/

  FILE *fp;
  nodo_igualdad *cabeza;
  short correcto = 0;
  cad240 fm_ld_str;
/*** Alvaro
  texpr         comando;
  cad6          siga;
  ***/
  trio_edicion edicion[MAX_EDICION];

  assert (p != NULL && s != NULL);	/* VTP. 7/May/1997 */
  if (!p->modif)
    p->modif = 1;

    /*-- Creara un archivo con un programa Lisp-emacs que abre dos ventanas,
      en una de las cuales carga la signatura. En el archivo "temp" queda
      el programa emacs que despliega "signature..." en una ventana --*/
  esc_arch_edicion_signatura (tad_ed);
  editar_2w ("signa_ed.tad", s->nombre);

  fp = fopen ("axiom_ed.tad", "w");
  FileOpen (fp, "axiom_ed.tad");
  for (cabeza = s->axiomas; cabeza != NULL; cabeza = cabeza->sig)
    {
      strcpy (fm_ld_str, "");
      formatee_ld_axioma (cabeza->lado_der, fm_ld_str,
			  strlen (cabeza->li_str) + 4);
      fprintf (fp, "%s = %s\n", cabeza->li_str, fm_ld_str);
    }
  fclose (fp);
  strcpy (edicion[0].nombre, "signa_ed.tad");
  strcpy (edicion[1].nombre, "axiom_ed.tad");
  edicion[0].linea = 1;
  edicion[1].linea = 1;

  if (p->status == 1)
    {				/* el tad no es modificable */
      edicion[0].modo = 0;
      edicion[1].modo = 0;
      llamar_al_editor (2, edicion, "temp");
    }
  else
    {				/* el tad es modificable */
      edicion[0].modo = 0;
      edicion[1].modo = 1;
      llamar_al_editor (2, edicion, "temp");
      if (arch_modificado ("axiom_ed.tad"))
	correcto = cargue_axiomas (s);
      else
	correcto = 1;
    }
/*** VTP. Decia system("rm -f temp");  ***/
  unlink ("temp");
/*** VTP. Decia system("rm -f signa_ed.tad");  ***/
  unlink ("signa_ed.tad");
/*** VTP. Decia system("rm -f axiom_ed.tad");  ***/
  unlink ("axiom_ed.tad");
  return (correcto);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void edite_todos_axiomas_selectora(nodo_select *cab_ops)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Desplegar los axiomas de un conjunto de selectoras en un   *
*	archivo.                                                   	  *
* DESCRIPCION: Crea un archivo de edicion, escribe los axiomas de las ope-*
*	raciones (selectoras un tad u operaciones auxs. de una represent.)*
*	de la lista apun. por cab_ops formateadas en el archivo y los des-*
*	pliega (usando vi) sin permitir modificaciones.                   *
* PARAMETROS:                                                             *
*    nodo_select  	*cab_ops        Ap. a la cabeza de la lista de ope*
*					raciones cuyos axiomas deben ser  *
*					escritos.	  		  *
* PRE!:  { cab_ops -> < op1,...,opn > ,  n >= 0.  y para todo i, 0<=i<=n: *
*	   opi.axiomas -> <ai1,...,aimi>,  mi >= 0, 			} *
* POST!: { Los axiomas a11...a1m1.......an1...anmn han sido mostrados al  *
*	   usuario sin permitir que este los modifique.			} *
* LLAMA A:								  *
*	void	ed_arch_axiomas_ops 			                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
* MODIFICACIONES:
* VTP 13/Ene/97
*	Antes borraba un archivo empleando system("rm ... ahora
*	se usa unlink (portable para DOS y UNIX)
**************************************************************************/

void
edite_todos_axiomas_selectora (nodo_select * cab_ops)
{
/*----------------------------------------------------------------------*/

  FILE *fp;
/*** Alvaro
  texpr         comando;
  cad6          siga;
***/
  trio_edicion edicion[MAX_EDICION];

  fp = fopen ("axiom_ed.tad", "w");
  FileOpen (fp, "axiom_ed.tad");
  ed_arch_axiomas_ops (fp, cab_ops);
  fclose (fp);
  edicion[0].modo = 0;
  edicion[0].linea = 1;
  strcpy (edicion[0].nombre, "axiom_ed.tad");
  llamar_al_editor (1, edicion, "");
/*** VTP. Decia system("rm -f axiom_ed.tad"); ***/
  unlink ("axiom_ed.tad");

/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short es_parte_inicial(cad50 inic_simple,cad50 inic_comp)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si una inicial hace parte de una inicial compues*
*	       ta del producto Cartesiano explicito.			  *
* DESCRIPCION: Parte la inicial compuesta <inic_comp>, que es una inicial *
*        de un tad producto Cartesiano, en sus dos componentes y chequea  *
*        si alguno de estos es <inic_simple>.                             *
* PARAMETROS:                                                             *
*    cad50	inic_simple	Nombre de una inicial de un tad T.	  *
*    cad50	inic_comp  	Nombre de una inicial de un tad producto  *
*   				Cartesiano de T con otro tad.		  *
* RESULTADO:  short		1, Si <inic_simple> es una de las inicia- *
*				   les que componen <inic_comp>.	  *
*				0, Sino.				  *
* PRE!:  { inic_comp = INIC1-INIC2, inic_simple = INICS			} *
* POST!: { es_parte_inicial = (INIC1 = INICS) OR (INIC2 = INICS)        } *
* LLAMA A: void	  parta_expresion                                         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

short
es_parte_inicial (cad50 inic_simple, cad50 inic_comp)
{
/*----------------------------------------------------------------------*/

  cad50 inic1, inic2;

  assert (inic_comp != NULL && inic_simple != NULL);	/* VTP. 7/May/1997 */
  strcpy (inic1, inic_comp);
  parta_expresion (inic1, inic2, '_');
  if (!strcmp (inic_simple, inic1) || !strcmp (inic_simple, inic2))
    return (1);
  else
    return (0)
      ;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void escriba_arch_ed_ax_restr(FILE *fp,texpr ni,texpr nd,               *
*                               texpr li_axioma, texpr cond_ld_axioma)    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Escribe en el archivo la plantilla para la edicion de un   *
*	       axioma de un tad RESTRICCION.				  *
*	<li_axioma> = IF NOT(<cond_ls_axioma>) THEN ERROR  ELSE 	  *
* PARAMETROS:                                                             *
*   FILE 	   *fp 			Ap. al archivo donde se escribe el*
*					formato del axioma.		  *
*   texpr 	   li_axioma,	  	Cadena con el lado izq. del axioma*
*   texpr 	   cond_ld_axioma	Cadena con la condicion para que  *
*					definicion de la selectora sea co-*
*					rrecta, esto es que los argums. de*
*					TDI cumplan el presicado de restr.*
* PRE!:  { fp-> [///], li_axioma = LA, cond_ld_axioma = CLA,              *
*	   axs_viejos_select = <a1,...,an>,  n >= 0			} *
* POST:  { fp-> [///LA = IF NOT(CLA) THEN ERROR  ELSE  }                  *
* LLAMA A:                                                                *
*   short busque_li_axioma		void formatee_ld_axioma		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
escriba_arch_ed_ax_restr (FILE * fp, texpr ni, texpr nd, texpr li_axioma,
			  texpr cond_ld_axioma)
{
/*----------------------------------------------------------------------*/


/*** Alvaro
  texpr 	ld_axioma;
  cad240	fm_ld_str;
  ***/
  assert (li_axioma != NULL && fp != NULL && ni != NULL && nd != NULL);		/* VTP. 7/May/1997 */
  strcat (li_axioma, ")");
  fprintf (fp, "%s%s = \n\t\t%s NOT(%s) %s %s\n", ni, li_axioma, claves[8],
	   cond_ld_axioma, claves[9], claves[11]);
  if (strcmp (nd, ""))
    fprintf (fp, "\t\t%s %s%s %s\n", claves[10], nd, li_axioma, claves[15]);
  else
    fprintf (fp, "\t\t%s ... %s\n", claves[10], claves[15]);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void escriba_arch_ed_axiomas(FILE *fp,texpr li_axioma,		  *
*     			       nodo_igualdad *axs_viejos_select)          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Escribe en el archivo el lado izq. de un axioma.           *
* DESCRIPCION: Busca en la lista encabezada por <axs_viejos_select> uno   *
*	con lado izq. isomorfo a <li_axioma>. Si lo encuentra, escribe es-*
*	axioma formateado en el archivo. Sino, escribe en el archivo el   *
*	lado izq. <li_axioma>.						  *
* PARAMETROS:                                                             *
*   FILE 	   *fp 			Ap. al archivo donde se escribe el*
*					formato del axioma.		  *
*   texpr 	   li_axioma,	  	Cadena con el lado izq. del axioma*
*					TDI cumplan el presicado de restr.*
*   nodo_igualdad   *axs_viejos_select  Ap. a la lista de axiomas ya cono-*
*					cidos del tad.			  *
* PRE!:  { fp-> [///], li_axioma = LA, 					  *
*	   axs_viejos_select = <a1,...,an>,  n >= 0			} *
* POST!: { fp-> [///a1],    Si  existe i, 1 <= i <= n, tq. li_axioma y    *
*               a1->lado_izq son expresiones isomorfas, o                 *
*          fp-> [///LA = ], Sino.					  *
* LLAMA A:                                                                *
*   short busque_li_axioma		void formatee_ld_axioma		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
escriba_arch_ed_axiomas (FILE * fp, texpr li_axioma)
{
/*----------------------------------------------------------------------*/


/*** ALvaro
  texpr 	ld_axioma;
  cad240	fm_ld_str;
 ***/
  assert (li_axioma != NULL);	/* VTP. 7/May/1997 */
  /* VTP. 5/Jun/1998.  Selectors with empty domain don't need () */
  if (strchr(li_axioma,'(')!=NULL)
    {
      strcat (li_axioma, ")");
    }
    /************   Arreglar lo de busque_li_axioma
    if ((axs_viejos_select != NULL) &&
	(busque_li_axioma(li_axioma,axs_viejos_select,ld_axioma))) {
	strcpy(fm_ld_str,"");
	formatee_ld_axioma(ld_axioma,fm_ld_str,strlen(ld_axioma) + 4);
	fprintf(fp,"%s=%s\n",li_axioma,fm_ld_str);
     }else {
	fprintf(fp,"%s=\n",li_axioma);
    }
    ***************/
  fprintf (fp, "%s=\n", li_axioma);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short expr_isomorfas(nodo_expresion *expr1,nodo_expresion *expr2,	  *
*                      nodo_expresion *expr_der) 			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si las expresiones expr1 y expr2 son isomorfas. *
* DESCRIPCION: Recorre los arboles sintacticos de la expresion almacenadas*
*        en <expr1> y <expr2> verificando que sean iguales, salvo renom-  *
*        bramiento de variables. Al mismo tiempo va "renombrando" las va- *
*        riables en la expresion <expr_der>.                              *
* PARAMETROS:                                                             *
*   nodo_expresion 	*expr1		Ap. a la 1a. expresion.		  *
*   nodo_expresion 	*expr2,	        Ap. a la 2a. expresion.		  *
*   nodo_expresion 	*expr_der	Ap. a la expresion que corresponde*
*					al lado der. de un axioma cuyo la-*
*					do izq. es expr2.		  *
* RESULTADO:  short	1, Si expr1 y expr2 apuntan a expresiones isomorf.*
*                   	0, Sino.					  *
* PRE!:  { expr1 = EXP1, expr2 = EXPR2, expr_der = EXPD                 } *
* POST!: { expr_isomorfas = 1, Si EXP1 y EXP2 son expresiones identicas,  *
*		   salvo renombremiento de variables. expr_der = REN(EXPD)*
*                  donde, REN(EXPD) es la expresion que resulta de renom- *
*		   brar las variables de EXPD que aparecen en EXP2 por los*
*		   nombres de variables que estas tienen en EXP1.         *
*          expr_isomorfas = 0, Sino                                   	} *
* LLAMA A: 								  *
*	void	reemplace_var		short	expr_isomorfas		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

short
expr_isomorfas (nodo_expresion * expr1, nodo_expresion * expr2,
		nodo_expresion * expr_der)
{
/*----------------------------------------------------------------------*/

  short isomorf;
  nodo_argum *a1, *a2;

  assert (expr1 != NULL && expr2 != NULL);	/* VTP. 7/May/1997 */
  isomorf = 1;
  if (strcmp (expr1->nombre, expr2->nombre))
    {
      if ((!strcmp (expr1->tipo, "VAR")) && (!strcmp (expr1->tipo, "VAR")))
	{
	  reemplace_var (expr2->nombre, expr1->nombre, expr_der);
	}
      else
	{
	  isomorf = 0;
	}
    }
  a1 = expr1->argumentos;
  a2 = expr2->argumentos;
  while (((a1 != NULL) || (a2 != NULL)) && isomorf)
    {
      if ((a1 == NULL) || (a2 == NULL))
	{
	  isomorf = 0;
	}
      else
	{
	  isomorf = expr_isomorfas (a1->expr, a2->expr, expr_der);
	  a1 = a1->sig;
	  a2 = a2->sig;
	}

    }
  return (isomorf);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void formatee_ld_axioma(nodo_expresion *expr,cad240 linea,int margen) *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Formatear (preaty-print) el lado derecho de un axioma.     *
* DESCRIPCION: Retorna en <linea> la expresion correspondiente a arbol    *
*        sintactico apunt. por <expr> formateada dejando <margen> blancos *
*        a la izquierda.                                                  *
* PARAMETROS:                                                             *
*   nodo_expresion	*expr	Ap. al arbol sintactico de la expresion   *
*				que debe ser formateada.		  *
*   cad240		linea   Cadena en la que se retorna la expresion  *
*				formateada.				  *
*   short		margen  Numero de blancos de margen izquierda 	  *
* PRE!:  { expr-> EXP, margen = M                                       } *
* POST!: { linea = L, donde L es la cadena con la expresion EXP formateada*
*          para ser impresa "bonito".					  *
* LLAMA A: void formatee_ld_axioma					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
formatee_ld_axioma (nodo_expresion * expr, cad240 linea, int margen)
{
/*----------------------------------------------------------------------*/

  short num_arg, num_blancos;
  nodo_argum *a;
  int margen_inic;

  assert (expr != NULL && linea != NULL && margen >= 0);	/* VTP. 7/May/1997 */
  margen_inic = margen;
  strcat (linea, expr->nombre);
  if (expr->argumentos != NULL)
    {
      if (!strcmp (expr->nombre, claves[8]))
	{
	  strcat (linea, " ");
	  margen += 4;
	}
      else
	{
	  strcat (linea, "(");
	}
      for (a = expr->argumentos, num_arg = 1;
	   a != NULL;
	   a = a->sig, num_arg++)
	{
	  if (num_arg > 1)
	    {
	      if (!strcmp (expr->nombre, claves[8]))
		{
		  strcat (linea, "\n");
		  for (num_blancos = 0; num_blancos < margen;
		       num_blancos++)
		    {
		      strcat (linea, " ");
		    }
		  if (num_arg == 2)
		    {
		      strcat (linea, claves[9]);
		    }
		  else
		    {
		      strcat (linea, claves[10]);
		    }
		  strcat (linea, " ");
		}
	      else
		{
		  strcat (linea, ",");
		}
	    }
	  formatee_ld_axioma (a->expr, linea, margen);
	}
      if (strcmp (expr->nombre, claves[8]))
	strcat (linea, ")");
      else
	{
	  strcat (linea, "\n");
	  for (num_blancos = 1; num_blancos < margen_inic; num_blancos++)
	    strcat (linea, " ");
	  strcat (linea, claves[15]);
	}
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_ax_eq_pc(nodo_select *s,cad20 nom_tad1,cad20 nom_tad2)	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar los axiomas del predicado de igualdad de un tad que*
*	       es un producto Cartesiano.  				  *
* DESCRIPCION: Construye el axioma del predicado de igualdad de un tad pro*
*        ducto Cartesiano de los tads <nom_tad1> y <nom_tad2> que dice que*
*	 dos objetos del producto son iguales si sus primeras proyecciones*
*	 son EQ_<nom_tad1> iguales y sus segundas proyecciones son 	  *
*	 EQ_<nom_tad2> iguales.		  				  *
* PARAMETROS:                                                             *
*    nodo_select 	*s	 Ap. al predicado de igualdad del producto*	
*    cad20 		nom_tad1 Nombre del 1er. tad componente del produc*
*    cad20 		nom_tad2 Nombre del 2o.  tad componente del produc*
* PRE!:  { s ap. al predicado de igualdad producto de los tads nom_tad1 y *
*	   nom_tad2.							} *
* POST!: { s->axiomas =							  *
*		<EQ_<tdi>(X,Y) = AND(EQ_<nom_tad1>(PROY1(X),PROY1(Y)),    *
*			             EQ_<nom_tad1>(PROY2(X),PROY2(Y)))> } *
* LLAMA A: void genere_plantilla_selectora				  *
*	   void genere_eq_tipo		    void transf_estruct_iguald    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
genere_ax_eq_pc (nodo_select * s, cad20 nom_tad1, cad20 nom_tad2)
{
/*----------------------------------------------------------------------*/

  nodo_expresion *plantilla_select = NULL, *exp_and;
  nodo_igualdad *ax;
  char cont;

  assert (s != NULL);		/* VTP. 7/May/1997 */
  /* Generar axioma
     EQ_<tdi>(X,Y) = AND(EQ_<p>(PROY1(X),PROY1(Y)),
     EQ_<q>(PROY2(X),PROY2(Y))) */

  cont = '1';
  genere_plantilla_selectora (s, &plantilla_select, &cont);
  exp_and = (nodo_expresion *) malloc (sizeof (nodo_expresion));
  HayMem (exp_and, sizeof (nodo_expresion));
  strcpy (exp_and->nombre, "AND");
  strcpy (exp_and->tipo, "BOOL");
  strcpy (exp_and->clase, "SELECT");
  genere_eq_tipo (nom_tad1, plantilla_select, "1", &(exp_and->argumentos));
  genere_eq_tipo (nom_tad2, plantilla_select, "2", &(exp_and->argumentos->sig));
  ax = (nodo_igualdad *) malloc (sizeof (nodo_igualdad));
  HayMem (ax, sizeof (nodo_igualdad));
  ax->lado_izq = plantilla_select;
  ax->lado_der = exp_and;
  ax->sig = NULL;
  transf_estruct_iguald (ax);
  s->axiomas = ax;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_axiomas(FILE *fp,nodo_dom *d,texpr li_axioma,		  *
*                     char cont,nodo_igualdad *axs_viejos_select)	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar los argumentos de los lados izqs. de los axiomas de*
*              una selectora.                                             *
* DESCRIPCION: Genera los args. de los lados izqs. de una selectora usan- *
*        do lema de forma normal sobre los argumentos del TDI que estan   *
*        marcados como de induccion. Si el lado izq. generado es isomorfo *
*        al lado izq. de algun axioma en la lista apunt. por <axs_viejos_ *
*        select> entonces escribe en el archivo este axioma. Sino, escribe*
*	 el lado izq. generado.						  *
* PARAMETROS:                                                             *
*   FILE          *fp			Ap. al archivo donde se escriben  *
*					los axiomas.			  *
*   nodo_dom      *d			Ap. a una sublista del dominio de *
*					la selectora a la que se le estan *
*					generando los axiomas.		  *
*   texpr         li_axioma		Cadena con el segmento de lado izq*
*					que se ha formado hasta ahora.    *
*   char          cont			Indice para la siguiente variable *
*					que se deba usar.		  *
*   nodo_igualdad *axs_viejos_select	Ap. a la lista de axiomas conoci- *
*					dos para la selectora.	    	  *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_ed    	Apunt. al tad que esta siendo editado.    *
* PRE!:  { fp-> [///], d-> <d1,...,dn>, n >= 0, li_axioma = LIA, cont = C,*
*	   axs_viejos_select-> <a1,...,am>,  m >= 0			} *
* POST!: { fp-> [///li1,...,lik], k > 0, donde li1,...,lik  son los lados *
*          izqs. de la forma LIA.expr_argum(d1).expr_argums(<d2,...,dn>), *
*	   donde                   				   	  *
*	   expr_argum(d1) esta definida como:				  *
*	   - "", Si d1 es vacio (NULL).       				  *
*	   - Una variable de tipo d1, Si d1 <> TDI			  *
*	   - Nombres del TDI generados con el lema de forma normal, Si    *
*		d1 = TDI.						  *
*	   expr_argums(<c1,...,cj> esta definido por:		  	  *
*	   - "", Si j = 0.						  *
*	   - expr_argum(c1).expr_argums(<c2,...,cj>), Sino	    	} *
* LLAMA A: 								  *
*  	void escriba_arch_ed_axiomas 	void ponga_generad_argum	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
genere_axiomas (FILE * fp, nodo_dom * d, texpr li_axioma,
		char cont, nodo_igualdad * axs_viejos_select)
{
/*----------------------------------------------------------------------*/

  texpr mens_err;
  nodo_generadora *g;
  cad12 temp;
  nodo_tad *tg;

  assert (li_axioma != NULL);	/* VTP. 7/May/1997 */
  if (d == NULL)
    {
      escriba_arch_ed_axiomas (fp, li_axioma);
    }
  else if (!d->induccion)
    {
      /* Fixing problem, before this, it generated 1,2,...,8,9,:,;,<.. */
      sprintf(temp,"%c%i",(char)toupper((d->nombre)[0]),cont-'0');
      if (d->sig != NULL)
	strcat(temp,",");
      strcat(li_axioma,temp);
      cont = (char) (cont + 1);
      genere_axiomas (fp, d->sig, li_axioma, cont, axs_viejos_select);
    }
  else
    {				/* Es parametro de induccion */
      if (strcmp (d->nombre, tad_ed->nombre)==0)
	{ tg=tad_ed; }
      else 
	{ 
	  mens_err[0]='\0';
 	  tg=tipoAmbiente(&tad,d->nombre,mens_err);
	  assert(tg!=NULL); /* No estaba el TAD.
			       Debe informarse al usuario y terminar */
	}

      for (g = tg->iniciales; g != NULL; g = g->sig)
	{
	  ponga_generad_argum (fp, g, d, li_axioma, cont, axs_viejos_select);
	}
      for (g = tg->constructoras; g != NULL; g = g->sig)
	{
	  ponga_generad_argum (fp, g, d, li_axioma, cont, axs_viejos_select);
	}
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_axiomas_restr(FILE *fp,texpr ni, texpr nd, nodo_dom *d,     *
*                           texpr li_axioma,texpr cond_ld_axioma,         *
*                           cad50 pred_restr,cad12 nombre_tad_r,          *
*                           char cont,nodo_igualdad *axs_viejos_select)   *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar los plantillas de axiomas de una selectora de un   *
*	       tad RESTRICCION.						  *
* DESCRIPCION: Genera formas normales usando las generadoras a partir de  *
*        d poniendo como lado derecho del axioma la plantilla : 	  *
*        IF NOT(AND(<pred_restr>(arg_tdi1),...,<pred_restr>(arg_tdin)))   *
*	   THEN ERROR 	 ELSE 						  *
*        Si el lado izq. generado es isomorfo al lado izq. de algun axioma*
*	 en la lista apunt. por <axs_viejos_select> entonces escribe en el*
*	 archivo este axioma. Sino, escribe el formato generado.	  *
* PARAMETROS:                                                             *
*   FILE          *fp			Ap. al archivo donde se escriben  *
*					los axiomas.			  *
*   texpr         ni                    Nombre de la selectora que se     *
*                                       esta definiendo.                  *
*   texpr         nd                    Nombre de la selectora que        *
*                                       corresponde en el tad que se esta *
*                                       restringiendo.                    *
*   nodo_dom      *d			Ap. a una sublista del dominio de *
*					la selectora a la que se le estan *
*					generando los axiomas.		  *
*   texpr         li_axioma		Cadena con el segmento de lado izq*
*					que se ha formado hasta ahora.    *
*   texpr         cond_ld_axioma	Cadena con el segmento de condi-  *
*					cion (sobre los args. del TDI que *
*					deben cumplir el pred. de restric)*
*					que se ha formado hasta ahora.    *
*   cad50 	  pred_restr		Nombre del pred. de restriccion.  *
*   cad12   	  nombre_tad_r 		Nombre del tad es restriccion.    *
*   char          cont			Indice para la siguiente variable *
*					que se deba usar.		  *
*   nodo_igualdad *axs_viejos_select	Ap. a la lista de axiomas conoci- *
*					dos para la selectora.	    	  *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_ed    	Apunt. al tad que esta siendo editado.    *
* PRE!:  { fp-> [///], d-> <d1,...,dn>, n >= 0, li_axioma = LIA, cont = C,*
*	   cond_ld_axioma = CLA, pred_restr = P, nom_tad_r = T,		  *
*	   axs_viejos_select-> <a1,...,am>,  m >= 0			} *
* POST!: { fp-> [///li1=ld1,...,lik=ldk], k > 0, donde li1,...,lik  son   *
*	   los lados izqs. de la forma 					  *
*	   LIA.expr_argum(d1).expr_argums(<d2,...,dn>), donde 		  *
*	   expr_argum(d1) esta definida como:				  *
*	   - "", Si d1 es vacio (NULL).       				  *
*	   - Una variable de tipo d1, Si d1 <> TDI			  *
*	   - Nombres del TDI generados con el lema de forma normal, Si    *
*		d1 = TDI.						  *
*	   expr_argums(<c1,...,cj> esta definido por:		  	  *
*	   - "", Si j = 0.						  *
*	   - expr_argum(c1).expr_argums(<c2,...,cj>), Sino	    	  *
*	   ld1,...,ldk son los lados derechos de la forma:		  *
*	   ldi = IF NOT(<condi>) THEN ERROR ELSE <sel> FI   , con 	  *
*	   <condi> = AND(P(xi1),...,P(xif)) donde xi1,...,xif son todos   *
*          <sel> = <selectora>(xi1,...xin)                                *
*	   los argumentos del TDI que aparecen en lii.			} *
* LLAMA A:   void genere_axiomas_restr					  *
*  	void escriba_arch_ed_ax_restr	void ponga_generad_argum_restr	  *
**************************************************************************/

void
genere_axiomas_restr (FILE * fp, texpr ni, texpr nd, nodo_dom * d,
		      texpr li_axioma, texpr cond_ld_axioma,
		      cad50 pred_restr, cad12 nombre_tad_r,
		      char cont, nodo_igualdad * axs_viejos_select)
{
/*----------------------------------------------------------------------*/

  nodo_generadora *g;
/*** Alvaro
  nodo_tad		*p;
 ***/
  cad20 exp_generad;
  texpr ld_aux;

  assert (ni != NULL && nd != NULL && cond_ld_axioma != NULL);	/* VTP. 7/May/1997 */
  /* Genera formas normales usando las generadoras a partir de
     <cab_generad> en el argumento <arg> de <plant_li_ax> y pone
     como lado derecho del axioma la plantilla :
     IF NOT(AND(<pred_restr>(arg_tdi1),...,<pred_restr>(arg_tdin)))
     THEN ERROR
     ELSE <selectora>(arg_tdi1,...arg_tdin) FI                 */

  if (d == NULL)
    {
      escriba_arch_ed_ax_restr (fp, ni, nd,
				li_axioma, cond_ld_axioma);
    }
  else if (!(d->induccion))
    {
      exp_generad[0] = (char) toupper ((d->nombre)[0]);
      exp_generad[1] = cont;
      exp_generad[2] = '\0';
      if (!strcmp (d->nombre, nombre_tad_r))
	{
	  if (!strcmp (cond_ld_axioma, ""))
	    {
	      sprintf (ld_aux, "%s(%s)", pred_restr, exp_generad);
	    }
	  else
	    {
	      sprintf (ld_aux, "AND(%s,%s(%s))", cond_ld_axioma, pred_restr,
		       exp_generad);
	    }
	}
      else
	{
	  strcpy (ld_aux, cond_ld_axioma);
	}
      if (d->sig != NULL)
	strcat (exp_generad, ",")
	  ;
      strcat (li_axioma, exp_generad);
      cont = (char) (cont + 1);
      genere_axiomas_restr (fp, ni, nd, d->sig, li_axioma, ld_aux, pred_restr,
			    nombre_tad_r, cont, axs_viejos_select);
    }
  else
    {				/* El dominio es el TADI y es de induccion */
      for (g = tad_ed->iniciales; g != NULL; g = g->sig)
	{
	  ponga_generad_argum_restr (fp, ni, nd, g, d, li_axioma, cond_ld_axioma,
				     pred_restr,
				     nombre_tad_r, cont, axs_viejos_select);
	}
      for (g = tad_ed->constructoras; g != NULL; g = g->sig)
	{
	  ponga_generad_argum_restr (fp, ni, nd, g, d, li_axioma, cond_ld_axioma,
				     pred_restr,
				     nombre_tad_r, cont, axs_viejos_select);
	}
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_axiomas_cruzados(nodo_select *s,nodo_generadora *cab,	  *
*		 	       nodo_generadora *cola) 			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar los axiomas de una selectora del producto Cartesian*
*	       Explicito.						  *
* DESCRIPCION: Genera los axiomas de la selectora s de un tad producto Car*
*        tesiano T de dos tads T1 y T2 tq. s es heredada del tad Ti y se  *
*        define sobre los elementos de Tj, para i,j = 1,2 , i <> j.       *
*        Se aplica lema de forma normal sobre los args. de tipo Tj que son*
*	 de induccion.							  *
*	 Si gj es una generadora heredada de Tj y t es un objeto de T:    *
*	 s(...gj(...t...)...)  = gj(...s(...t...)...), Si s es de tipo T o*
*	 s(...gj(...t...)...)  = s(...t...), 	     Sino                 *
* PARAMETROS:                                                             *
*    nodo_select	*s	Selectora a la cual se le definen los axs.*
*    nodo_generadora	*cab    Ap. a la generadora a partir de la cual se*
*				deben generar la formas normales.	  *
*    nodo_generadora	*cola   Ap. a la siguiente generadora que no debe *
*				ser tenida en cuenta para generar la 	  *
*				formas normales.	  		  *
* V.GLOBALES:                                                             *
*    nodo_tad	*ult_tad	Apunt. al ultimo tad de la lista de tads  *
*				del ambiente, que contiene el producto    *
*				Cartesiano que se esta generando.	  *
* PRE!:  { ult_tad-> T = T1 x T2 tads del ambiente,			  *
*	   T->selectoras = <...,s,...>, s es una selectora heredada de Ti *
*	   s->axiomas = <a1,...,ak> , k >= 0			 	  *
*	   Generadoras de T = <..g1,..,gm..>, m >= 0, g1,...,gm son las ge*
*	   neradoras de T heredadas de Tj , i,j = 1,2   i<>j,		  *
*	   cab-> g1, cola->g(m+1)					} *
* POST!: { s->axiomas = <a1,...,ak,ac1,...,acl> l >= 0, donde ac1,...,acl *
*	  es la lista de axiomas "cruzados" que definen a s sobre los ele-*
*	  mentos de Tj.							} *
* LLAMA A: 								  *
*	short tiene_argums_induccion 	 void genere_plantilla_selectora  *
*	void copie_lista_expresion_apunt void genere_plantilla_generadora *
*	void busque_argum_var_tdi 	 void transf_estruct_iguald	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
genere_axiomas_cruzados (nodo_select * s, nodo_generadora * cab,
			 nodo_generadora * cola)
{
/*----------------------------------------------------------------------*/

  nodo_expresion *ld_axioma = NULL, *plantilla_li_ax = NULL, *nuevo_li_ax = NULL,
   *plantilla_construct = NULL, *expr_const;
  nodo_dom *d;
  nodo_argum *a, *na = NULL, *lda = NULL, *arg_tdi = NULL;
  nodo_generadora *c;
  nodo_igualdad *ax, *ult_ax;
  char cont;

  assert (s != NULL);		/* VTP. 7/May/1997 */
  if (tiene_argums_induccion (s))
    {
      for (ult_ax = s->axiomas; (ult_ax != NULL) && (ult_ax->sig != NULL);
	   ult_ax = ult_ax->sig);
      cont = '1';
      genere_plantilla_selectora (s, &plantilla_li_ax, &cont);
      for (d = s->dominio, a = plantilla_li_ax->argumentos;
	   d != NULL; d = d->sig, a = a->sig)
	{
	  if (d->induccion)
	    {
	      for (c = cab; c != cola; c = c->sig)
		{
		  copie_lista_expresion_apunt (plantilla_li_ax, &nuevo_li_ax,
					       a, &na);
		  genere_plantilla_generadora (c, ult_tad->nombre,
					       &plantilla_construct, &cont);
		  na->expr = plantilla_construct;
		  copie_lista_expresion_apunt (nuevo_li_ax, &ld_axioma, na, &lda);
		  busque_argum_var_tdi (lda->expr->argumentos, ult_tad->nombre,
					&arg_tdi);
		  if (strcmp (s->codominio, ult_tad->nombre))
		    {
		      lda->expr = arg_tdi->expr;
		      arg_tdi->sig = NULL;
		    }
		  else
		    {
		      expr_const = lda->expr;
		      lda->expr = arg_tdi->expr;
		      arg_tdi->expr = ld_axioma;
		      ld_axioma = expr_const;
		    }
		  ax = (nodo_igualdad *) malloc (sizeof (nodo_igualdad));
		  HayMem (ax, sizeof (nodo_igualdad));
		  ax->lado_izq = nuevo_li_ax;
		  ax->lado_der = ld_axioma;
		  ax->sig = NULL;
		  transf_estruct_iguald (ax);
		  if (s->axiomas == NULL)
		    s->axiomas = ax;
		  else
		    ult_ax->sig = ax
		      ;
		  ult_ax = ax;
		}
	    }
	}
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_axiomas_proy_constr(nodo_select *s,nodo_generadora *cab,    *
*                                 nodo_generadora *cola) 		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar los axiomas de una proyeccion del producto Cartesia*
*	       Explicito.						  *
* DESCRIPCION: Genera los axiomas de la proyeccion s de un tad producto   *
*        Cartesiano T de dos tads T1 y T2. Se aplica lema de forma normal *
*	 sobre los args. de TDI que son de induccion, usando las construc-*
*	 toras de la lista apunt. por <cab> hasta antes de la apunt. por  *
*	 <cola> que son las constructoras heredadas del padre que corres- *
*	 ponden a la proyeccion a ser aplicada.				  *
*	 Si Ci es una constructora de tipo Ti,  1 <= i <= 2		  *
*	 PROYi(Ci(...t...)) = C(PROYi(...t...))				  *
* PARAMETROS:                                                             *
*    nodo_select	*s	Selectora proyeccion del prod. Cartesiano.*
*    nodo_generadora	*cab    Ap. a la constructora a partir de la cual *
*				se deben generar la formas normales.	  *
*    nodo_generadora	*cola   Ap. a la siguiente constructora que no de-*
*				be ser tenida en cuenta para generar la   *
*				formas normales.	  		  *
* V.GLOBALES:                                                             *
*    nodo_tad	*ult_tad	Apunt. al ultimo tad de la lista de tads  *
*				del ambiente, que contiene el producto    *
*				Cartesiano que se esta generando.	  *
* PRE!:  { ult_tad-> T = T1 x T2 tads del ambiente,			  *
*	   T->selectoras = <...,s,...>, s es una selectora de i-proyeccion*
*	   Constructoras de T = <..g1,..,gm..>, m >= 0, g1,...,gm son las *
*	   generadoras de T heredadas de Ti , 1 <= i <= 2          	  *
*	   cab-> g1, cola->g(m+1) 					} *
* POST!: { s->axiomas = <a1,...,ak> k >= 0, donde a1,...,ak es la lista de*
*	  axiomas correspondiente a la proyeccion I-esima de las constr.} *
* LLAMA A: 								  *
*	void genere_plantilla_selectora void copie_lista_expresion	  *
*	void parta_expresion 		void transf_estruct_iguald	  *
*	void busque_argum_var_tdi					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
genere_axiomas_proy_constr (nodo_select * s, nodo_generadora * cab,
			    nodo_generadora * cola)
{
/*----------------------------------------------------------------------*/

  nodo_expresion *ld_axioma = NULL, *nuevo_li_ax = NULL, *plantilla_construct = NULL,
   *plantilla_select = NULL, *exp;
  nodo_argum *arg_tdi = NULL;
  nodo_generadora *constr;
  nodo_igualdad *ax, *ult_ax;
  char cont;

  assert (s != NULL);		/* VTP. 7/May/1997 */
  for (ult_ax = s->axiomas; (ult_ax != NULL) && (ult_ax->sig != NULL);
       ult_ax = ult_ax->sig);
  cont = '1';
  genere_plantilla_selectora (s, &plantilla_select, &cont);
  for (constr = cab; constr != cola; constr = constr->sig)
    {
      copie_lista_expresion (plantilla_select, &nuevo_li_ax, "", NULL,"");
      genere_plantilla_generadora (constr, ult_tad->nombre,
				   &plantilla_construct, &cont);
      nuevo_li_ax->argumentos->expr = plantilla_construct;
      copie_lista_expresion (plantilla_construct, &ld_axioma, "", NULL,"");
      (ld_axioma->nombre)[strlen (ld_axioma->nombre) - 1] = '\0';
      busque_argum_var_tdi (ld_axioma->argumentos, ult_tad->nombre, &arg_tdi);
      exp = (nodo_expresion *) malloc (sizeof (nodo_expresion));
      HayMem (exp, sizeof (nodo_expresion));
      strcpy (exp->nombre, plantilla_select->nombre);
      strcpy (exp->tipo, plantilla_select->tipo);
      strcpy (exp->clase, plantilla_select->clase);
      exp->argumentos = (nodo_argum *) malloc (sizeof (nodo_argum));
      HayMem (exp->argumentos, sizeof (nodo_argum));
      exp->argumentos->expr = arg_tdi->expr;
      exp->argumentos->sig = NULL;
      arg_tdi->expr = exp;
      ax = (nodo_igualdad *) malloc (sizeof (nodo_igualdad));
      HayMem (ax, sizeof (nodo_igualdad));
      ax->lado_izq = nuevo_li_ax;
      ax->lado_der = ld_axioma;
      ax->sig = NULL;
      transf_estruct_iguald (ax);
      if (s != NULL && s->axiomas == NULL)
	s->axiomas = ax;
      else
	ult_ax->sig = ax
	  ;
      ult_ax = ax;
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_axiomas_proy_inic(nodo_tad *t, nodo_select *s, cad5 indice) *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar los axiomas de una proyeccion del producto Cartesia*
*	       Explicito.						  *
* DESCRIPCION: Genera los axiomas de la proyeccion s de un tad producto   *
*        Cartesiano T de dos tads T1 y T2. Se aplica lema de forma normal *
*	 sobre los args. de TDI que son de induccion, usando las iniciales*
*	 de la lista apunt. por <cab>.				  	  *
*	 <indice> indica si se trata de la primera o segunda proyeccion.  *
*	 PROY<indice>(<inic1>_<inic2>) = <inic1>,  Si indice = 1   o	  *
*	 PROY<indice>(<inic1>_<inic2>) = <inic2>,  Si indice = 2    	  *
* PARAMETROS:                                                             *
*    nodo_select	*s	Selectora proyeccion del prod. Cartesiano.*
*    nodo_tad           *t      Apuntador al tad que se esta creando      *
*    cad5		indice	"1" Si es la primera proyeccion o	  *
*				"2" Si es la segunda proyeccion.	  *
* PRE!:  { ult_tad-> T = T1 x T2 tads del ambiente,			  *
*	   T->selectoras = <...,s,...>, s es una selectora de proyeccion  *
*	   Iniciales de T = <g1,..,gm>, m >= 0, cab-> g1 ,		  *
*	   indice = I, I = "1" o I = "2"  				} *
* POST!: { s->axiomas = <a1,...,ak> k >= 0, donde a1,...,ak es la lista de*
*	  axiomas correspondiente a la proyeccion I-esima de las inics. } *
* LLAMA A: 								  *
*	void genere_plantilla_selectora void copie_lista_expresion	  *
*	void transf_estruct_iguald	                                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
genere_axiomas_proy_inic (nodo_tad * t, nodo_select * s, cad5 indice)
{
/*----------------------------------------------------------------------*/
  nodo_expresion *ld_axioma, *nuevo_li_ax = NULL, *plantilla_select = NULL;
  nodo_generadora *inic, *cab;
  nodo_igualdad *ax, *ult_ax;
  char cont;
  nodo_tad *tad_obj1;
/*-- Tad objeto 1 de la selectora --*/
  nodo_tad *tad_obj2;
/*-- Tad objeto 2 de la selectora --*/
  nodo_generadora *inic_obj;
/*-- Inicial del tad objeto       --*/
  int num_inic;
/*-- Numero de iniciales del tad  --*/
  int i;

  assert (t != NULL && s != NULL && indice != NULL);	/* VTP. 7/May/1997 */

  cab = t->iniciales;
  for (ult_ax = s->axiomas; (ult_ax != NULL) && (ult_ax->sig != NULL);
       ult_ax = ult_ax->sig);
  cont = '1';
  genere_plantilla_selectora (s, &plantilla_select, &cont);
  strcpy (plantilla_select->argumentos->expr->clase, "INIC");
  tad_obj1 = existe_tad (t->info_tipo->nombre);
  tad_obj2 = existe_tad (t->info_tipo->sig->nombre);
  if ((tad_obj1 == NULL) || (tad_obj2 == NULL))
    return;
  for (inic_obj = tad_obj2->iniciales, num_inic = 0;
       inic_obj != NULL; inic_obj = inic_obj->sig, num_inic++);
  if (!strcmp (indice, "1"))
    inic_obj = tad_obj1->iniciales;
  else
    inic_obj = tad_obj2->iniciales;
  for (inic = cab, i = 0; inic != NULL; inic = inic->sig)
    {
      copie_lista_expresion (plantilla_select, &nuevo_li_ax, "", NULL,"");
      strcpy (nuevo_li_ax->argumentos->expr->nombre, inic->nombre);
      ld_axioma = (nodo_expresion *) malloc (sizeof (nodo_expresion));
      HayMem (ld_axioma, sizeof (nodo_expresion));
      strcpy (ld_axioma->nombre, inic_obj->nombre);
      if (!strcmp (indice, "1"))
	{
	  i++;
	  if (i == num_inic)
	    {
	      inic_obj = inic_obj->sig;
	      i = 0;
	    }
	}
      else
	{
	  inic_obj = inic_obj->sig;
	  if (inic_obj == NULL)
	    inic_obj = tad_obj2->iniciales;
	}
      strcpy (ld_axioma->tipo, s->codominio);
      strcpy (ld_axioma->clase, "INIC");
      ld_axioma->argumentos = NULL;
      ax = (nodo_igualdad *) malloc (sizeof (nodo_igualdad));
      HayMem (ax, sizeof (nodo_igualdad));
      ax->lado_izq = nuevo_li_ax;
      ax->lado_der = ld_axioma;
      ax->sig = NULL;
      transf_estruct_iguald (ax);
      if (s->axiomas == NULL)
	s->axiomas = ax;
      else
	ult_ax->sig = ax
	  ;
      ult_ax = ax;
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_axiomas_tipo(nodo_select *s,char componente)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar los axiomas de una selectora del producto Cartesia-*
*	       no Explicito.						  *
* DESCRIPCION: Genera los axiomas de la selectora s de un tad producto Car*
*        tesiano T de dos tads T1 y T2 tq. s es heredada del tad Ti y se  *
*        define sobre los elementos de Ti, para i = 1,2 		  *
*        Se aplica lema de forma normal sobre los args. de tipo Ti que son*
*	 de induccion.							  *
*	 Si gi es una generadora heredada de Ti y t es un objeto de T y   *
*	 se tiene el axioma s(...(g(...ti...)...) = ld entonces:	  *
*	 s(...gi(...t...)...)  = ldi, donde ldi es isomorfo a ld pero se  *
*	 ha reemplazado g por gi y ti por t				  *
* PARAMETROS:                                                             *
*    nodo_select *s		Selectora a la cual se le definen los axs.*
*    char	 componente	'1' Si i = 1				  *
*    				'2' Si i = 2				  *
* PRE!:  { ult_tad-> T = T1 x T2 tads del ambiente,			  *
*	   T->selectoras = <...,s,...>, s es una selectora heredada de Ti *
*	   s->axiomas = <a1,...,ak> , k >= 0			 	  *
* POST!: { s->axiomas = <a1,...,ak,ac1,...,acl> l >= 0, donde ac1,...,acl *
*	  es la lista de axiomas que definen a s sobre los elementos de Ti*
* LLAMA A: void transf_axioma_pc					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
genere_axiomas_tipo (nodo_select * s, char componente)
{
/*----------------------------------------------------------------------*/

  nodo_igualdad *ax, *ult_ax = NULL;

  assert (s != NULL && (componente == '1' || componente == '2'));	/* VTP. 7/May/1997 */

  for (ax = s->axiomas, s->axiomas = NULL, ult_ax = NULL;
       ax != NULL; ax = ax->sig)
    {
      transf_axioma_pc (s, &ult_ax, ax->lado_izq->argumentos, ax->lado_izq,
			ax->lado_der, componente, 0);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
*void genere_axs_tdi(FILE *fp,nodo_expresion *plant_li_ax,nodo_argum *arg,*
*		     nodo_generadora *cab_generad,char cont)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar formas normales en un argum. de una expresion.     *
* DESCRIPCION: Genera formas normales usando las generadoras a partir de  *
* 	<cab_generad> en el argumento <arg> de <plant_li_ax>. Ademas escri*
*	be la expresion resultante en el archivo apunt. por fp.		  *
* PARAMETROS:                                                             *
*    FILE 	     *fp		Ap. al archivo de edicion de axs. *
*    nodo_expresion  *plant_li_ax 	Ap. al lado izq. de un axioma.	  *
*    nodo_argum      *arg		Ap. a un argumento de <plant_li_ax*
*    nodo_generadora *cab_generad 	Ap. a una lista de generadoras del*
*					TDI.				  *
*    char 	     cont		Indice de la siguiente variable a *
*					ser utilizada.			  *
* PRE!:  { fp -> [///], plant_li_ax-> LI_AX, arg ap. a ARG un argumento en*
*	   LI_AX, cab_generad -> <g1,...,gn>, cont = C			} *
* POST!: { fp -> [///l_izq1...l_izqm], m > 0, tq. para todo i, 1<= i <=m. *
*	   l_izqi es el lado izq. de un axioma que se obtiene ponindo una *
*	   forma normal del TDI en el ARG de LI_AX.		        } *
* LLAMA A:   void copie_lista_expresion_apunt				  *
*	void genere_plantilla_generadora 	void construya_exp	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
* VTP. 11/Jun/1997 Parametro adic agregado.  Esta funcion es usada para
*                    generar plantillas de TADs nuevos (Extension/Suma-directa)
*                    Creaba mal las plantillas, porque no agregaba al final
*                    de los nombres de funciones el nombre del tipo (_tipo)
*                    Si el parametro adic, no es vacio lo agrega a las
*                    funciones (no a variables)
**************************************************************************/

void
genere_axs_tdi (FILE * fp, nodo_expresion * plant_li_ax, nodo_argum * arg,
		nodo_generadora * cab_generad, char cont, char *adic)
{
/*----------------------------------------------------------------------*/


  /* Genera formas normales usando las generadoras a partir de
     <cab_generad> en el argumento <arg> de <plant_li_ax>     */

  nodo_generadora *g;
  nodo_expresion *plant_generad = NULL, *li_ax = NULL;
  nodo_argum *arg_tdi = NULL;
  texpr exp_li_ax;

  for (g = cab_generad; g != NULL; g = g->sig)
    {
      copie_lista_expresion_apunt (plant_li_ax, &li_ax, arg, &arg_tdi);
      genere_plantilla_generadora (g, ult_tad->nombre, &plant_generad,
				   &cont);
      arg_tdi->expr = plant_generad;
      construya_exp (li_ax, exp_li_ax,adic); /* VTP. 11/Jun/1997. Agregado adic */
      fprintf (fp, "%s=\n", exp_li_ax);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_eq_tipo(cad20 nom_tad,nodo_expresion *plant_select,	  *
*                     cad5 indice,nodo_argum **arg_exp) 		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar una subexpresion para el axioma del predicado de   *
*	       igualdad de un tad producto Cartesiano.  		  *
* DESCRIPCION: Construye la expresion					  *
* 	 EQ_<nom_tad>(PROY<indice>(X),PROY<indice>(Y))) 		  *
*        y la retorna en (*arg_exp).                                      *
* PARAMETROS:                                                             *
*    cad20 	    nom_tad		Nombre de uno de los tad componen-*
*					del producto Cartesiano.	  *
*    nodo_expresion *plant_select	Ap. a la expresion: EQ_<tdi>(X,Y) *
*    cad5 	    indice		"1" Si se debe hacer la proyec. 1 *
*    					"2" Si se debe hacer la proyec. 2 *
*    nodo_argum     **arg_exp		(*arg_exp) ap. a la expresion cons*
*					truida.				  *
* PRE!:  { nom_tad = <ti>, plant_select = EQ_<tdi>(X,Y), indice = i    	} *
* POST!: { (*arg_exp)->  EQ_<ti>(PROYi(X),PROYi(Y)))> 			} *
* LLAMA A: void genere_proy_tipo					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
genere_eq_tipo (cad20 nom_tad, nodo_expresion * plant_select,
		cad5 indice, nodo_argum ** arg_exp)
{
/*----------------------------------------------------------------------*/

  nodo_expresion *exp_eq;

  /* Genera la expresion EQ_<nom_tad>(PROY<indice>(X),PROY<indice>(Y))) */

  assert (arg_exp != NULL && nom_tad != NULL);	/* VTP. 7/May/1997 */
  (*arg_exp) = (nodo_argum *) malloc (sizeof (nodo_argum));
  HayMem (*arg_exp, sizeof (nodo_argum));
  exp_eq = (nodo_expresion *) malloc (sizeof (nodo_expresion));
  HayMem (exp_eq, sizeof (nodo_expresion));
  (*arg_exp)->expr = exp_eq;
  strcpy (exp_eq->nombre, "EQ_");
  strcat (exp_eq->nombre, nom_tad);
  strcpy (exp_eq->tipo, "BOOL");
  strcpy (exp_eq->clase, "SELECT");
  genere_proy_tipo (nom_tad, plant_select->argumentos, indice,
		    &(exp_eq->argumentos));
  genere_proy_tipo (nom_tad, plant_select->argumentos->sig, indice,
		    &(exp_eq->argumentos->sig));
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_exp_generadora(nodo_nombre *n,texpr exp,char *cont)	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar argumentos variables para una generadora           *
* DESCRIPCION: Produce en <exp> la expresion que resulta de generar argu- *
*	 mentos variables para una generadora cuya lista de dominio esta  *
*        apuntada por <n>.                                                *
* PARAMETROS:                                                             *
*   nodo_nombre	*n	Ap. a la lista con el dominio de la generadora.	  *
*   texpr 	exp	Cadena para retornar la expresion construida.	  *
*   char 	*cont	Indice de la proxima variable a ser utilizada.	  *
* PRE!:  { n-> <d1,...,dm>, m >= 0, cont = C              		} *
* POST!: { exp = "(vC,...,v(C+m-1))" donde v(C+i-1) es una variable de    *
*          tipo di, para todo i, 1 <= i <= m.                           } *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
genere_exp_generadora (nodo_nombre * n, texpr exp, char *cont)
{
/*----------------------------------------------------------------------*/

  cad12 tstr;

  assert (exp != NULL && cont != NULL);		/* VTP. 7/May/1997 */
  if (n != NULL)
    {
      strcat (exp, "(");
      for (; n != NULL; n = n->sig)
	{
	  /* Again fixing problem of variables numbering */
	  sprintf(tstr,"%c%i",(char) toupper ((n->nombre)[0]),(*cont)-'0');
	  *cont = (char) (*cont + 1);
	  if (n->sig != NULL)
	    strcat(tstr,",");
	  strcat(exp,tstr);
	}
      strcat(exp,")");
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_proy_tipo(cad20 nom_tad,nodo_argum *arg,cad5 indice,	  *
*                       nodo_argum **arg_exp) 				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar una subexpresion para el axioma del predicado de   *
*	       igualdad de un tad producto Cartesiano.  		  *
* DESCRIPCION: Construye la expresion PROY<indice>(X) y la retorna en     *
*              (*arg_exp).                                      	  *
* PARAMETROS:                                                             *
*    cad20 	    nom_tad		Nombre de uno de los tad componen-*
*					del producto Cartesiano.	  *
*    nodo_argum     *arg         	Ap. al argum. X que se va a proyec*
*    cad5 	    indice		"1" Si se debe hacer la proyec. 1 *
*    					"2" Si se debe hacer la proyec. 2 *
*    nodo_argum     **arg_exp		(*arg_exp) ap. a la expresion cons*
*					truida.				  *
* PRE!:  { nom_tad = <ti>, arg-> X, indice = i    			} *
* POST!: { (*arg_exp)->  PROYi(X)					} *
* LLAMA A: void copie_lista_argum					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
genere_proy_tipo (cad20 nom_tad, nodo_argum * arg, cad5 indice,
		  nodo_argum ** arg_exp)
{
/*----------------------------------------------------------------------*/

  nodo_expresion *exp;
  nodo_argum *arg_var = NULL;
  assert (arg_exp != NULL && ult_tad != NULL);	/* VTP. 7/May/1997 */
  assert (ult_tad->nombre != NULL && nom_tad != NULL);	/* VTP. 7/May/1997 */
  /* Genera la arg_expresion PROY<indice>(X)  */
  (*arg_exp) = (nodo_argum *) malloc (sizeof (nodo_argum));
  HayMem (*arg_exp, sizeof (nodo_argum));
  exp = (nodo_expresion *) malloc (sizeof (nodo_expresion));
  HayMem (exp, sizeof (nodo_expresion));
  (*arg_exp)->expr = exp;
  (*arg_exp)->sig = NULL;
  sprintf (exp->nombre, "PROY_%s%s", ult_tad->nombre, indice);
  strcpy (exp->tipo, nom_tad);
  strcpy (exp->clase, "SELECT");
  copie_lista_argum (arg, &arg_var, "", NULL,"");
  arg_var->sig = NULL;
  exp->argumentos = arg_var;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void ponga_generad_argum(FILE *fp,nodo_generadora *g,nodo_dom *d,	  *
*	      texpr li_axioma,char cont,nodo_igualdad *axs_viejos_select) *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Producir una expresion de una generadora con todos sus ar- *
*              gumentos variables y producir formas normales en los stes. *
*	       argumentos de la expresion.				  *
* DESCRIPCION: Concatena a <li_axioma> una "plantilla" de la generadora   *
*        <g> con todos sus argumentos variables e invoca a la funcion que *
*        se encarga de generar, usando lema de forma normal, los demas ar-*
*        gumentos del lado izq. del axioma a partir de <d>.               *
* PARAMETROS:                                                             *
*   FILE          *fp			Ap. al archivo donde se escriben  *
*					los axiomas.			  *
*   nodo_generadora *g			Ap. a la generadora a la cual se  *
*				  	le construye la plantilla.	  *
*   nodo_dom      *d			Ap. a una sublista del dominio de *
*					la selectora a la que se le estan *
*					generando los axiomas.		  *
*   texpr         li_axioma		Cadena con el segmento de lado izq*
*					que se ha formado hasta ahora.    *
*   char          cont			Indice para la siguiente variable *
*					que se deba usar.		  *
*   nodo_igualdad *axs_viejos_select	Ap. a la lista de axiomas conoci- *
*					dos para la selectora.	    	  *
* PRE!:  { fp-> [///], d-> <d1,...,dn>, n >= 0, li_axioma = LIA, cont = C,*
*	   g-> G , G.dominio = <dg1,...,dgk>,  k > 0			  *
*	   axs_viejos_select-> <a1,...,am>,  m >= 0			} *
* POST!: { fp-> [///li1,...,lik], k > 0, donde li1,...,lik  son los lados *
*          izqs. de la forma LIA.expr_gener(g).expr_argums(<d2,...,dn>),  *
*	   donde                   				   	  *
*	   expr_gener(g) = <nom_G>(vC,...,v(C+k-1)) con v(C+i-1) una va-  *
*	   riable de tipo dgi.						} *
* LLAMA A:								  *
*	void genere_exp_generadora 		void genere_axiomas	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
ponga_generad_argum (FILE * fp, nodo_generadora * g, nodo_dom * d,
	      texpr li_axioma, char cont, nodo_igualdad * axs_viejos_select)
{
/*----------------------------------------------------------------------*/

  texpr li_aux;

  assert (li_axioma != NULL && g != NULL && g->nombre != NULL);		/* VTP. 7/May/1997 */
  strcpy (li_aux, li_axioma);
  strcat (li_aux, g->nombre);
  genere_exp_generadora (g->dominio, li_aux, &cont);
  if (d->sig != NULL)
    strcat (li_aux, ",")
      ;
  genere_axiomas (fp, d->sig, li_aux, cont, axs_viejos_select);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void ponga_generad_argum_restr(FILE *fp, texpr ni, texpr nd,            *
*         nodo_generadora *g,nodo_dom *d,                                 *
* 	  texpr li_axioma,texpr cond_ld_axioma,cad50 pred_restr,          *
*	  cad12 nombre_tad_r,char cont,nodo_igualdad *axs_viejos_select)  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Producir una expresion de una generadora de un tad restric-*
*	       cion con todos sus argumentos variables y producir formas  *
*	       normales en los stes. argumentos de la expresion.	  *
* DESCRIPCION: Concatena a <li_axioma> una "plantilla" de la generadora   *
*        <g> con todos sus argumentos variables e invoca a la funcion que *
*        se encarga de generar, usando lema de forma normal, los demas ar-*
*        gumentos del lado izq. del axioma a partir de <d>.               *
* PARAMETROS:                                                             *
*   FILE          *fp			Ap. al archivo donde se escriben  *
*					los axiomas.			  *
*   texpr         ni                    Nombre de la selectora que se     *
*                                       esta definiendo.                  *
*   texpr         nd                    Nombre de la selectora que        *
*                                       corresponde en el tad que se esta *
*                                       restringiendo.                    *
*   nodo_generadora *g			Ap. a la generadora a la cual se  *
*				  	le construye la plantilla.	  *
*   nodo_dom      *d			Ap. a una sublista del dominio de *
*					la selectora a la que se le estan *
*					generando los axiomas.		  *
*   texpr         li_axioma		Cadena con el segmento de lado izq*
*					que se ha formado hasta ahora.    *
*   texpr         cond_ld_axioma	Cadena con el segmento de condi-  *
*					cion (sobre los args. del TDI que *
*					deben cumplir el pred. de restric)*
*					que se ha formado hasta ahora.    *
*   cad50 	  pred_restr		Nombre del pred. de restriccion.  *
*   cad12   	  nombre_tad_r 		Nombre del tad es restriccion.    *
*   char          cont			Indice para la siguiente variable *
*					que se deba usar.		  *
*   nodo_igualdad *axs_viejos_select	Ap. a la lista de axiomas conoci- *
*					dos para la selectora.	    	  *
* PRE!:  { fp-> [///], d-> <d1,...,dn>, n >= 0, li_axioma = LIA, cont = C,*
*	   cond_ld_axioma = CLA, pred_restr = P, nom_tad_r = T,		  *
*	   g-> G , G.dominio = <dg1,...,dgk>,  k > 0			  *
*	   axs_viejos_select-> <a1,...,am>,  m >= 0			} *
* POST!: { fp-> [///li1=ld1,...,lik=ldk], k > 0, donde li1,...,lik  son   *
*	   los lados izqs. de la forma 					  *
*	   LIA.expr_gener(g).expr_argums(<d2,...,dn>), donde 		  *
*	   expr_argum(d1) esta definida como:				  *
*	   expr_gener(g) = <nom_G>(vC,...,v(C+k-1)) con v(C+i-1) una va-  *
*	   riable de tipo dgi.						} *
* LLAMA A:   void genere_axiomas_restr  				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
ponga_generad_argum_restr (FILE * fp, texpr ni, texpr nd,
			   nodo_generadora * g, nodo_dom * d,
		    texpr li_axioma, texpr cond_ld_axioma, cad50 pred_restr,
	   cad12 nombre_tad_r, char cont, nodo_igualdad * axs_viejos_select)
{
/*----------------------------------------------------------------------*/

  texpr li_aux, ld_aux;
  cad50 exp_generad;

  assert (g != NULL && li_axioma != NULL && pred_restr != NULL);	/* VTP. 7/May/1997 */
  assert (exp_generad != NULL);	/* VTP. 7/May/1997 */
  strcpy (exp_generad, g->nombre);
  genere_exp_generadora (g->dominio, exp_generad, &cont);
  strcpy (li_aux, li_axioma);
  strcat (li_aux, exp_generad);
  if (!strcmp (cond_ld_axioma, ""))
    {
      sprintf (ld_aux, "%s(%s)", pred_restr, exp_generad);
    }
  else
    {
      sprintf (ld_aux, "AND(%s,%s(%s))", cond_ld_axioma, pred_restr, exp_generad);
    }
  if (d->sig != NULL)
    strcat (li_aux, ",")
      ;
  genere_axiomas_restr (fp, ni, nd, d->sig, li_aux, ld_aux, pred_restr,
			nombre_tad_r, cont, axs_viejos_select);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void reemplace_dom_inicial(cad50 nom_inic,nodo_expresion *cab_expr,     *
*			     nodo_argum *cab_dom)     			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Reemplazar los argumentos de una inicial por otros.        *
* DESCRIPCION: Reemplaza los argumentos de <nom_inic> por los apuntados   *
*        por <cab_dom> en todas las ocurrencias de <nom_inic> en la expre-*
*        sion apunt. por <cab_expr>.                                      *
* PARAMETROS: 								  *
*   cad50 		nom_inic	Nombre de la inical a la cual hay *
*					que reemplazarle los args.	  *
*   nodo_expresion	*cab_expr	Ap. a la expresion en donde se    *
*					reemplazan los args. de <nom_inic>*
*					en todas las ocurrencias.	  *
*   nodo_argum 		*cab_dom	Ap. a los nuevos args. de         *
*					<nom_inic>.			  *
* PRE!:  { nom_inic = NI, cab_expr ap. a EXP, cab_dom ap. a CD          } *
* POST!: { cab_expr ap. a REEMP(EXP) que es la expresion que resulta de   *
*	   reemplazar en todas las ocurrencias de de <nom_inic> su dominio*
*	   por CD 							} *
* LLAMA A:								  *
*	void copie_lista_argum 		void reemplace_dom_inicial 	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
reemplace_dom_inicial (cad50 nom_inic, nodo_expresion * cab_expr,
		       nodo_argum * cab_dom)
{
/*----------------------------------------------------------------------*/

  nodo_argum *ncab_dom = NULL, *a;

  assert (nom_inic != NULL && cab_expr != NULL);	/* VTP. 7/May/1997 */
  if (!strcmp (cab_expr->nombre, nom_inic) &&
      !strcmp (cab_expr->clase, "INIC"))
    {
      copie_lista_argum (cab_dom, &ncab_dom, "", NULL,"");
      cab_expr->argumentos = ncab_dom;
    }
  for (a = cab_expr->argumentos; a != NULL; a = a->sig)
    {
      reemplace_dom_inicial (nom_inic, a->expr, cab_dom);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void reemplace_inicial(cad50 nom_inic,cad50 nom_final,		  *
*			 nodo_expresion *cabeza)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Reemplazar <nom_inic> por <nom_final> en la expresion ap.  *
*	       por <cabeza>.						  *
* DESCRIPCION: Hace un recorrido en preorden en la expresion apunt. por   *
*	 <cabeza> cambiando todas las ocurrencias de la inicial <nom_inic>*
*	 por <nom_final> 						  *
* PARAMETROS:                                                             *
*   cad50 	   nom_inic	Nombre original a ser cambiado.		  *
*   cad50 	   nom_final    Nuevo nombre.				  *
*   nodo_expresion *cabeza 	Ap. a la expresion donde debe hacerse el  *
*				cambio.					  *
* PRE!:  { nom_inic = NI, nom_final = NF, cabeza-> EXP.                 } *
* POST!: { cabeza-> REEMP(EXP) que es la expresion que resulta de reempla-*
*          todas las ocurrencias de NI por NF en EXP.                   } *
* LLAMA A: void reemplace_inicial					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
reemplace_inicial (cad50 nom_inic, cad50 nom_final, nodo_expresion * cabeza)
{
/*----------------------------------------------------------------------*/

  nodo_argum *a;

  assert (nom_inic != NULL && nom_final != NULL);	/* VTP. 7/May/1997 */
  if (cabeza != NULL)
    {
      if (!strcmp (cabeza->nombre, nom_inic) &&
	  !strcmp (cabeza->clase, "INIC"))
	strcpy (cabeza->nombre, nom_final)
	  ;
      for (a = cabeza->argumentos; a != NULL; a = a->sig)
	reemplace_inicial (nom_inic, nom_final, a->expr)
	  ;
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void reemplace_var(cad20 var_vieja,cad20 var_nueva,nodo_expresion *exp) *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Reemplazar <var_vieja> por <var_nueva> en la expresion ap. *
*	       por <exp>.						  *
* DESCRIPCION: Hace un recorrido en preorden en la expresion apunt. por   *
*	 <exp> cambiando todas las ocurrencias de la variable <var_vieja> *
*	 por <var_nueva> 						  *
* PARAMETROS:                                                             *
*   cad50 	   var_vieja	Nombre original a ser cambiado.		  *
*   cad50 	   var_nueva    Nuevo nombre.				  *
*   nodo_expresion *exp 	Ap. a la expresion donde debe hacerse el  *
*				cambio.					  *
* PRE!:  { var_vieja = VV, var_nueva = VN, exp-> EXP.                 	} *
* POST!: { exp-> REEMP(EXP) que es la expresion que resulta de reemplazar *
*          todas las ocurrencias de VV por VN en EXP.                   } *
* LLAMA A: void reemplace_var    					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
reemplace_var (cad20 var_vieja, cad20 var_nueva, nodo_expresion * exp)
{
/*----------------------------------------------------------------------*/

  nodo_argum *a;

  assert (exp != NULL && var_nueva != NULL);	/* VTP. 7/May/1997 */
  if ((!strcmp (exp->clase, "VAR")) && (!strcmp (exp->nombre, var_vieja)))
    {
      strcpy (exp->nombre, var_nueva);
    }
  for (a = exp->argumentos; a != NULL; a = a->sig)
    {
      reemplace_var (var_vieja, var_nueva, a->expr);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void renombre_var_inic(nodo_argum *arg,nodo_argum *cab_dom,		  *
*		         nodo_expresion *cab_ld_ax,char componente)	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Renombrar variables que son args. de una inicial.          *
* DESCRIPCION: Dados los argumentos ARGC de una inicial compuesta 	  *
*	INIC = INIC1-INIC2 y los argumentos ARG de una de sus componentes *
*	INICi ( 1 <= i <= 2),  reemplaza en la expresion apunt. por       *
*	<cab_ld_ax> las ocurrencias de las variables de ARG por las corres*
*	pondientes en ARGC.					          *
*       Esto se hace cuando se esta transformando un axioma de un tad, en *
*	un axioma de un tad producto Cartesiano que lo involucra.	  *
* PARAMETROS:                                                             *
*   nodo_argum 		*arg  		Ap. a los args. de la inicial que *
*					deben ser reemplazados.		  *
*   nodo_argum 		*cab_dom	Ap. a los args. de la inicial com-*
*					puesta.                    	  *
*   nodo_expresion 	*cab_ld_ax 	Ap. al lado derecho del axioma en *
*					transformacion.			  *
*   char	    	componente	'1' Si i = 1			  *
*    					'2' Si i = 2			  *
* PRE!:  { arg -> ARG, cab_dom-> ARGC, cab_ld_ax-> EXP, componente = C  } *
* POST!: { cab_ld_ax-> REEMP(EXP), que es al expresion que resulta de reem*
*	   plazar las ocurrencias de las variables en ARG por las corres- *
*	   pondientes (de acuerdo a C) en ARGC en EXP.			} *
* LLAMA A:                                                                *
*	short	longitud_lista_argum 		void	reemplace_var	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
renombre_var_inic (nodo_argum * arg, nodo_argum * cab_dom,
		   nodo_expresion * cab_ld_ax, char componente)
{
/*----------------------------------------------------------------------*/

  nodo_argum *prim, *a;
  int i, k;

  assert (componente == '1' || componente == '2');	/* VTP. 7/May/1997 */
  if (componente == '1')
    {
      prim = cab_dom;
    }
  else
    {
      i = longitud_lista_argum (cab_dom) - longitud_lista_argum (arg);
      for (prim = cab_dom, k = 1; k <= i; prim = prim->sig, k++)
	;
    }
  for (a = arg; a != NULL; a = a->sig, prim = prim->sig)
    {
      if (strcmp (a->expr->nombre, prim->expr->nombre))
	{
	  reemplace_var (a->expr->nombre, prim->expr->nombre, cab_ld_ax);
	}
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void transf_axioma_pc(nodo_select *s,nodo_igualdad **ult_ax,		  *
*  		nodo_argum *l_izq, nodo_expresion *cab_li_ax,     	  *
*   		nodo_expresion *cab_ld_ax,char componente,short nivel)    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Transformar un axioma de una selectora de un tad T en el co*
*	       rrespondiente del producto Cartesiano Explicito.		  *
* DESCRIPCION: Genera los axiomas de la selectora s de un tad producto Car*
*        tesiano T de dos tads T1 y T2 tq. s es heredada del tad Ti y se  *
*        define sobre los elementos de Ti, para i = 1,2 		  *
*	 Transforma los axiomas conocidos de s sobre los elementos de Ti: *
*	 Si gi es una generadora heredada de Ti y t es un objeto de T y   *
*	 se tiene el axioma s(...(g(...ti...)...) = ld entonces:	  *
*	 s(...gi(...t...)...)  = ldi, donde ldi es isomorfo a ld pero se  *
*	 ha reemplazado g por gi y ti por t				  *
* PARAMETROS:                                                             *
*    nodo_select    *s		Selectora a la cual se le definen los axs.*
*    nodo_igualdad  **ult_ax 	(*ult_ax) apunt. al ultimo axioma transfor*
*				mado de s.				  *
*    nodo_argum     *l_izq      Ap. a un argumento del lado izq. del axio-*
*				ma en transformacion.			  *
*    nodo_expresion *cab_li_ax  Ap. a la cabeza del lado izq. del axioma  *
*				transformado. 				  *
*    nodo_expresion *cab_ld_ax  Ap. a la cabeza del lado der. del axioma  *
*				transformado. 				  *
*    char	    componente	'1' Si i = 1				  *
*    				'2' Si i = 2				  *
*    short          nivel	Indica el nivel en el arbol sintactico del*
*				lado izq. del axioma original que esta    *
*				siendo analizado.			  *
* PRE!:  { ult_tad-> T = T1 x T2 tads del ambiente,			  *
*	   T->selectoras = <...,s,...>, s es una selectora heredada de Ti *
*	   s->axiomas = <a1,...,ak> , k >= 0			 	  *
* POST!: { s->axiomas = <a1,...,ak,ac1,...,acl> l >= 0, donde ac1,...,acl *
*	  es la lista de axiomas que definen a s sobre los elementos de Ti*
* LLAMA A: void transf_axioma_pc					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
transf_axioma_pc (nodo_select * s, nodo_igualdad ** ult_ax, nodo_argum * l_izq,
		  nodo_expresion * cab_li_ax, nodo_expresion * cab_ld_ax,
		  char componente, short nivel)
{
/*----------------------------------------------------------------------*/

  nodo_expresion *ncab_li_ax = NULL, *ncab_ld_ax = NULL;
  nodo_igualdad *nuevo_ax;
  nodo_argum *nl_izq = NULL, *a;
  nodo_generadora *i;

/*** Alvaro
  texpr cad;
 ***/
  assert (s != NULL && ult_ax != NULL && (componente == '1' || componente == '2'));	/* VTP. 7/May/1997 */

  if (l_izq == NULL)
    {
      if (nivel == 0)
	{
	  nuevo_ax = (nodo_igualdad *) malloc (sizeof (nodo_igualdad));
	  HayMem (nuevo_ax, sizeof (nodo_igualdad));
	  nuevo_ax->lado_izq = cab_li_ax;
	  nuevo_ax->lado_der = cab_ld_ax;
	  nuevo_ax->sig = NULL;
	  transf_estruct_iguald (nuevo_ax);
	  if (*ult_ax == NULL)
	    s->axiomas = nuevo_ax;
	  else
	    (*ult_ax)->sig = nuevo_ax
	      ;
	  *ult_ax = nuevo_ax;
	}
    }
  else
    {
      if ((!strcmp ((l_izq->expr)->clase, "INIC")) &&
	  (!strcmp ((l_izq->expr)->tipo, ult_tad->nombre)))
	{
	  for (i = ult_tad->iniciales; i != NULL; i = i->sig)
	    {
	      copie_lista_expresion_apunt (cab_li_ax, &ncab_li_ax,
					   l_izq, &nl_izq);
	      strcpy (nl_izq->expr->nombre, i->nombre);
	      copie_lista_expresion (cab_ld_ax, &ncab_ld_ax, "", NULL,"");
	      reemplace_inicial (l_izq->expr->nombre,
				 i->nombre, ncab_ld_ax);
	      arregle_dom_inicial (i, nl_izq, ncab_ld_ax, componente);
	      transf_axioma_pc (s, ult_ax, nl_izq->sig, ncab_li_ax,
				ncab_ld_ax, componente, nivel);
	    }
	}
      else
	{
	  for (a = l_izq->expr->argumentos; a != NULL; a = a->sig)
	    {
	      transf_axioma_pc (s, ult_ax, a, cab_li_ax, cab_ld_ax, componente,
				(short) (nivel + 1));
	    }
	  transf_axioma_pc (s, ult_ax, l_izq->sig, cab_li_ax, cab_ld_ax,
			    componente, nivel);
	}
    }
/*----------------------------------------------------------------------*/
}
