/* ED_TIPOS.C                                                   */
/* Arhivo de manejo de edicion de tipos.                        */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include "os_dep.h"		/* Agregada por VTP en 7/Mar/1997 */
#include "definic.h"
 
/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* v-tamara 7/May/1997 */



/**************************************************************************
* int analice_cambios(void)  						  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Analizar las modificaciones hechas a la signatura de un tad*
*	       existente en el ambiente.    				  *
* DESCRIPCION: Analiza cada modificacion del archivo que contenia la signa*
*        tura. Si encuentra algun error, lo informa al usuario y permite  *
*	 que edite nuevamente el archivo.				  *
* PRE!:  { tad -> <t1,...,tk,...,tn> <- ult_tad, donde t1,...,tn son los  *
*	   tads en el ambiente, n >= 2, tad_actual-> tk, 1 <= k <= n 	  *
*	   "signa_ed.tad" contiene la signatura modificada SM de tk } *
* POST!: { Si la signatura en el arch. "signa_ed.tad" es correcta,    *
*             tad -> <t1,..,t(k-1),t(k+1)..,tn,t(n+1)> <- ult_tad, donde  *
*	      t(n+1) = MODIF(tk), esto es, la modificacion de la definic. *
*	      de tk usando la informacion del archivo,  o	 	  *
*	   Sino,							  *
*             tad -> <t1,...,tk,...,tn> <- ult_tad			} *
**************************************************************************/

int
analice_cambios (void)
{
/*----------------------------------------------------------------------*/
  nodo_select *sel;
  nodo_select *sel_fte;
  nodo_generadora *gen;
  int cambio_gen;
  int correcto;
/*** Alvaro
    cad6            siga;
 ***/



  cambio_gen = !iguales_parametros (tad_actual, tad_ed);
  for (gen = tad_ed->iniciales;
       !cambio_gen && gen != NULL; gen = gen->sig)
    {
      cambio_gen = (igual_generadora (tad_actual, gen, 0) == NULL);
    }
  for (gen = tad_ed->constructoras;
       !cambio_gen && gen != NULL; gen = gen->sig)
    {
      cambio_gen = (igual_generadora (tad_actual, gen, 1) == NULL);
    }
  if (!cambio_gen)
    {
/*-- Se editan unicamente los axiomas de las nuevas selectoras --*/
      for (sel = tad_ed->selectoras, correcto = 1;
	   sel != NULL && correcto; sel = sel->sig)
	{
	  sel_fte = igual_selectora (tad_actual, sel);
	  if (sel_fte != NULL)
	    copie_lista_igualdades (sel_fte->axiomas, &sel->axiomas, "", "", "",
				    tad_ed,"");
	  else
	    {
	      sel->axiomas = NULL;
	      if (tad_ed->tipo == RESTRICCION)
		correcto =
		  construya_axiomas (sel, 1, tad_ed->info_tipo->sig->nombre,
				     tad_ed->nombre,NULL);
	      else
		correcto = construya_axiomas (sel, 0, "", 
					      tad_ed->nombre,NULL);
	    }
	}
    }
  else
    {
      /*-- Se editan todos los axiomas por que hubo cambio en las generadoras
	y selectoras --*/
      correcto = edite_axiomas ();
    }
  if (correcto)
    correcto = edite_igualdad (1);
  return (correcto);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* int comparar_generadoras(nodo_generadora *g1, nodo_generadora *g2)      *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* POST : comparar_generadoras = 1 ssi g1 == g2                            *
**************************************************************************/

int
comparar_generadoras (nodo_generadora * g1, nodo_generadora * g2)
{
/*----------------------------------------------------------------------*/
  nodo_nombre *d1, *d2;

  if (strcmp (g1->nombre, g2->nombre))
    return (0);
  for (d1 = g1->dominio, d2 = g2->dominio; d1 != NULL && d2 != NULL;
       d1 = d1->sig, d2 = d2->sig)
    {
      if (strcmp (d1->nombre, d2->nombre))
	return (0);
    }
  if (d1 == NULL && d2 == NULL)
    return (1);
  return (0);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* int comparar_selectoras(nodo_select *s1, nodo_select *s2)               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* POST : comparar_selectoras = 1 ssi s1 == s2 (en la signatura).          *
**************************************************************************/

int
comparar_selectoras (nodo_select * s1, nodo_select * s2)
{
/*----------------------------------------------------------------------*/
  nodo_dom *d1, *d2;

  assert (s1 != NULL && s2 != NULL);	/* VTP. 7/May/1997 */
  if (strcmp (s1->nombre, s2->nombre) ||
      strcmp (s1->codominio, s2->codominio))
    return (0);
  for (d1 = s1->dominio, d2 = s2->dominio; d1 != NULL && d2 != NULL;
       d1 = d1->sig, d2 = d2->sig)
    {
      if (strcmp (d1->nombre, d2->nombre) ||
	  d1->induccion != d2->induccion)
	return (0);
    }
  if (d1 == NULL && d2 == NULL)
    return (1);
  return (0);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void cree_arch_edicion_signatura(texpr nombre)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Crear la plantilla de edicion de un nuevo tad.             *
* DESCRIPCION: Crea una plantilla para la definicion inductiva de la signa*
*	 tura de un nuevo tad.						  *
* PARAMETROS:                                                             *
*    texpr	nombre		Nombre del nuevo tad.			  *
* PRE!:  { nombre = NT.                                                 } *
* POST!: { el archivo "signa_ed.tad" contiene la plantilla para la    *
*          creacion de la signatura del tad NT.                         } *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
cree_arch_edicion_signatura (texpr nombre)
{
/*----------------------------------------------------------------------*/

  FILE *fp;
  assert (nombre != NULL);	/* VTP. 7/May/1997 */
  fp = fopen ("signa_ed.tad", "w");
  FileOpen (fp, "signa_ed.tad");
  fprintf (fp, "%s %s [ /* T1,T2 */ ]\n\n", claves[0], nombre);
  fprintf (fp, "%s\n", claves[1]);
  fprintf (fp, "/* ini :  T1    * T2    -> %s */\n\n", nombre);
  fprintf (fp, "%s\n", claves[2]);
  fprintf (fp, "/* con : %s    * T1    * T2    -> %s */\n\n", nombre, nombre);
  fprintf (fp, "%s\n", claves[3]);
  fprintf (fp, "/* sel (%s): %s (%s)    * T1    * T2    -> T2 */\n\n",
	   claves[13], nombre, claves[14]);
  fprintf (fp, "EQ_%s (%s): %s (%s)    * %s (%s)    -> BOOL\n\n",
	   nombre, claves[13], nombre, claves[14], nombre, claves[14]);
  fclose (fp);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void ed_arch_signatura(FILE *fp,nodo_tad *p)			    	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Escribir la signatura de un tad en un archivo para edicion.*
* DESCRIPCION: Escribe el nombre, la informacion del tipo y las operacio- *
*        nes de la signatura de <p> en el archivo apunt. por fp.          *
* PARAMETROS:                                                             *
*   FILE	*fp		Ap. al archivo de edicion. 		  *
*   nodo_tad	*p		Ap. al tad cuya signatura debe ser escrita*
*				en el archivo.			  	  *
* PRE!:  { fp ->[////],                                                   *
*          tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2.					  *
*	   p-> tk, 1 <= k <= n.						} *
* POST!: { fp ->[////							  *
*	         <tk.nombre>						  *
*		 <tk.info_tipo> 					  *
*		 claves[1]: <lista de iniciales de p>			  *
*		 claves[2]: <lista de constructoras de p>		  *
*		 claves[3]: <lista de selectoras de p>			  *
*	        ],  donde para cada operacion se escribe su nombre y su   *
*		    funcionalidad.					} *
* LLAMA A:								  *
*   void  ed_escriba_nom_parametros 	void  ed_escriba_info_tipo	  *
*   void  ed_escriba_generadoras 	void  ed_escriba_selectoras	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
ed_arch_signatura (FILE * fp, nodo_tad * p)
{
/*----------------------------------------------------------------------*/
  nodo_tad *t;

  assert (fp != NULL && p != NULL);	/* VTP. 7/May/1997 */
  ed_escriba_nom_parametros (fp, p);
  ed_escriba_info_tipo (fp, p);
  fprintf (fp, "\n%s\n", claves[1]);
  if (p->tipo == RESTRICCION)
    t = tad_restriccion (p);
  else
    t = p;
  ed_escriba_generadoras (fp, p->iniciales, t->nombre);
  fprintf (fp, "\n%s\n", claves[2]);
  ed_escriba_generadoras (fp, p->constructoras, t->nombre);
  fprintf (fp, "\n%s\n", claves[3]);
  ed_escriba_selectoras (fp, p->selectoras);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void ed_escriba_generadoras(FILE *fp,nodo_generadora *cabeza,		  *
*			      cad12 nombre_tad)	                 	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Escribir una lista de generadoras en un arch. para edicion.*
* DESCRIPCION: Escribe el nombre y la funcionalidad de las generadoras de *
*	 la lista apunt. por <cabeza> en el archivo apunt. por fp.        *
* PARAMETROS:                                                             *
*   FILE		*fp		Ap. al archivo de edicion.	  *
*   nodo_generadora	*cabeza		Ap. a la lista de generadoras.	  *
*   cad12		nombre_tad	Nombre del tad codominio de la ge-*
*					neradoras.			  *
* PRE!:  { fp ->[////],                                                   *
*	   cabeza-> <g1,...,gn>, n >= 0,
*	   nombre_tad = T, nombre del tad que posee a g1,..,gn como genera*
*	   doras.   							  *
* POST!: { fp ->[////							  *
*		 <g1.nombre> : <g1->dominio> -> <T>	        	  *
*		 ...							  *
*		 <gn.nombre> : <gn->dominio> -> <T>]              }       *
* LLAMA A: void ed_escriba_nombres					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
ed_escriba_generadoras (FILE * fp, nodo_generadora * cabeza,
			cad12 nombre_tad)
{
/*----------------------------------------------------------------------*/

  texpr linea;

  assert (fp != NULL && nombre_tad != NULL);	/* VTP. 7/May/1997 */

  for (; cabeza != NULL; cabeza = cabeza->sig)
    {
      fprintf (fp, "%-12s: ", cabeza->nombre);
      ed_escriba_nombres (linea, cabeza->dominio, " * ");
      fprintf (fp, "%-32s -> %s\n", linea, nombre_tad);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void ed_escriba_igualdades(FILE *fp,nodo_igualdad *cabeza)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Escribir una lista de igualdades  en un arch. para edicion.*
* DESCRIPCION: Escribe las igualdades  de la lista apunt. por <cabeza> en *
*	  el archivo apunt. por fp.        				  *
* PARAMETROS:                                                             *
*   FILE		*fp		Ap. al archivo de edicion.	  *
*   nodo_select		*cabeza		Ap. a la lista de igualdades .	  *
* PRE!:  { fp ->[////],                                                   *
*	   cabeza-> <ig1,...,ign>, n >= 0,				  *
* POST!: { fp ->[////							  *
*	         <ig1->li_str> = <ig1->ld_str>				  *
*		 ...							  *
*	         <ign->li_str> = <ign->ld_str>]			  	} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
ed_escriba_igualdades (FILE * fp, nodo_igualdad * cabeza)
{
/*----------------------------------------------------------------------*/

  assert (fp != NULL);		/* VTP. 7/May/1997 */
  for (; cabeza != NULL; cabeza = cabeza->sig)
    {
      fprintf (fp, " %s=%s\n", cabeza->li_str, cabeza->ld_str);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void ed_escriba_info_tipo(FILE *fp,nodo_tad *p)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Escribir el tipo de un tad en un archivo para edicion.	  *
* DESCRIPCION: Escribe la informacion de tipo de <p> en el archivo apunt. *
*	por fp.          						  *
* PARAMETROS:                                                             *
*   FILE	*fp		Ap. al archivo de edicion. 		  *
*   nodo_tad	*p		Ap. al tad cuya info_tipo debe ser escrita*
*				en el archivo.			  	  *
* PRE!:  { fp ->[////], p-> T						} *
* POST!: { fp ->[////<informacion de tipo de p>], donde:		  *
*		<informacion de tipo de p> depende de p->tipo asi:	  *
*		- NORMAL         : ""*
*		- PROD_CARTESIANO: <nom padre 1> x <nom padre 2>	  *
*		- RESTRICCION    : <nom padre> : <predic. de restriccion> *
*		- COMPLETACION   : <nom padre> + <nom elem. de completac.>*
*		- NT_EXTENSION   : <nom padre 1> + <nom padre 2>  	  *
*	        - VARIACION      : <nom padre 1> U <nom padre 2>  	} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90
MODIFICAIONES:
VTP 2/May/1998 Now the grammar don't supports =T1 x T2 or things like
that in the ADT's name, now it has to be:
ADT nombre [parameters]
***************************************************************************/

void
ed_escriba_info_tipo (FILE * fp, nodo_tad * p)
{
/*----------------------------------------------------------------------*/

  cad50 lin_info_tipo;

  assert (p != NULL && fp != NULL);	/* VTP. 7/May/1997 */
  switch (p->tipo)
    {
    case NORMAL:
      strcpy (lin_info_tipo, "\n");
      break;
    case PROD_CARTESIANO:
      sprintf (lin_info_tipo, "\n/*  =  %s x %s */\n",
	       p->info_tipo->nombre,
	       p->info_tipo->sig->nombre);
      break;
    case RESTRICCION:
      sprintf (lin_info_tipo, "\n/*  =  ( %s : %s )*/\n",
	       p->info_tipo->nombre,
	       p->info_tipo->sig->nombre);
      break;
    case COMPLETACION:
      sprintf (lin_info_tipo, "\n/*  =  %s + { %s }*/\n",
	       p->info_tipo->nombre,
	       p->info_tipo->sig->nombre);
      break;
    case NT_EXTENSION:		/* VTP.  7/Mar/1997 */
      sprintf (lin_info_tipo, "\n/*  =  %s + %s */\n",
	       p->info_tipo->nombre,
	       p->info_tipo->sig->nombre);
      break;
    case VARIACION:
      sprintf (lin_info_tipo, "\n/*  =  %s U %s */\n",
	       p->info_tipo->nombre,
	       p->info_tipo->sig->nombre);
      break;
    case REPRESENTACION:
      sprintf (lin_info_tipo, "\n/*  %s : %s -> %s */\n",
	       p->nombre,
	       p->info_tipo->nombre,
	       p->info_tipo->sig->nombre);
      break;
    default:
      strcpy (lin_info_tipo, "\n");
      break;
    }
  fprintf (fp, lin_info_tipo);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void ed_escriba_nom_parametros(FILE *fp,nodo_tad *p)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Escribir una lista de parametros  en un arch. para edicion.*
* DESCRIPCION: Escribe el nombre de los parametros del tad <p>.		  *
* PARAMETROS:                                                             *
*   FILE		*fp		Ap. al archivo de edicion.	  *
*   nodo_tad		*p		Ap. al tad del cual se van a es-  *
*					cribir los parametros en el arch. *
* PRE!:  { fp ->[////],                                                   *
*	   p->parametros = <p1,...,pn>, n >= 0.                         } *
* POST!: { fp ->[////claves[0] <p->nombre [<p1> , <p2>...... , <pm>] ] }  *
* LLAMA A: void ed_escriba_nombres					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
ed_escriba_nom_parametros (FILE * fp, nodo_tad * p)
{
/*----------------------------------------------------------------------*/

  texpr linea;
  assert (fp != NULL && p != NULL && p->nombre != NULL);	/* VTP. 7/May/1997 */
  ed_escriba_nombres (linea, p->parametros, " , ");
  fprintf (fp, "%s %s [ %s ]", claves[0], p->nombre, linea);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void ed_escriba_nombres(texpr linea,nodo_nombre *cabeza,texpr separador)*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Escribir una lista de nombres en un string.                *
* DESCRIPCION: Escribe la lista de nombres apunt. por <cabeza> , separados*
*	 <separador> en la cadena <linea>.		  		  *
* PARAMETROS:                                                             *
*   texpr		linea		Cadena donde debe quedar escrita  *
*					la lista de nombres.        	  *
*   nodo_nombre		*cabeza		Ap. a la lista de nombres.    	  *
*   texpr 		separador	Caracteres para separar los nom-  *
*					bres de la lista.		  *
* PRE!:  { cabeza-> <n1,...,nm>, m >= 0, separador = S			} *
* POST!: { linea = "<n1> <separador> <n2>......<separador> <nm>"        } *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
ed_escriba_nombres (texpr linea, nodo_nombre * cabeza, texpr separador)
{
/*----------------------------------------------------------------------*/

  strcpy (linea, "");
  for (; cabeza != NULL; cabeza = cabeza->sig)
    {
      strcat (linea, cabeza->nombre);
      if (cabeza->sig != NULL)
	strcat (linea, separador)
	  ;
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void ed_escriba_selectoras(FILE *fp,nodo_select *cabeza)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Escribir una lista de selectoras  en un arch. para edicion.*
* DESCRIPCION: Escribe el nombre y la funcionalidad de las selectoras  de *
*	 la lista apunt. por <cabeza> en el archivo apunt. por fp.        *
* PARAMETROS:                                                             *
*   FILE		*fp		Ap. al archivo de edicion.	  *
*   nodo_select		*cabeza		Ap. a la lista de selectoras .	  *
* PRE!:  { fp ->[////],                                                   *
*	   cabeza-> <s1,...,sn>, n >= 0,
*	   nombre_tad = T, nombre del tad que posee a g1,..,gn como genera*
*	   doras.   							  *
*	   operac = O, O = 'I' si las operaciones son iniciales o,	  *
*	               O = 'C' si las operaciones son constructoras	} *
* POST!: { fp ->[////							  *
*  (S) <s1.nombre><s1.caracterizadora> : <s1->dominio> -> <S1.codominio>  *
*		 ....							  *
*  (S) <sn.nombre><sn.caracterizadora> : <sn->dominio> -> <Sn.codominio>]}*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
ed_escriba_selectoras (FILE * fp, nodo_select * cabeza)
{
/*----------------------------------------------------------------------*/

  texpr linea;
  nodo_dom *q;

  for (; cabeza != NULL; cabeza = cabeza->sig)
    {
      fprintf (fp, "%-9s", cabeza->nombre);
      if (cabeza->caracterizadora)
	fprintf (fp, "(%s): ", claves[13]);
      else
	fprintf (fp, "   : ")
	  ;
      strcpy (linea, "");
      for (q = cabeza->dominio; q != NULL; q = q->sig)
	{
	  strcat (linea, q->nombre);
	  if (q->induccion)
	    {
	      strcat (linea, "(");
	      strcat (linea, claves[14]);
	      strcat (linea, ")");
	    }
	  if (q->sig != NULL)
	    strcat (linea, " * ");
	}
      fprintf (fp, "%-31s  -> ", linea);
      fprintf (fp, "%s\n", cabeza->codominio);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void edite_axs_existente(int num_menu_axs,nodo_select *cab_ops)	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Editar los axiomas existentes de un tad.                   *
* DESCRIPCION: Edita los axiomas del tad_ed. Permite consultar y/o modifi-*
*        car (en caso de que el status de tad_ed sea MODIFICABLE) cual-   *
*	 quier cjto. de axiomas del tad.				  *
* PARAMETROS:								  *
*    int	num_menu_axs	Determina si se despliega el menu de axio-*
*				mas de una selectora de un tad o de una   *
*				operacion aux. de una representacion.	  *
*    nodo_select *cab_ops	Ap. a la lista de selectoras u operacs.aux*
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_ed    	Apunt. al tad actual/. en edicion.        *
* PRE!:  { num_menu_axs = AXIOMAS o AXIOMAS_REPRESENT,			  *
*	   cab_ops-> <op1,...,opn>, n >= 0, opi->axiomas = AXSi i=1,..,n }*
* POST!: { opi->axiomas = MODIF(AXSi) esto es una posible version diferent*
*	   pero correcta de AXSi.					} *
**************************************************************************/

void
edite_axs_existente (int num_menu_axs, nodo_select * cab_ops)
{
/*----------------------------------------------------------------------*/

  texpr nombre;
  nodo_nombre *lis;
  nodo_select *s;
/*** Alvaro
    cad6        siga;
 ***/

  lis = lista_selectoras (cab_ops);
  admon_menu_scroll (menus_manta[num_menu_axs], lis, nombre);
  while (strcmp (nombre, "") && strcmp (nombre, mensajes[150]))
    {
      s = busque_selectora (nombre, cab_ops);
      if (s != NULL)
	edite_axiomas_selectora (tad_ed, s);
      else			/* Se deben editar todos los axiomas del tad_actual */
	edite_todos_axiomas_selectora (cab_ops);
      admon_menu_scroll (menus_manta[num_menu_axs], lis, nombre);
    }
  liberar_nombres (&lis);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* int edite_igualdad(short pida_axiomas)				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Sugerir que se defina el predicado de iguadad para un tad. *
* DESCRIPCION: Verifica que el tad_ed tenga definido el pred. de igualdad.*
*        Si no lo tiene, sugiere al usuario que lo defina y pide sus axio-*
*	 mas si <pida_axiomas> = 1.	                                  *
*        Retorna 1 si pudo definirlo, 0 si no				  *
* PARAMETROS:                                                             *
*   short	pida_axiomas	1, Si se deben pedir inmediata/. los axiom*
*                               0, Sino.				  *
**************************************************************************/

int
edite_igualdad (short pida_axiomas)
{
/*----------------------------------------------------------------------*/

  nodo_select *s = NULL;
  short c;
/*** Alvaro
    cad6      resp;
  ***/

  /* VTP. 5/May/1998.  Now extensions don't have equality */
  assert(tad_ed!=NULL);

  c = 1;
  if (!tiene_igualdad_definida (tad_ed) && 
      (tad_ed->iniciales!=NULL || tad_ed->constructoras!=NULL))
    {
      if (desea_def_igualdad ())
	{
	  adic_select_eq (tad_ed, &s);
	  if (pida_axiomas)
	    c = construya_axiomas (s, 0, "", tad_ed->nombre,NULL);
	}
    }
  return (c);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void edite_sign_existente()						  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Editar la signatura de un tad existente.			  *
* DESCRIPCION: Edita la signatura del tad_ed. Permite consultar y/o modifi*
*        carla, en caso de que el status de tad_ed sea MODIFICABLE.	  *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_actual	Apunt. al tad actual de la lista de tads  *
*				del ambiente.				  *
* PRE!:  { tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2.  tad_actual-> tk = T,  1 <= k <= n.  } *
* POST!: { tk  = MODIF(T) es una posible version modificada pero correcta *
*	   de T, cuya signatura puede haber cambiado y causado alteracio- *
*	   nes en los axiomas.						} *
* LLAMA A: 	void	esc_arch_edicion_signatura			  *
*	                  		int	system			  *
*	short   arch_modificado 	void	analice_cambios	   	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Mar-90 *
* MODIFICACIONES:
* VTP 13/Ene/97
*	Antes borraba un archivo empleando system("rm ... ahora
*	se usa unlink (portable para DOS y UNIX)
**************************************************************************/

void
edite_sign_existente ()
{
/*----------------------------------------------------------------------*/
  texpr /*** Alvaro comando ,***/ mens_error;
  int correcto;

/*** Alvaro
    cad6  siga;
  ***/
  trio_edicion edicion[MAX_EDICION];


  esc_arch_edicion_signatura (tad_actual);
  edicion[0].linea = 1;
  strcpy (edicion[0].nombre, "signa_ed.tad");
  if (tad_actual->status == 1)
    {				/* es TAD inmodificable */
      edicion[0].modo = 0;
      llamar_al_editor (1, edicion, "");
    }
  else
    {				/* es TAD modificable */
      edicion[0].modo = 1;
      llamar_al_editor (1, edicion, "");
      if (arch_modificado ("signa_ed.tad"))
	{
	  tad_ed = crear_tad ();
	  while ((!(correcto = analizador_sintactico (mens_error))) &&
		 (desea_corregir_edicion (1, mens_error)))
	    {
	      edicion[0].linea = num_ed;
	      llamar_al_editor (1, edicion, "");
	    }
	  if (correcto)
	    {
	      correcto = analice_cambios ();
	      if (correcto)
		{
		  reemplace_def (tad_actual, tad_ed);
		  liberar_tad (&tad_actual);
		  tad_ed->modif = 1;
		  tad_actual = tad_ed;
		}
	      else
		{
		  liberar_tad (&tad_ed);
		  tad_ed = tad_actual;
		}
	    }
	}
    }
/***VTP. Decia   system("rm -f signa_ed.tad");   ***/
  unlink ("signa_ed.tad");
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void edite_tad_existente()						  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Editar un tad existente.			  		  *
* DESCRIPCION: Edita la signatura,los axiomas y/o el status del tad_ed.   *
*	 Permite consultar y/o modificar la definicion de tad_ed, solo si *
*	 el status de tad_ed es MODIFICABLE.	  			  *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_actual	Apunt. al tad actual de la lista de tads  *
*				del ambiente.				  *
*    nodo_tad	*tad_ed      	Apunt. a una copia del tad actual.	  *
* PRE!:  { tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2.  tad_actual-> tk = T,  1 <= k <= n.  } *
* POST!: { tk  = MODIF(T) es una posible version modificada pero correcta *
*	   de T, cuya signatura, axiomas o status pueden haber cambiado } *
* LLAMA A:                                                                *
*   void   edite_sign_existente                                 	  *
*   void   edite_axs_existente 		void	edite_stat_existente	  *
*   void   edite_todo_tad_existente 	 			          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Mar-90 *
**************************************************************************/

void
edite_tad_existente ()
{
/*----------------------------------------------------------------------*/

  int opcion;
  texpr merr;

  opcion = admon_menu (menus_manta[EDICION]);
  while (opcion != NO_OPCION &&
	 opcion != menus_manta[EDICION]->num_op - 2)
    {
      tad_ed = tad_actual;
      switch (opcion)
	{
	case 0:		/* Debe editarse la Signatura */
	  edite_sign_existente ();
	  break;
	case 1:		/* Debe editarse un Axioma */
	  edite_axs_existente (AXIOMAS, tad_ed->selectoras);
	  break;
	case 2:		/* Debe editarse el Status */
	  edite_stat_existente ();
	  break;
	case 3:		/* Debe mostrarse la Signatura y los Axiomas */
	  edite_todo_tad_existente ();
	  break;
	case 5:
	  escriba_mensaje (156);
	  break;
	default:
	  escriba_mensaje (146);
	  break;
	}
      /* VTP. 4/May/1998. Administration of symbol table*/
      if (tad_ed!=NULL && updateSymbols(&symtable,tad_ed,merr)!=0)
	{
	  escriba_cadena(merr,1);
	}
      opcion = admon_menu (menus_manta[EDICION]);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void edite_tad_nuevo(cad20 nombre)					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Crear un nuevo tad inductivamente.                         *
* DESCRIPCION: Crea la plantilla para edicion de un tad, invoca al editor *
*        de texto y analiza sintacticamente el archivo. Si es correcto,   *
*        crea un nuevo tad con esa informacion.                           *
* PARAMETROS:                                                             *
*    cad20	nombre  	Nombre del tad que va a ser creado.       *
* V.GLOBALES:                                                             *
*    nodo_tad	*ult_tad	Apunt. al ultimo tad de la lista de tads  *
*				del ambiente.				  *
*    nodo_tad	*tad_actual	Apunt. al tad actual de la lista de tads  *
*				del ambiente.				  *
*    nodo_tad 	*tad_ed    	Ap. a la estructura que contiene la infor-*
*				macion analizada hasta el momento de la   *
*				signatura contenida en el archivo *fp.    *
* PRE!:  { tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2, 					} *
* POST!: { tad -> <t1,t2,...,tn,nt> <- ult_tad, donde nt es un nuevo tad  *
*	   definido por el usuario., tad_actual = tad_ed -> nt          } *
* LLAMA A: void	cree_arch_edicion_signatura 	  			  *
*    short	arch_modificado		short	analizador_sintactico	  *
*    short	desea_corregir_edicion	void	edite_igualdad		  *
*    short	edite_axiomas		void	informacion_tipo_normal   *
*    void	esc_nom_tad_actual				  	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Mar-90 *
* VTP 7/Mar/1997
*	Antes borraba un archivo empleando system("rm ... ahora
*	se usa unlink (portable para DOS y UNIX)
**************************************************************************/

void
edite_tad_nuevo (cad20 nombre)
{
/*----------------------------------------------------------------------*/

  texpr mens_error;
/*** Alvaro,comando;***/
  int correcto;

 /*** Alvaro
  cad6     resp;
  ***/
  trio_edicion edicion[MAX_EDICION];

  cree_arch_edicion_signatura (nombre);
  strcpy (edicion[0].nombre, "signa_ed.tad");
  edicion[0].modo = 1;
  edicion[0].linea = 1;
  llamar_al_editor (1, edicion, "");

  if (arch_modificado ("signa_ed.tad"))
    {
      tad_ed = crear_tad ();
      while ((!(correcto = analizador_sintactico (mens_error))) &&
	     (desea_corregir_edicion (1, mens_error)))
	{
	  edicion[0].linea = num_ed;
	  llamar_al_editor (1, edicion, "");
	}
      if (correcto)
	{
	  edite_igualdad (0);
	  if (edite_axiomas ())
	    {
	      correcto = 1;
	      tad_ed->status = pida_status ();
	      informacion_tipo_normal (tad_ed);
	      ult_tad->sig = tad_ed;
	      ult_tad = tad_ed;
	      tad_actual = tad_ed;
	      esc_nom_tad_actual ();
	    }
	  else
	    correcto = 0;
	}
      if (!correcto)
	{
	  liberar_tad (&tad_ed);
	  tad_ed = tad_actual;
	}
    }
/*** VTP. Decia system("rm -f signa_ed.tad"); ***/
  unlink ("signa_ed.tad");
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void edite_todo_tad_existente()					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Imprimir un archivo con la informacion del tad_ed.         *
* DESCRIPCION: Escribe en el archivo "tad_ed.tad" la informacion del tad  *
*	 tad_ed y lo muestra al usuario sin permitir que lo modifique.	  *
* PRE!:  { tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2, tad_ed-> tk , 1 <= k <= n.           } *
* POST!: { Se "mostro" al usuario la signatura y los axiomas de tk.	} *
* LLAMA A:                                                                *
*   void 	ed_arch_signatura	void	ed_arch_axiomas           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-May-90 *
* MODIFICACIONES:
* VTP 13/Ene/97
*	Antes borraba un archivo empleando system("rm ... ahora
*	se usa unlink (portable para DOS y UNIX)
**************************************************************************/

void
edite_todo_tad_existente ()
{
/*----------------------------------------------------------------------*/

  FILE *fp;
/*** Alvaro
  texpr         comando;
  cad6          siga;
 ***/
  trio_edicion edicion[MAX_EDICION];

  fp = fopen ("tad_ed.tad", "w");
  FileOpen (fp, "tad_ed.tad");
  ed_arch_signatura (fp, tad_ed);
  fprintf (fp, "\n%s\n\n", claves[4]);
  ed_arch_axiomas_ops (fp, tad_ed->selectoras);
  fclose (fp);
  strcpy (edicion[0].nombre, "tad_ed.tad");
  edicion[0].modo = 0;
  edicion[0].linea = 1;
  llamar_al_editor (1, edicion, "");
/*** VTP. Decia:  system("rm -f tad_ed.tad"); ***/
  unlink ("tad_ed.tad");

/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void esc_arch_edicion_signatura(nodo_tad *p)				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Escribir la signatura de un tad en un archivo para edicion.*
* DESCRIPCION: Escribe el nombre, la informacion del tipo y las operacio- *
*        nes de la signatura de <p> en el archivo "signature_ed.adt"      *
* PARAMETROS:                                                             *
*   nodo_tad	*p		Ap. al tad cuya signatura debe ser escrita*
*				en el archivo.			  	  *
* PRE!:  { tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2.					  *
*	   p-> tk, 1 <= k <= n.						} *
* POST!: { "sinature_ed.tad" =
*	        [<tk.nombre>						  *
*		 <tk.info_tipo> 					  *
*		 claves[1] <lista de iniciales de p>			  *
*		 claves[2] <lista de constructoras de p>		  *
*		 claves[3] <lista de selectoras de p>			  *
*	        ],  donde para cada operacion se escribe su nombre y su   *
*		    funcionalidad.					} *
* LLAMA A: void	ed_arch_signatura					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
esc_arch_edicion_signatura (nodo_tad * p)
{
/*----------------------------------------------------------------------*/

  FILE *fp;
  fp = fopen ("signa_ed.tad", "w");
  FileOpen (fp, "signa_ed.tad");
  ed_arch_signatura (fp, p);
  fclose (fp);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* nodo_generadora *igual_generadora(nodo_tad *t, nodo_generadora *g,      *
*                                   int tipo)                             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Buscar una generadora en t que sea exactamente igual a g   *
*              Si tipo = 0 busca en las iniciales.                        *
*              Si tipo = 1 busca en las constructoras.                    *
**************************************************************************/

nodo_generadora *
igual_generadora (nodo_tad * t, nodo_generadora * g,
		  int tipo)
{
/*----------------------------------------------------------------------*/
  nodo_generadora *gen;
  assert (t != NULL);		/* VTP. 7/May/1997 */
  if (!tipo)
    gen = t->iniciales;
  else
    gen = t->constructoras;
  for (; gen != NULL && !comparar_generadoras (gen, g); gen = gen->sig);
  return (gen);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* nodo_select *igual_selectora(nodo_tad *t, nodo_select *s)               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Buscar una selectora en t que sea exactamente igual a s    *
*              en su signatura.                                           *
**************************************************************************/

nodo_select *
igual_selectora (nodo_tad * t, nodo_select * s)
{
/*----------------------------------------------------------------------*/
  nodo_select *sel;
  assert (t != NULL);		/* VTP. 7/May/1997 */
  for (sel = t->selectoras;
       sel != NULL && !comparar_selectoras (sel, s); sel = sel->sig);
  return (sel);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* int iguales_parametros(nodo_tad *t1, nodo_tad *t2)                      *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Compara si t1 y t2 tienen el mismo nombre y los mismos     *
*              parametros.                                                *
**************************************************************************/

int
iguales_parametros (nodo_tad * t1, nodo_tad * t2)
{
/*----------------------------------------------------------------------*/
  nodo_nombre *p1, *p2;
  int ig;

  assert (t1 != NULL && t2 != NULL);	/* VTP. 7/May/1997 */
  ig = 1;
  if (strcmp (t1->nombre, t2->nombre))
    ig = 0;
  for (p1 = t1->parametros, p2 = t2->parametros;
       ig && p1 != NULL && p2 != NULL; p1 = p1->sig, p2 = p2->sig)
    {
      if (strcmp (p1->nombre, p2->nombre))
	ig = 0;
    }
  if (p1 != NULL || p2 != NULL)
    ig = 0;
  return (ig);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void informacion_tipo_normal(nodo_tad *p)				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Asignar tipo NORMAL a un tad.                              *
* DESCRIPCION: Asigna NORMAL al tipo de p y NULL a la informacion asociada*
*        al tipo.	                                                  *
* PARAMETROS:                                                             *
*    nodo_tad		*p	Ap. al tad que es de tipo normal.	  *
* PRE!:  { p-> T }                                                        *
* POST!: { T.tipo = NORMAL, T.info_tipo = NULL                          } *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Mar-90 *
**************************************************************************/

void
informacion_tipo_normal (nodo_tad * p)
{
/*----------------------------------------------------------------------*/
  assert (p != NULL);		/* VTP. 7/May/1997 */
  p->tipo = NORMAL;
  p->info_tipo = NULL;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short pida_status()							  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Pedir el status (MODIFICABE o INMODIFICABLE) de un tad.    *
* DESCRIPCION: Despliega un menu con los posibles status: MODIFICABE o    *
*        INMODIFICABLE de un tad y retorna la escogencia del usuario.     *
* RESULTADO:  short		0, Si es MODIFICABLE  			  *
* 				1, Si es INMODIFICABLE  		  *
* PRE!:  { TRUE }                                                         *
* POST!: { pida_status = 0, Si el usuario escogio MODIFICABLE  o, 	  *
* 	   pida_status = 1, Si el usuario escogio INMODIFICABLE		} *
* LLAMA A:                                                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Mar-90 *
**************************************************************************/

short
pida_status ()
{
/*----------------------------------------------------------------------*/

  short opcion;

  do
    {
      opcion = (short) admon_menu (menus_manta[STATUS]);
      switch (opcion)
	{
	case 0:		/* MODIFICABLE */
	  break;
	case 1:		/* INMODIFICABLE */
	  break;
	case 2:		/* AYUDA */
	  escriba_mensaje (157);
	  break;
	case NO_OPCION:
	  break;
	default:
	  escriba_mensaje (146);
	  break;
	}
    }
  while (opcion > 1);
  if (opcion == NO_OPCION)
    return (0);
  return (opcion);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void reemplace_def(nodo_tad *p,nodo_tad *tad_ed)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Reemplazar la definicion de un tad.                        *
* DESCRIPCION: Cambia la estructura apuntada por p por la apuntada por    *
*        tad_ed. Estas contienen informacion diferente de un mismo tad.   *
* PARAMETROS:                                                             *
*    nodo_tad	*p		Apunt. a la estructura que contiene la vie*
*                               ja informacion de un tad.                 *
*    nodo_tad	*tad_ed		Apunt. a la estructura que contiene la nue*
*                               va informacion del tad.                   *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad		Apunt. a la cabeza de la lista de los tads*
*				del ambiente.				  *
*    nodo_tad	*ult_tad	Apunt. al ultimo tad de la lista de tads  *
*				del ambiente.				  *
* PRE!:  { tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2, p -> ti, para algun  1 <= i <= n,      *
*	   tad_ed -> t   tq  ti.nombre = t.nombre.			} *
* POST!: { tad -> <t1,t2,...,t(i-1),t(i+1),tn,t> <- ult_tad		} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Mar-90 *
**************************************************************************/

void
reemplace_def (nodo_tad * p, nodo_tad * tad_ed)
{
/*----------------------------------------------------------------------*/

  nodo_tad *q;
  assert (ult_tad != NULL && p != NULL);	/* VTP. 7/May/1997 */
  ult_tad->sig = tad_ed;
  ult_tad = tad_ed;
  if (p != NULL)
    {
      if (tad == p)
	{
	  tad = p->sig;
	}
      else
	{
	  for (q = tad; q->sig != p; q = q->sig);
	  q->sig = p->sig;
	}
    }
/*----------------------------------------------------------------------*/
}
