 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "definic.h"

#include <assert.h>		/* VTP. 5/May/1997 */
/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* v-tamara 7/May/1997 */

/* EXPRESIONES.C                                                        */
/* Contiene el manejo de expresiones (axiomas y teoremas)               */

/**************************************************************************
* void aisle_param(texpr exp,texpr exp_param)				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Separar el primer argumento de los demas en una cadena.    *
* DESCRIPCION: Recibe en <exp> una expresion de la forma a1,...,an donde  *
*        a1,...,an son a su vez expresiones y deja en <exp_param> a a1 y  *
*	 en <exp>  a los demas: a2,...,an.	 n >= 0			  *
* PARAMETROS:                                                             *
*   texpr	exp		Expresion con argumentos.                 *
*   texpr	exp_param	Cadena con el primer argumento.		  *
* PRE!:  { exp = "<a1>,....,<an>",  n >=0                          	} *
* POST!: { exp_param = "<a1>" , exp = "<a2>,....,<an>"			} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
aisle_param (texpr exp, texpr exp_param)
{
/*----------------------------------------------------------------------*/

  short num_parentesis, fin_param;
  int i, j;
  texpr aux;

  assert (exp_param != NULL && exp != NULL);	/* VTP. 5/May/1997 */

  strcpy (exp_param, "");
  for (fin_param = 0, num_parentesis = 0, i = 0;
       (exp[i] != '\0') && (!fin_param); i++)
    {
      switch (exp[i])
	{
	case ',':
	  if (num_parentesis == 0)
	    {
	      fin_param = 1;
	      exp_param[i] = '\0';
	    }
	  else
	    exp_param[i] = exp[i]
	      ;
	  break;
	case '(':
	  num_parentesis++;
	  exp_param[i] = exp[i];
	  break;
	case ')':
	  num_parentesis--;
	  exp_param[i] = exp[i];
	  break;
	default:
	  exp_param[i] = exp[i];
	  break;
	}
    }
  exp_param[i] = '\0';
  if (exp[i] == '\0')
    strcpy (exp, "");
  else
    {
      for (j = 0; (exp[i] != '\0'); i++, j++)
	aux[j] = exp[i]
	  ;
      aux[j] = '\0';
      strcpy (exp, aux);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void construya_exp(nodo_expresion *cabeza,texpr exp)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Construir la cadena asociada con el arbol sintactico de una*
*	expresion.							  *
* DESCRIPCION: Recibe un ap. <cabeza> a una estructura que almacena una ex*
*	 presion y retorna en <exp> la cadena que representa dicha estruc-*
*	 tura. Esta construccion se hace con un recorrido en preorden del *
*	 arbol sintactico concatenando a <exp> el nombre de cada nodo visi*
*	 tado, usando parentesis para encerrar el resultado de recorrer   *
*	 sus hijos.							  *
* PARAMETROS:                                                             *
*    nodo_expresion	*cabeza		Ap. al arbol sintactico de una ex-*
*					presion.			  *
*    texpr		exp		Cadena con la expresion.	  *
* PRE!:  { cabeza -> ARBOL_SINTAC(EXP).                                 } *
* POST!: { exp = EXP.                                                   } *
* LLAMA A: void   construya_exp						  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
* MODIFICACIONES:
*   VTP. 11/Jun/1997 Parametro adic agregado.  Esta funcion es usada para
*                    generar plantillas de TADs nuevos (Extension/Suma-directa)
*                    Creaba mal las plantillas, porque no agregaba al final
*                    de los nombres de funciones el nombre del tipo (_tipo)
*                    Si el parametro adic, no es vacio lo agrega a las
*                    funciones (no a variables)
**************************************************************************/

void
construya_exp (nodo_expresion * cabeza, texpr exp, char *adic)
{
/*----------------------------------------------------------------------*/

  nodo_argum *p;
  char *exp_arg;
  int sc,sa;

  assert (cabeza != NULL && exp != NULL);
  assert (strlen (exp) < sizeof (texpr));	/* VTP. 5/May/1997 */

  exp_arg = (char *) malloc (sizeof (char) * 500);
  HayMem (exp_arg, sizeof (char) * 500);

  strcpy (exp, cabeza->nombre);
  if (adic!=NULL && adic[0]!='\0' && cabeza->clase!=NULL &&
      strcmp(cabeza->clase,"VAR"))
    {
      sc=strlen(cabeza->nombre);
      sa=strlen(adic);
      if (sc<=sa || strcmp(cabeza->nombre+sc-sa,adic))
        {
          strcat (exp,adic); /* VTP 11/Jun/1997, Ver modificaciones */
        }
    }
  if (cabeza->argumentos != NULL)
    {
      strcat (exp, "(");
      for (p = cabeza->argumentos; p != NULL; p = p->sig)
	{
	  strcpy (exp_arg, "");
	  construya_exp (p->expr, exp_arg,adic);
	  strcat (exp, exp_arg);
	  if (p->sig != NULL)
	    strcat (exp, ",")
	      ;
	}
      strcat (exp, ")");
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void construya_exp_tipo_clase(nodo_expresion *cabeza,texpr exp,	  *
*			        texpr tipo,texpr clase)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Construir las cadenas de nombres, tipos y clase asociados  *
*	con el arbol sintactico de una expresion.			  *
* DESCRIPCION: Recibe un ap. <cabeza> a una estructura que almacena una ex*
*	 presion y retorna en <exp> la cadena que representa dicha estruc-*
*	 tura, en <tipo> los tipos y en <clase> las clases de los tokens  *
*	 que forman la expresion.					  *
*	 Esta construccion se hace con un recorrido en preorden del arbol *
*	 sintactico, concatenando a <exp> el nombre , a <tipo> el tipo y a*
*	 <clase> la clase de cada nodo visitado, usando parentesis para en*
*	 cerrar el resultado de recorrer sus hijos.			  *
* PARAMETROS:                                                             *
*    nodo_expresion	*cabeza		Ap. al arbol sintactico de una ex-*
*					presion.			  *
*    texpr		exp		Cadena con la expresion.	  *
*    texpr		tipo		Cadena con los tipos.   	  *
*    texpr		clase		Cadena con las clases.  	  *
* PRE!:  { cabeza -> ARBOL_SINTAC(EXP).                                 } *
* POST!: { exp = EXP, tipo = TIPOS(EXP), clase = CLASES(EXP).           } *
* LLAMA A: void   construya_exp_tipo_clase				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
construya_exp_tipo_clase (nodo_expresion * cabeza, texpr exp, texpr tipo,
			  texpr clase)
{
/*----------------------------------------------------------------------*/

  nodo_argum *p;
  texpr exp_arg_nom, exp_arg_tip, exp_arg_cla;

  assert (cabeza != NULL && exp != NULL);	/* VTP. 5/May/1997 */
  assert (tipo != NULL && clase != NULL);	/* VTP. 5/May/1997 */
  strcpy (exp, cabeza->nombre);
  strcpy (tipo, cabeza->tipo);
  strcpy (clase, cabeza->clase);
  if (cabeza->argumentos != NULL)
    {
      strcat (exp, "(");
      strcat (tipo, "(");
      strcat (clase, "(");
      for (p = cabeza->argumentos; p != NULL; p = p->sig)
	{
	  strcpy (exp_arg_nom, "");
	  strcpy (exp_arg_tip, "");
	  strcpy (exp_arg_cla, "");
	  construya_exp_tipo_clase (p->expr, exp_arg_nom, exp_arg_tip,
				    exp_arg_cla);
	  strcat (exp, exp_arg_nom);
	  strcat (tipo, exp_arg_tip);
	  strcat (clase, exp_arg_cla);
	  if (p->sig != NULL)
	    {
	      strcat (exp, ",");
	      strcat (tipo, ",");
	      strcat (clase, ",");
	    }
	}
      strcat (exp, ")");
      strcat (tipo, ")");
      strcat (clase, ")");
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void construya_param(texpr exp,nodo_expresion **cabeza)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Construir los arboles sintacticos asociados a los argumen- *
*	 tos de una expresion.    					  *
* DESCRIPCION: Recibe <exp> y va recorriendo la cadena construyendo el ar-*
*	 bol sintactico asociado a cada uno de los argumentos asociados.  *
* PARAMETROS:                                                             *
*    texpr		exp		Cadena con los argumentos de una  *
*					expresion.			  *
*    nodo_expresion	**cabeza	Ap. al comienzo de los arboles sin*
*					tacticos de una expresion.	  *
* PRE!:  { exp =  "<arg1>,...,<argn>", n>= 0    			} *
* POST!: { *cabeza -> ARBOL_SINTAC(<arg1>)-> ... -> ARBOL_SINTAC(<argn>)} *
* LLAMA A:                                                                *
*	void	aisle_param 		void	construya_subestruct	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
construya_param (texpr exp, nodo_expresion ** cabeza)
{
/*----------------------------------------------------------------------*/

  nodo_argum *p, *ult_param;
  texpr exp_param;

  assert (exp != NULL && cabeza != NULL);	/* VTP. 5/May/1997 */
  (*cabeza)->argumentos = NULL;
  ult_param = NULL;
  while (strcmp (exp, ""))
    {
      aisle_param (exp, exp_param);
      p = (nodo_argum *) malloc (sizeof (nodo_argum));
      HayMem (p, sizeof (nodo_argum));	/* VTP. 5/May/1997 */
      p->sig = NULL;
      if (ult_param == NULL)
	(*cabeza)->argumentos = p;
      else
	ult_param->sig = p
	  ;
      ult_param = p;
      construya_subestruct (exp_param, &(p->expr));
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void construya_subestruct(texpr exp,nodo_expresion **cabeza)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Construir el arbol sintactico asociado a una expresion.    *
* DESCRIPCION: Recibe <exp> y va recorriendo la cadena construyendo el ar-*
*	 bol sintactico asociado.					  *
* PARAMETROS:                                                             *
*    texpr		exp		Cadena con la expresion.	  *
*    nodo_expresion	**cabeza	Ap. al arbol sintactico de una ex-*
*					presion.			  *
* PRE!:  { exp = EXP.                                                   } *
* POST!: { *cabeza -> ARBOL_SINTAC(EXP).                                } *
* LLAMA A:                       				  	  *
*	void	predicado_ppal 		void	construya_param	   	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
construya_subestruct (texpr exp, nodo_expresion ** cabeza)
{
/*----------------------------------------------------------------------*/

  texpr pred;

  assert (exp != NULL && cabeza != NULL);	/* VTP. 5/May/1997 */
  *cabeza = (nodo_expresion *) malloc (sizeof (nodo_expresion));
  HayMem (*cabeza, sizeof (nodo_expresion));	/* VTP. 5/May/1997 */
  predicado_ppal (exp, pred);
  strcpy ((*cabeza)->nombre, pred);
  construya_param (exp, cabeza);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void parta_expresion(texpr exp, texpr exp_der,char simbolo)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Partir una cadena en dos por la primera ocurrencia de un   *
* 	simbolo.							  *
* DESCRIPCION: Toma <exp>, la parte por la 1a. ocurrencia de <simbolo> y  *
*        deja en <exp> la parte izquierda y en <exp_der> la derecha.      *
* PARAMETROS:                                                             *
*   texpr	exp		Cadena de caracteres.		          *
*   texpr       exp_der 	Cadena con la parte derecha de la expre-  *
*				a partir de <simbolo>.			  *
*   char	simbolo		Simbolo que determina el lugar por el cual*
*				se parte la cadena.			  *
* PRE!:  { simbolo = S, exp = "/////S\\\\\\" tq en ///// no ocurre S.   } *
* POST!: { exp = "/////",  exp_der = "\\\\\\".	                        } *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
parta_expresion (texpr exp, texpr exp_der, char simbolo)
{
/*----------------------------------------------------------------------*/

  int i;

  assert (exp != NULL && exp_der != NULL);	/* VTP. 5/May/1997 */
  for (i = 0; exp[i] != simbolo; i++)
    ;
  strcpy (exp_der, &(exp[i + 1]));
  exp[i] = '\0';
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void predicado_ppal(texpr exp,texpr pred)				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Separar el funtor de sus parametros correspondientes.      *
* DESCRIPCION: Recibe en <exp> una expresion de la forma F(a1,...,an) don-*
*        de a1,...,an son a su vez expresiones y deja en <pred> a F y en  *
*	 <exp> los argumentos a1,...,an.	n >= 0			  *
* PARAMETROS:                                                             *
*   texpr	exp		Expresion con functor y posible/. argums. *
*   texpr	pred     	Cadena con el funtor.      	  	  *	
* PRE!:  { exp = "<F>(<a1>,....,<an>)",  n >=0                          } *
* POST!: { pred = "<F>", exp = "<a1>,....,<an>"				} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
predicado_ppal (texpr exp, texpr pred)
{
/*----------------------------------------------------------------------*/

  int i, j, longitud;
  texpr aux;

  assert (exp != NULL && pred != NULL);		/* VTP. 5/May/1997 */
  longitud = strlen (exp);
  strcpy (pred, "");
  for (i = 0; (i < longitud) && (exp[i] != '('); i++)
    pred[i] = exp[i]
      ;
  pred[i] = '\0';
  if (i == longitud)
    strcpy (exp, "");
  else
    {				/* exp = (<param>, ... ,<param>) */
      exp[longitud - 1] = '\0';
      for (j = 0, i++; (i < longitud); i++, j++)
	aux[j] = exp[i]
	  ;
      strcpy (exp, aux);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void transf_estruct_iguald(nodo_igualdad *cabeza)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Transformar el arbol sintactico de una igualdad a la expre-*
*	       sion correspondiente.					  *
* DESCRIPCION: Recibe <cabeza> apunt. a una estructura donde se almacena  *
*	 una igualdad y deja en <cabeza>->li_str la cadena correspondiente*
*        a la expresion izq. y en <cabeza>->ld_str la cadena correspondien*
*        te a la expresion der. de la igualdad.				  *
* PARAMETROS:                                                             *
*   nodo_igualdad	*cabeza		Ap. a la estructura que almacena  *
*					la igualdad.			  *
* PRE!:  { cabeza -> IG                                                 } *
* POST!: { IG->li_str = EXP(IG->lado_izq), IG->li_str = EXP(IG->lado_izq)}*
* LLAMA A: void construya_exp						  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
transf_estruct_iguald (nodo_igualdad * cabeza)
{
/*----------------------------------------------------------------------*/
  assert (cabeza != NULL);	/* VTP. 5/May/1997 */
  construya_exp (cabeza->lado_izq, cabeza->li_str,"");
  construya_exp (cabeza->lado_der, cabeza->ld_str,"");
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void transf_iguald_estruct(texpr exp, nodo_igualdad **cabeza)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:  Transformar una igualdad al arbol sintactico correspondiente*
* DESCRIPCION: Recibe en <exp> una cadena con una ecuacion y deja a 	  *
*	 <*cabeza> apunt. a una estructura donde se almacena la igualdad  *
* PARAMETROS:                                                             *
*   texpr		exp		Cadena con una ecuacion.	  *
*   nodo_igualdad	**cabeza	Ap. a la estructura que almacena  *
*					la igualdad.			  *
* PRE!:  { exp = "<li> = <ld>"						} *
* POST!: { *cabeza -> IG,						  *
*	   IG->lado_izq = ARBOL_SINTAC(<li>),				  *
*	   IG->lado_der = ARBOL_SINTAC(<ld>),				  *
*	   IG->li_str = <li>,  IG->li_str = <li>   			} *
* LLAMA A:								  *
*    void  parta_expresion 	void	construya_subestruct		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Feb-90 *
**************************************************************************/

void
transf_iguald_estruct (texpr exp, nodo_igualdad ** cabeza)
{
/*----------------------------------------------------------------------*/

  texpr ld;
  assert (exp != NULL && cabeza != NULL);	/* VTP. 5/May/1997 */
  *cabeza = (nodo_igualdad *) malloc (sizeof (nodo_igualdad));
  HayMem (*cabeza, sizeof (nodo_igualdad));	/* VTP. 5/May/1997 */
  (*cabeza)->sig = NULL;
  parta_expresion (exp, ld, '=');
  strcpy ((*cabeza)->li_str, exp);
  strcpy ((*cabeza)->ld_str, ld);
  construya_subestruct (exp, &((*cabeza)->lado_izq));
  construya_subestruct (ld, &((*cabeza)->lado_der));
/*----------------------------------------------------------------------*/
}

/************************************************************************
* void liberar_igualdad(nodo_igualdad **ig)                             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Libera la memoria de ig.                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio  91 *
************************************************************************/

void
liberar_igualdad (nodo_igualdad ** ig)
{
/*--------------------------------------------------------------------*/

  assert (ig != NULL);		/* VTP. 5/May/1997 */
  if ((*ig) == NULL)
    return;
  liberar_expresion (&((*ig)->lado_izq));
  liberar_expresion (&((*ig)->lado_der));
  free (*ig);
  (*ig) = NULL;
/*--------------------------------------------------------------------*/
}

/************************************************************************
* void liberar_igualdades(nodo_igualdad **ig)                           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Libera la memoria de ig.                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio  91 *
************************************************************************/

void
liberar_igualdades (nodo_igualdad ** ig)
{
/*--------------------------------------------------------------------*/
  nodo_igualdad *i;
  assert (ig != NULL);		/* VTP. 5/May/1997 */
  while ((*ig) != NULL)
    {
      i = (*ig)->sig;
      (*ig)->sig = NULL;	/* VTP. 5/May/1997 */
      liberar_igualdad (ig);
      (*ig) = i;
    }
  (*ig) = NULL;

/*--------------------------------------------------------------------*/
}

/************************************************************************
* void liberar_dominios(nodo_dom **d)                                   *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Libera la memoria de d.                                 *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio  91 *
************************************************************************/

void
liberar_dominios (nodo_dom ** d)
{
/*--------------------------------------------------------------------*/
  nodo_dom *d_aux;

  assert (d != NULL);		/* VTP. 5/May/1997 */
  while ((*d) != NULL)
    {
      d_aux = (*d)->sig;
      (*d)->sig = NULL;		/* VTP. 5/May/1997 */
      free (*d);
      (*d) = d_aux;
    }
  (*d) = NULL;
/*--------------------------------------------------------------------*/
}

/************************************************************************
* void liberar_generadoras(nodo_generadora **gen)                       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Libera la memoria de gen.                               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio  91 *
************************************************************************/

void
liberar_generadoras (nodo_generadora ** gen)
{
/*--------------------------------------------------------------------*/
  nodo_generadora *g;

  assert (gen != NULL);		/* VTP. 5/May/1997 */
  while ((*gen) != NULL)
    {
      g = (*gen)->sig;
      (*gen)->sig = NULL;	/* VTP. 5/May/1997 */
      liberar_nombres (&((*gen)->dominio));
      free (*gen);
      (*gen) = g;
    }
  (*gen) = NULL;
/*--------------------------------------------------------------------*/
}

/************************************************************************
* void liberar_selectoras(nodo_select **sel)                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Libera la memoria de sel.                               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio  91 *
************************************************************************/

void
liberar_selectoras (nodo_select ** sel)
{
/*--------------------------------------------------------------------*/
  nodo_select *s;

  assert (sel != NULL);		/* VTP. 5/May/1997 */
  while ((*sel) != NULL)
    {
      s = (*sel)->sig;
      (*sel)->sig = NULL;	/* VTP. 5/May/1997 */
      liberar_dominios (&((*sel)->dominio));
      liberar_igualdades (&((*sel)->axiomas));
      free (*sel);
      (*sel) = s;
    }
  (*sel) = NULL;
/*--------------------------------------------------------------------*/
}

/************************************************************************
* void liberar_expresion(nodo_expresion **expr)                         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Libera la memoria de expr.                              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio  91 *
************************************************************************/

void
liberar_expresion (nodo_expresion ** expr)
{
/*--------------------------------------------------------------------*/
  nodo_argum *arg, *p;

  assert (expr != NULL);	/* VTP. 5/May/1997 */
  if ((*expr) == NULL)
    return;
  arg = (*expr)->argumentos;

  while (arg != NULL)
    {
      p = arg->sig;
      arg->sig = NULL;
      liberar_expresion (&arg->expr);
      free (arg);
      arg = p;
    }
  free (*expr);
/*  (*expr) = NULL;*/
/*--------------------------------------------------------------------*/
}


/************************************************************************
* nodo_argum *crear_argumento(nodo_expresion *expr)                     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Crea un argumento con la expresion expr.                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio 91  *
************************************************************************/
nodo_argum *
crear_argumento (nodo_expresion * expr)
{
/*-------------------------------------------------------------------*/
  nodo_argum *arg;

  arg = (nodo_argum *) malloc (sizeof (nodo_argum));
  HayMem (arg, sizeof (nodo_argum));	/* VTP. 5/May/1997 */

  arg->expr = copia_expresion(expr);
  arg->sig = NULL;
  return (arg);

/*--------------------------------------------------------------------*/
}


/************************************************************************
* nodo_argum *copia_argumento(nodo_argumento *argin)                    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Crea una copia del  argumento argin                     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: V. TAMARA     .                            FECHA: Febrero 1998 *
************************************************************************/
nodo_argum *
copia_argum(nodo_argum * inarg)
{
/*-------------------------------------------------------------------*/
  nodo_argum *arg;
  if (inarg==NULL)
    {
      return NULL;
    }
  arg = (nodo_argum *) malloc (sizeof (nodo_argum));
  HayMem (arg, sizeof (nodo_argum));
  arg->expr = copia_expresion(inarg->expr);
  arg->sig = copia_argum(inarg->sig);
  return (arg);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* nodo_expresion *copia_expresion(nodo_expresion *expr)                 *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Copia en una nueva estructura la expresion que recibe   *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: V. Tamara                                 FECHA: Febrero 1998  *
************************************************************************/
nodo_expresion *
copia_expresion (nodo_expresion * inexpr)
{
/*--------------------------------------------------------------------*/
  nodo_expresion *expr;
  if (inexpr==NULL)
    {
      return NULL;
    }
  expr = (nodo_expresion *)malloc(sizeof(nodo_expresion));
  HayMem (expr, sizeof (nodo_expresion));
  strcpy(expr->nombre,inexpr->nombre);
  strcpy(expr->tipo,inexpr->tipo);
  strcpy(expr->clase,inexpr->clase);
  expr->argumentos=copia_argum(inexpr->argumentos);

  return (expr);
/*--------------------------------------------------------------------*/
}
