/* EXTENSION.C  						*/
/* Archivo de manejo de extensiones de tipos.                    */
 
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include "os_dep.h" /*** VTP. 7/Mar/1997 ***/
#include "definic.h"

/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* j-tamara 29/May/1997 */


/**************************************************************************
* short construya_axs_selec_sum_direc(nodo_tad *tad,nodo_select *s)       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Construir los axiomas de una selectora de un tad NT_EXTENSION * 
* DESCRIPCION: Si tad_ed = T1 + T2 y s es una selectora heredada de Ti    *
*        (para i = 1 o 2) entonces escribe en el archivo de edicion los   *
*	 axiomas conocidos que definen a s sobre los elementos de Ti y los*
*	 lados izqs. de los axiomas que la definen sobre los elementos de *
*	 Tj (con j = 1 o 2, i <> j).					  *
*	 Permite que el usuario edite tal archivo y lo analiza sintactica *
*	 y semantica/. 							  *
* PARAMETROS:                                                             *
*   nodo_tad	*tad		Tad componente de cual NO se heredo s (Tj)*
*   nodo_select *s		Ap. a la selectora que se va a extender.  *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_ed     	Apunt. al tad de que se esta generando por*
*			  	NT_EXTENSION				  *
* RESULTADO:  short		1, Si la edicion de los axiomas tuvo exito*
*				0, Sino.				  *
* PRE!:  { tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2.					  *
*	   tad-> Tj,  existe un tk = Ti ( 1 <= k <= n) tq.		  *
*	   tad_ed-> TC = Ti + Tj   o    tad_ed-> TC = Tj + Ti  donde      * 
*	   Ti.selectoras = <...s...> y s.axiomas = <a1,...,ak>, k >= 0	} *
* POST!: { construya_axs_selec_sum_direc = 1, Si			  *
*	 	TC.selectoras = <...,s...>, tq.          		  *
*	        s.axiomas = <a1,...,ak,b1,...,bl>, para l >= 0  	  *
*		tq. bj1,...,bjl extiende la definicion de s sobre los ele-*
*	 	mentos de Tj, 						  *
*          construya_axs_selec_sum_direc = 0, Sino.                     } *
* !OJO!: Esta funcion se usa para la generacion de axs. de COMPLETACION y *
*	 de NT_EXTENSION. En el primer caso, Tj es el tad que consiste unica-*
*	 mente del elemento de completacion.				  *
* LLAMA A: short  es_prefijo						  *
*   void   genere_axs_extens 		void   genere_axs_compl		  *
*       			        short  arch_modificado		  *
*   short  cargue_axiomas 		int    system			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
* MODIFICACIONES:
* VTP 13/Ene/97
*	Antes borraba un archivo empleando system("rm ... ahora
*	se usa unlink (portable para DOS y UNIX)
* VTP. 11/Jun/1997 Parametro adic agregado.  Esta funcion es usada para
*                    generar plantillas de TADs nuevos (Extension/Suma-directa)
*                    Creaba mal las plantillas, porque no agregaba al final
*                    de los nombres de funciones el nombre del tipo (_tipo)
*                    Si el parametro adic, no es vacio lo agrega a las
*                    funciones (no a variables)
 VTP. 24/Jun/1997  Se agrego codigo para que cambie las ocurrencias de
                   EQ_TadOrigen por EQUAL al crear un nuevo tipo por
                   completacion
      12/Jul/1997  Se agrego parametro fp2, para que agrege a un archivo
                   las definiciones que haga.  El comportamiento sera el
                   original si fp2 viene en NULL
**************************************************************************/

short construya_axs_selec_sum_direc(nodo_tad *tad,nodo_select *s,char *adic,
                                    FILE *fp2)

{/*----------------------------------------------------------------------*/

  short		correcto=1;
  FILE		*fp;
  nodo_igualdad	*ax;
  trio_edicion edicion[MAX_EDICION];
  char          tmpbus[100]; /* VTP. 24/Jun/1997. */
  char          *posbus;     /* VTP. 24/Jun/1997. */
  cad240         lder,slder;        /* VTP. 24/Jun/1997. */
  char *feq;

    assert(s!=NULL);            /* j-tamara 29/May/1997 */
    assert(tad!=NULL);          /* j-tamara 29/May/1997 */
    if (fp2==NULL) {
        fp = fopen("axiom_ed.tad","w");
        FileOpen(fp,"axiom_ed.tad");                /* j-tamara 29/May/1997 */
    }
    else
    {
        fp=fp2;
    }

    if (es_prefijo("EQ_",s->nombre))
      {
    	fprintf(fp,"%s(%c1,%c2) =\n",s->nombre,(tad->nombre)[0],
	                                       (tad->nombre)[0]);
      }
    else
      {
        for (ax = s->axiomas; ax != NULL; ax = ax->sig) {
            /* VTP. 24/Jun/1997. */
            /* Cambia ocurrencias de EQ_TadOrigen por EQUAL en
               el lado derecho de la expresion */
	  /*            strcpy(lder,ax->ld_str);
			lder[strlen(ax->ld_str)]='\0';
			Probemos asi: */
	  lder[0]='\0';
	  formatee_ld_axioma (ax->lado_der, lder,
			      strlen (ax->li_str) + 4);
	  /* Si hay EQ_ algo lo cambiamos por EQUAL */
	  sprintf(tmpbus,"EQ_");
	  posbus=strstr(lder,tmpbus);
	  for (;posbus!=NULL;)
	    { /* Lo encontro toca cambiarlo */
	      *posbus='\0';
	      feq=strstr(posbus+1,"(");
	      sprintf(slder,"%sEQUAL%s",lder,feq);
	      strcpy(lder,slder);
	      lder[strlen(slder)]='\0';
	      posbus=strstr(lder,tmpbus);
	    }
	  fprintf(fp,"%s = %s\n",ax->li_str,lder);
        }
    }
    s->axiomas = NULL;
    if (tad_ed->tipo == NT_EXTENSION)
        genere_axs_extens(fp,tad,s,adic); /* VTP. 11/Jun/1997, Agregado adic */
    else  /* tad_ed->tipo = COMPLETACION */
        genere_axs_compl(fp,s)
    ;
    if (fp2==NULL)
      {
        fclose(fp);
	edicion[0].modo = 0;
	edicion[0].linea = 1;
	strcpy (edicion[0].nombre, "signa_ed.tad");
	edicion[1].modo = 1;
	edicion[1].linea = num_ed;
	strcpy (edicion[1].nombre, "axiom_ed.tad");
	llamar_al_editor (2, edicion, "temp");
	/*  VTP. 22/Jun/1999 Invocaba al editor de forma chanda
	    sprintf (comando, "%s axiom_ed.tad", editor[0]);
	  system (comando); */
        if (arch_modificado("axiom_ed.tad"))
          {
            correcto = cargue_axiomas(s);
          }
        else
          {
            correcto = 0;
          }
        /*** VTP. Decia: system("rm -f axiom_ed.tad");  ***/
        unlink("axiom_ed.tad");
      }

    return(correcto);
/*----------------------------------------------------------------------*/}

/**************************************************************************
* void extension(cad12 nom_tad_e)					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar un tad de tipo NT_EXTENSION.                          *
* DESCRIPCION: Pide los nombre de los tads T1 y T2 que se va a sumar, gene*
*	 ra automatica/. la signatura del nuevo tad <nom_tad_e> y solicita*
*	 al usuario que extienda las operaciones selectoras del tad T1 so-*
*	 los elementos de T2 y viceversa.				  *
* PARAMETROS:                                                             *
*    cad12	nom_tad_e	Nombre del nuevo tad de tipo NT_EXTENSION.	  *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_ed     	Apunt. al tad de que se esta generando por*
*			  	NT_EXTENSION				  *
* PRE!:  { nom_tad_e = NT,                                                *
*          tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2.					} *
* POST!: { tad -> <t1,...,tn,t(n+1)> <- ult_tad = tad_ed, donde		  * 
*	        p-> ti, q-> tj,  1 <= i,j <= n,				  *
*		t(n+1) = <p> + <q>, t(n+1).nombre = NT, o  		  *	
*          tad -> <t1,...,tn> <- ult_tad, Si la generacion del nuevo tad  *
*	   	no fue exitosa.						  *
* LLAMA A:nodo_tad  *pida_tad_ambiente					  *
*   void  informacion_tipo_extension  void  genere_param_sum_direc 	  *
*   void  genere_iniciales_sum_direc  void  genere_constructoras_sum_direc*
*   void  genere_selectoras_extension void  edite_igualdad		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void extension(cad12 nom_tad_e)
{/*----------------------------------------------------------------------*/

  nodo_tad	*p,*q;
  
  p = pida_tad_ambiente(mensajes[52]);
  if (p != NULL) {
    /* The first has to be a basic type */
    if (p->parametros!=NULL ||
	(p->iniciales==NULL && p->constructoras==NULL))
      {
	escriba_mensaje(183);
	return;
      }
    q = pida_tad_ambiente(mensajes[51]);
    if (q != NULL)
      {
	/* Different types to make direct sum */
	if (q==p)
	  {
	    escriba_mensaje(188);
	    return;
	  }
	/* The second has to be a basic type */
	if (q->parametros!=NULL ||
	    (q->iniciales==NULL && q->constructoras==NULL))
	  {
	    escriba_mensaje(183);
	    return;
	  }
	tad_ed = crear_tad();
	strcpy(tad_ed->nombre,nom_tad_e);
	informacion_tipo_extension(tad_ed,p->nombre,q->nombre);
	genere_param_sum_direc(p,q);
	genere_iniciales_sum_direc(p,q);
	genere_constructoras_sum_direc(p,q);
	if (genere_selectoras_extension(p,q))
	  {
	    tad_ed->status = pida_status();
	  }
      }
  }
  /*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_axs_extens(FILE *fp,nodo_tad *tad,nodo_select *s)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar los lados izqs. de los axiomas para definir una    *
*	       selectora heredada de un padre en los elementos del otro   *
*	       padre de la extension.		  			  *
* DESCRIPCION: Genera lados izq. para definir a s en el elementos de <tad>*
*        Los lados izqs. se construyen poniendo formas normales de <tad>  *
*	 en los distintos argumentos de TDI de s y variables en los demas *
*	 argumentos.						  	  *
* PARAMETROS:                                                             *
*   FILE	*fp		Ap. Al archivo de edicion de axiomas.     *
*   nodo_tad	*tad		Ap. al tad componente en cuyos elementos  *
*				debe definirse s.			  *
*   nodo_select	*s 		Ap. a la selectora que se va a extender.  *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_ed     	Apunt. al tad de que se esta generando por*
*			  	extension				  *
* PRE!:  { fp-> [////], tad-> Tj, tad_ed-> TC = Ti + Tj  o TC = Tj + Ti   *
*          para algun tad Ti en el ambiente,				  *
*	   s-> S, selectora heredada de Ti, tq, S.axiomas = <a1,...,al>,  *
*	   l >= 0, axiomas que definen a s sobre Ti.			} *
* POST!: { fp-> [////a1,...,al,li1,...,lik], donde k = # de argumentos del*
*	        TDI de s   y para j = 1,...,k:  			  *
*	        lij = s(v1,...,v(j-1),F,v(j+1),...,vk) con vm variable de *
*		tipo correspondiente al argumento m-esimo de s y          * 
*		F es una forma normal de Tj.				} *
* LLAMA A:    								  *
*	void   genere_plantilla_selectora 	void	genere_axs_tdi	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
* VTP. 11/Jun/1997 Parametro adic agregado.  Esta funcion es usada para
*                    generar plantillas de TADs nuevos (Extension/Suma-directa)
*                    Creaba mal las plantillas, porque no agregaba al final
*                    de los nombres de funciones el nombre del tipo (_tipo)
*                    Si el parametro adic, no es vacio lo agrega a las
*                    funciones (no a variables)
**************************************************************************/

void genere_axs_extens(FILE *fp,nodo_tad *tad,nodo_select *s,char *adic)
{/*----------------------------------------------------------------------*/

  nodo_expresion	*plant_li_ax=NULL;      /* j-tamara 29/May/1997 */
  nodo_argum		*arg;
  char			cont;

    assert(tad_ed!=NULL);               /* j-tamara 29/May/1997 */
    cont = '1';
    genere_plantilla_selectora(s,&plant_li_ax,&cont);
    for (arg = plant_li_ax->argumentos; arg != NULL; arg = arg->sig) {
	if (!strcmp(arg->expr->tipo,tad_ed->nombre)) {
	    genere_axs_tdi(fp,plant_li_ax,arg,tad->iniciales,cont,adic);
	    genere_axs_tdi(fp,plant_li_ax,arg,tad->constructoras,cont,adic);
            /* VTP. 11/Jun/1997.  Agregado parametro adic */
	}
    }
/*----------------------------------------------------------------------*/}

/**************************************************************************
* void genere_constructoras_sum_direc(nodo_tad *p, nodo_tad *q)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar las constructoras de un tad de tipo NT_EXTENSION. 	  *
* DESCRIPCION: El tad_ed hereda las constructoras de <p> y de <q>.Esto es,*
*	 el conjunto de las constructoras de tad_ed es la union disyunta  *
*	 de las constructoras de sus padres.  				  *
* PARAMETROS:                                                             *
*   nodo_tad		*p	Ap. al primer tad padre de la NT_EXTENSION.  *
*   nodo_tad		*q	Ap. al segundo tad padre de la NT_EXTENSION. *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_ed     	Apunt. al tad de que se esta generando por*
*			  	NT_EXTENSION				  *
* PRE!:  { p-> T1, q-> T2, tad_ed-> TC = T1 + T2, 	  		  *
*	   T1.constructoras = <c11,...,c1m1>, m1 >= 0,                    *
*	   T2.constructoras = <c21,...,c2m2>, m2 >= 0,			} * 
* POST!: { TC.constructoras = <c11,...,c1m1,c21,...,c2m2>		} *
* LLAMA A: 			void  copie_lista_generadoras  		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void genere_constructoras_sum_direc(nodo_tad *p, nodo_tad *q)
{/*----------------------------------------------------------------------*/

  nodo_generadora	*cabeza_lista=NULL,             /* j-tamara 29/May/1997 */
	      		*ult_lista1=NULL,               /* j-tamara 29/May/1997 */
	      		*ult_lista2=NULL;               /* j-tamara 29/May/1997 */

    assert(p!=NULL);            /* j-tamara 29/May/1997 */
    assert(q!=NULL);            /* j-tamara 29/May/1997 */
    assert(tad_ed!=NULL);       /* j-tamara 29/May/1997 */
    copie_lista_generadoras(p->constructoras,&cabeza_lista,&ult_lista1,
			      p->nombre,tad_ed->nombre,"");
    tad_ed->constructoras = cabeza_lista;
    copie_lista_generadoras(q->constructoras,&cabeza_lista,&ult_lista2,
			      q->nombre,tad_ed->nombre,"");
    if (tad_ed->constructoras == NULL)
	tad_ed->constructoras = cabeza_lista;
      else
	ult_lista1->sig = cabeza_lista;
/*----------------------------------------------------------------------*/}

/**************************************************************************
* void genere_iniciales_sum_direc(nodo_tad *p, nodo_tad *q)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar las iniciales de un tad de tipo NT_EXTENSION. 	  *
* DESCRIPCION: El tad_ed hereda las iniciales de <p> y de <q>.	Esto es,  *
*	 el conjunto de las iniciales de tad_ed es la union disyunta  	  *
*	 de las iniciales de sus padres.  				  *
* PARAMETROS:                                                             *
*   nodo_tad		*p	Ap. al primer tad padre de la NT_EXTENSION.  *
*   nodo_tad		*q	Ap. al segundo tad padre de la NT_EXTENSION. *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_ed     	Apunt. al tad de que se esta generando por*
*			  	NT_EXTENSION				  *
* PRE!:  { p-> T1, q-> T2, tad_ed-> TC = T1 + T2, 	  		  *
*	   T1.iniciales = <i11,...,i1m1>, m1 >= 0,                    	  *
*	   T2.iniciales = <i21,...,i2m2>, m2 >= 0,			} * 
* POST!: { TC.iniciales = <i11,...,i1m1,i21,...,i2m2>			} *
* LLAMA A: 			void  copie_lista_generadoras  		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void genere_iniciales_sum_direc(nodo_tad *p, nodo_tad *q)
{/*----------------------------------------------------------------------*/

  nodo_generadora	*cabeza_lista=NULL,             /* j-tamara 29/May/1997 */
	      		*ult_lista1=NULL,               /* j-tamara 29/May/1997 */
	      		*ult_lista2=NULL;               /* j-tamara 29/May/1997 */

    assert(p!=NULL);            /* j-tamara 29/May/1997 */
    assert(q!=NULL);            /* j-tamara 29/May/1997 */
    assert(tad_ed!=NULL);       /* j-tamara 29/May/1997 */
    copie_lista_generadoras(p->iniciales,&cabeza_lista,&ult_lista1,
			      p->nombre,tad_ed->nombre,"");
    tad_ed->iniciales = cabeza_lista;
    copie_lista_generadoras(q->iniciales,&cabeza_lista,&ult_lista2,
			      q->nombre,tad_ed->nombre,"");
    if (tad_ed->iniciales == NULL)
	tad_ed->iniciales = cabeza_lista;
      else
	ult_lista1->sig = cabeza_lista
    ;
/*----------------------------------------------------------------------*/}

/**************************************************************************
* void genere_param_sum_direc(nodo_tad *p, nodo_tad *q)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar los parametros de un tad de tipo NT_EXTENSION. 	  *
* DESCRIPCION: El tad_ed hereda los parametros de <p> y de <q>.Esto es,	  *
*	 el conjunto de los parametros de tad_ed es la union disyunta  	  *
*	 de las parametros de sus padres.  				  *
* PARAMETROS:                                                             *
*   nodo_tad		*p	Ap. al primer tad padre de la NT_EXTENSION.  *
*   nodo_tad		*q	Ap. al segundo tad padre de la NT_EXTENSION. *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_ed     	Apunt. al tad de que se esta generando por*
*			  	NT_EXTENSION				  *
* PRE!:  { p-> T1, q-> T2, tad_ed-> TC = T1 + T2, 	  		  *
*	   T1.parametros = <c11,...,c1m1>, m1 >= 0,                    	  *
*	   T2.parametros = <c21,...,c2m2>, m2 >= 0,			} *
* POST!: { TC.parametros = <c11,...,c1m1,c21,...,c2m2>			} *
* LLAMA A: 			void  copie_lista_nombres      		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void genere_param_sum_direc(nodo_tad *p, nodo_tad *q)
{/*----------------------------------------------------------------------*/

  nodo_nombre *cabeza_lista=NULL,               /* j-tamara 29/May/1997 */
	      *ult_lista1=NULL,                 /* j-tamara 29/May/1997 */
	      *ult_lista2=NULL;                 /* j-tamara 29/May/1997 */

    assert(p!=NULL);            /* j-tamara 29/May/1997 */
    assert(q!=NULL);            /* j-tamara 29/May/1997 */
    assert(tad_ed!=NULL);       /* j-tamara 29/May/1997 */
    copie_lista_nombres(p->parametros,&cabeza_lista,&ult_lista1,"","","");
    tad_ed->parametros = cabeza_lista;
    cabeza_lista = NULL;
    copie_lista_nombres(q->parametros,&cabeza_lista,&ult_lista2,"","","");
    if (tad_ed->parametros == NULL)
	tad_ed->parametros = cabeza_lista;
      else
	ult_lista1->sig = cabeza_lista
    ;
/*----------------------------------------------------------------------*/}

/**************************************************************************
* void genere_selectoras_extension(nodo_tad *tad_origen1,        	  *
*				   nodo_tad *tad_origen2)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar las selectoras de un tad de tipo NT_EXTENSION.     	  *
* DESCRIPCION: El tad_ed hereda las selectoras del <tad_origen1> y del    *
*	 <tad_origen2> que deben ser extendidas definiendolas sobre los   *
*	 elementos y de la otra componente de la suma. Esto se hace gene- *
*	 rando lados izqs. de los axiomas que logren tal definicion y soli*
*	 citando al usuario los lados derechos respectivos.  		  *
*	 Si este proceso tiene exito, se solicita el status del tad y se  *
*	 adiciona a la lista de tads del ambiente.		  	  *
* PARAMETROS:                                                             *
*   nodo_tad	*tad_origen1	Ap. al primer tad padre de la NT_EXTENSION.  *
*   nodo_tad	*tad_origen2	Ap. al segundo tad padre de la NT_EXTENSION. *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_ed     	Apunt. al tad de que se esta generando por*
*			  	NT_EXTENSION				  *
* PRE!:  { tad_origen1-> T1, tad_origen2-> T2, tad_ed-> TC = T1 + T2, 	  *
*	   T1.selectoras = <s11,...,s1m1>, m1 >= 0,                       *
*	   T2.selectoras = <s21,...,s2m2>, m2 >= 0,			  * 
*	   y para j = 1,2  y  i = 1,...,mj :				  *
*	     sji.axiomas = <aji1,...,ajikji>, kji >= 0			  *
*          tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2.					} *
* POST!: { tad -> <t1,...,tn,TC)> <- ult_tad = tad_ed, donde		  *
*               TC.selectoras = <s11,...,s1m1,s21,...,s2m2>, tq.          *
*	        para j = 1,2  y  i = 1,...,mj :				  *
*	        sji.axiomas = <aji1,...,ajikji,bj1,...,bjl>, para l >= 0  *
*		tq. bj1,...,bjl extiende la definicion de s,   o	  *
*          tad -> <t1,...,tn> <- ult_tad, Si la generacion del nuevo tad  *
*	   	no fue exitosa.						} *
* !OJO!: El predicado de igualdad del tad_ed no esta siendo generado.     *
* LLAMA A: 			void  copie_lista_selectoras		  *
*   short es_prefijo 		short construya_axs_selec_sum_direc	  *
*   short pida_status 		void  esc_nom_tad_actual		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
* VTP. 11/Jun/1997   Se invocan funciones modificadas, para arreglar mala
*                    generacion de plantillas.  Se asume que en tad_ed esta
*                    el tad nuevo y el nombre
**************************************************************************/

int genere_selectoras_extension(nodo_tad *tad_origen1,nodo_tad *tad_origen2)
{/*----------------------------------------------------------------------*/

  nodo_select	*s,*seq,
		*cab_select=NULL,               /* j-tamara 29/May/1997 */
		*ult_select1=NULL,              /* j-tamara 29/May/1997 */
		*ult_select2=NULL;              /* j-tamara 29/May/1997 */
  short		correcto;
  char *adic;

    assert(tad_origen1!=NULL);                  /* j-tamara 29/May/1997 */
    assert(tad_ed!=NULL);                       /* j-tamara 29/May/1997 */
    assert(tad_origen2!=NULL);                  /* j-tamara 29/May/1997 */
    assert(tad_ed->nombre!=NULL); /*v-tamara 11/Jun/1997 */

    /* VTP. 11/Jun/1997 Se agregaron las siguientes 3 lineas. Ver Modificaciones */
    adic=(char *)malloc(sizeof(char)*(strlen(tad_ed->nombre)+3));
    HayMem(adic,sizeof(char)*(strlen(tad_ed->nombre)+3));
    sprintf(adic,"_%s",tad_ed->nombre);

    copie_lista_selectoras(tad_origen1->selectoras,&cab_select,&ult_select1,
			   tad_origen1->nombre,tad_ed->nombre,"",
                           tad_origen1,tad_ed->nombre);
    /* VTP. 17/Jun/1997 Se agrego el ultimo parametro */
        /* Suprimir la igualdad */
    s = cab_select;
    while ((s != NULL) && (s->sig != NULL)) {
        if (es_prefijo("EQ_",s->sig->nombre)) {
	    if (s->sig == ult_select1)
		ult_select1 = s;
            s->sig = s->sig->sig;
         }else {
            s = s->sig;
        }
    }
    if ((cab_select != NULL) && es_prefijo("EQ_",cab_select->nombre)) {
	cab_select = cab_select->sig;
    }
    tad_ed->selectoras = cab_select;

    copie_lista_selectoras(tad_origen2->selectoras,&cab_select,&ult_select2,
			   tad_origen2->nombre,tad_ed->nombre,"",
                           tad_origen2,tad_ed->nombre);
                           /* VTP. 17/Jun/1997. Ultimo parametro agregado para
                                   arreglar Bug con Nuevo/Tipo/Extension */
        /* Suprimir la igualdad */
    s = cab_select;
    while ((s != NULL) && (s->sig != NULL)) {
        if (es_prefijo("EQ_",s->sig->nombre)) {
            s->sig = s->sig->sig;
        }else {
            s = s->sig;
        }
    }
    if ((cab_select != NULL) && es_prefijo("EQ_",cab_select->nombre)) {
	cab_select = cab_select->sig;
    }
    if (tad_ed->selectoras == NULL)
        tad_ed->selectoras = cab_select;
      else
    	ult_select1->sig = cab_select
    ;
    /* VTP. 6/May/1998.  Equality administration fixed */
    seq=NULL;
    if (tad_ed->iniciales!=NULL || tad_ed->constructoras!=NULL)
      {
	adic_select_eq (tad_ed, &seq);
      }
    for (s = tad_ed->selectoras,correcto = 1; 
	 s!=NULL && s!=cab_select && correcto;
	 s = s->sig) 
      {
	correcto= construya_axs_selec_sum_direc(tad_origen2,s,adic,NULL);
      }
    for (; s!=NULL && s!=seq && correcto;
	 s = s->sig) 
      {
	correcto= construya_axs_selec_sum_direc(tad_origen1,s,adic,NULL);
      }
    if (correcto && s==seq && seq!=NULL)
      {
	correcto = construya_axiomas (seq, 0, "", tad_ed->nombre,NULL);
      }
    
    if (correcto) 
      {
	ult_tad->sig = tad_ed;
	ult_tad = tad_ed;
	tad_actual = tad_ed;
      }
    else
      {
	liberar_tad(&tad_ed); /* VTP. 6/May/1998  Fixing memory admin. */
      }
    free(adic);
    return correcto;
/*----------------------------------------------------------------------*/}

/**************************************************************************
* void informacion_tipo_extension(nodo_tad *p,cad12 nom_tad_origen1,	  *
*				  cad12 nom_tad_origen2)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Asignar tipo NT_EXTENSION a un tad.  			  *
* DESCRIPCION: Asigna NT_EXTENSION al tipo de p, <nom_tad_origen1> a la      *
*	 primera componente y <nom_tad_origen2> a la segunda componente de*
*	 la informacion asociada al tipo.				  *
* PARAMETROS:                                                             *
*    nodo_tad	*p		Ap. al tad que es de tipo NT_EXTENSION.	  *
*    cad12	nom_tad_origen1 Nombre del primer tad componente de la    *
*				suma directa.				  *
*    cad12	nom_tad_origen2 Nombre del segundo tad componente de la   *
*				suma directa.				  *
* PRE!:  { p-> T, nom_tad_origen1 = NTO1, nom_tad_origen2 = NTO2.	} *
* POST!: { T.tipo = NT_EXTENSION, T.info_tipo = <NTO1 , NTO2>		} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void informacion_tipo_extension(nodo_tad *p,cad12 nom_tad_origen1,
				cad12 nom_tad_origen2)
{/*----------------------------------------------------------------------*/

  nodo_nombre	*n;

    assert(p!=NULL);                            /* j-tamara 29/May/1997 */
    p->tipo = NT_EXTENSION;
    n = (nodo_nombre *) malloc(sizeof(nodo_nombre));
    HayMem(n,sizeof(nodo_nombre));              /* j-tamara 29/May/1997 */
    strcpy(n->nombre,nom_tad_origen1);
    p->info_tipo = n;
    n->sig = (nodo_nombre *) malloc(sizeof(nodo_nombre));
    HayMem(n->sig,sizeof(nodo_nombre));         /* j-tamara 29/May/1997 */
    n = n->sig;
    strcpy(n->nombre,nom_tad_origen2);
    n->sig = NULL;
/*----------------------------------------------------------------------*/}
