/* INTERF_TIPOS.C                                               */
/* Arhivo de manejo de la interfase                             */
 

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "definic.h"

/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* v-tamara 7/May/1997 */

/**************************************************************************
* short desea_corregir_edicion(int tipo_error,texpr mens_error)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Preguntar al usuario si desea corregir un error detectado  *
*        en la edicion.                                                   *
* DESCRIPCION: Despliega el tipo de error (sintactico o semantico) detec- *
*        tado y un mensaje de explicacion. Pregunta al usuario si desea   *
*        corregirlo o no y retorna un valor "booleano" de acuedo a su res-*
*        puesta.                                                          *
* PARAMETROS:                                                             *
*   int	  tipo_error		1: Error Sintactico en la Signatura.	  *
*                   		2: Error Sintactico en los Axiomas. 	  *
*                   		3: Error Semantico en los Axiomas: la de- *
*                   		   definicion no es completa.             *
*				4,5,6: Error semantico: programas de una  *
*				       representacion incompletos.	  *
                                7:  Otro (el mensaje viene en mens_error) *
*   texpr mens_error		Explicacion acerca del error detectado.	  *
* V.GLOBALES:                                                             *
*   int	num_ed			Si el error es sintactico,num. de la linea*
*				en el archivo donde aparece el error.	  *
*   cad50 posic_param_incompl   Si el error es semantico,num. del argumen-*
*				to de la selectora en el cual no es "com- *
*				pleta" la definicion de los axiomas (arg. *
*				que no cubre todos los posibles casos).	  *
* short desea_ayuda_tuplas_comp Booleano que indica (en caso de que el    *
*				<tipo_error>=3 y que se desee corregir el *
*				error) si el usuario desea ayuda en la de-*
*			 	finicion de los lados izqs. de los axiomas*
* RESULTADO:  short		1, Si el usuario desea corregir el error. *
*                               0, Sino.                           	  *
* PRE!:  { tipo_error=TE tipo del error detectado, mens_error=ME explica- *
*	   cion acerca del error, num_ed=NE linea del archivo donde apare-*
*	   ce el error en caso de ser sintactico, posic_param_incompl=PP  *
*	   argumento "no completo" en los axiomas en caso de error seman- *
*	   tico.							} *
* POST!: { Se informo al usuario el tipo de error TE y la explicacion ME, *
*	   desea_corregir_edicion = 1 Si el usuario desea corregir el     *
*		error y si TE=3  entonces				  *
*		desea_ayuda_tuplas_comp = 1 Si el usuario desea ayuda en  *
*			la definicion de los lados izqs. de los axiomas.  *
*		desea_ayuda_tuplas_comp = 0 Sino.                         *
*	   desea_corregir_edicion = 0 Sino.                             } *
* LLAMA A:                       					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
MODIFICADO POR: Vladimir Tamara
 Presenta advertencia al usuario de axiomas no lineales a la izquierda
**************************************************************************/

short
desea_corregir_edicion (int tipo_error, texpr mens_error)
{
/*----------------------------------------------------------------------*/

  texpr tipo_error_str, mens_dinamico;
  char resp;
  short salida;
/*** Alvaro
    cad6        respuesta;
 ***/
  assert (mens_error != NULL);	/* VTP. 7/May/1997 */
  tipo_error_str[0]='\0';
  switch (tipo_error)
    {
    case 1:
      sprintf (tipo_error_str, mensajes[57], num_ed);
      /* 57:error sintactico detectado en la signatura */
      break;
    case 2:
      sprintf (tipo_error_str, mensajes[56], num_ed);
      /* 56:error sintactico detectado en el axioma */
      break;
    case 3: /* posic_param_incompl has garbage VTP. 25.Nov.99 */
      sprintf (tipo_error_str, mensajes[55], ""); /* posic_param_incompl); */
      /* 55:definicion parcial sobre el argumento... */
      break;
    case 4:
    case 5:
    case 6:
      strcpy (tipo_error_str, mensajes[54]);
      /*error semantico: Definicion parcial de un programa */
      break;
    case 7:
      tipo_error_str[0]='\0';
      break;
    }
  if (tipo_error<7)
    {
      sprintf (mens_dinamico, "%s\n%s\n%s\n", mensajes[108], tipo_error_str,
	       mens_error);
    }
  else
    {
      sprintf(mens_dinamico,"%s\n%s\n",mensajes[108],mens_error);
    }
  escriba_mensaje_dinam (mens_dinamico);
  escriba_mensaje (2);		/* 2:Editar el archivo nuevamente? (s/n) */

  resp = lea_caracter ();
  if (toupper (resp) == mensajes[0][0])
    salida = 1;
  else
    salida = 0
      ;
  if (salida && (tipo_error == 3))
    {
      escriba_mensaje (3);	/* 3:quiere ayuda? (s/n) */
      resp = lea_caracter ();
      if (toupper (resp) == mensajes[0][0])
	desea_ayuda_tuplas_comp = 1;
      else
	desea_ayuda_tuplas_comp = 0
	  ;
    }
  return (salida);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short desea_def_igualdad()						  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Preguntar al usuario si desea definir la igualdad del tad  *
*	 que esta siendo editado.                             		  *
* DESCRIPCION: Pregunta al usuario si desea anadir el predicado de igual- *
*        dad a la signatura del tad en edicion (dicho tad no contiene aun *
*        tal definicion).                                                 *
* V.GLOBALES:                                                             *
*   nodo_tad	*tad_actual	Apuntador a la estructura que contiene el *
*				tad que esta siendo editado.           	  *
* RESULTADO:  short		1, Si el usuario desea definir la igualdad*
*                               0, Sino.                           	  *
* PRE!:  { tad_ed -> t tq. t no tiene en su signatura el predicado de     *
*	   igualdad.                                                    } *
* POST!: { desea_def_igualdad = 1 Si el usuario desea anadir el predicado *
*		de igualdad al tad t.					  *
*	   desea_def_igualdad = 0 Sino.                             	} *
* LLAMA A: int	lea_tecla_vt220						  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

short
desea_def_igualdad ()
{
/*----------------------------------------------------------------------*/

  short salida;
  texpr linea;


  escriba_mensaje (112);	/* 112:IGUALDAD */
  escriba_mensaje (4);		/* 4:Este TAD no incluye una def. de igualdad */
  sprintf (linea, "EQ_%s : %s x %s  ->  BOOL",
	   tad_ed->nombre, tad_ed->nombre, tad_ed->nombre);
  escriba_mensaje_dinam (linea);
  escriba_mensaje (5);		/* 5:Esta sera definida...presione cualquier... */
  lea_caracter ();
  salida = 1;
  return (salida);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void edite_stat_existente()						  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Editar el status de un tad existente.                      *
* DESCRIPCION: Informa cual es el status del tad actual y si este no es   *
*        primitivo permite que el usuario lo cambie.                      *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_actual	Apunt. al tad actual de la lista de tads  *
*				del ambiente.				  *
* PRE!:  { tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2, tad_actual -> ti,  1 <= i <= n       } *
* POST!: { tad_actual -> MODIF_STATUS(ti) donde MODIF_STATUS(ti) es un tad*
*	   identico a ti excepto posiblemente en el status. 		  *
*	   Si ti es un tipo primitivo (NAT o BOOL) entonces		  *
*	    MODIF_STATUS(ti) = ti.					} *
* LLAMA A: int	lea_tecla_vt220						  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
edite_stat_existente ()
{
/*----------------------------------------------------------------------*/

  char resp;
/*** Alvaro
    cad6        siga;
 ***/

  if (!flag_emacs)
    escriba_mensaje (111);	/* 111:E s t a d o */
  if (tad_actual->status == 1)
    escriba_mensaje (6);	/* 6:Este TAD es inmodificable */
  else
    escriba_mensaje (7);	/* 7:Este TAD es modificable */

  if (!tipo_primitivo (tad_actual->nombre))
    {
      escriba_mensaje (8);	/* 8:Desea modificar esto? (s/n) */
      resp = lea_caracter ();
      if (toupper (resp) == mensajes[0][0])
	{
	  tad_actual->status = (short) (1 - (tad_actual->status));
	  tad_actual->modif = 1;
	}
    }

/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void encuentre_min(cad20 lineas[50],int pos_inic,int pos_fin,int *min)  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Encontrar la posicion del minimo elemento de un arreglo.   *
* DESCRIPCION: Busca la cadena menor lexicograficamente en el arreglo     *
*        <lineas> desde la posicion <pos_inic> hasta la <pos_fin> y retor-*
*        na la posicion de dicho elemento.                                *
* PARAMETROS:                                                             *
*   cad20	lineas[50]	Arreglo de Strings.			  *
*   int 	pos_inic 	Posicion a partir de la cual se busca el  *
*				minimo del arreglo.			  *
*   int 	pos_fin 	Posicion hasta la cual se busca el minimo *
*				del arreglo.			  	  *
*   int 	*min		Posicion del minimo string de <lineas>    *
*				desde <pos_inic> hasta <pos_fin>  	  *
* PRE!:  { pos_inic = PI, pos_fin = PF, 1 <= PI <= PF <= 50,              *
*          lineas = [_______///////////////_________]                     *
*			    PI           PF				} *
* POST!: { (*min) = posicion del string minimo en //////	   	} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 04-Jul-90 *
**************************************************************************/

void
encuentre_min (cad20 lineas[50], int pos_inic, int pos_fin, int *min)
{
/*----------------------------------------------------------------------*/

  int i;

  assert (min != NULL);		/* VTP. 7/May/1997 */
  for (i = pos_inic + 1, (*min) = pos_inic; (i <= pos_fin); i++)
    {
      if (strcmp (lineas[i], lineas[(*min)]) < 0)
	(*min) = i;
    }
/*----------------------------------------------------------------------*/
}



/**************************************************************************
* void lea_nombre_nuevo_tad(cad12 nombre)	  	                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Leer el nombre de un nuevo tad.                            *
* DESCRIPCION: Lee cadenas de caracteres hasta encontrar una que no corres*
*	ponda al nombre de un tad (en el ambiente o en memoria secundaria)*
* PARAMETROS:                                                             *
*    cad12	nombre		Nombre del nuevo tad.                     *
*    int	fila		Fila en la cual debe leer la cadena.      *
*    int	columna		Columna a partir de la cual debe leer la  *
*				cadena.      				  *
* PRE!:  { tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2,                                        *
*	   fila = F, columna = C					} *
* POST!: { nombre = NT tq. NT <> ti.nombre para todo 1 <= i <= n.       } *
* LLAMA A:                                                                *
*    void	lea_cadena		void	espere_tecla              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
lea_nombre_nuevo_tad (cad12 nombre)
{
/*----------------------------------------------------------------------*/

  assert (nombre != NULL);	/* VTP. 7/May/1997 */
  escriba_dir (direct[2], "adt");
  lea_nombre (mensajes[9], nombre);	/* 9:nuevo TAD */
  if (existe_tad (nombre) != NULL)
    {
      escriba_mensaje (10);	/* 10:Un TAD con este nombre ya existe */
      strcpy (nombre, "");
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void lista_parametros_tad(cad20 lineas[50],short *num_params)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Almacenar en el arreglo los parametros de una lista    	  *
* DESCRIPCION: Guarda en cada posicion del arreglo el nombre de un parame-*
*        tro de la lista ap. por <parametros>  y retorna en <*num_params> *
*	 el numero de tales parametros.					  *
* PARAMETROS:                                                             *
*    cad20	lineas[50]	Arreglo en donde se almacenan los nombres *
*				de los parametros.			  *
*    short	*num_params	Contiene el numero de params. de la lista.*
* V.GLOBALES:                                                             *
*   nodo_nombre *parametros	Ap. a una lista de parametros, correspon- *
*				dientes a un tad o a una representacion.  *
* PRE!:  { parametros-> <p1,...,pn>, n >= 0.                            } *
* POST!: { lineas[i] = pi.nombre, para todo 1 <= i <= n,		  *
*	   (*num_params) = n						} *
* !OJO!: El arreglo se usa apartir de la posicion 1.                      *
*	 y (*num_params) es en realidad el numero de lineas en el arreglo *
*	 contando la marca de terminacion "EXIT"			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
lista_parametros_tad (cad20 lineas[50], short *num_params)
{
/*----------------------------------------------------------------------*/

  short i;
  nodo_nombre *n;
  assert (num_params != NULL);	/* VTP. 7/May/1997 */
  for (n = parametros, i = 1; (n != NULL); n = n->sig, i++)
    strcpy (lineas[i], n->nombre)
      ;
  strcpy (lineas[i], "EXIT");
  (*num_params) = i;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* nodo_nombre *lista_selectoras(nodo_select *cab_op)                      *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Retorna una lista de los nombres de las selectoras         *
**************************************************************************/

nodo_nombre *
lista_selectoras (nodo_select * cab_op)
{
/*----------------------------------------------------------------------*/

  nodo_select *s;
  nodo_nombre *l;

  l = NULL;
  for (s = cab_op; s != NULL; s = s->sig)
    insertar_nombre_ord (&l, s->nombre);
  adicionar_nombre (&l, mensajes[41]);	/* 41:Todo */
  return (l);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* nodo_nombre *lista_tads_ambiente(void)                                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Retornar una lista con los nombres de los tad en el        *
*              ambiente.                                                  *
**************************************************************************/

nodo_nombre *
lista_tads_ambiente (void)
{
/*----------------------------------------------------------------------*/

  nodo_nombre *l;
  nodo_tad *p;

  l = NULL;
  for (p = tad; p != NULL; p = p->sig)
    {
      insertar_nombre_ord (&l, p->nombre);
    }
  return (l);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void ordene_alfab(cad20 lineas[50],int pos_inic,int pos_fin)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Ordenar alfabeticamente un arreglo de strings.             *
* DESCRIPCION: Ordena un arreglo desde la posicion <pos_inic> hasta <pos_ *
*        fin> usando el metodo de seleccion, esto es, escoge el minimo de *
*	 ese segmento del arreglo, lo intercambia con el string que esta  *
*	 en <pos_inic> y ordena recursivamente el segmento de arreglo des-*
*	 de la posicion <pos_inic>+1 hasta <pos_fin>.		  	  *
* PARAMETROS:                                                             *
*   cad20 	lineas[50]	Arreglo que sera ordenado alfabetica/.    *
*   int 	pos_inic	Posicion inicial del segmento a ordenar.  *
*   int 	pos_fin		Posicion final   del segmento a ordenar.  *
* PRE!:  { pos_inic = PI,  pos_fin = PF,  1 <= PI <= PF <= 50,            *
*	   lineas = [......../////////////............]			  *
*			     PI         PF				} *
* POST!:   lineas = [........\\\\\\\\\\\\\............]			  *
*	   donde \\\\\ es una permutacion ordenada de //////		} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 06-Jul-90 *
**************************************************************************/

void
ordene_alfab (cad20 lineas[50], int pos_inic, int pos_fin)
{
/*----------------------------------------------------------------------*/

  int min;
  cad20 temp;


  if (pos_inic < pos_fin)
    {
      encuentre_min (lineas, pos_inic, pos_fin, &min);
      if (min != pos_inic)
	{
	  strcpy (temp, lineas[pos_inic]);
	  strcpy (lineas[pos_inic], lineas[min]);
	  strcpy (lineas[min], temp);
	}
      ordene_alfab (lineas, pos_inic + 1, pos_fin);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void pida_elem_completacion(cad12 nom_tad_c,cad12 nom_tad_padre,	  *
*			      cad12 nom_elem_com)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Pedir el nombre del elemento de completacion.              *
* DESCRIPCION: Despliega una ventana y captura el nombre del elemento de  *
*        completacion.                                                    *
* PARAMETROS:                                                             *
*   cad12 	nom_tad_c	Nombre del tad COMPLETACION.		  *
*   cad12 	nom_tad_padre	Nombre del tad padre de la completacion.  *
*   cad12 	nom_elem_com    Nombre del elemento de completacion.      *
* PRE!:  { nom_tad_c = TC, nom_tad_padre = T.                           } *
* POST!: { nom_elem_com = C tq.  TC = T + { C }                         } *
* LLAMA A: 	void	lea_cadena 	                                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
pida_elem_completacion (cad12 nom_tad_c, cad12 nom_tad_padre,
			cad12 nom_elem_com)
{
/*----------------------------------------------------------------------*/

  cad50 linea;

  assert (nom_tad_c != NULL && nom_tad_padre != NULL && nom_elem_com != NULL);
  /* VTP. 7/May/1997 */
  escriba_mensaje (110);	/* 110:Completacion */
  escriba_mensaje (12);		/* 12:Elemento de la extension */
  sprintf (linea, "%s = %s + {              }", nom_tad_c, nom_tad_padre);
  escriba_mensaje_dinam (linea);
  lea_cadena (nom_elem_com);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* nodo_nombre *pida_nodo_param(nodo_nombre *cab_param)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Seleccionar un nodo en una lista de parametros.            *
* DESCRIPCION: Despliega un menu con todos los nombres en la lista apunt. *
*        por <cab_param> y retorna un apuntador al nodo de la lista que   *
*	 contiene el nombre selecionado.				  *
* PARAMETROS:                                                             *
*   nodo_nombre	*cab_param	Ap. a la cabeza de la lista de parametros.*
* V.GLOBALES:                                                             *
*   nodo_nombre *parametros	Ap. a una lista de parametros, correspon- *
*				dientes a un tad o a una representacion.  *
* RESULTADO:  nodo_nombre *     Ap. al nodo seleccionado de la lista o a  *
*				    NULL si no se selecciono ninguno.     *
* PRE!:  { cab_param -> <p1,...,pn>, n >= 0.                            } *
* POST!: { pida_nodo_param-> pi , Si el parametro i fue seleccionado por  *
*				     el usuario,  1 <= i <= n.		  *
*	   pida_nodo_param= NULL, Si no se selecciono ninguno.		} *
**************************************************************************/

nodo_nombre *
pida_nodo_param (nodo_nombre * cab_param)
{
/*----------------------------------------------------------------------*/
  texpr nombre;

  admon_menu_scroll (menus_manta[PARAMETROS], cab_param, nombre);
  return (busque_nodo_nom (cab_param, nombre));
/*----------------------------------------------------------------------*/
}


/**************************************************************************
* void pida_predicado_restr(cad12 nom_tad_padre,cad12 nom_pred_r)	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Pedir el nombre del predicado de restriccion.              *
* DESCRIPCION: Despliega una ventana y captura el nombre del predicado de *
*        restriccion.                                                     *
* PARAMETROS:                                                             *
*   cad12 	nom_tad_padre	Nombre del tad padre de la completacion.  *
*   cad12 	nom_pred_r      Nombre del predicado de restriccion.      *
* PRE!:  { tad_ed->nombre = TR, nom_tad_padre = T.                      } *
* POST!: { nom_pred_r   = P tq.  TR = ( T : P )                         } *
* LLAMA A: 	void	lea_cadena 	                                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
pida_predicado_restr (cad12 nom_tad_padre, cad12 nom_pred_r)
{
/*----------------------------------------------------------------------*/

  texpr sign_restric;

  assert (nom_tad_padre != NULL && nom_pred_r != NULL);		/* VTP. 7/May/1997 */
  escriba_mensaje (109);	/* 109:Predicado de restriccion */
  sprintf (sign_restric, " : %s -> BOOL", nom_tad_padre);
  escriba_mensaje_dinam (sign_restric);
  lea_cadena (nom_pred_r);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* nodo_tad *pida_tad_ambiente(texpr mensaje)                              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Seleccionar un tad de la lista de tads del ambiente.       *
* DESCRIPCION: Pide el nombre de un tad del ambiente y retorna un apunta- *
*        dor a la estructura que lo almacena en la lista de tads del amb. *
* RESULTADO:  								  *
*    nodo_tad	*pida_tad_ambiente	Apuntador al tad seleccionado por *
*					usuario.			  *
* PRE!:  { tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2, 					} *
* POST!: { pida_tad_ambiente -> ti, 1 <= i <= n, tq ti.nombre es el nombre*
*	   seleccionado por el usuario.					} *
**************************************************************************/

nodo_tad *
pida_tad_ambiente (texpr mensaje)
{
/*----------------------------------------------------------------------*/

  texpr nombre;
  nodo_tad *p;
  nodo_nombre *lis;

  assert (mensaje != NULL);	/* VTP. 7/May/1997 */
  lis = lista_tads_ambiente ();
  p = NULL;
  admon_menu_scroll (menus_manta[TADS_AMBIENTE], lis, nombre);
  while (strcmp (nombre, "") && strcmp (nombre, mensajes[150]) && (p == NULL))
    {
      p = existe_tad_amb (nombre);
      if (p == NULL)
	{
	  escriba_mensaje (146);
	  admon_menu_scroll (menus_manta[TADS_AMBIENTE], lis, nombre);
	}
    }
  liberar_nombres (&lis);
  return (p);
/*----------------------------------------------------------------------*/
}
