/* LIB_LISTAS.C                                                 */
/* Libreria de rutinas de listas.                               */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "definic.h"
#include "apimanta.h"

#include <assert.h>		/* VTP. 5/May/1997 */
/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* v-tamara 7/May/1997 */

/**************************************************************************
* void asignar_nombre_sel(nodo_select *sel_nuevo, nodo_select *sel_viejo  *
*                         texpr tad_nuevo, texpr indice,char *tad_viejo)
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Asignar un nuevo nombre a una selectora.                   *
MODIFICACIONES:
5/MAY/1998.  VTP.  Las parameter added to support good EQUAL support
If its NULL it won't use it
**************************************************************************/

void
asignar_nombre_sel (nodo_select * sel_nuevo, nodo_select * sel_viejo,
		    texpr tad_nuevo, texpr indice, char *tad_viejo)
{
/*----------------------------------------------------------------------*/
  cad12 eqstr;
  assert (sel_nuevo != NULL && sel_viejo != NULL);
  if (!strcmp (sel_viejo->nombre_fte, "EQ"))
    strcpy (sel_nuevo->nombre_fte, sel_viejo->nombre_fte);
  else
    strcpy (sel_nuevo->nombre_fte, sel_viejo->nombre);
  /* VTP. 3/Jun/1997.  No existia este if, siempre agregaba
     _ al nombre del tad creado */
  if (tad_nuevo[0]!='\0' || indice[0]!='\0')
  {
    if (tad_viejo!=NULL)
      {
	sprintf(eqstr,"EQ_%s",tad_viejo);
      }
    if (tad_viejo!=NULL && tad_nuevo[0]!='\0' && 
	strcmp(sel_viejo->nombre,eqstr)==0)
      {
	sprintf(sel_nuevo->nombre,"EQ_%s",tad_nuevo);
      }
    else
      {

	sprintf (sel_nuevo->nombre, "%s_%s%s",
		 sel_viejo->nombre_fte, tad_nuevo, indice);
      }
  }
  else
  {
      sprintf (sel_nuevo->nombre, "%s",sel_viejo->nombre_fte);
  }

/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void asignar_nombre_gen(nodo_generadora *gen_nuevo,                     *
*                         nodo_generadora *gen_viejo,                     *
*                         texpr tad_nuevo, texpr indice)                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Asignar un nuevo nombre a una generadora.                  *
**************************************************************************/

void
asignar_nombre_gen (nodo_generadora * gen_nuevo,
		    nodo_generadora * gen_viejo,
		    texpr tad_nuevo, texpr indice)
{
/*----------------------------------------------------------------------*/

  assert (gen_nuevo != NULL && gen_viejo != NULL && tad_nuevo != NULL && indice != NULL);
  /* VTP. 5/May/1997 */
  assert (strlen (tad_nuevo) < sizeof (texpr) && strlen (indice) < sizeof (texpr));
  /* VTP. 5/May/1997 */

  strcpy (gen_nuevo->nombre_fte, gen_viejo->nombre);
  /* VTP. 3/Jun/1997.  No existia este if, siempre agregaba
     _ al nombre del tad creado */
  if (tad_nuevo[0]!='\0' || indice[0]!='\0')
  {
      sprintf (gen_nuevo->nombre, "%s_%s%s",
		gen_viejo->nombre_fte, tad_nuevo, indice);
  }
  else
  {
      sprintf (gen_nuevo->nombre, "%s",gen_viejo->nombre_fte);
  }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void asigne_nombre(texpr nombre_final,texpr nombre_viejo,		  *
*                    texpr nom_cambio,texpr nuevo_nom,cad12 indice)       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Asignar un nombre a una cadena con opciones.               *
* DESCRIPCION: Asigna <nom_final> un nombre <nom> seguido de <indice>,    *
*        donde <nom> es:                                                  *
*        - <nombre_viejo>, Si <nombre_viejo> <> <nom_cambio>.             *
*        - <nuevo_nom>,    Si <nombre_viejo> =  <nom_cambio>.             *
**************************************************************************/

void
asigne_nombre (texpr nombre_final, texpr nombre_viejo,
 		texpr nom_cambio, texpr nuevo_nom, cad12 indice)
{
/*----------------------------------------------------------------------*/
/*** Alvaro
    texpr exp_aux;
 ***/
  assert (nombre_viejo != NULL && nom_cambio != NULL && nombre_final != NULL);
  assert (indice != NULL && nuevo_nom != NULL);
  if (strcmp (nombre_viejo, nom_cambio))
    {
      strcpy (nombre_final, nombre_viejo);
      strcat (nombre_final, indice);
    }
  else
    strcpy (nombre_final, nuevo_nom);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void busque_argum_var_tdi(nodo_argum *cab,cad20 nom_tad,		  *
*			    nodo_argum **arg_tdi)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Buscar un argumento del TDI en una lista de argumentos.    *
* DESCRIPCION: <cab> apunta a una lista de argumentos y (*arg_tdi) debe   *
*        quedar apuntando al primer argumento de tipo <nom_tad> de la     *
*        lista.                                                           *
* PARAMETROS:                                                             *
*   nodo_argum		*cab		Ap. a una lista de argumentos.    *
*   cad20		nom_tad		Nombre del TDI.			  *
*   nodo_argum		**arg_tdi 	Ap. a el 1er. argumento del TDI de*
*					la lista de argumentos.    	  *
* PRE!:  { cab-> <a1,....,an>,  n >= 0,  nom_tad = TDI                  } *
* POST!: { (*arg_tdi)-> ai, 1 <= i <= n, tq. TIPO(ai) = TDI y para todo j *
*          1 <= j < i : TIPO(aj) <> TDI.                                } *
* LLAMA A: void busque_argum_var_tdi					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
busque_argum_var_tdi (nodo_argum * cab, cad20 nom_tad, nodo_argum ** arg_tdi)
{
/*----------------------------------------------------------------------*/
  assert (nom_tad != NULL && strlen (nom_tad) <= 20 && arg_tdi != NULL);
  /* VTP. 5/May/1997 */

  if (cab == NULL)
    {
      (*arg_tdi = NULL);
    }
  else if (!strcmp ((cab->expr)->tipo, nom_tad) &&
	   !strcmp ((cab->expr)->clase, "VAR"))
    {
      (*arg_tdi) = cab;
    }
  else
    {
      busque_argum_var_tdi (cab->sig, nom_tad, arg_tdi);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* nodo_nombre *busque_nodo_nom(nodo_nombre *cab_lista, texpr nombre)      *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Buscar en la lista cab_lista un nodo cuyo nombre sea el que*
*              entra como parametro.                                      *
**************************************************************************/

nodo_nombre *
busque_nodo_nom (nodo_nombre * cab_lista, texpr nombre)
{
/*----------------------------------------------------------------------*/

  nodo_nombre *n;

  assert (nombre != NULL);	/* VTP. 5/May/1997 */
  for (n = cab_lista; n != NULL && strcmp (n->nombre, nombre); n = n->sig);
  return (n);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void construya_dom_generad(nodo_generadora *inic,nodo_argum **cab_dom   *
*			     char *cont)				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar una lista de argumentos variables para una opera-  *
*	       cion generadora.						  *
* DESCRIPCION: Construye una estructura apunt. por (*cab_dom) que consiste*
*	 en una lista de argumentos variables para la generadora <inic>   *
*	 donde las variables estan subindicadas a partir de <cont>.       *
* PARAMETROS:                                                             *
*   nodo_generadora	*inic		Ap. a la generadora a la cual se  *
*					construyen los argumentos.	  *
*   nodo_argum		**cab_dom	Ap. a la cabeza de la lista de ar-*
*					gumentos.			  *
*   char		*cont		Valor con el cual se debe subindi-*
*					car la ste. variable que se use.  *
* PRE!:  { inic-> G, cont = V, G.dominio = <d1,...,dn>, n >= 0          } *
* POST!: { *cab_dom-> <a1,...,an> donde ai es una variable de tipo di,    *
*	   i = 1,...,n.							} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
construya_dom_generad (nodo_generadora * inic, nodo_argum ** cab_dom,
			char *cont)
{
/*----------------------------------------------------------------------*/

  nodo_nombre *d;
  nodo_argum *a, *ult_arg;

  assert (inic != NULL && cab_dom != NULL);	/* VTP. 5/May/1997 */
  for (d = inic->dominio, *cab_dom = NULL, ult_arg = NULL;
	d != NULL; d = d->sig, (*cont)++)
    {
      a = (nodo_argum *) malloc (sizeof (nodo_argum));
      HayMem (a, sizeof (nodo_argum));
      a->sig = NULL;
      if ((*cab_dom) == NULL)
	*cab_dom = a;
      else
	ult_arg->sig = a
	  ;
      ult_arg = a;
      a->expr = (nodo_expresion *) malloc (sizeof (nodo_expresion));
      HayMem (a->expr, sizeof (nodo_expresion));
      a->expr->argumentos = NULL;
      (a->expr->nombre)[0] = (char) toupper ((d->nombre)[0]);
      (a->expr->nombre)[1] = (*cont);
      (a->expr->nombre)[2] = '\0';
      strcpy (a->expr->tipo, d->nombre);
      strcpy (a->expr->clase, "VAR");
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
*void construya_dom_select(nodo_select *s,nodo_argum **cab_dom,char *cont)*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar una lista de argumentos variables para una opera-  *
*	       cion selectora.						  *
* DESCRIPCION: Construye una estructura apunt. por (*cab_dom) que consiste*
*	 en una lista de argumentos variables para la selectora <s>       *
*	 donde las variables estan subindicadas a partir de <cont>.       *
* PARAMETROS:                                                             *
*   nodo_select   	*inic		Ap. a la selectora a la cual se   *
*					construyen los argumentos.	  *
*   nodo_argum		**cab_dom	Ap. a la cabeza de la lista de ar-*
*					gumentos.			  *
*   char		*cont		Valor con el cual se debe subindi-*
*					car la ste. variable que se use.  *
* PRE!:  {   s -> S, cont = V, S.dominio = <d1,...,dn>, n >= 0          } *
* POST!: { *cab_dom-> <a1,...,an> donde ai es una variable de tipo di,    *
*	   i = 1,...,n.							} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
construya_dom_select (nodo_select * s, nodo_argum ** cab_dom, char *cont)
{
/*----------------------------------------------------------------------*/

  nodo_dom *d;
  nodo_argum *a, *ult_arg;

  assert (s != NULL && cab_dom != NULL && cont != NULL);	/* VTP. 5/May/1997 */
  for (d = s->dominio, *cab_dom = NULL, ult_arg = NULL;
	d != NULL; d = d->sig, (*cont)++)
    {
      a = (nodo_argum *) malloc (sizeof (nodo_argum));
      HayMem (a, sizeof (nodo_argum));
      a->sig = NULL;
      if ((*cab_dom) == NULL)
	*cab_dom = a;
      else
	ult_arg->sig = a
	  ;
      ult_arg = a;
      a->expr = (nodo_expresion *) malloc (sizeof (nodo_expresion));
      HayMem (a->expr, sizeof (nodo_expresion));
      a->expr->argumentos = NULL;
      (a->expr->nombre)[0] = (char) toupper ((d->nombre)[0]);
      (a->expr->nombre)[1] = (*cont);
      (a->expr->nombre)[2] = '\0';
      strcpy (a->expr->tipo, d->nombre);
      strcpy (a->expr->clase, "VAR");
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
*void copie_lado_der(nodo_expresion *cabeza,nodo_expresion **nueva_cabeza,*
*                    nodo_tad *p,cad12 indice)				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Copiar el lado_der de una igualdad subindicando todas las  *
*	       operaciones de la signatura de un tad.			  *
* DESCRIPCION: (*nueva_cabeza) queda apunt. a una estructura identica a la*
*        apunt. por <cabeza>, excepto que las operaciones de la signatura *
*        de <p> que aparecen en la estructura han sido subindicadas con   *
*	 <indice>.							  *
* PARAMETROS:                                                             *
*   nodo_expresion	*cabeza		Ap. a la estructura a ser copiada.*
*   nodo_expresion	**nueva_cabeza	Ap. a la estructura copiada.	  *
*   nodo_tad	 	*p		Ap. al tad cuyas operaciones seran*
*					subindicadas.			  *
*   cad12		indice		Cadena con el indice.		  *
* PRE!:  { cabeza-> EXP, p-> T, indice = I.                             } *
* POST!: { *nueva_cabeza-> COPIA_IND(EXP) que es identica a EXP, excepto  *
*	   que los nombres de la operaciones de la signatura de T estan   *
*	   concatenadas con I.						} *
* LLAMA A: 	short	es_generadora					  *
*	short	es_selectora 			void	copie_ld_argum	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
copie_lado_der (nodo_expresion * cabeza, nodo_expresion ** nueva_cabeza,
		nodo_tad * p, cad12 indice)
{
/*----------------------------------------------------------------------*/
/*** Alvaro
  nodo_expresion *e,*ne;
  ***/

  assert (nueva_cabeza != NULL && cabeza != NULL && p != NULL && indice != NULL);
  /* VTP. 5/May/1997 */

  *nueva_cabeza = (nodo_expresion *) malloc (sizeof (nodo_expresion));
  HayMem (*nueva_cabeza, sizeof (nodo_expresion));
  strcpy ((*nueva_cabeza)->nombre, cabeza->nombre);
  if (es_generadora (cabeza->nombre, p->constructoras) ||
      es_selectora (cabeza->nombre, p->selectoras))
    strcat ((*nueva_cabeza)->nombre, indice)
      ;
  copie_ld_argum (cabeza->argumentos, &((*nueva_cabeza)->argumentos), p, indice);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void copie_ld_argum(nodo_argum *cabeza,nodo_argum **nueva_cabeza,	  *
*		      nodo_tad *p,cad12 indice)				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Copiar los argumentos del lado_der de una igualdad subindi-*
*	       cando todas las operaciones de la signatura de un tad.     *
* DESCRIPCION: (*nueva_cabeza) queda apunt. a una estructura identica a la*
*        apunt. por <cabeza>, excepto que las operaciones de la signatura *
*        de <p> que aparecen en la estructura han sido subindicadas con   *
*	 <indice>.							  *
* PARAMETROS:                                                             *
*   nodo_argum    	*cabeza		Ap. a la estructura a ser copiada.*
*   nodo_argum    	**nueva_cabeza	Ap. a la estructura copiada.	  *
*   nodo_tad	 	*p		Ap. al tad cuyas operaciones seran*
*					subindicadas.			  *
*   cad12		indice		Cadena con el indice.		  *
* PRE!:  { cabeza-> EXP, p-> T, indice = I.                             } *
* POST!: { *nueva_cabeza-> COPIA_IND(EXP) que es identica a EXP, excepto  *
*	   que los nombres de la operaciones de la signatura de T estan   *
*	   concatenadas con I.						} *
* LLAMA A: void  copie_lado_der 					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
copie_ld_argum (nodo_argum * cabeza, nodo_argum ** nueva_cabeza,
		nodo_tad * p, cad12 indice)
{
/*----------------------------------------------------------------------*/

  nodo_argum *a, *na, *ult_argum;

  assert (nueva_cabeza != NULL && p != NULL && indice != NULL);		/* VTP. 5/May/1997 */
  *nueva_cabeza = NULL;
  ult_argum = NULL;
  for (a = cabeza; a != NULL; a = a->sig)
    {
      na = (nodo_argum *) malloc (sizeof (nodo_argum));
      HayMem (na, sizeof (nodo_argum));
      na->sig = NULL;
      if (ult_argum == NULL)
	*nueva_cabeza = na;
      else
	ult_argum->sig = na
	  ;
      ult_argum = na;
      copie_lado_der (a->expr, &(na->expr), p, indice);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void copie_lista_argum(nodo_argum *cabeza,nodo_argum **nueva_cabeza,    *
*       		 cad50 nom, nodo_tad *t)                          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Copiar una lista de argumentos con reemplazo de nombres.   *
* DESCRIPCION: (*nueva_cabeza) queda apunt. a una estructura identica a la*
*        apunt. por <cabeza>, excepto que las ocurrencias de <nom> han si-*
*	 do reemplazadas por <nuevo_nom> y se ha concatenado <indice> a to*
*	 dos los nombres de la estructura.				  *
* PARAMETROS:                                                             *
*   nodo_argum    	*cabeza		Ap. a la estructura a ser copiada.*
*   nodo_argum    	**nueva_cabeza	Ap. a la estructura copiada.	  *
*   texpr		nom		Cadena con el nombre a cambiar.	  *
*   texpr		nuevo_nom 	Cadena con el nombre por el cual  *
*					se debe cambiar <nom>.	          *
*   cad12		indice		Cadena con el indice.		  *
* PRE!:  { cabeza-> EXP, nom = N, nuevo_nom = NN , indice = I.   	} *
* POST!: { *nueva_cabeza-> COPIA_IND(EXP) que es identica a EXP, excepto  *
*          que las ocurrencias de N han sido reemplazadas por NN y se ha  *
*	   concatenado <indice> a todos los nombres de la estructura.   } *
* LLAMA A: void  copie_lista_expresion					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
copie_lista_argum (nodo_argum * cabeza, nodo_argum ** nueva_cabeza,
		   cad50 nom, nodo_tad * t,char *nom_adic)
{
/*----------------------------------------------------------------------*/

  nodo_argum *a, *na, *ult_argum;

  assert (nueva_cabeza != NULL);
  *nueva_cabeza = NULL;
  ult_argum = NULL;
  for (a = cabeza; a != NULL; a = a->sig)
    {
      na = (nodo_argum *) malloc (sizeof (nodo_argum));
      HayMem (na, sizeof (nodo_argum));
      na->sig = NULL;
      if (ult_argum == NULL)
	*nueva_cabeza = na;
      else
	ult_argum->sig = na
	  ;
      ult_argum = na;
      copie_lista_expresion (a->expr, &(na->expr), nom, t,nom_adic);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void copie_lista_argum_apunt(nodo_argum *cabeza,			  *
*               nodo_argum **nueva_cabeza,nodo_argum *ap,nodo_argum **nap)*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Copiar una lista de argumentos marcando uno en particular. *
* DESCRIPCION: (*nueva_cabeza) queda apunt. a una estructura identica a la*
*        apunt. por <cabeza> y (*na) apunt. al argumento correspondiente  *
*	 en la lista (*nueva_cabeza) al apuntado por <ap> en la lista     *
*	 <cabeza>.							  *
* PARAMETROS:                                                             *
*   nodo_argum    	*cabeza		Ap. a la estructura a ser copiada.*
*   nodo_argum    	**nueva_cabeza	Ap. a la estructura copiada.	  *
*   nodo_argum    	*ap		Ap. a un argumento en la estructu-*
*					apunt. por <cabeza>.		  *
*   nodo_argum    	**nap		Ap. al argumento en la estructura *
*					apunt. por (*nueva_cabeza) corres-*
*					pondiente al apunt. por <ap>.     *
* PRE!:  { cabeza-> EXP, ap-> ARG, tq. ARG es un argumento en EXP  	} *
* POST!: { *nueva_cabeza-> COPIA(EXP) que es identica a EXP,              *
*	   *nap-> CORR(ARG) argumento correspond. a ARG en COPIA(EXP).	} *
* LLAMA A: void  copie_lista_expresion_apunt				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
copie_lista_argum_apunt (nodo_argum * cabeza, nodo_argum ** nueva_cabeza,
			  nodo_argum * ap, nodo_argum ** nap)
{
/*----------------------------------------------------------------------*/

  nodo_argum *a, *na, *ult_argum;

  assert (ap != NULL && nap != NULL);	/* VTP. 5/May/1997 */
  *nueva_cabeza = NULL;
  ult_argum = NULL;
  for (a = cabeza; a != NULL; a = a->sig)
    {
      na = (nodo_argum *) malloc (sizeof (nodo_argum));
      HayMem (na, sizeof (nodo_argum));
      na->sig = NULL;
      if (ult_argum == NULL)
	*nueva_cabeza = na;
      else
	ult_argum->sig = na
	  ;
      ult_argum = na;
      copie_lista_expresion_apunt (a->expr, &(na->expr), ap, nap);
      if (a == ap)
	{
	  *nap = na;
	}
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void copie_lista_componentes(nodo_componente *cabeza,			  *
*              nodo_componente **nueva_cabeza,nodo_componente **ult_comp) *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Copiar una lista de componentes .  			  *
* DESCRIPCION: (*nueva_cabeza) queda apunt. a una lista identica a la	  *
*        apunt. por <cabeza>.						  *
* PARAMETROS:                                                             *
*   nodo_componente	*cabeza		Ap. a la estructura a ser copiada.*
*   nodo_componente	**nueva_cabeza	Ap. a la estructura copiada.	  *
*   nodo_componente	**ult_comp    	Ap. al ult.elem. de la nueva lista*
* PRE!:  { cabeza-> LISTA                                          	} *
* POST!: { *nueva_cabeza->COPIA(LISTA) que es identica a LISTA,           *
*	   *ult_comp   -> a la ultima componente de COPIA(LISTA).	} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
copie_lista_componentes (nodo_componente * cabeza,
			  nodo_componente ** nueva_cabeza,
			  nodo_componente ** ult_comp)
{
/*----------------------------------------------------------------------*/

  nodo_componente *m, *n;

  assert (nueva_cabeza != NULL && ult_comp != NULL);	/* VTP. 5/May/1997 */
  *nueva_cabeza = NULL;
  *ult_comp = NULL;
  for (m = cabeza; m != NULL; m = m->sig)
    {
      n = (nodo_componente *) malloc (sizeof (nodo_componente));
      HayMem (n, sizeof (nodo_componente));
      if (*ult_comp == NULL)
	*nueva_cabeza = n;
      else
	(*ult_comp)->sig = n
	  ;
      *ult_comp = n;
      strcpy (n->nombre, m->nombre);
      strcpy (n->tipo, m->tipo);
      strcpy (n->clase, m->clase);
      n->sig = NULL;
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void copie_lista_expresion(nodo_expresion *cabeza,                      *
*                            nodo_expresion **nueva_cabeza,               *
*   			     cad50 nom, nodo_tad *t)                      *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Copiar una expresion con reemplazo de nombres.   	  *
* DESCRIPCION: (*nueva_cabeza) queda apunt. a una estructura identica a la*
*        apunt. por <cabeza>, excepto que las ocurrencias de <nom> han si-*
*	 do reemplazadas por <nuevo_nom> y se ha concatenado <indice> a to*
*	 dos los nombres de la estructura.				  *
* PARAMETROS:                                                             *
*   nodo_expresion	*cabeza		Ap. a la estructura a ser copiada.*
*   nodo_expresion	**nueva_cabeza	Ap. a la estructura copiada.	  *
*   texpr		nom		Cadena con el nombre del TAD 	  *
*                                       que se esta cambiando.            *
*   nodo_tad            *t              Tad sobre el cual se copian las   *
*                                       expresiones.                      *
* PRE!:  { cabeza-> EXP, nom = N, nuevo_nom = NN , indice = I.   	} *
* POST!: { *nueva_cabeza-> COPIA_IND(EXP) que es identica a EXP, excepto  *
*          que las ocurrencias de N han sido reemplazadas por NN y se ha  *
*	   concatenado <indice> a todos los nombres de la estructura.   } *
* LLAMA A: void  copie_lista_argum    					  *
**************************************************************************/

void
copie_lista_expresion (nodo_expresion * cabeza,
			nodo_expresion ** nueva_cabeza,
			cad50 nom, nodo_tad * t,char *adic_nom)
/* VTP. 3/Jun/1997 Parametro adic_nom aumentado para arreglar
	falla que ocurria al crear tipos nuevos por extension */
{
/*----------------------------------------------------------------------*/

/*** Alvaro
    nodo_expresion 	*e,*ne;
    cad20		exp_aux;
 ***/
  nodo_select *s, *s_eq;
  nodo_generadora *g;
  texpr cad_auxd, cad_auxi;

  assert (nueva_cabeza != NULL && nom != NULL && cabeza != NULL);	/* VTP. 5/May/1997 */
  *nueva_cabeza = (nodo_expresion *) malloc (sizeof (nodo_expresion));
  HayMem (*nueva_cabeza, sizeof (nodo_expresion));
  if (t != NULL)
    {
      for (g = t->iniciales; g != NULL &&
	   strcmp (g->nombre_fte, cabeza->nombre); g = g->sig);
      if (g != NULL)
	{
	  strcpy ((*nueva_cabeza)->nombre, g->nombre);
	  if (adic_nom[0]!='\0') /* VTP 11/Jun/1997 */
	    {
		sprintf((*nueva_cabeza)->nombre,"%s_%s",g->nombre,adic_nom);
	    }
	}
      else
	{
	  for (g = t->constructoras; g != NULL &&
		strcmp (g->nombre_fte, cabeza->nombre); g = g->sig);
	  if (g != NULL)
	    {
	      strcpy ((*nueva_cabeza)->nombre, g->nombre);
		if (adic_nom[0]!='\0') /* VTP 11/Jun/1997 */
		{
		    sprintf((*nueva_cabeza)->nombre,"%s_%s",g->nombre,adic_nom);
		}
	    }
	  else
	    {
	      for (s = t->selectoras, s_eq = NULL; s != NULL &&
		   strcmp (s->nombre_fte, cabeza->nombre); s = s->sig)
		if (!strcmp (s->nombre_fte, "EQ"))
		  s_eq = s;
	      if (s != NULL) {
		strcpy ((*nueva_cabeza)->nombre, s->nombre);
		/* VTP. La siguiente linea se agrego para arreglar
			 falla en Tipos/Nuevo/Extension */
		if (adic_nom[0]!='\0')
		{
		    sprintf((*nueva_cabeza)->nombre,"%s_%s",s->nombre,adic_nom); /* VTP Por Quitar*/
		}
	      }
	      else if ((s_eq != NULL) &&
			es_prefijo ("EQ_", cabeza->nombre))
		{
		  strcpy (cad_auxd, cabeza->nombre);
		  parta_expresion (cad_auxd, cad_auxi, '_');
		  if (!strcmp (cad_auxi, nom))
		    strcpy ((*nueva_cabeza)->nombre, s_eq->nombre);
		  else
		    strcpy ((*nueva_cabeza)->nombre, cabeza->nombre);
		}
	      else
		strcpy ((*nueva_cabeza)->nombre, cabeza->nombre);
	    }
	}
    }
  else
    strcpy ((*nueva_cabeza)->nombre, cabeza->nombre);
  strcpy ((*nueva_cabeza)->clase, cabeza->clase);
  if (!strcmp (cabeza->tipo, nom) && t != NULL) {
    strcpy ((*nueva_cabeza)->tipo, t->nombre);
		/* VTP. La siguiente linea se agrego para arreglar
			 falla en Tipos/Nuevo/Extension */
		if (adic_nom[0]!='\0')
		{
		    sprintf((*nueva_cabeza)->tipo,"%s_%s",
			      t->nombre,adic_nom); /* VTP Por Quitar*/
		}
  }
  else
    strcpy ((*nueva_cabeza)->tipo, cabeza->tipo);
  copie_lista_argum (cabeza->argumentos, &((*nueva_cabeza)->argumentos), nom, t,adic_nom);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void revisar_lista_expresion(nodo_expresion *cabeza, nodo_tad *t,       *
*                              char nom[])                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:  Revisa que las expresiones sean consistentes despues del    *
*             cambio de nombre.                                           *
*             Cabeza es la expresion que se revisa                        *
*             t es el tad que se esta creando.                            *
*             nom es el nombre del tad generico                           *
*             nom_param es el nombre del tipo parametro.                  *
**************************************************************************/

void
revisar_lista_expresion (nodo_expresion * cabeza, nodo_tad * t,
			  char nom[])
{
/*----------------------------------------------------------------------*/
/*** Alvaro
    nodo_expresion 	*e;
    cad20		exp_aux;
 ***/
  nodo_select *s;
  nodo_generadora *g;
  nodo_argum *arg;
  nodo_select *s_eq;
  texpr cad_auxd, cad_auxi;

  assert (cabeza != NULL && nom != NULL);	/* VTP. 5/May/1997 */
  if (t != NULL)
    {
      for (g = t->iniciales; g != NULL &&
	   strcmp (g->nombre_fte, cabeza->nombre); g = g->sig);
      if (g != NULL)
	strcpy (cabeza->nombre, g->nombre);
      else
	{
	  for (g = t->constructoras; g != NULL &&
		strcmp (g->nombre_fte, cabeza->nombre); g = g->sig);
	  if (g != NULL)
	    strcpy (cabeza->nombre, g->nombre);
	  else
	    {
	      for (s = t->selectoras, s_eq = NULL; s != NULL &&
		   strcmp (s->nombre_fte, cabeza->nombre); s = s->sig)
		if (!strcmp (s->nombre_fte, "EQ"))
		  s_eq = s;
	      if (s != NULL)
		strcpy (cabeza->nombre, s->nombre);
	      else if ((s_eq != NULL) &&
			es_prefijo ("EQ_", cabeza->nombre))
		{
		  strcpy (cad_auxd, cabeza->nombre);
		  parta_expresion (cad_auxd, cad_auxi, '_');
		  if (!strcmp (cad_auxi, nom))
		    strcpy (cabeza->nombre, s_eq->nombre);
		}
	    }
	}
      if (!strcmp (cabeza->tipo, nom) && t != NULL)
	strcpy (cabeza->tipo, t->nombre);
      for (arg = cabeza->argumentos; arg != NULL; arg = arg->sig)
	revisar_lista_expresion (arg->expr, t, nom);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void copie_lista_expresion_apunt(nodo_expresion *cabeza,		  *
*          nodo_expresion **nueva_cabeza,nodo_argum *ap,nodo_argum **nap) *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Copiar una expresion marcando un argumento  en particular. *
* DESCRIPCION: (*nueva_cabeza) queda apunt. a una estructura identica a la*
*        apunt. por <cabeza> y (*na) apunt. al argumento correspondiente  *
*	 en la lista (*nueva_cabeza) al apuntado por <ap> en la lista     *
*	 <cabeza>.							  *
* PARAMETROS:                                                             *
*   nodo_expresion	*cabeza		Ap. a la estructura a ser copiada.*
*   nodo_expresion	**nueva_cabeza	Ap. a la estructura copiada.	  *
*   nodo_argum    	*ap		Ap. a un argumento en la estructu-*
*					apunt. por <cabeza>.		  *
*   nodo_argum    	**nap		Ap. al argumento en la estructura *
*					apunt. por (*nueva_cabeza) corres-*
*					pondiente al apunt. por <ap>.     *
* PRE!:  { cabeza-> EXP, ap-> ARG, tq. ARG es un argumento en EXP  	} *
* POST!: { *nueva_cabeza-> COPIA(EXP) que es identica a EXP,              *
*	   *nap-> CORR(ARG) argumento correspond. a ARG en COPIA(EXP).	} *
* LLAMA A: void  copie_lista_argum_apunt				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
copie_lista_expresion_apunt (nodo_expresion * cabeza,
			      nodo_expresion ** nueva_cabeza,
			      nodo_argum * ap, nodo_argum ** nap)
{
/*----------------------------------------------------------------------*/
/*** Alvaro
  nodo_expresion *e,*ne;
 ***/

  assert (cabeza != NULL && nueva_cabeza != NULL && nap != NULL && ap != NULL);
  /* VTP. 5/May/1997 */

  *nueva_cabeza = (nodo_expresion *) malloc (sizeof (nodo_expresion));
  HayMem (*nueva_cabeza, sizeof (nodo_expresion));	/* VTP. 5/May/1997 */
  strcpy ((*nueva_cabeza)->nombre, cabeza->nombre);
  strcpy ((*nueva_cabeza)->tipo, cabeza->tipo);
  strcpy ((*nueva_cabeza)->clase, cabeza->clase);
  copie_lista_argum_apunt (cabeza->argumentos, &((*nueva_cabeza)->argumentos),
			    ap, nap);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void copie_lista_generadoras(nodo_generadora *cabeza,                   *
* 			     nodo_generadora **nueva_cabeza,              *
*			     nodo_generadora **ult_generad,               *
*			     texpr nom,texpr nuevo_nom,cad12 indice,      *
*                            nodo_tad *t)                                 *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Copiar una lista de generadoras con reemplazo de nombres.  *
* DESCRIPCION: (*nueva_cabeza) queda apunt. a una lista identica a la	  *
*        apunt. por <cabeza>, excepto que las ocurrencias de <nom> han si-*
*	 do reemplazadas por <nuevo_nom> y se ha concatenado <indice> a to*
*	 dos los nombres de la lista .				  	  *
* PARAMETROS:                                                             *
*   nodo_generadora	*cabeza		Ap. a la estructura a ser copiada.*
*   nodo_generadora	**nueva_cabeza	Ap. a la estructura copiada.	  *
*   nodo_generadora	**ult_generad 	Ap. al ult.elem. de la nueva lista*
*   texpr		nom		Cadena con el nombre a cambiar.	  *
*   texpr		nuevo_nom 	Cadena con el nombre por el cual  *
*					se debe cambiar <nom>.	          *
*   cad12		indice		Cadena con el indice.		  *
* PRE!:  { cabeza-> LISTA, nom = N, nuevo_nom = NN , indice = I.   	} *
* POST!: { *nueva_cabeza->COPIA_IND(LISTA) que es identica a LISTA,excepto*
*          que las ocurrencias de N han sido reemplazadas por NN y se ha  *
*	   concatenado <indice> a todos los nombres de la estructura.     *
*	   *ult_generad-> a la ultima generadora de COPIA_IND(LISTA).	} *
* LLAMA A:                                                                *
*    void	asigne_nombre 		void	copie_lista_nombres	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
copie_lista_generadoras (nodo_generadora * cabeza,
			  nodo_generadora ** nueva_cabeza,
			  nodo_generadora ** ult_generad,
			  texpr nom, texpr nuevo_nom, cad12 indice)
{
/*----------------------------------------------------------------------*/

  nodo_generadora *m, *n;
  nodo_nombre *ult = NULL;

  assert (nueva_cabeza != NULL && ult_generad != NULL);		/* VTP. 5/May/1997 */
  assert (nom != NULL && nuevo_nom != NULL && indice != NULL);	/* VTP. 5/May/1997 */

  *ult_generad = NULL;
  *nueva_cabeza = NULL;
  for (m = cabeza; m != NULL; m = m->sig)
    {
      n = (nodo_generadora *) malloc (sizeof (nodo_generadora));
      HayMem (n, sizeof (nodo_generadora));	/* VTP. 5/May/1997 */
      if (*ult_generad == NULL)
	*nueva_cabeza = n;
      else
	(*ult_generad)->sig = n
	  ;
      *ult_generad = n;
      asignar_nombre_gen (n, m, nuevo_nom, indice);
      copie_lista_nombres (m->dominio, &(n->dominio), &ult,
			    nom, nuevo_nom, "");
      n->sig = NULL;
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void copie_lista_igualdades(nodo_igualdad *cabeza,                      *
*                             nodo_igualdad **nueva_cabeza,               *
*	 		      cad50 nom,cad50 nuevo_nom,cad12 indice,     *
*                             nodo_tad *t)                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Copiar una lista de igualdades con reemplazo de nombres.   *
* DESCRIPCION: (*nueva_cabeza) queda apunt. a una lista identica a la	  *
*        apunt. por <cabeza>, excepto que las ocurrencias de <nom> han si-*
*	 do reemplazadas por <nuevo_nom> y se ha concatenado <indice> a to*
*	 dos los nombres de la lista .				  	  *
* PARAMETROS:                                                             *
*   nodo_igualdad	*cabeza		Ap. a la estructura a ser copiada.*
*   nodo_igualdad	**nueva_cabeza	Ap. a la estructura copiada.	  *
*   texpr		nom		Cadena con el nombre a cambiar.	  *
*   texpr		nuevo_nom 	Cadena con el nombre por el cual  *
*					se debe cambiar <nom>.	          *
*   cad12		indice		Cadena con el indice.		  *
* PRE!:  { cabeza-> LISTA, nom = N, nuevo_nom = NN , indice = I.   	} *
* POST!: { *nueva_cabeza->COPIA_IND(LISTA) que es identica a LISTA,excepto*
*          que las ocurrencias de N han sido reemplazadas por NN y se ha  *
*	   concatenado <indice> a todos los nombres de la estructura.   } *
* LLAMA A:                                                                *
*    void  copie_lista_expresion 	void  transf_estruct_iguald	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
copie_lista_igualdades (nodo_igualdad * cabeza, nodo_igualdad ** nueva_cabeza,
			 cad50 nom, cad50 nuevo_nom, cad12 indice,
			 nodo_tad * t,char *adic_nom)
/* VTP. 17/Jun/1997 Ultimo parametro agregado */
{
/*----------------------------------------------------------------------*/

  nodo_igualdad *p, *np, *ult_ig;

  assert (nueva_cabeza != NULL);		/* VTP. 5/May/1997 */
  assert (indice != NULL);	/* VTP. 5/May/1997 */
  assert (adic_nom!=NULL);      /* VTP. 17/Jun/1997 */
  assert (nuevo_nom!=NULL);      /* VTP. 17/Jun/1997 */


  *nueva_cabeza = NULL;
  ult_ig = NULL;
  for (p = cabeza; p != NULL; p = p->sig)
    {
      np = (nodo_igualdad *) malloc (sizeof (nodo_igualdad));
      HayMem (np, sizeof (nodo_igualdad));	/* VTP. 5/May/1997 */
      np->sig = NULL;
      if (ult_ig == NULL)
	*nueva_cabeza = np;
      else
	ult_ig->sig = np;
      ult_ig = np;
      copie_lista_expresion (p->lado_izq, &(np->lado_izq), nom, t,adic_nom);
      copie_lista_expresion (p->lado_der, &(np->lado_der), nom, t,adic_nom);
      if (!strcmp (indice, ""))	/* No se esta haciendo Prod. Cartesiano */
	transf_estruct_iguald (np);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void revisar_lista_igualdades(nodo_igualdad *cabeza, nodo_tad *t,       *
*                               char nom[])                               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Revisa que todas las igualdades de una selectoras sean     *
*              consistentes, despues del cambio de nombres.               *
**************************************************************************/

void
revisar_lista_igualdades (nodo_igualdad * cabeza, nodo_tad * t,
			   char nom[])
{
/*----------------------------------------------------------------------*/

  nodo_igualdad *p;
  assert (cabeza != NULL && t != NULL);		/* VTP. 5/May/1997 */

  for (p = cabeza; p != NULL; p = p->sig)
    {
      revisar_lista_expresion (p->lado_izq, t, nom);
      revisar_lista_expresion (p->lado_der, t, nom);
      construya_exp (p->lado_izq, p->li_str,"");
      construya_exp (p->lado_der, p->ld_str,"");
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void copie_lista_nombres(nodo_nombre *cabeza,nodo_nombre **nueva_cabeza,*
*	   nodo_nombre **ult_nom,texpr nom,texpr nuevo_nom,cad12 indice)  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Copiar una lista de nombres con reemplazos.  		  *
* DESCRIPCION: (*nueva_cabeza) queda apunt. a una lista identica a la	  *
*        apunt. por <cabeza>, excepto que las ocurrencias de <nom> han si-*
*	 do reemplazadas por <nuevo_nom> y se ha concatenado <indice> a to*
*	 dos los nombres de la lista .				  	  *
* PARAMETROS:                                                             *
*   nodo_nombre		*cabeza		Ap. a la estructura a ser copiada.*
*   nodo_nombre		**nueva_cabeza	Ap. a la estructura copiada.	  *
*   nodo_nombre		**ult_nom     	Ap. al ult.elem. de la nueva lista*
*   texpr		nom		Cadena con el nombre a cambiar.	  *
*   texpr		nuevo_nom 	Cadena con el nombre por el cual  *
*					se debe cambiar <nom>.	          *
*   cad12		indice		Cadena con el indice.		  *
* PRE!:  { cabeza-> LISTA, nom = N, nuevo_nom = NN , indice = I.   	} *
* POST!: { *nueva_cabeza->COPIA_IND(LISTA) que es identica a LISTA,excepto*
*          que las ocurrencias de N han sido reemplazadas por NN y se ha  *
*	   concatenado <indice> a todos los nombres de la estructura.     *
*	   *ult_nom    -> a la ultima nombre de COPIA_IND(LISTA).	} *
* LLAMA A:   void	asigne_nombre 				    	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
copie_lista_nombres (nodo_nombre * cabeza, nodo_nombre ** nueva_cabeza,
		     nodo_nombre ** ult_nom,
		     texpr nom, texpr nuevo_nom, cad12 indice)
{
/*----------------------------------------------------------------------*/

  nodo_nombre *m, *n;
  assert (nueva_cabeza != NULL);	/* VTP. 5/May/1997 */
  assert (ult_nom != NULL && nom != NULL && nuevo_nom != NULL && indice != NULL);
  /* VTP. 5/May/1997 */

  *ult_nom = NULL;
  *nueva_cabeza = NULL;
  for (m = cabeza; m != NULL; m = m->sig)
    {
      n = (nodo_nombre *) malloc (sizeof (nodo_nombre));
      HayMem (n, sizeof (nodo_nombre));		/* VTP. 5/May/1997 */
      if (*ult_nom == NULL)
	*nueva_cabeza = n;
      else
	(*ult_nom)->sig = n
	  ;
      *ult_nom = n;
      asigne_nombre (n->nombre, m->nombre, nom, nuevo_nom, indice);
      n->sig = NULL;
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void copie_lista_selectoras(nodo_select *cabeza,                        *
* 			      nodo_select **nueva_cabeza,                 *
*  			      nodo_select **ult_select,                   *
* 			      texpr nom,texpr nuevo_nom,cad12 indice,     *
*                             nodo_tad *t)                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Copiar una lista de selectoras con reemplazo de nombres.   *
* DESCRIPCION: (*nueva_cabeza) queda apunt. a una lista identica a la	  *
*        apunt. por <cabeza>, excepto que las ocurrencias de <nom> han si-*
*	 do reemplazadas por <nuevo_nom> y se ha concatenado <indice> a to*
*	 dos los nombres de la lista .				  	  *
* PARAMETROS:                                                             *
*   nodo_select   	*cabeza		Ap. a la estructura a ser copiada.*
*   nodo_select   	**nueva_cabeza	Ap. a la estructura copiada.	  *
*   nodo_select   	**ult_select  	Ap. al ult.elem. de la nueva lista*
*   texpr		nom		Cadena con el nombre a cambiar.	  *
*   texpr		nuevo_nom 	Cadena con el nombre por el cual  *
*					se debe cambiar <nom>.	          *
*   cad12		indice		Cadena con el indice.		  *
*   nodo_tad            *t              Tad donde se copian las selectoras*
* PRE!:  { cabeza-> LISTA, nom = N, nuevo_nom = NN , indice = I.   	} *
* POST!: { *nueva_cabeza->COPIA_IND(LISTA) que es identica a LISTA,excepto*
*          que las ocurrencias de N han sido reemplazadas por NN y se ha  *
*	   concatenado <indice> a todos los nombres de la estructura.     *
*	   *ult_select -> a la ultima selectora de COPIA_IND(LISTA).	} *
* LLAMA A:                                                                *
*    void	asigne_nombre 		void	copie_lista_igualdades	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
copie_lista_selectoras (nodo_select * cabeza,
			 nodo_select ** nueva_cabeza,
			 nodo_select ** ult_select,
			 texpr nom, texpr nuevo_nom, cad12 indice,
			 nodo_tad * t,char *adic_nom)
/* VTP. 17/Jun/1997.  Ultimo parametro agregado para arreglar
		      Bug con Tipos/Nuevo/Extension */
{
/*----------------------------------------------------------------------*/

  nodo_select *m, *n;
/*** Alvaro
    nodo_nombre *ult;
 ***/
  nodo_dom *md, *nd, *ult_nom;
  assert (nueva_cabeza != NULL && ult_select != NULL); /* VTP. 3/Jun/1997 */
  assert(adic_nom!=NULL); /* VTP. 17/Jun/1997 */
  *ult_select = NULL;
  *nueva_cabeza = NULL;
  for (m = cabeza; m != NULL; m = m->sig)
    {
      n = (nodo_select *) malloc (sizeof (nodo_select));
      HayMem (n, sizeof (nodo_select));		/* VTP. 5/May/1997 */
      n->sig = NULL;		/* VTP. 6/May/1997 */
      if (*ult_select == NULL)
	*nueva_cabeza = n;
      else
	(*ult_select)->sig = n
	  ;
      *ult_select = n;
      asignar_nombre_sel (n, m, nuevo_nom, indice,nom);
      n->caracterizadora = m->caracterizadora;
      ult_nom = NULL;		/* Se copia la lista de dominios */
      n->dominio = NULL;
      for (md = m->dominio; md != NULL; md = md->sig)
	{
	  nd = (nodo_dom *) malloc (sizeof (nodo_dom));
	  HayMem (nd, sizeof (nodo_dom));	/* VTP. 5/May/1997 */
	  if (ult_nom == NULL)
	    n->dominio = nd;
	  else
	    ult_nom->sig = nd
	      ;
	  ult_nom = nd;
	  asigne_nombre (nd->nombre, md->nombre, nom, nuevo_nom, "");
	  nd->induccion = md->induccion;
	  nd->sig = NULL;
	}
      asigne_nombre (n->codominio, m->codominio, nom, nuevo_nom, "");
      copie_lista_igualdades (m->axiomas, &(n->axiomas),
			       nom, nuevo_nom, "", t,adic_nom);
      /* Ultimo parametro pasado para arreglar bug con Tipos/Extension/Nuevos */
    }
  for (m = *nueva_cabeza; m != NULL; m = m->sig)
    revisar_lista_igualdades (m->axiomas, t, nom);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_plantilla_generadora(nodo_generadora *g,cad20 nom_codom,	  *
*                            nodo_expresion **plant_construct,char *cont) *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Construye una expresion para una generadora con todos sus  *
*	       argumentos variables.					  *
* DESCRIPCION: (*plant_construct) apunt. a una estructura expresion de la *
*        generadora <g> con todos sus argumentos variables del tipo       *
*        apropiado.                                                       *
* PARAMETROS:                                                             *
*   nodo_generadora	*g		Ap. a la operacion a la que se le *
*					va a construir la plantilla.	  *
*   cad20		nom_codom	Nombre del codominio de <g>.      *
*   nodo_expresion **plant_construct	Ap. a la plantilla construida de g*
*   char		*cont		Valor con el cual se debe subindi-*
*					car la sig. variable a ser usuada.*
* PRE!:  { g-> G, G.dominio = <d1,...,dn>,  n >= 0, nom_codom = TDI,      *
*	   cont = V.                            			} *
* POST!: { *plant_construct-> EXP que es una estructura que contiene la   *
*	   expresion: <G.nombre>(v1,...,vn), donde vi es una variable de  *
*	   tipo di,  i = 1,...,n.					} *
* LLAMA A: void  construya_dom_generad 					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
genere_plantilla_generadora (nodo_generadora * g, cad20 nom_codom,
			      nodo_expresion ** plant_construct, char *cont)
{
/*----------------------------------------------------------------------*/

  nodo_expresion *expr;

  assert (g != NULL && nom_codom != NULL && cont != NULL);	/* VTP. 5/May/1997 */

  expr = (nodo_expresion *) malloc (sizeof (nodo_expresion));
  HayMem (expr, sizeof (nodo_expresion));	/* VTP. 5/May/1997 */
  strcpy (expr->nombre, g->nombre);
  strcpy (expr->tipo, nom_codom);
  strcpy (expr->clase, "CONSTR");
  construya_dom_generad (g, &(expr->argumentos), cont);
  (*plant_construct) = expr;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_plantilla_selectora(nodo_select *s,			  *
*                               nodo_expresion **plant_select,char *cont) *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Construye una expresion para una selectora con todos sus   *
*	       argumentos variables.					  *
* DESCRIPCION: (*plant_select) apunt. a una estructura expresion de la    *
*        selectora <s> con todos sus argumentos variables del tipo        *
*        apropiado.                                                       *
* PARAMETROS:                                                             *
*   nodo_selectora	*s		Ap. a la operacion a la que se le *
*					va a construir la plantilla.	  *
*   nodo_expresion 	**plant_select  Ap. a la plantilla construida de s*
*   char		*cont		Valor con el cual se debe subindi-*
*					car la sig. variable a ser usuada.*
* PRE!:  { s-> S, S.dominio = <d1,...,dn>,  n >= 0, cont = V.   	} *
* POST!: { *plant_select   -> EXP que es una estructura que contiene la   *
*	   expresion: <S.nombre>(v1,...,vn), donde vi es una variable de  *
*	   tipo di,  i = 1,...,n.					} *
* LLAMA A: void  construya_dom_generad 					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
genere_plantilla_selectora (nodo_select * s, nodo_expresion ** plant_select,
			     char *cont)
{
/*----------------------------------------------------------------------*/

  nodo_expresion *expr;

  assert (s != NULL && plant_select != NULL);	/* VTP. 5/May/1997 */
  expr = (nodo_expresion *) malloc (sizeof (nodo_expresion));
  HayMem (expr, sizeof (nodo_expresion));	/* VTP. 5/May/1997 */
  strcpy (expr->nombre, s->nombre);
  strcpy (expr->tipo, s->codominio);
  strcpy (expr->clase, "SELECT");
  construya_dom_select (s, &(expr->argumentos), cont);
  (*plant_select) = expr;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* int longitud_lista_argum(nodo_argum *cabeza)			 	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Contar el numero de argumentos de una lista.               *
* DESCRIPCION: Recorre la lista contando sus elementos.                   *
* PARAMETROS:                                                             *
*   nom_argum		*cabeza		Ap. a la cabeza de la lista.      *
* RESULTADO:  int			Num. de elementos de la lista.    *
* PRE!:  { cabeza -> <a1,...,an>,  n >= 0.                              } *
* POST!: { longitud_lista_argum = n.                                    } *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

int
longitud_lista_argum (nodo_argum * cabeza)
{
/*----------------------------------------------------------------------*/

  int i;


  for (i = 0; cabeza != NULL; cabeza = cabeza->sig, i++)
    ;
  return (i);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void subindique_axs(nodo_igualdad *cabeza,cad12 indice)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Subindicar todas las operaciones de la signatura de un tad *
*	       que aparecen en una lista de igualdades.			  *
* DESCRIPCION: <cabeza> queda apunt. a la misma estructura donde las opera*
*	 ciones de la signatura de ult_tad que aparecen en la estructura  *
*	 han sido subindicadas con <indice>.				  *
* PARAMETROS:                                                             *
*   nodo_igualdad 	*cabeza		Ap. a la estruc. a ser subindicada*
*   cad12		indice		Cadena con el indice.		  *
* V.GLOBALES:                                                             *
*    nodo_tad	*ult_tad	Apunt. al ultimo tad de la lista de tads  *
*				del ambiente.				  *
* PRE!:  { cabeza-> EXP, ult_tad-> T, indice = I.                      	} *
* POST!: { cabeza-> COPIA_IND(EXP) que es identica a EXP, excepto  	  *
*	   que los nombres de la operaciones de la signatura de T estan   *
*	   concatenadas con I.						} *
* LLAMA A: 	void	subindique_expr   				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
subindique_axs (nodo_igualdad * cabeza, cad12 indice)
{
/*----------------------------------------------------------------------*/

  nodo_igualdad *ax;
  assert (cabeza != NULL && indice != NULL);	/* VTP. 5/May/1997 */
  for (ax = cabeza; ax != NULL; ax = ax->sig)
    {
      subindique_expr (ax->lado_izq, indice);
      subindique_expr (ax->lado_der, indice);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void subindique_expr(nodo_expresion *cabeza,cad12 indice)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Subindicar todas las operaciones de la signatura de un tad *
*	       que aparecen en una expresion.			   	  *
* DESCRIPCION: <cabeza> queda apunt. a la misma estructura donde las opera*
*	 ciones de la signatura de ult_tad que aparecen en la estructura  *
*	 han sido subindicadas con <indice>.				  *
* PARAMETROS:                                                             *
*   nodo_expresion	*cabeza		Ap. a la estruc. a ser subindicada*
*   cad12		indice		Cadena con el indice.		  *
* V.GLOBALES:                                                             *
*    nodo_tad	*ult_tad	Apunt. al ultimo tad de la lista de tads  *
*				del ambiente.				  *
* PRE!:  { cabeza-> EXP, ult_tad-> T, indice = I.                      	} *
* POST!: { cabeza-> COPIA_IND(EXP) que es identica a EXP, excepto  	  *
*	   que los nombres de la operaciones de la signatura de T estan   *
*	   concatenadas con I.						} *
* LLAMA A: 	short	es_generadora					  *
*	short	es_selectora 			void	subindique_expr   *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
subindique_expr (nodo_expresion * cabeza, cad12 indice)
{
/*----------------------------------------------------------------------*/

  cad50 nom_aux;
  nodo_argum *a;
  assert (cabeza != NULL && indice != NULL);	/* VTP. 5/May/1997 */
  strcpy (nom_aux, cabeza->nombre);
  strcat (nom_aux, indice);
  if ((!strcmp (cabeza->clase, "CONSTR") &&
	es_generadora (nom_aux, ult_tad->constructoras))
      || (!strcmp (cabeza->clase, "SELECT") &&
	  es_selectora (nom_aux, ult_tad->selectoras)))
    {
      /* Es una operacion del producto Cartesiano */
      strcpy (cabeza->nombre, nom_aux);
    }
  for (a = cabeza->argumentos; a != NULL; a = a->sig)
    {
      subindique_expr (a->expr, indice);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void suprima_param_tad(nodo_tad *p,cad50 nom_param)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Eliminar un parametro de un tad.                           *
* DESCRIPCION: Recorre la lista de parametros de <p> y eleimina a <nom_   *
*        param>.                                                          *
* PARAMETROS:                                                             *
*   nodo_tad	*p		Ap. al tad al que se elimina un parametro *
*   cad50	nom_param	Nombre del parametro a eliminar.	  *
* PRE!:  { nom_param = NP, p->parametros = <p1,...pi,NP,p(i+1),...,pn>,   *
*	   n >= 0.							} *
* POST!: { p->parametros = <p1,...pi,p(i+1),...,pn>   			} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

void
suprima_param_tad (nodo_tad * p, cad50 nom_param)
{
/*----------------------------------------------------------------------*/

  nodo_nombre *n;

  assert (p != NULL && nom_param != NULL && strlen (nom_param) <= 50);	/* VTP. 5/May/1997 */
  if (!strcmp (p->parametros->nombre, nom_param))
    {
      p->parametros = p->parametros->sig;
    }
  else
    {
      for (n = p->parametros;
	   (n->sig != NULL) && strcmp (n->sig->nombre, nom_param);
	   n = n->sig)
	;
      if (n->sig != NULL)
	n->sig = n->sig->sig;
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* nodo_nombre *lista_tipos(nodo_tad *t)	                                  *
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Crear una lista con todos los tipos que aparecen como      *
*              parametros en la signatura del tad t.                      *
* PRE :  lista_tipos = NULL                                               *
* POS :  lista_tipos = <t1,t2,...tn>, donde t1..tn aparecen como          *
*        dominio en las generadoras y selectoras del tad t. Ademas,       *
*        ti < ti+1, para todo i entre 1 y n-1.                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Cesar Munoz                                   Fecha: 15-Ene-92   *
**************************************************************************/

nodo_nombre *
lista_tipos (nodo_tad * t)
{
/*----------------------------------------------------------------------*/
  nodo_nombre *lista = NULL;
  nodo_nombre *n;
  nodo_dom *d;
  nodo_generadora *gen;
  nodo_select *sel;

  lista = NULL;
  if (t == NULL)
    return (lista);
  for (gen = t->iniciales; gen != NULL; gen = gen->sig)
    {
      for (n = gen->dominio; n != NULL; n = n->sig)
	if (strcmp (n->nombre, t->nombre)
	    && (busque_nodo_nom (t->parametros, n->nombre) == NULL))
	  insertar_nombre_ord (&lista, n->nombre);
    }
  for (gen = t->constructoras; gen != NULL; gen = gen->sig)
    for (n = gen->dominio; n != NULL; n = n->sig)
      if (strcmp (n->nombre, t->nombre)
	  && busque_nodo_nom (t->parametros, n->nombre) == NULL)
	insertar_nombre_ord (&lista, n->nombre);
  for (sel = t->selectoras; sel != NULL; sel = sel->sig)
    {
      if (strcmp (sel->codominio, t->nombre)
	  && busque_nodo_nom (t->parametros, sel->codominio) == NULL)
	insertar_nombre_ord (&lista, sel->codominio);
      for (d = sel->dominio; d != NULL; d = d->sig)
	if (strcmp (d->nombre, t->nombre)
	    && busque_nodo_nom (t->parametros, d->nombre) == NULL)
	  insertar_nombre_ord (&lista, d->nombre);
    }
  return (lista);

/*----------------------------------------------------------------------*/
}

/**
* Adds to a ordered list of types, a list of all the types used in a 
* given type
* @author Vladimir Tamara
**/
int                                     /* Amount of errors */
related_types_list(nodo_tad * t,        /* Type to add */
		  nodo_nombre** lista, /* List to complete */
		  char *merr)          /* Errors buffer */
{
  nodo_nombre *n;
  nodo_dom *d;
  nodo_generadora *gen;
  nodo_select *sel;
  int cr=0; /* Codigo retornado */

  assert(merr!=NULL);
  assert(lista!=NULL);



  if (t == NULL)
    return cr;
  for (gen = t->iniciales; gen != NULL; gen = gen->sig)
    {
      for (n = gen->dominio; n != NULL; n = n->sig)
	{ 
	  if (busque_nodo_nom(t->parametros,n->nombre)==NULL)
	    { cr+=add_type_recTypeList(n->nombre,lista,merr); }
	}
    }
  for (gen = t->constructoras; gen != NULL; gen = gen->sig)
    {
      for (n = gen->dominio; n != NULL; n = n->sig)
	{ 
	  if (busque_nodo_nom(t->parametros,n->nombre)==NULL)
	    { cr+=add_type_recTypeList(n->nombre,lista,merr); }
	}
    }

  for (sel = t->selectoras; sel != NULL; sel = sel->sig)
    {
      if (busque_nodo_nom(t->parametros,sel->codominio)==NULL)
	{ cr+=add_type_recTypeList(sel->codominio,lista,merr); }
      for (d = sel->dominio; d != NULL; d = d->sig)
	{ 
	  if (busque_nodo_nom(t->parametros,d->nombre)==NULL)
	    { cr+=add_type_recTypeList(d->nombre,lista,merr); }
	}
      /* We look for symbols from other ADT in the axioms */
      cr+=types_from_axioms(sel->axiomas,lista,merr);
    }
  return cr;
}



/**
 * Adds a type and all its related types to an ordered list of 
 * type names without repetition
 * @author Vladimir Tamara
 **/
int                                       /* Returns the amount of errors */
add_type_recTypeList(char *ntad,          /* Type name to add */
		     nodo_nombre **list, /* List of type names */
		     char *merr)         /* Errors Buffer */
{
 int cr=0;
 nodo_tad *tc;
 texpr mser;
 int fromdisk=0; /* Indicates if the type must be loaded from disk */
 
 assert(list!=NULL);
 assert(merr!=NULL);
 assert(ntad!=NULL);
 
 
 if (busque_nodo_nom(*list,ntad)==NULL)
   {
     if ((tc=buscaNombreLTipo(tad,ntad))==NULL)
	{  /* If the type is not in memory look for it in disk */
	 if ((tc=tipoAmbiente(&tad,ntad,merr))!=NULL)	  
	   { fromdisk=1; }
	 else
	   {
	     cr+=1;
	     sprintf(mser,mensajes[194],ntad);
	     strcat(merr,mser);
	     strcat(merr,"\n");
	     return cr;
	   }
	}
     insertar_nombre_ord (list, ntad);
     cr+=related_types_list(tc,list,merr);
     if (fromdisk)
 	{ elimElemLTipo(&tad,tc->nombre); 	}
   }
 return cr;
}


/**
 * Adds all the types related with a list of axioms to an ordered list 
 * of type names without repetitions
 * @author Vladimir Tamara
 **/
int                                                   /* Amount of errors */
types_from_axioms(
				 nodo_igualdad * ax,  /* Axioms */
				 nodo_nombre** list,  /* Partial list */
				 char *merr)          /* Errors buffer */
{
 int vr=0;

 assert(ax!=NULL);
 assert(list!=NULL);
 assert(merr!=NULL);

 for (;ax!=NULL;ax=ax->sig)
   {
     vr=vr + types_from_expression(ax->lado_izq,list,merr);
     vr=vr + types_from_expression(ax->lado_der,list,merr);
   }
 return vr;
}



/**
 * Adds all the types related with an expression to an ordered list of 
 * type names without repetitions
 * @author Vladimir Tamara
 **/
int                                           /* 0 iff OK */
types_from_expression(
		     nodo_expresion * expr,  /* Expression */
		     nodo_nombre** list,     /* Partial list */
		     char *merr)             /* Errors buffer */
{
 int vr=0;
 char ntad[500];
 nodo_argum *na;
 
 assert(expr!=NULL);
 assert(list!=NULL);
 assert(merr!=NULL);

 if (strcmp(expr->clase,"SELECT")==0 && 
     existsSymbol(symtable,expr->nombre,ntad)==SYM_SELECTOR)
   { vr=add_type_recTypeList(ntad,list,merr); }

 for (na=expr->argumentos;na!=NULL;na=na->sig)
   { vr+= types_from_expression(na->expr,list,merr); }
 return vr;
}


/**************************************************************************
* void liberar_nombres(nodo_nombre **l)	                          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Borrar la lista de nombres *l                              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Cesar Munoz                                   Fecha: 15-Ene-92   *
**************************************************************************/

void
liberar_nombres (nodo_nombre ** l)
{
/*----------------------------------------------------------------------*/
 nodo_nombre *aux;
 nodo_nombre *p;

 assert (l != NULL);		/* VTP. 5/May/1997 */
 for (p = (*l); p != NULL;)
   {
     aux = p;
     p = p->sig;
     aux->sig = NULL;		/* VTP. 5/May/1997 */
     free (aux);
   }
 (*l) = NULL;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* int existe_nombre(nodo_nombre *l, cad20 nombre)                         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE : l = lista de nombres, nombre = cadena                             *
* POST : existe_nombre = 0, si nombre no pertenece a l                    *
*        existe_nombre != 0, si nombre si pertence a l                    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Cesar Munoz                                   Fecha: 15-Ene-92   *
**************************************************************************/

int
existe_nombre (nodo_nombre * l, cad20 nombre)
{
/*----------------------------------------------------------------------*/
 nodo_nombre *p;
 assert (nombre != NULL && strlen (nombre) <= 20);	/* VTP. 5/May/1997 */
 for (p = l; (p != NULL) && (strcmp (p->nombre, nombre)); p = p->sig);
 return (p != NULL);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void insertar_nombre_ord(nodo_nombre **l, cad20 nombre)                 *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Inserta nombre en la lista *l, en orden alfabetico. Si     *
*              nombre ya existe, *l no es modificada.                     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Cesar Munoz                                   Fecha: 15-Ene-92   *
**************************************************************************/

void
insertar_nombre_ord (nodo_nombre ** l, cad20 nombre)
{
/*----------------------------------------------------------------------*/
 nodo_nombre *aux;
 nodo_nombre *p;

 assert (l != NULL && nombre != NULL);	/* VTP. 5/May/1997 */
 aux = (nodo_nombre *) malloc (sizeof (nodo_nombre));
 HayMem (aux, sizeof (nodo_nombre));	/* VTP. 5/May/1997 */
 strcpy (aux->nombre, nombre);
 aux->sig = NULL;
 if ((*l) == NULL)
   (*l) = aux;
 else if (strcmp ((*l)->nombre, nombre) > 0)
   {
     aux->sig = (*l);
     (*l) = aux;
   }
 else if (strcmp ((*l)->nombre, nombre) < 0)
   {
     for (p = (*l); (p->sig != NULL) &&
	  (strcmp (p->sig->nombre, nombre) < 0); p = p->sig);
     if ((p->sig == NULL) || (strcmp (p->sig->nombre, nombre) > 0))
	{
	 aux->sig = p->sig;
	 p->sig = aux;
	}
   }

/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void adicionar_nombre(nodo_nombre **l, cad20 nombre)                    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Coloca como primer nombre de la lista a nombre.            *
*              No verifica su existencia en la lista.                     *
**************************************************************************/

void
adicionar_nombre (nodo_nombre ** l, cad20 nombre)
{
/*----------------------------------------------------------------------*/
 nodo_nombre *aux;
/*** Alvaro
   nodo_nombre *p=NULL;
***/

 assert (l != NULL && nombre != NULL && strlen (nombre) <= 20);	/* VTP. 5/May/1997 */
 aux = (nodo_nombre *) malloc (sizeof (nodo_nombre));
 HayMem (aux, sizeof (nodo_nombre));	/* VTP. 5/May/1997 */
 strcpy (aux->nombre, nombre);
 aux->sig = *l;
 (*l) = aux;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void insertar_expresion(nodo_argum **l, nodo_expresion *expr)           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Inserta expr en la lista *l, en orden alfabetico. Si       *
*              expr->nombre ya existe, *l no es modificada.               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Cesar Munoz                                   Fecha: 15-Ene-92   *
**************************************************************************/

void
insertar_expresion (nodo_argum ** l, nodo_expresion * expr)
{
/*----------------------------------------------------------------------*/
 nodo_argum *aux;
 nodo_argum *p;

 assert (l != NULL && expr != NULL);	/* VTP. 5/May/1997 */
 aux = crear_argumento (expr);
 if ((*l) == NULL)
   (*l) = aux;
 else if (strcmp ((*l)->expr->nombre, expr->nombre) > 0)
   {
     aux->sig = (*l);
     (*l) = aux;
   }
 else if (strcmp ((*l)->expr->nombre, expr->nombre) < 0)
   {
     for (p = (*l); (p->sig != NULL) &&
	  (strcmp (p->sig->expr->nombre, expr->nombre) < 0);
	  p = p->sig);
     if ((p->sig == NULL) ||
	 (strcmp (p->sig->expr->nombre, expr->nombre) > 0))
	{
	 aux->sig = p->sig;
	 p->sig = aux;
	}
   }

/*----------------------------------------------------------------------*/
}
