/* MAN_TIPOS.C							*/
/* Arhivo de manejo de definicion de tipos.                     */
 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "definic.h"
#include "os_dep.h" /*** VTP. 13/Mar/1997 ***/

/* A este codigo fuente se le han aplicado estas tecnicas preventivas:*/
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
      y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
      en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* v-tamara 7/May/1997 */


/* Definicion de la variable global VersionKernelManta */

char VersionKernelManta[20]="3.1 Beta 4";



/**************************************************************************
* void borre_tad(nodo_tad *p)		       				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Borrar un tad del ambiente y del directorio de tads.       *
* DESCRIPCION: recibe un TAD existente en el ambiente y lo elimina de la  *
*        lista de tads y del directorio de TADs.                          *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad		Apunt. a la cabeza de la lista de los tads*
*				del ambiente.				  *
*    nodo_tad	*ult_tad	Apunt. al ultimo tad de la lista de tads  *
*				del ambiente.				  *
*    nodo_tad	*tad_actual	Apunt. al tad actual de la lista de tads  *
*				del ambiente.				  *
* PRE!:  { tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2, tad_actual -> ti,  1 <= i <= n y       *
*          D_ADT = <a1.adt,...,am.adt>, m >= 0				} *
* POST!: { p -> tk, 1 <= k <= n,					  *
*	    tad -> <t1,...t(k-1),t(k+1),...,tn> <- ult_tad, 		  *
*	   Si i = k, Tad_actual -> t1, Sino tad_actual -> ti. y           *
*	   Si tk.nombre = aj, para algun 1 <= j <= m entonces             *
*	    D_ADT = <a1.adt,...,a(j-1).adt,a(j+1).adt,...,am.adt>        }*
* !OJO!:                                                                  *
* LLAMA A:                                                                *
*   nodo_tad	*pida_tad_ambiente	  void 	nombre_archivo		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
* VTP 13/Mar/1997                                                         *
*	Antes borraba un archivo empleando system("rm ... ahora           *
*	se usa unlink (portable para DOS y UNIX)                          *
**************************************************************************/

void borre_tad(nodo_tad *p)
{/*----------------------------------------------------------------------*/

    cad20    nom_arch;
    nodo_tad *q;
    char     resp;
    /*** VTP Declaraba texpr    linea; ***/
    FILE     *fp;


    if (p != NULL) {
      if (p->status == 1)
        escriba_mensaje(14);       /* 14:este TAD no puede ser borr...!*/
      else {
        escriba_mensaje(58);       /* 58:Borrar TAD */
        escriba_mensaje(63);       /* 63:El TAD sera removido del amb...*/
        escriba_mensaje(24);       /* 24:Esta seguro? (s/n) */
        resp = (char)toupper(lea_caracter());
        if (resp == mensajes[0][0]) {
	  /*VTP. 4/May/1998. Symbols table administration*/
	  removeSymbolADT(&symtable,p->nombre);

          for (q = tad; (q->sig != p); q = q->sig)
            ;
          q->sig = p->sig;
          if (ult_tad == p)
            ult_tad = q
         ;
          nombre_archivo(p->nombre,nom_arch);
          if ((fp = fopen(nom_arch,"r")) != NULL) {
            fclose(fp);
   /*** Decia sprintf(linea,"rm -f %s",nom_arch); system(linea); ***/
            unlink(nom_arch);
          }

          if (tad_actual == p)
            tad_actual = tad;
        }
        }
      }
/*----------------------------------------------------------------------*/}

/**************************************************************************
* void cargar_tad(char *nombre_tad)                                       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Cargar el tad de nombre <nombre_tad> del ambiente.         *
*	         							  *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_actual	Apunt. al tad actual de la lista de tads  *
*				del ambiente.				  *
*    texpr	token		Contiene el nombre del tad que se desea   *
*				cargar de memoria secundaria.		  *
* PRE!:  { D_ADT = <a1.adt,...,am.adt>, m >= 0,			  	  *
*	   tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2, tad_actual -> ti,  1 <= i <= n .     } *
* POST!: { token = aj, 1<= j <= m, t(n+1) contine la informacion del tad  *
*          contenido en el archivo aj.adt.                                *
*	   Si aj = tk.nombre, para algun k con 1 <= k <= n, entonces 	  *
*            tad -> <t1,...t(k-1),t(k+1),...,tn,t(n+1)> <- ult_tad, 	  *
*	   Sino, tad -> <t1,...,tn,t(n+1)> <- ult_tad.	  		  *
*	   tad_actual -> t(n+1).                                        } *
* !OJO!: Carga el nuevo tad sobre la posible version pre-existente en el  *
*        ambiente.                                                        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Cesar A. Munoz H.                               FECHA: 25-Oct-91 *
**************************************************************************/

void cargar_tad(char *nombre_tad)
{/*----------------------------------------------------------------------*/
    nodo_nombre *l=NULL;
    nodo_tad    *t;

    assert(nombre_tad!=NULL); /* VTP. 7/May/1997 */

    if (!tipo_primitivo(nombre_tad) &&
	busque_arch_tad(nombre_tad)) {
        t = ult_tad;
        for (l = lista_tipos(ult_tad); l!= NULL; l = l->sig)
            busque_arch_tad(l->nombre);
        liberar_nombres(&l);
        tad_actual = t;
      }
    else if (strcmp(nombre_tad,""))
      escriba_mensaje(146);
/*----------------------------------------------------------------------*/}

/**************************************************************************
* void cargue_tad()							  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Cargar la informacion de un tad del archivo al ambiente.   *
* DESCRIPCION: Pide el nombre de un tad que se encuentre almacenado en el *
*        directorio D_ADT y lo carga en el ambiente llamando la funcion   *
*        cargar_tad(tad).                                                 *
* !OJO!: Carga el nuevo tad sobre la posible version pre-existente en el  *
*        ambiente.                                                        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Cesar A. Munoz H.                               FECHA: 25-Oct-91 *
**************************************************************************/

void cargue_tad()
{/*----------------------------------------------------------------------*/

    texpr       nombre;
    nodo_nombre *lis=NULL;

    lis = lista_dir(direct[2],"adt",0);
    elimine_primitivos(&lis);
    if (lis != NULL) {
      admon_menu_scroll(menus_manta[TADS_DIRECTORIO],lis,nombre);
      if ( strcmp(nombre,"") && strcmp(nombre,mensajes[150]))
	cargar_tad(nombre);
      liberar_nombres(&lis);
    }
/*----------------------------------------------------------------------*/}


/**************************************************************************
* void elimine_primitivos(nodo_nombre **lis)				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Borrar de una lista de TADS para cargar los tipos primiti  *
*              vos NAT y BOOL. Estos se borran puesto que ya estan carga  *
*              dos                                                        *
* DESCRIPCION:                                                            *
*                                                                         *
* PRE:                                                                    *
* !OJO!:                                                                  *
*                                                                         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Hernan Aponte                                   FECHA: 16-Mar-94 *
**************************************************************************/

void elimine_primitivos(nodo_nombre **lis)
{/*----------------------------------------------------------------------*/

    nodo_nombre   *aux;
    nodo_nombre   *ant;
    int           i;

    assert(lis!=NULL); /* VTP. 7/May/1997 */

    for (i=0; i<2 && ((*lis) != NULL); i++) {
      aux = (*lis);
      ant = NULL;
      while ( (aux != NULL) && strcmp(aux->nombre,"NAT") &&
	      strcmp(aux->nombre,"BOOL") ) {
         ant = aux;
         aux = aux->sig;
      }
      if (aux != NULL)
	{
	  if (ant == NULL) {
	    (*lis) = (*lis)->sig;
	    aux->sig = NULL;
	    free(aux);
	  }
	  else {
	    ant->sig = aux->sig;
	    aux->sig = NULL;
	    free(aux);
	  }
	}
    }

/*----------------------------------------------------------------------*/}


/**************************************************************************
* void cargue_tads_primitivos()						  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Cargar en el ambiente los tads primitivos (NAT y BOOL).    *
* DESCRIPCION: Inicializa la lista de tads en el ambiente con los tads    *
*        primitivos (NAT y BOOL) cargandolos de memoria secundaria.       *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad		Apunt. a la cabeza de la lista de los tads*
*				del ambiente.				  *
*    nodo_tad	*ult_tad	Apunt. al ultimo tad de la lista de tads  *
*				del ambiente.				  *
*    nodo_tad	*tad_actual	Apunt. al tad actual de la lista de tads  *
*				del ambiente.				  *
* PRE!:  { tad = NULL, ult_tad = NULL, tad_actual = NULL                  *
*          D_ADT = <a1.adt,...,BOOL.adt,...,NAT.adt,...,am.adt>, m >= 2 } *
* POST!: { tad -> <t1,t2> <- ult_tad,  t1.nombre = NAT, t2.nombre = BOOL  *
*	   tad_actual -> t1.                                            } *
* LLAMA A:                                                                *
*   void	nombre_archivo		void	lea_arch_tad              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void cargue_tads_primitivos()
{/*----------------------------------------------------------------------*/

    cad20 nom,nom_arch;
    texpr mens_error;
    FILE *fp;


    proved = NULL;
    unproved = NULL;
    ult_proved = NULL;
    ult_unproved = NULL;
    tad = NULL;
    ult_tad = NULL;
    tad_actual = NULL;
    strcpy(nom,"BOOL");
    nombre_archivo(nom,nom_arch);
    fp = fopen(nom_arch,"r");
    FileOpen(fp,nom_arch);
    lea_arch_tad(fp);
    fclose(fp);
    strcpy(nom,"NAT");
    nombre_archivo(nom,nom_arch);
    fp = fopen(nom_arch,"r");
    FileOpen(fp,nom_arch);
    lea_arch_tad(fp);
    fclose(fp);
    tad_actual = tad;
    tad_ed = tad_actual;
    fp = fopen("D_THEO/AXS.axs","r");
    FileOpen(fp,"D_THEO/AXS.axs");
    especiales = analice_conjeturas(fp,mens_error);
    fclose(fp);
/*----------------------------------------------------------------------*/}

/************************************************************************
* nodo_tad *crear_tad(void)                                             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Crea un tad vacio.                                      *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio  91 *
************************************************************************/

nodo_tad *crear_tad(void)
{/*--------------------------------------------------------------------*/
    nodo_tad *t;

    t = (nodo_tad *) malloc (sizeof(nodo_tad));
    HayMem(t,sizeof(nodo_tad));
    t->info_tipo = NULL;
    t->parametros = NULL;
    t->iniciales = NULL;
    t->constructoras = NULL;
    t->selectoras = NULL;
    strcpy(t->nombre,"");
    t->status = 0;
    t->tipo = 0;
    t->sig = NULL;
    t->modif = 1;
    return(t);
/*--------------------------------------------------------------------*/}

/**************************************************************************
* void edite_tad_amb()							  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Editar un tad existente en el ambiente.                    *
* DESCRIPCION: Pide el nombre de un tad del ambiente e invoca a la funcion*
*        encargada de la edicion de tads existentes.                      *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_actual	Apunt. al tad actual de la lista de tads  *
*				del ambiente.				  *
* PRE!:  { tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2, tad_actual -> ti,  1 <= i <= n       } *
* POST!: { tad_actual -> MODIF(tk), 1 <= k <= n,			  *
*	   tad -> <t1,...,t(k-1),t(k+1),...,tn,MODIF(tk)> <- ult_tad,     *
*	   donde MODIF(tk) denota una version modificada (posiblemente    *
*  	   igual) de tk.						} *
* LLAMA A:                                                                *
*   nodo_tad	*pida_tad_ambiente	  void 	esc_nom_tad_actual	  *
*   void 	edite_tad_existente				     	  *	
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void edite_tad_amb()
{/*----------------------------------------------------------------------*/

  nodo_tad *p;

  p = pida_tad_ambiente(mensajes[61]);  /* 61: Editar tad */
  if (p != NULL) {
  tad_actual = p;
  	edite_tad_existente();
      }
/*----------------------------------------------------------------------*/}


/**************************************************************************
* void manejar_tipos()							  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Controlar el proceso de manejo de definicion de tipos.     *
* DESCRIPCION: Despliega el menu de opciones de manejo de tads e invoca a *
*        la funcion correspondiente a la seleccion del usuario.           *
* PRE!:  { tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2, 					  *
*	   D_ADT = <a1.adt,...,am.adt>, m >= 0,				  *
*          D_PRINT_ADT = <p1.adt,...,pl.adt>, l >= 0.			} *
* POST!: { La lista de tads en el ambiente, D_ADT, D_PRINT_ADT han sido   *
*	   posiblemente modificados de acuerdo al gusto del usuario.	} *
* LLAMA A:                                                                *
*                	                void	nuevo_tad                 *
*    void	edite_tad_amb  		void	cargue_tad                *
*    void	salve_tad      		void	imprima_tad               *
*    void	copie_tad      		void	borre_tad                 *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void manejar_tipos()
{/*----------------------------------------------------------------------*/

    int       opcion;

    opcion = admon_menu(menus_manta[TIPOS]);
    while ( opcion  != NO_OPCION &&
            opcion != menus_manta[TIPOS]->num_op-2 ) {
	switch (opcion) {
	  case 0:
	    escriba_dir(direct[2],"adt");
	    break;
	  case 1:
	    nuevo_tad();
	    break;
	  case 2:
	    edite_tad_amb();
	    break;
	  case 3:
	    cargue_tad();
	    break;
	  case 4:
	    salve_tad(tad_actual);
	    break;
	  case 5:
	    imprima_tad(tad_actual);
	    break;
	  case 6:
	    borre_tad(tad_actual);
	    break;
	  case 8:                   /* AYUDA */
	    escriba_mensaje(152);
	    break;
          default:
	    escriba_mensaje(146);
	    break;
        }
        opcion = admon_menu(menus_manta[TIPOS]);
    }

/*----------------------------------------------------------------------*/}


/**************************************************************************
* nodo_tad *existe_tad(cad12 nom)                                         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si existe el tad <nom> en el ambiente o en el   *
*        directorio.                                                      *
* DESCRIPCION: Busca al tad <nom> en el ambiente (en la lista de tads) y  *
*	 si no lo encuentra lo busca en memoria secundaria (en el directo-*
*	 rio D_ADT). Si lo encuentra retorna a la direccion a la estruc-  *
*	 tura que lo contiene. 						  *
* PARAMETROS: cad12  nom	    Nombre del pretendido tad.            *
*             nodo_tad **p 	    (*p) apunta a la estructura del tad   *
*                                   <nom> si este existe en el ambiente.  *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad		Apunt. a la cabeza de la lista de los tads*
*				del ambiente.				  *
*    nodo_tad	*ult_tad	Apunt. al ultimo tad de la lista de tads  *
*				del ambiente.				  *
* PRE!:  { nom = NT,                                                      *
*          tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2.					  *
*          D_ADT = <a1.adt,...,am.adt>, m >= 0				} *
* POST!: { existe_tad = tk, 						  *
*	     Si NT = tk.nombre, 1 <= k <= n y p -> tk,  o     		  *
*	        NT = ak, 1 <= k <= m,        t(n+1).nombre = ak		  *
*	         tad -> <t1,...,tn,t(n+1)> <- ult_tad = p. 		  *
*	   existe_tad = NULL, Sino.					} *
* LLAMA A:                                                                *
*   short	busque_arch_tad		                        	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

nodo_tad *existe_tad(cad12 nom)
{/*----------------------------------------------------------------------*/
    nodo_tad *p;

    assert(nom!=NULL); /* VTP. 7/May/1997 */
    if (!strcmp(nom,""))
        return 0;
    p = existe_tad_amb(nom);
    if (p == NULL) {
	if (busque_arch_tad(nom)) {
	    p = ult_tad;
	    return(p);
       } else {
	    return(NULL);
	 }
   }
   return(p);
/*----------------------------------------------------------------------*/}

/**************************************************************************
* nodo_tad *existe_tad_amb(cad12 nom)	                 		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si existe el tad <nom> en el ambiente.          *
* DESCRIPCION: Busca al tad <nom> en el ambiente (en la lista de tads) y  *
*	 si lo encuentra retorna su direccion, si no retorna NULL.        *
* PARAMETROS: cad12  nom	    Nombre del pretendido tad.            *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad		Apunt. a la cabeza de la lista de los tads*
*				del ambiente.				  *
* PRE!:  { nom = NT,                                                      *
*          tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2.					} *
* POST!: { existe_tad_amb = tk, 	       				  *
*	     Si NT = tk.nombre, 1 <= k <= n y p -> tk                     *
*	   existe_tad_amb = NULL, Sino.					} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

nodo_tad *existe_tad_amb(cad12 nom)
{/*----------------------------------------------------------------------*/
    nodo_tad *p_aux;
    assert(nom!=NULL); /* VTP. 7/May/1997 */
    if (!strcmp(nom,""))
        return(NULL);
    for (p_aux = tad;(p_aux != NULL) && (strcmp(p_aux->nombre,nom));
        p_aux = p_aux->sig);
    return(p_aux);
/*----------------------------------------------------------------------*/}

/**************************************************************************
* void cambiar_nombre_sel(nodo_tad *p,cad12 nuevo_nom)                    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Agregar a todas las selectoras de p el sufijo nuevo_nom    *
**************************************************************************/

void cambiar_nombre_sel(nodo_tad *p,cad12 nuevo_nom)
{/*----------------------------------------------------------------------*/
    nodo_select *s;

    assert(p!=NULL && nuevo_nom!=NULL); /* VTP. 7/May/1997 */
    for (s = p->selectoras; s!=NULL; s = s->sig) {
        strcat(s->nombre,"_");
        strcat(s->nombre,nuevo_nom);
    }
/*----------------------------------------------------------------------*/}


/**************************************************************************
* void genere_copia_tad(nodo_tad *p,cad12 viejo_nom,nodo_tad *np,    	  *
*			     cad12 nuevo_nom) 				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Copiar el tad p en el tad np remplazando las ocurrencias de*
*	 <viejo_nom> por <nuevo_nom>.              			  *
* DESCRIPCION: Copia en la estructura apuntada por np la informacion del  *
*	 tad p reemplazando las ocurrencias de <viejo_nom> por <nuevo_nom>*
* V.GLOBALES:                                                             *
*    nodo_tad	*ult_tad	Apunt. al ultimo tad de la lista de tads  *
*				del ambiente.				  *
*    nodo_tad	*tad_actual	Apunt. al tad actual de la lista de tads  *
*				del ambiente.				  *
* PRE!:  { tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2, p -> tk, np -> ti,  1 <= k,i <= n    } *
* POST!: { np -> COPIA(tk), donde COPIA(tk) es un tad identico a tk excep-*
*	   to que todas las ocurrencias de <viejo_nom> han sido reemplaza-*
*	   das por <nuevo_nom>.						} *
* !OJO!: Usa funciones de FENIX.                                          *
* LLAMA A:                                                                *
*   void	copie_lista_nombres   	  void 	copie_lista_generadoras	  *
*   void	copie_lista_selectoras	  void 	copie_lista_igualdades 	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Mar-90 *
**************************************************************************/

void genere_copia_tad(nodo_tad *p,cad12 viejo_nom,nodo_tad *np,cad12 nuevo_nom)
{/*----------------------------------------------------------------------*/

    nodo_nombre     	*ult_nom=NULL;
    nodo_generadora	*ult_generad=NULL;
    nodo_select		*ult_select=NULL;

    assert(p!=NULL && viejo_nom!=NULL && np!=NULL && nuevo_nom!=NULL);
      /* VTP. 7/May/1997 */
    asigne_nombre(np->nombre,p->nombre,viejo_nom,nuevo_nom,"");
    np->status = p->status;
    np->tipo = p->tipo;
    copie_lista_nombres(p->info_tipo,&(np->info_tipo),&ult_nom,
    			viejo_nom,nuevo_nom,"");
    copie_lista_nombres(p->parametros,&(np->parametros),&ult_nom,
    			viejo_nom,nuevo_nom,"");
    copie_lista_generadoras(p->iniciales,&(np->iniciales),
			    &ult_generad,viejo_nom,nuevo_nom,"");
    ult_generad = NULL;
    copie_lista_generadoras(p->constructoras,&(np->constructoras),
			    &ult_generad,viejo_nom,nuevo_nom,"");
    copie_lista_selectoras(p->selectoras,&(np->selectoras),
    			   &ult_select,viejo_nom,nuevo_nom,"",np,"");
/*----------------------------------------------------------------------*/}

/**************************************************************************
* void cambiar_clase_expresion(nodo_expresion *e, texpr nom_viejo,        *
*                              texpr nom_nuevo)                           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Cambiar de la expresion e, la clase nom_viejo por nom_nuevo*
**************************************************************************/

void cambiar_clase_expresion(nodo_expresion *e, texpr nom_viejo,
                             texpr nom_nuevo)
{/*----------------------------------------------------------------------*/
    nodo_argum *arg;

    assert(e!=NULL && nom_viejo!=NULL && nom_nuevo!=NULL); /* VTP. 7/May/1997 */
    if (!strcmp(e->tipo,nom_viejo))
        strcpy(e->tipo,nom_nuevo);
    if (strcmp(nom_viejo,"") &&
	!strcmp(e->clase,"EQUAL") &&
	!strcmp(e->argumentos->expr->tipo,nom_viejo)){
       sprintf(e->nombre,"EQ_%s",nom_nuevo);
    }
    for (arg=e->argumentos;arg!=NULL;arg=arg->sig)
        cambiar_clase_expresion(arg->expr,nom_viejo,nom_nuevo);
/*----------------------------------------------------------------------*/}

/**************************************************************************
* void cambiar_nombre(nodo_tad *t, texpr nom_viejo, texpr nom_nuevo)      *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Cambiar del tad t, las ocurrencias de nom_viejo por        *
*              nom_nuevo. (No cambia los nombres de las funciones)        *
**************************************************************************/

void cambiar_nombre(nodo_tad *t, texpr nom_viejo, texpr nom_nuevo)
{/*----------------------------------------------------------------------*/
    nodo_nombre     	*nom;
    nodo_dom            *dom;
    nodo_generadora	*gen;
    nodo_select		*sel;
    nodo_igualdad       *axs;

    assert(t!=NULL && nom_viejo!=NULL && nom_nuevo!=NULL); /* VTP. 7/May/1997 */
    for (nom = t->info_tipo; nom != NULL; nom = nom->sig)
        if (!strcmp(nom->nombre,nom_viejo))
            strcpy(nom->nombre,nom_nuevo);
    for (nom = t->parametros; nom != NULL; nom = nom->sig)
        if (!strcmp(nom->nombre,nom_viejo))
            strcpy(nom->nombre,nom_nuevo);
    for (gen = t->iniciales; gen!= NULL; gen = gen->sig)
        for (nom = gen->dominio; nom != NULL; nom = nom->sig)
            if (!strcmp(nom->nombre,nom_viejo))
                strcpy(nom->nombre,nom_nuevo);
    for (gen = t->constructoras; gen!= NULL; gen = gen->sig)
        for (nom = gen->dominio; nom != NULL; nom = nom->sig)
            if (!strcmp(nom->nombre,nom_viejo))
                strcpy(nom->nombre,nom_nuevo);
    for (sel = t->selectoras; sel!= NULL; sel = sel->sig) {
        for (dom = sel->dominio; dom != NULL; dom = dom->sig)
            if (!strcmp(dom->nombre,nom_viejo))
                strcpy(dom->nombre,nom_nuevo);
        if (!strcmp(sel->codominio,nom_viejo))
            strcpy(sel->codominio,nom_nuevo);
        for(axs=sel->axiomas;axs!=NULL;axs=axs->sig) {
            cambiar_clase_expresion(axs->lado_izq,nom_viejo,nom_nuevo);
            cambiar_clase_expresion(axs->lado_der,nom_viejo,nom_nuevo);
            construya_exp(axs->lado_izq,axs->li_str,"");
            construya_exp(axs->lado_der,axs->ld_str,"");
        }
    }
/*----------------------------------------------------------------------*/}

/**************************************************************************
* void imprima_tad(nodo_tad *p)						  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Producir un archivo con la informacion de un tad.          *
* DESCRIPCION: Recibe el TAD <p> y escribe en el archivo         	  *
*	 D_PRINT_ADT/<nombre del tad>.adt la informacion del tad recibido *
* PRE!:  { tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, *p = tk, 1<=k<=n, n >= 2                     } *
* POST!: { p -> tk, 1 <= k <= n y el archivo D_PRINT_ADT/<tk.nombre>.adt  *
*	   contiene la informacion de tk.				} *
* LLAMA A:                                                                *
*   nodo_tad	*pida_tad_ambiente	  void 	ed_arch_signatura  	  *
*   void	ed_arch_axiomas                                       	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-May-90 *
**************************************************************************/

void imprima_tad(nodo_tad *p)
{/*----------------------------------------------------------------------*/

    FILE 		*fp;
    cad50		nom_arch;

    if (p != NULL) {
	     sprintf(nom_arch,"%s/%s.adt",direct[4],p->nombre);
        fp = fopen(nom_arch,"w");
        if (fp==NULL)
	     {
	        printf("%s\n",nom_arch);
	       return;
	     }
        ed_arch_signatura(fp,p);
	     fprintf(fp,"\n%s\n\n",claves[4]);
        ed_arch_axiomas_ops(fp,p->selectoras);
        fclose(fp);
        escriba_mensaje(117);        /* 117: Imprimir OK! */
    }
/*----------------------------------------------------------------------*/}

/************************************************************************
* void liberar_tad(nodo_tad **t)                                        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Libera la memoria de t.                                 *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz H.                                 FECHA: Junio  91 *
************************************************************************/

void liberar_tad(nodo_tad **t)
{/*--------------------------------------------------------------------*/
    assert(t!=NULL); /* VTP. 7/May/1997 */
    if ((*t) == NULL)
       return;
    liberar_nombres(&((*t)->info_tipo));
    liberar_nombres(&((*t)->parametros));
    liberar_generadoras(&((*t)->iniciales));
    liberar_generadoras(&((*t)->constructoras));
    liberar_selectoras(&((*t)->selectoras));
    free(*t);
    (*t) = NULL;
/*--------------------------------------------------------------------*/}


/**************************************************************************
* void salvar_backup(char dir[], char nombre[], char ext[])               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Sacar una copia deyl archivo dir/nombre.ext                 *
**************************************************************************/

void salvar_backup(char dir[], char nombre[], char ext[])
{/*----------------------------------------------------------------------*/
    texpr comando;
    texpr fuente,destino; /*** VTP 13/Mar/1997 ***/

    /*** VTP Decia sprintf(comando,"touch %s/%s.%s",dir,nombre,ext);
                   system(comando); ***/
    assert(dir!=NULL && nombre!=NULL && ext!=NULL); /* VTP. 7/May/1997 */
    sprintf(comando,"%s/%s.%s",dir,nombre,ext);
    touch(comando);

/*** VTP. Decia
    sprintf(comando,"cp %s/%s.%s %s/%s.%s~",dir,nombre,ext,dir,nombre,ext);
    system(comando); ***/
    sprintf(fuente,"%s/%s.%s",dir,nombre,ext);
    sprintf(destino,"%s/%s.%s~",dir,nombre,ext);
    copia_archivo(fuente,destino);
/*----------------------------------------------------------------------*/}

/**************************************************************************
* void salve_tad(nodo_tad *t)                                             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Salvar un tad en memoria secundaria.                       *
**************************************************************************/

void salve_tad(nodo_tad *t)
{/*----------------------------------------------------------------------*/
    assert(t!=NULL); /* VTP. 7/May/1997 */
    if (t->modif) {
        t->modif = 0;
        salvar_backup(direct[2],t->nombre,"adt");
	escriba_arch_tad(t->nombre,t);
	escriba_mensaje(116);          /* 116: salvar OK! */
      }
    else
      escriba_mensaje(15);      /* 15:Este TAD no tiene modificaciones */
/*----------------------------------------------------------------------*/}

/**************************************************************************
* void salve_todo_tad(void)                                               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Salvar todos los tad del ambiente que se hayan modificado. *
**************************************************************************/

void salve_todo_tad(void)
{/*----------------------------------------------------------------------*/
    nodo_tad    *t;
    texpr       men;
    char        resp;
/*** Alvaro
    cad6        siga;
 ***/

    for(t= tad; t!= NULL; t= t->sig) {
      if (t->status == 0 && t->modif) {
	t->modif = 0;
	if (!flag_emacs)
	  escriba_mensaje(13);         /* 13:Salvar TAD */
	sprintf(men,mensajes[115],t->nombre);
	escriba_mensaje_dinam(men); /* 115:Quiere salvar el tad...? (s/n) */

	resp = (char)toupper(lea_caracter());
	if (resp == mensajes[0][0]) {
	  salvar_backup(direct[2],t->nombre,"adt");
	  escriba_arch_tad(t->nombre,t);
	  escriba_mensaje(116);   /* 116:Salvar OK! */
	}
      }
    }

/*----------------------------------------------------------------------*/}

/************************************************************************
* int tipo_primitivo(cad20 nombre)                                      *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE : nombre es el nombre de un TAD                                   *
* POST: tipo_primitivo = 1, ssi nombre es el nombre de un TAD primitivo *
*       tipo_primitivo = 0, si no.                                      *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

int tipo_primitivo(cad20 nombre)
{/*--------------------------------------------------------------------*/
    assert(nombre!=NULL); /* VTP. 7/May/1997 */
    return(!strcmp(nombre,"NAT") || !strcmp(nombre,"BOOL"));

/*--------------------------------------------------------------------*/}



