/* PRINC_DEF.C                                                          */
/* Verifica que un conjunto de tuplas sea completo.                     */
 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "definic.h"

/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* j-tamara 29/May/1997 */



/**************************************************************************
* void aumente_variables(nodo_tupla *s,int num_var)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Insertar variables nuevas a cominzo de cada tupla de un con*
*	       junto de tuplas.						  *
* DESCRIPCION: Inserta <num_var> variables nuevas al comienzo de cada tu- *
*	pla en la lista apunt. por <s>.					  *
* PARAMETROS:                                                             *
*   nodo_tupla		*s		Ap. a la lista de tuplas.	  *
*   int			num_var		Num. de variables nuevas que hay  *
*					que agregar a cada tupla.	  *
* PRE!:  { s->S = < <arg11,...,arg1k>,...,<argn1,...,argnk> >, 		  *
*	   num_var = NV							} *
* POST!: { s-> < <v1,...,vNV,arg11,...,arg1k>,				  *
*		....							  *
*              < <v1,...,vNV,argn1,...,argnk> >,			  *
*	   donde  v1,...,vNV  son nuevas variables.			  *
* LLAMA A:                                                                *
*	void	predicado_ppal		void	aisle_param		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

void
aumente_variables (nodo_tupla * s, int num_var)
{
/*----------------------------------------------------------------------*/

  int i;
  nodo_componente *nc, *prim_comp, *ult_comp;
  texpr expr_tipo, tipo_args, tipo_arg;

  assert(s!=NULL);              /* j-tamara 29/May/1997 */

  strcpy (expr_tipo, s->componentes->tipo);
  predicado_ppal (expr_tipo, tipo_arg);
  for (; (s != NULL); s = s->sig)
    {
      strcpy (tipo_args, expr_tipo);
      for (i = 1, prim_comp = NULL, ult_comp = NULL; (i <= num_var); i++)
	{
	  nc = (nodo_componente *) malloc (sizeof (nodo_componente));
          HayMem(nc,sizeof (nodo_componente));          /* j-tamara 29/May/1997 */
	  sprintf (nc->nombre, "X%d", i);
	  aisle_param (tipo_args, tipo_arg);
	  strcpy (nc->tipo, tipo_arg);
	  strcpy (nc->clase, "VAR");
	  nc->sig = NULL;
	  if (prim_comp == NULL)
	    prim_comp = nc;
	  else
	    ult_comp->sig = nc
	      ;
	  ult_comp = nc;
	  ult_comp->sig = s->componentes->sig;
	}
      s->componentes = prim_comp;
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void ayuda_tuplas_complet_comp(FILE *fp,nodo_select *s)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Ayuda a generar lados izq. al usuario para que la defini-  *
*	       cion de los axiomas sea total.				  *
* DESCRIPCION: Genera una plantilla de <s> con todos sus argumentos varia-*
*        bles y aplica lema de forma normal sobre el argumento que no es  *
*	 total en los axiomas que se encuentran en el archivo ap. por fp. *
* PARAMETROS:                                                             *
*   FILE	*fp		Ap. al arch que contiene los axiomas.     *
*   nodo_select	*s		Ap. a la selectora a la que pertenecen los*
*				axiomas.				  *
* V.GLOBALES:                                                             *
*   short	num_param	Numero del argumento de <s> que no es to- *
*				dados los axiomas del archivo.		  *
* PRE!:  { s-> S selectora n-aria,  num_param = NP, 1 <= NP <= n,	  *
*          fp -> [S(...) = ld1, ...,  S(...) = ldm ] , m > 0, cjto de axio*
*	   mas para S que no es total en la NP-componente.		} *
* POST!: { fp -> [S(...)=ld1,...,S(...)=ldm,S(..FN1..)= ,...,S(..FNj..)= ]*
*	   donde, FN1,...,FNj son todas las formas normales de tad dominio*
*	   NP-esimo de S, que aparecen en el argumento NP y en los demas  *
*	   aparecen variables.						} *
* LLAMA A:                                                                *
*   void  genere_plantilla_selectora 		void  genere_axs_tdi	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

void
ayuda_tuplas_complet_comp (FILE * fp, nodo_select * s)
{
/*----------------------------------------------------------------------*/

  nodo_tad *p;
  nodo_expresion *plant_li_ax=NULL;             /* j-tamara 29/May/1997 */
  nodo_argum *arg;
  char cont;
  int num_param, i;

  cont = '1';
  genere_plantilla_selectora (s, &plant_li_ax, &cont);
  num_param = ((int) posic_param_incompl[0]) - ((int) '0');
  for (arg = plant_li_ax->argumentos, i = 1;
       (arg != NULL) && (i < num_param);
       arg = arg->sig, i++)
    ;
  if (arg != NULL)
    {
      if (!strcmp (arg->expr->tipo, tad_ed->nombre))
	{
	  p = tad_ed;
	}
      else
	p = existe_tad (arg->expr->tipo);
      if (p != NULL)
	{
	  genere_axs_tdi (fp, plant_li_ax, arg, p->iniciales, cont,"");
	  genere_axs_tdi (fp, plant_li_ax, arg, p->constructoras, cont,"");
	  /* Codigo util cuando se implemente herencia
	     switch (p->tipo) {
	     case RESTRICCION:
	     case COMPLETACION:
	     p = existe_tad(p->info_tipo->nombre);
	     if (p!=NULL) {
	     genere_axs_tdi(fp,plant_li_ax,arg,p->iniciales,cont);
	     genere_axs_tdi(fp,plant_li_ax,arg,p->constructoras,cont);
	     }
	     break;
	     case NT_EXTENSION:
	     case VARIACION:
	     p = existe_tad(p->info_tipo->nombre);
	     if (p!=NULL) {
	     genere_axs_tdi(fp,plant_li_ax,arg,p->iniciales,cont);
	     genere_axs_tdi(fp,plant_li_ax,arg,p->constructoras,cont);
	     }
	     p = existe_tad(p->info_tipo->sig->nombre);
	     if (p!=NULL) {
	     genere_axs_tdi(fp,plant_li_ax,arg,p->iniciales,cont);
	     genere_axs_tdi(fp,plant_li_ax,arg,p->constructoras,cont);
	     }
	     break;
	     default:
	     break;
	     }
	   */
	}
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short cubre_generad(nodo_tupla *cab_s, texpr mens_error,		  *
*              nodo_generadora *cabeza,cad50 posic_param,cad50 max_posic) *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Verificar que la primera componente de un conjunto de tu-  *
*	plas cubra todas las generadoras de una lista.			  *
* DESCRIPCION: Se verifica que si		  			  *
*	<cab_s> =  < <arg11,...,arg1k>,...,<argn1,...,argnk> > entonces:  *
*	  Para toda G operacion generadora de la lista ap. por <cabeza>:  *
*	  < <pg1,..,pgm,argi2,...,argik> | argi1 = G(pg1,...,pgm)> U	  *
*	  < <va1,..,vam,argi2,...,argik> | argi1 es variable>    es total.*
*	  donde m >= 0 es la aridad de G, va1,...,vam son nuevas variables*
* PARAMETROS:                                                             *
*   nodo_tupla		*cab_s		Ap. a la lista de tuplas.	  *
*   texpr		mens_error	Cadena para escribir un eventual  *
*					error.				  *
*   nodo_generadora	*cabeza		Ap. a la lista de generadoras.    *
*   cad50		posic_param     Posicion del argumento que se esta*
*					analizando, dentro de la subexpre-*
*					sion mas pequena que lo contiene. *
*   cad50		max_param     	Num. de argumentos de la subexpre-*
*					que se esta analizando.		  *	
* RESULTADO:  short			1, Si el cjto de tuplas  es total.*
*					0, Sino.			  *
* PRE!:  { s->S = < <arg11,...,arg1k>,...,<argn1,...,argnk> >, 		  *
*	   cabeza-> <g1,...,gm>, m >= 0.			 	} *
* POST!: { cubre_generad     = 1, 	Si 				  *
*	        Para toda G operacion generadora en <g1,...,gm>   	  *
*	        < <pg1,..,pgm,argi2,...,argik> | argi1 = G(pg1,...,pgm)> U*
*	        < <va1,..,vam,argi2,...,argik> | argi1 es variable> es    *
*	        total, donde m >= 0 es la aridad de G, va1,...,vam son    *
*	        nuevas variables					  *
*          cubre_generad     = 0, 	Sino.                           } *
* LLAMA A:								  * 	
*   void  predicado_ppal 		void  insertar_tupla		  *
*   void  expanda_argum 		void  aumente_variables		  *
*   void  elimine_prim_comp 		short es_completo		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
cubre_generad (nodo_tupla * cab_s, texpr mens_error, nodo_generadora * cabeza, cad50 posic_param, cad50 max_posic)
{
/*----------------------------------------------------------------------*/

  nodo_generadora *generad;
  nodo_tupla *ns=NULL, *ult_g=NULL, *nv=NULL, *ult_v=NULL, *s;  /* j-tamara 29/May/1997 */
  short cubre;
  int num_var;
  cad20 pred;
  texpr expr;
  cad50 posic_param_aux, max_posic_aux;


  for (cubre = 1, generad = cabeza;(generad != NULL) && cubre; generad = generad->sig)
    {
      for (ns = NULL, ult_g = NULL, nv = NULL, ult_v = NULL, s = cab_s;
	   (s != NULL); s = s->sig)
	{
	  strcpy (expr, s->componentes->nombre);
	  predicado_ppal (expr, pred);
	  if (!strcmp (pred, generad->nombre))
	    {
	      insertar_tupla (s, &ns, &ult_g);
	    }
	  else if (!strcmp (s->componentes->clase, "VAR"))
	    {
	      insertar_tupla (s, &nv, &ult_v);
	    }
	}
      if (ns == NULL)
	{
	  sprintf (mens_error, mensajes[66], generad->nombre);
	  strcpy (posic_param_incompl, posic_param);
	  cubre = 0;
	}
      else
	{
	  if (generad->dominio != NULL)
	    expanda_argum (ns, &num_var);
	  else
	    num_var = 0;
	  if ((nv != NULL) && (num_var > 0))
	    {
	      aumente_variables (nv, num_var);
	    }
	  ult_g->sig = nv;
	  strcpy (posic_param_aux, posic_param);
	  strcpy (max_posic_aux, max_posic);
	  if (num_var == 0)
	    {
	      elimine_prim_comp (ns, posic_param_aux, max_posic_aux);
	    }
	  else
	    {
	      strcat (posic_param_aux, ".1");
	      sprintf (max_posic_aux, "%s.%d", max_posic_aux, num_var);
	    }
	  cubre = es_completo (ns, mens_error, posic_param_aux, max_posic_aux);
	}
    }
  return (cubre);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
*short cubre_generadoras(nodo_tupla *s,texpr mens_error,cad50 posic_param,*
*			 cad50 max_posic)				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Verificar que la primera componente de un conjunto de tu-  *
*	plas cubra todos las posibles formas normales.			  *
* DESCRIPCION: Se verifica que si		  			  *
*	<s> =  < <arg11,...,arg1k>,...,<argn1,...,argnk> > entonces:   	  *
*	  Para toda G operacion generadora de tipo de arg11:		  *
*	  < <pg1,..,pgm,argi2,...,argik> | argi1 = G(pg1,...,pgm)> U	  *
*	  < <va1,..,vam,argi2,...,argik> | argi1 es variable>    es total.*
*	  donde m >= 0 es la aridad de G, va1,...,vam son nuevas variables*
* PARAMETROS:                                                             *
*   nodo_tupla		*s		Ap. a la lista de tuplas.	  *
*   texpr		mens_error	Cadena para escribir un eventual  *
*					error.				  *
*   cad50		posic_param     Posicion del argumento que se esta*
*					analizando, dentro de la subexpre-*
*					sion mas pequena que lo contiene. *
*   cad50		max_param     	Num. de argumentos de la subexpre-*
*					que se esta analizando.		  *	
* RESULTADO:  short			1, Si el cjto de tuplas  es total.*
*					0, Sino.			  *
* PRE!:  { s->S = < <arg11,...,arg1k>,...,<argn1,...,argnk> >		} *
* POST!: { cubre_generadoras = 1, 	Si 				  *
*	        Para toda G operacion generadora de tipo de arg11:	  *
*	        < <pg1,..,pgm,argi2,...,argik> | argi1 = G(pg1,...,pgm)> U*
*	        < <va1,..,vam,argi2,...,argik> | argi1 es variable> es    *
*	        total, donde m >= 0 es la aridad de G, va1,...,vam son    *
*	        nuevas variables					  *
*          cubre_generadoras = 0, 	Sino.                           } *
* LLAMA A: 	void  predicado_ppal					  *
*	short	existe_tad 		short	cubre_generad		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
cubre_generadoras (nodo_tupla * s, texpr mens_error, cad50 posic_param,
		   cad50 max_posic)
{
/*----------------------------------------------------------------------*/

  nodo_tad *p;
  short salida;
  texpr expr_tipo;
  cad20 tipo;

  assert(tad_ed!=NULL);
  strcpy (expr_tipo, s->componentes->tipo);
  predicado_ppal (expr_tipo, tipo);
  if (!strcmp (tad_ed->nombre, tipo))
    p = tad_ed;
  else
    p = existe_tad (tipo);
  if (p != NULL)
    {
      salida = (short)
	(cubre_generad (s, mens_error, p->iniciales, posic_param, max_posic)
	 &&
	 cubre_generad (s, mens_error, p->constructoras, posic_param, max_posic));
    }
  else
    {
      salida = 1;
    }
  /* Codigo util cunado se implemente la herencia.
     if (salida) {
     switch (p->tipo) {
     case RESTRICCION:
     case COMPLETACION:
     p = existe_tad(p->infor->nombre);
     if (p!=NULL) {
     salida=
     (cubre_generad(s,mens_error,p->iniciales,posic_param,
     max_posic)
     &&
     cubre_generad(s,mens_error,p->constructoras,posic_param,
     max_posic));
     }
     break;
     case NT_EXTENSION:
     case VARIACION:
     p = existe_tad(p->info_tipo->nombre);
     if (p!=NULL) {
     salida=
     (cubre_generad(s,mens_error,p->iniciales,posic_param,
     max_posic)
     &&
     cubre_generad(s,mens_error,p->constructoras,posic_param,
     max_posic));
     }
     p = existe_tad(p->info_tipo->sig->nombre);
     if (salida && (p!=NULL)) {
     salida=
     (cubre_generad(s,mens_error,p->iniciales,posic_param,
     max_posic)
     &&
     cubre_generad(s,mens_error,p->constructoras,posic_param,
     max_posic));
     }
     break;
     defaul:
     break;
     }
     }    */
  return (salida);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void elimine_prim_comp(nodo_tupla *ns,cad50 posic_param,cad50 max_posic)*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Eliminar la 1a. componente de todas la tuplas de un conjunt*
* DESCRIPCION: Elimina la primera componente de toda las tuplas en la     *
*        lista apunt. por <ns>.                                           *
* PARAMETROS:                                                             *
*   nodo_tupla		*ns		Ap. a la lista de tuplas.	  *
*   cad50		posic_param     Posicion del argumento que se esta*
*					analizando, dentro de la subexpre-*
*					sion mas pequena que lo contiene. *
*   cad50		max_param     	Num. de argumentos de la subexpre-*
*					que se esta analizando.		  *	
* PRE!:  { ns->S = < <arg11,...,arg1k>,...,<argn1,...,argnk> >		} *
* POST!: { ns->    < <arg12,...,arg1k>,...,<argn2,...,argnk> >		} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

void
elimine_prim_comp (nodo_tupla * ns, cad50 posic_param, cad50 max_posic)
{
/*----------------------------------------------------------------------*/

  for (; (ns != NULL); ns = ns->sig)
    ns->componentes = ns->componentes->sig
      ;
  while (posic_param[strlen (posic_param) - 1] ==
	 max_posic[strlen (max_posic) - 1])
    {
      posic_param[strlen (posic_param) - 2] = '\0';
      max_posic[strlen (max_posic) - 2] = '\0';
    }
  posic_param[strlen (posic_param) - 1] =
    (char) (posic_param[strlen (posic_param) - 1] + 1);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short es_completo(nodo_tupla *s,texpr mens_error,cad50 posic_param,	  *
*		    cad50 max_posic)					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Verificar que un conjunto de tuplas sea total.             *
* DESCRIPCION: Se verifica que <s> sea total. Un conjunto de tuplas       *
*       S = < <arg11,...,arg1k>,...,<argn1,...,argnk> > es total si:	  *
*	- S = < <>,....,<> >  o,					  *
*	- < <argi2,...,argik> | argi1 es variable > es total, o		  *
*	- Para toda G operacion generadora de tipo de arg11:		  *
*	  < <pg1,..,pgm,argi2,...,argik> | argi1 = G(pg1,...,pgm)> U	  *
*	  < <va1,..,vam,argi2,...,argik> | argi1 es variable>    es total.*
*	  donde m >= 0 es la aridad de G, va1,...,vam son nuevas variables*
* PARAMETROS:                                                             *
*   nodo_tupla		*s		Ap. a la lista de tuplas.	  *
*   texpr		mens_error	Cadena para escribir un eventual  *
*					error.				  *
*   cad50		posic_param     Posicion del argumento que se esta*
*					analizando, dentro de la subexpre-*
*					sion mas pequena que lo contiene. *
*   cad50		max_param     	Num. de argumentos de la subexpre-*
*					que se esta analizando.		  *	
* RESULTADO:  short			1, Si el cjto de tuplas  es total.*
*					0, Sino.			  *
* PRE!:  { s->S = < <arg11,...,arg1k>,...,<argn1,...,argnk> >		} *
* POST!: { es_completo = 1, 	Si S es total,  o        		  *
*          es_completo = 0, 	Sino.                            	} *
* LLAMA A:  short  es_vacio						  *
*    short  tuplas_prim_comp_var_completo 	short  cubre_generadoras  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
es_completo (nodo_tupla * s, texpr mens_error, cad50 posic_param,
	     cad50 max_posic)
{
/*----------------------------------------------------------------------*/

  cad50 posic_param_aux, max_posic_aux;

  strcpy (posic_param_aux, posic_param);
  strcpy (max_posic_aux, max_posic);
  if ((s != NULL) &&
      ((es_vacio (s)) ||
       (tuplas_prim_comp_var_completo (s, mens_error, posic_param_aux,
				       max_posic_aux)) ||
       (cubre_generadoras (s, mens_error, posic_param, max_posic))))
    {
      return (1);
    }
  else
    {
      return (0);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short es_definicion_completa(nodo_igualdad *axiomas,texpr mens_error)	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Verificar que un conjunto de axiomas sea total.            *
* DESCRIPCION: Se normaliza el conjunto de tuplas (de argumentos en los   *
*        lados izqs. de los axiomas) y se verifica que sea total.         *
* PARAMETROS:                                                             *
*   nodo_igualdad	*axiomas	Ap. a la lista de axiomas.        *
*   texpr		mens_error	Cadena para escribir un eventual  *
*					error.				  *
* RESULTADO:  short			1, Si el cjto de axiomas es total.*
*					0, Sino.			  *
* PRE!:  { axiomas -> <a1,...,an>, n >= 0.                              } *
* POST!: { es_definicion_completa = 1, Si <a1,...,an> es total,  o        *
*          es_definicion_completa = 0, Sino.                            } *
* LLAMA A:                                                                *
*	void	normalice 		short	es_completo		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
es_definicion_completa (nodo_igualdad * axiomas, texpr mens_error)
{
/*----------------------------------------------------------------------*/

  nodo_tupla *s=NULL;                           /* j-tamara 29/May/1997 */
  cad50 posic_param, max_posic;

  normalice (axiomas, &s);
  strcpy (posic_param, "1");
  strcpy (max_posic, "9");
  if (es_completo (s, mens_error, posic_param, max_posic))
    {
      return (1);
    }
  else
    {
      return (0);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short es_vacio(nodo_tupla *s)						  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Verificar si un  conjunto de tuplas es vacio.              *
* DESCRIPCION: Determina si <s> = < <>,....,<> > .			  *
* PARAMETROS:                                                             *
*   nodo_tupla		*s		Ap. a la lista de tuplas.	  *
* RESULTADO:  short			1, Si el cjto de tuplas  es vacio.*
*					0, Sino.			  *
* PRE!:  { s->S = < <arg11,...,arg1k>,...,<argn1,...,argnk> >		} *
* POST!: { es_vacio    = 1, 	Si k = 0,       o        		  *
*          es_vacio    = 0, 	Sino.                            	} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
es_vacio (nodo_tupla * s)
{
/*----------------------------------------------------------------------*/

  for (; (s != NULL) && (s->componentes == NULL); s = s->sig)
    ;
  if (s == NULL)
    {
      return (1);
    }
  else
    {
      return (0);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void expanda_argum(nodo_tupla *s,int *num_var)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Expandir los argumentos de la operacion que aparece en la  *
*	       primera componente de las tuplas de un conjunto.		  *
* DESCRIPCION: Pone como primeros componentes de cada tupla (de la lista  *
*        apuntada por <s>) los argumentos de la operacion que aparece como*
*        primera componente y eleimina esta. Ademas retorna en (*num_var) *
*	 el numero de args. de la operacion.				  *
* PARAMETROS:                                                             *
*   nodo_tupla		*s		Ap. a la lista de tuplas.	  *
*   int			*num_var	Num. de argumentos expandidos.    *
*					(aridad de la operacion)          *
* PRE!:  { s->S = < <g(x11,...,x1m),arg12,...,arg1k>,			  *
*		    ....						  *
*                   <g(xn1,...,xnm),argn2,...,argnk> >, 		  *
*	   donde g es una operacion generadora de aridad m		} *
* PRE!:  { s->    < <x11,...,x1m,arg12,...,arg1k>,			  *
*		    ....						  *
*                   <xn1,...,xnm,argn2,...,argnk> >, 		  	  *
*	   *num_var = m.						} *
* LLAMA A:                                                                *
*   void  predicado_ppal 		void  aisle_param		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

void
expanda_argum (nodo_tupla * s, int *num_var)
{
/*----------------------------------------------------------------------*/

  texpr expr_nom, expr_tip, expr_cla, argum_nom, argum_tip, argum_cla;
  cad20 pred;
  nodo_componente *nc, *ult_comp, *prim_comp;
/*** Alvaro
  int			narg;
 ***/

  for (; (s != NULL); s = s->sig)
    {
      strcpy (expr_nom, s->componentes->nombre);
      predicado_ppal (expr_nom, pred);
      strcpy (expr_tip, s->componentes->tipo);
      predicado_ppal (expr_tip, pred);
      strcpy (expr_cla, s->componentes->clase);
      predicado_ppal (expr_cla, pred);
      prim_comp = NULL;
      ult_comp = NULL;
      (*num_var) = 0;
      while (strcmp (expr_nom, ""))
	{
	  (*num_var)++;
	  aisle_param (expr_nom, argum_nom);
	  aisle_param (expr_tip, argum_tip);
	  aisle_param (expr_cla, argum_cla);
	  nc = (nodo_componente *) malloc (sizeof (nodo_componente));
          HayMem(nc,sizeof (nodo_componente));            /* j-tamara 29/May/1997 */
	  strcpy (nc->nombre, argum_nom);
	  strcpy (nc->tipo, argum_tip);
	  strcpy (nc->clase, argum_cla);
	  nc->sig = NULL;
	  if (prim_comp == NULL)
	    prim_comp = nc;
	  else
	    ult_comp->sig = nc
	      ;
	  ult_comp = nc;
	  ult_comp->sig = s->componentes->sig;
	}
      if ((*num_var) > 0)
	{
	  s->componentes = prim_comp;
	}
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void insertar_tupla(nodo_tupla *s, nodo_tupla **ns,			  *
*		      nodo_tupla **ult_tupla)			  	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Adicionar una nueva tupla a un conjunto de tuplas.         *
* DESCRIPCION: Adiciona una copia de la tupla apunt. por <s> al final del *
*        conjunto de tuplas (*ns). (*ult_tupla) apunt. a la ultima tupla  *
*	 de la lista apunt. por (*ns).					  *
* PARAMETROS:                                                             *
*   nodo_tupla		*s		Ap. a una tuplas.	  	  *
*   nodo_tupla		**ns		Ap. a una lista de tuplas.	  *
*   nodo_tupla		**ult_tupla	Ap. a la ultima tupla de la lista *
*					apunt. por (*ns).		  *
* PRE!:  { s-> t , *ns-> <t1,...,tn> <-*ult_tupla, n >= 0               } *
* POST!: { *ns-> <t1,...,tn,t> <-*ult_tupla				} *
* LLAMA A: void  copie_lista_componentes				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

void
insertar_tupla (nodo_tupla * s, nodo_tupla ** ns, nodo_tupla ** ult_tupla)
{
/*----------------------------------------------------------------------*/

  nodo_tupla *n;
  nodo_componente *cabeza=NULL, *ult_comp=NULL;                 /* j-tamara 29/May/1997 */

  assert(s!=NULL);                      /* j-tamara 29/May/1997 */
  copie_lista_componentes (s->componentes, &cabeza, &ult_comp);
  n = (nodo_tupla *) malloc (sizeof (nodo_tupla));
  HayMem(n,sizeof (nodo_tupla));        /* j-tamara 29/May/1997 */
  n->componentes = cabeza;
  n->sig = NULL;
  if ((*ns) == NULL)
    (*ns) = n;
  else
    (*ult_tupla)->sig = n
      ;
  (*ult_tupla) = n;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void normalice(nodo_igualdad *axiomas,nodo_tupla **s)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Construir una estructura de tuplas a partir de los axiomas.*
* DESCRIPCION: Recorre los axiomas construyendo para cada uno una tupla   *
*	 los argumentos de la selectora.				  *
* PARAMETROS:                                                             *
*   nodo_igualdad	*axiomas	Ap. a la lista de axiomas.        *
*   nodo_tupla		**s		Ap. a la lista de tuplas generadas*
*             tipo   nombre         comentario                            *
* PRE!:  { axiomas -> <a1,...,an>,  n >= o, tq. i = 1,...,n:              *
*	   ai = <selec>(argi1,...,aik)  donde k es la aridad de <select>} *
* POST!: { *s-> < <arg11,...,arg1k>,.....,<argn1,...,argnk> >           } *
* LLAMA A: void  construya_exp_tipo_clase				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

void
normalice (nodo_igualdad * axiomas, nodo_tupla ** s)
{
/*----------------------------------------------------------------------*/

  nodo_igualdad *ax;
  texpr expr, tipo, clase;
  nodo_tupla *nt, *ult_tupla;
  nodo_componente *nc, *ult_comp;
  nodo_argum *arg;

  for (ax = axiomas, (*s) = NULL, ult_tupla = NULL;
       (ax != NULL); ax = ax->sig)
    {
      nt = (nodo_tupla *) malloc (sizeof (nodo_tupla));
      HayMem(nt,sizeof (nodo_tupla));                   /* j-tamara 29/May/1997 */
      nt->componentes = NULL;
      nt->sig = NULL;
      for (arg = ax->lado_izq->argumentos, ult_comp = NULL;
	   arg != NULL;
	   arg = arg->sig)
	{
	  nc = (nodo_componente *) malloc (sizeof (nodo_componente));
          HayMem(nc,sizeof (nodo_componente));          /* j-tamara 29/May/1997 */
	  nc->sig = NULL;
	  construya_exp_tipo_clase (arg->expr, expr, tipo, clase);
	  strcpy (nc->nombre, expr);
	  strcpy (nc->tipo, tipo);
	  strcpy (nc->clase, clase);
	  if (nt->componentes == NULL)
	    nt->componentes = nc;
	  else
	    ult_comp->sig = nc
	      ;
	  ult_comp = nc;
	}
      if ((*s) == NULL)
	(*s) = nt;
      else
	ult_tupla->sig = nt
	  ;
      ult_tupla = nt;
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short tuplas_prim_comp_var_completo(nodo_tupla *s,texpr mens_error,	  *
*			              cad50 posic_param,cad50 max_posic)  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Verificar que un conjunto de tuplas cuyas primeras componen*
*	tes son variables sea total.             			  *
* DESCRIPCION: Se determina si 						  *
*       <s> = < <arg11,...,arg1k>,...,<argn1,...,argnk> > cumple que  	  *
*	 < <argi2,...,argik> | argi1 es variable > es total.  		  *
* PARAMETROS:                                                             *
*   nodo_tupla		*s		Ap. a la lista de tuplas.	  *
*   texpr		mens_error	Cadena para escribir un eventual  *
*					error.				  *
*   cad50		posic_param     Posicion del argumento que se esta*
*					analizando, dentro de la subexpre-*
*					sion mas pequena que lo contiene. *
*   cad50		max_param     	Num. de argumentos de la subexpre-*
*					que se esta analizando.		  *	
* RESULTADO:  short			1, Si el cjto de tuplas  es total.*
*					0, Sino.			  *
* PRE!:  { s->S = < <arg11,...,arg1k>,...,<argn1,...,argnk> >		} *
* POST!: { tuplas_prim_comp_var_completo = 1, 	Si			  *
*	 		< <argi2,...,argik> | argi1 es variable > es total*
*          tuplas_prim_comp_var_completo = 0, 	Sino.     		} *
* LLAMA A: 	void  insertar_tupla				  	  *
*      void	elimine_prim_comp 		short	es_completo	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
tuplas_prim_comp_var_completo (nodo_tupla * s, texpr mens_error,
			       cad50 posic_param, cad50 max_posic)
{
/*----------------------------------------------------------------------*/

  nodo_tupla *ns=NULL, *ult_tupla=NULL;                 /* j-tamara 29/May/1997 */

  for (ns = NULL, ult_tupla = NULL; (s != NULL); s = s->sig)
    {
      if (!strcmp (s->componentes->clase, "VAR"))
	insertar_tupla (s, &ns, &ult_tupla)
	  ;
    }
  if (ns == NULL)
    {
      return (0);
    }
  else
    {
      elimine_prim_comp (ns, posic_param, max_posic);
      return (es_completo (ns, mens_error, posic_param, max_posic));
    }
/*----------------------------------------------------------------------*/
}
