 
/* PROD_CART_IMPLIC.C                                           */
/* Arhivo de rutinas del producto cartesiano implicito de TADs. */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "definic.h"
#include "apimanta.h"
#include <assert.h>		/*  VTP. 3/May/1997 */
/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* v-tamara 7/May/1997 */

/**************************************************************************
* void adic_igualdad(nodo_tad *p,nodo_tad *q)				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Adicionar el predicado de igualdad a un PROD_CARTESIANO.   *
* DESCRIPCION: Si <p> y <q> tienen definido el predicado de igualdad, se  *
*        define automatica/. el del producto, estableciendo que dos pares *
*        ordenados son iguales si lo son por componentes.                 *
* PARAMETROS:                                                             *
*   nodo_tad	*p	Ap. al primer tad padre del PROD_CARTESIANO.      *
*   nodo_tad	*q	Ap. al segundo tad padre del PROD_CARTESIANO.     *
* V.GLOBALES:                                                             *
*    nodo_tad	*ult_tad     	Apunt. al tad de que se esta generando por*
*			  	PROD_CARTESIANO				  *
* PRE!:  { p-> T1, q-> T2, ult_tad-> TC = T1 x T2, 	  	 	  *
*	   TC.selectoras = S = <s1,...,sn>,  n >= 0.			} *
* POST!: { TC.selectoras = <s1,...,sn,EQ_TC>, con el axioma:	          *
*		  EQ_TC((X1,Y1),(X2,Y2)) = AND(EQ_T1(X1,X2),EQ_T2(Y1,Y2)) *
*	 	  Si:    T1.selectoras = <...,EQ_T1,...>  y		  *
*	 	         T2.selectoras = <...,EQ_T2,...>    	  	  *
*	   TC.selectoras = S, Sino.					  *
* LLAMA A: 	void	tiene_igualdad_definida				  *
*	void	adic_select_eq 			void	genere_ax_eq_pc   *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 04-Jul-90 *
**************************************************************************/

void
adic_igualdad (nodo_tad * p, nodo_tad * q)
{
/*----------------------------------------------------------------------*/

  nodo_select *s = NULL;	/* VTP. 5/May/1997 */

  assert (p != NULL && q != NULL);	/* VTP. 5/May/1997 */
  if (tiene_igualdad_definida (p) && tiene_igualdad_definida (q))
    {
      adic_select_eq (ult_tad, &s);
      genere_ax_eq_pc (s, p->nombre, q->nombre);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_param_pc(nodo_tad *p, nodo_tad *q)		 	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar los parametros de un tad de tipo PROD_CARTESIANO.  *
* DESCRIPCION: El ult_tad hereda los parametros de <p> y de <q>.Esto es,  *
*	 el conjunto de los parametros de ult_tad es la union disyunta    *
*	 de las parametros de sus padres.  				  *
* PARAMETROS:                                                             *
*   nodo_tad	*p	Ap. al primer tad padre de la PROD_CARTESIANO.    *
*   nodo_tad	*q	Ap. al segundo tad padre de la PROD_CARTESIANO.   *
* V.GLOBALES:                                                             *
*    nodo_tad	*ult_tad     	Apunt. al tad de que se esta generando por*
*			  	PROD_CARTESIANO				  *
* PRE!:  { p-> T1, q-> T2, ult_tad-> TC = T1 x T2, 	  		  *
*	   T1.parametros = <p11,...,p1m1>, m1 >= 0,                    	  *
*	   T2.parametros = <p21,...,p2m2>, m2 >= 0,			} *
* POST!: { TC.parametros = <p11'1',...,p1m1'1',p21'2',...,p2m2'2'>	} *
* LLAMA A: 			void  copie_lista_nombres      		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void
genere_param_pc (nodo_tad * p, nodo_tad * q)
{
/*----------------------------------------------------------------------*/

  nodo_nombre *cabeza_lista = NULL,	/* VTP. 5/May/1997 */
   *ult_lista1 = NULL,		/* VTP. 5/May/1997 */
   *ult_lista2 = NULL;		/* VTP. 5/May/1997 */

  assert (p != NULL && q != NULL);	/* VTP. 5/May/1997 */
  copie_lista_nombres (p->parametros, &cabeza_lista, &ult_lista1, "", "", "1");
  ult_tad->parametros = cabeza_lista;
  cabeza_lista = NULL;
  copie_lista_nombres (q->parametros, &cabeza_lista, &ult_lista2, "", "", "2");
  if (ult_tad->parametros == NULL)
    ult_tad->parametros = cabeza_lista;
  else
    ult_lista1->sig = cabeza_lista
      ;
/*----------------------------------------------------------------------*/
}
/**************************************************************************
* void informacion_tipo_prod(nodo_tad *p,cad12 nom_tad1,cad12 nom_tad2)	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Asignar tipo PROD_CARTESIANO a un tad.  			  *
* DESCRIPCION: Asigna PROD_CARTESIANO al tipo de p, <nom_tad1> a la       *
*	 primera componente y <nom_tad2> a la segunda componente de       *
*	 la informacion asociada al tipo.				  *
* PARAMETROS:                                                             *
*    nodo_tad	*p		Ap. al tad que es de tipo PROD_CARTESIANO.*
*    cad12	nom_tad1 	Nombre del primer tad componente de la    *
*				suma directa.				  *
*    cad12	nom_tad2 	Nombre del segundo tad componente de la   *
*				suma directa.				  *
* PRE!:  { p-> T, nom_tad1 = NTO1, nom_tad2 = NTO2.			} *
* POST!: { T.tipo = PROD_CARTESIANO, T.info_tipo = <NTO1 , NTO2>	} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void
informacion_tipo_prod (nodo_tad * p, cad12 nom_tad1, cad12 nom_tad2)
{
/*----------------------------------------------------------------------*/
  nodo_nombre *n;

  assert (p != NULL && nom_tad1 != NULL && nom_tad2 != NULL);	/* VTP. 5/May/1997 */
  p->tipo = PROD_CARTESIANO;
  n = (nodo_nombre *) malloc (sizeof (nodo_nombre));
  HayMem (n, sizeof (nodo_nombre));	/* VTP. 5/May/1997 */
  strcpy (n->nombre, nom_tad1);
  p->info_tipo = n;
  n->sig = (nodo_nombre *) malloc (sizeof (nodo_nombre));
  HayMem (n->sig, sizeof (nodo_nombre));	/* VTP. 5/May/1997 */
  n = n->sig;
  strcpy (n->nombre, nom_tad2);
  n->sig = NULL;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void adic_proy_implic(cad20 nom_codom,cad5 indice,			  *
*			nodo_expresion *plant_inic,			  *
*       	        nodo_argum *arg,nodo_select **cab_select)	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Adicionar una operacion de proyeccion a un tad de tipo     *
*	       PROD_CARTESIANO implicito.				  *
* DESCRIPCION: Adiciona una nueva selectora (y sus axiomas) a ult_tad:	  *
*	 PROY<indice>: ult_tad -> <nom_codom> 				  *
*        que obtiene de una pareja en su <indice> componente.    	  *
* PARAMETROS:                                                             *
*   cad20 		nom_codom	Nombre del tad al cual se proyecta*
*   cad5 		indice		"1", Si es la 1a. proyeccion.	  *
*					"2", Si es la 2a. proyeccion.	  *
*   nodo_expresion	*plant_inic	Plantilla de la expresion inicial:*
*					PAIR_<ult_tad>(X,Y).		  *
*   nodo_argum      	*arg         	Ap. al argumento de la inicial que*
*					es el resultado de la proyeccion: *
*					( X o Y )			  *
*   nodo_select 	**cab_select    (*cab_select) apunta a la estructu*
*					ra que contiene la nueva select.  *
* V.GLOBALES:                                                             *
*    nodo_tad	*ult_tad     	Apunt. al tad de que se esta generando por*
*			  	PROD_CARTESIANO				  *
* PRE!:  { nom_codom = NC, indice = I, I = "1" o I = "2"                } *
*	   plant_inic-> PAIR_<ult-tad>(X,Y), 				  *
*	   arg-> X, Si I = 1, o arg-> Y, si I = 2.			} *
* POST!: { cab_select -> PROY<indice> donde 				  *
*	   PROY<indice> : TC -> <nom_codom>, 				  *
*	   es la proyeccion sobre la i-esima componente.		} *
* LLAMA A: void genere_axiomas_proy_implic				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 04-Jul-90 *
**************************************************************************/

void
adic_proy_implic (cad20 nom_codom, cad5 indice, nodo_expresion * plant_inic,
		  nodo_argum * arg, nodo_select ** cab_select)
{
/*----------------------------------------------------------------------*/

  nodo_select *s;

  assert (nom_codom != NULL && indice != NULL);
  assert (ult_tad != NULL);	/* VTP. 5/May/1997 */
  s = (nodo_select *) malloc (sizeof (nodo_select));
  HayMem (s, sizeof (nodo_select));	/* VTP. 5/May/1997 */
  s->sig = NULL;
  sprintf (s->nombre, "PROY_%s%s", ult_tad->nombre, indice);
  strcpy (s->nombre_fte, s->nombre);
  strcpy (s->codominio, nom_codom);
  s->caracterizadora = 1;
  s->dominio = (nodo_dom *) malloc (sizeof (nodo_dom));
  HayMem (s->dominio, sizeof (nodo_dom));	/* VTP. 5/May/1997 */
  strcpy (s->dominio->nombre, ult_tad->nombre);
  s->dominio->induccion = 1;
  s->dominio->sig = NULL;
  genere_axiomas_proy_implic (s, indice, plant_inic, arg);
  (*cab_select) = s;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void adic_proyecciones_implic(nodo_tad *p,nodo_tad *q)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Adicionar las operaciones de proyeccion a un tad de tipo   *
*	       PROD_CARTESIANO implicito.				  *
* DESCRIPCION: Adiciona dos nuevas selectoras a ult_tad: PROY1 y PROY2,   *
*        que descomponen una pareja en su primera y segunda componente    *
*        respectiva/.                                                     *
* PARAMETROS:                                                             *
*   nodo_tad	*p	Ap. al primer tad padre del PROD_CARTESIANO.      *
*   nodo_tad	*q	Ap. al segundo tad padre del PROD_CARTESIANO.     *
* V.GLOBALES:                                                             *
*    nodo_tad	*ult_tad     	Apunt. al tad de que se esta generando por*
*			  	PROD_CARTESIANO				  *
* PRE!:  { p-> T1, q-> T2, ult_tad-> TC = T1 x T2, 	  	 	  *
*	   TC.selectoras =  <s1,...,sn>,  n >= 0.			} *
* POST!: { TC.selectoras = <PROY1,PROY2,s1,...,sn>, tq. para i = 1,2	  *
*		  PROYi : TC -> Ti,					  *
*		  es la proyeccion sobre la i-esima componente.		} *
* LLAMA A: 	void	adic_proy					  *
*   void  genere_plantilla_generadora 	void adic_proy_implic		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 04-Jul-90 *
**************************************************************************/

void
adic_proyecciones_implic (nodo_tad * p, nodo_tad * q)
{
/*----------------------------------------------------------------------*/

  nodo_expresion *plant_inic = NULL;	/* VTP. 5/May/1997 */
  nodo_select *cab_select = NULL;	/* VTP. 5/May/1997 */
  char cont;

  assert (p != NULL && q != NULL);	/* VTP. 5/May/1997 */
  cont = '1';
  genere_plantilla_generadora (ult_tad->iniciales, ult_tad->nombre,
			       &plant_inic, &cont);
  adic_proy_implic (p->nombre, "1", plant_inic, plant_inic->argumentos,
		    &cab_select);
  adic_proy_implic (q->nombre, "2", plant_inic, plant_inic->argumentos->sig,
		    &(cab_select->sig));
  cab_select->sig->sig = ult_tad->selectoras;
  ult_tad->selectoras = cab_select;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void convierta_constrs_selects(nodo_tad *p,nodo_select **cab_lista,	  *
*			         nodo_select **ult_lista,cad5 indice)	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Adicionar al conjunto de selectoras de un tad de tipo PROD_*
*	       CARTESIANO implicito las constructoras de uno de sus padres*
* DESCRIPCION: Adiciona al conjunto de las selectoras de ult_tad las cons-*
*	 tructoras de <p>
*	 yunta de los siguientes conjuntos:				  *
*        - constructoras de <p>.			  		  *
*        - constructoras de <q>.			  		  *
*        - selectoras de <p>.			  		  	  *
*        - selectoras de <q>.			  			  *
* PARAMETROS:                                                             *
*   nodo_tad	*p		Ap. al tad padre de la PROD_CARTESIANO.   *
*   nodo_select **cab_lista	Ap. a la primera select. del producto	  *
*   nodo_select **ult_lista	Ap. a la ultima  select. del producto	  *
*   cad5 	indice		"1", Si se adicionan las constructoras del*
*				     primer padre.			  *
*   				"2", Si se adicionan las constructoras del*
*				     segundo padre.			  *
* V.GLOBALES:                                                             *
*    nodo_tad	*ult_tad     	Apunt. al tad de que se esta generando por*
*			  	PROD_CARTESIANO				  *
* PRE!:  { p-> Ti, tq. existe un Tj:					  *
*          ult_tad-> TC = Ti x Tj  o  TC = Tj x Ti			  *
*	   indice = I,							  *
*	   TC.selectoras = < s1,.............,sn>,  n >= 0		  *
*	  		     cab_lista        ult_lista			  *
*	   Ti.constructoras = <c1,...,cm>, m >= 0,                    	  *
* POST!: { TC.selectoras = <s1,....,sn,c1I,......,cmI>			  *
*			    cab_lista             ult_lista		  *
* LLAMA A:	void	asigne_nombre					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void
convierta_constrs_selects (nodo_tad * p, nodo_select ** cab_lista,
			   nodo_select ** ult_lista, cad5 indice)
{
/*----------------------------------------------------------------------*/

  nodo_select *s;
  nodo_generadora *c;
  nodo_nombre *dc;
  nodo_dom *ds, *ult_dom;

  assert (p != NULL && cab_lista != NULL && ult_lista != NULL && indice != NULL);
  for (c = p->constructoras, (*cab_lista) = NULL, (*ult_lista) = NULL;
       c != NULL; c = c->sig)
    {
      s = (nodo_select *) malloc (sizeof (nodo_select));
      HayMem (s, sizeof (nodo_select));		/* VTP. 5/May/1997 */
      s->sig = NULL;
      strcpy (s->nombre, c->nombre);
      strcat (s->nombre, indice);
      strcpy (s->nombre_fte, s->nombre);
      s->caracterizadora = 0;
      for (dc = c->dominio, s->dominio = NULL, ult_dom = NULL;
	   dc != NULL; dc = dc->sig)
	{
	  ds = (nodo_dom *) malloc (sizeof (nodo_dom));
	  HayMem (ds, sizeof (nodo_dom));	/* VTP. 5/May/1997 */
	  ds->sig = NULL;
	  asigne_nombre (ds->nombre, dc->nombre, p->nombre, ult_tad->nombre, "");
	  ds->induccion = (short) !(strcmp (dc->nombre, ult_tad->nombre));
	  if (s->dominio == NULL)
	    s->dominio = ds;
	  else
	    ult_dom->sig = ds
	      ;
	  ult_dom = ds;
	}
      strcpy (s->codominio, ult_tad->nombre);
      if (*cab_lista == NULL)
	(*cab_lista) = s;
      else
	(*ult_lista)->sig = s
	  ;
      (*ult_lista) = s;
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_axiomas_proy_implic(nodo_select *s,cad5 indice,		  *
*			      nodo_expresion *plant_inic,nodo_argum *arg) *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Adicionar los axiomas a una operacion de proyeccion de un  *
*	       tad de tipo PROD_CARTESIANO implicito.			  *
* DESCRIPCION: Adiciona el axioma a la proyeccion <s>		  	  *
*        que obtiene de una pareja en su <indice> componente.    	  *
*	 Tal axioma es de la forma:					  *
*	 PROY<indice>(PAIR_<ult_tad>(X,Y)) = X, Si <indice> = 1  o	  *
*	 PROY<indice>(PAIR_<ult_tad>(X,Y)) = Y, Si <indice> = 2.  	  *
* PARAMETROS:                                                             *
*   nodo_select		*s		Ap. a la operacion de proyeccion  *
*   cad5 		indice		"1", Si es la 1a. proyeccion.	  *
*					"2", Si es la 2a. proyeccion.	  *
*   nodo_expresion	*plant_inic	Plantilla de la expresion inicial:*
*					PAIR_<ult_tad>(X,Y).		  *
*   nodo_argum      	*arg         	Ap. al argumento de la inicial que*
*					es el resultado de la proyeccion: *
*					( X o Y )			  *
* V.GLOBALES:                                                             *
*    nodo_tad	*ult_tad     	Apunt. al tad de que se esta generando por*
*			  	PROD_CARTESIANO				  *
* PRE!:  { s-> PROY<indice>, indice = I, I = "1" o I = "2"                *
*	   plant_inic-> PAIR_<ult-tad>(X,Y), 				  *
*	   arg-> A, donde A = X, Si I = 1, o A = Y, si I = 2.		} *
* POST!: { s->axiomas -> PROY<indice>(PAIR_<ult_tad>(X,Y)) = A		} *
* LLAMA A:								  *
*	void	copie_lista_expresion 	void	transf_estruct_iguald	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 04-Jul-90 *
**************************************************************************/

void
genere_axiomas_proy_implic (nodo_select * s, cad5 indice,
			    nodo_expresion * plant_inic, nodo_argum * arg)
{
/*----------------------------------------------------------------------*/

  nodo_expresion *cab_li_ax, *cab_ld_ax = NULL;		/* VTP. 5/May/1997 */
  nodo_igualdad *ax;

  assert (s != NULL && indice != NULL);		/* VTP. 5/May/1997 */
  assert (plant_inic != NULL && arg != NULL);	/* VTP. 5/May/1997 */
  cab_li_ax = (nodo_expresion *) malloc (sizeof (nodo_expresion));
  HayMem (cab_li_ax, sizeof (nodo_expresion));	/* VTP. 5/May/1997 */
  strcpy (cab_li_ax->nombre, s->nombre);
  strcpy (cab_li_ax->tipo, s->codominio);
  strcpy (cab_li_ax->clase, "SELECT");
  cab_li_ax->argumentos = (nodo_argum *) malloc (sizeof (nodo_argum));
  HayMem (cab_li_ax->argumentos, sizeof (nodo_argum));	/* VTP. 5/May/1997 */
  copie_lista_expresion (plant_inic, &(cab_li_ax->argumentos->expr),
			 "", NULL,"");
  copie_lista_expresion (arg->expr, &(cab_ld_ax), "", NULL,"");
  ax = (nodo_igualdad *) malloc (sizeof (nodo_igualdad));
  HayMem (ax, sizeof (nodo_igualdad));	/* VTP. 5/May/1997 */
  ax->sig = NULL;
  ax->lado_izq = cab_li_ax;
  ax->lado_der = cab_ld_ax;
  transf_estruct_iguald (ax);
  s->axiomas = ax;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_axs_select_implic(nodo_select *s,cad5 indice,cad12 nom_tad1,*
*			        cad12 nom_tad2)				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar los axiomas de una selectora de un tad de tipo     *
*	       PROD_CARTESIANO  implicito.				  *
* DESCRIPCION: Los axiomas de <s> expresan el comportamiento de la selecto*
*	 ra aplicada sobre la componente correspondiente al tad del que   *
*	 fue heredada. Esto es, si t es una variable del TDI		  *
*  s1(...t...) = PAIR_<ult_tad>(s(...PROY1(t)...),PROY2(t)), Si indice="1"*
*  s2(...t...) = PAIR_<ult_tad>(PROY1(t),s(...PROY2(t)...)), Si indice="2"*
* PARAMETROS:								  *
*   nodo_select		*s		Ap. a la operacion de proyeccion  *
*   cad5 		indice		"1", Si es la 1a. proyeccion.	  *
*					"2", Si es la 2a. proyeccion.	  *
*   cad12 		nom_tad1	Nombre del primer tad componente  *
*					del producto.			  *
*   cad12 		nom_tad2	Nombre del segundo tad componente *
*					del producto.			  *
* V.GLOBALES:                                                             *
*    nodo_tad	*ult_tad     	Apunt. al tad de que se esta generando por*
*			  	PROD_CARTESIANO				  *
* PRE!:  { nom_tad1 = T1, nom_tad2 = T2, ult_tad-> TC = T1 x T2, 	  *
*	   s-> S,  indice = I,  I = "1" o "2"				} *
* POST!: { TC.selectoras = <s11'1',...,s1m1'1',s21'2',...,s2m2'2'>, tq.   *
*	        para j = 1,2  y  i = 1,...,mj :				  *
*	        sji'j'.axiomas expresan el comportamiento de sji sobre la *
*	 	j-esima componente de las parejas.			  *
*		Esto es, si t es una variable del TDI		  	  *
*  s1(...t...) = PAIR_<ult_tad>(s(...PROY1(t)...),PROY2(t)),Si indice="1" *
*  s2(...t...) = PAIR_<ult_tad>(PROY1(t),s(...PROY1(t)...)),Si indice="2"}*
* LLAMA A: 								  *
*   void  genere_plantilla_selectora 	void  copie_lista_expresion	  *
*   void  proyecte_args_tdi 		void  busque_argum_var_tdi	  *
*   void  copie_lista_argum 		void  transf_estruct_iguald	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void
genere_axs_select_implic (nodo_select * s, cad5 indice, cad12 nom_tad1,
			  cad12 nom_tad2)
{
/*----------------------------------------------------------------------*/

  nodo_expresion *ld_axioma = NULL,	/* VTP. 5/May/1997 */
   *exp, *plantilla_select = NULL;	/* VTP. 5/May/1997 */
  nodo_argum *arg, *arg_tdi = NULL,	/* VTP. 5/May/1997 */
   *nuevo_arg_tdi = NULL;	/* VTP. 5/May/1997 */
  nodo_igualdad *ax;
  char cont;

  assert (s != NULL && indice != NULL && nom_tad1 != NULL);	/* VTP. 5/May/1997 */
  assert (nom_tad2 != NULL);	/* VTP. 5/May/1997 */
  s->axiomas = NULL;
  cont = '1';

  genere_plantilla_selectora (s, &plantilla_select, &cont);
  assert (plantilla_select != NULL);	/* VTP. 6/May/1997 */

  copie_lista_expresion (plantilla_select, &ld_axioma, "", NULL,"");
  assert (ld_axioma != NULL);	/* VTP. 6/May/1997 */

  /* VTP. 2/May/1998. Fixing error when it generates product types */
  strcpy(ld_axioma->nombre,s->nombre_fte);

  /* VTP. 2/May/1998. It said:
     (ld_axioma->nombre)[strlen (ld_axioma->nombre) - 1] = '\0';
     now it has an if as ld_axiom->type has*/
  if ((ld_axioma->nombre)[strlen (ld_axioma->nombre) -1]==indice[0])
    {
      (ld_axioma->nombre)[strlen (ld_axioma->nombre) -1]='\0';
    }
      
  if ((ld_axioma->tipo)[strlen (ld_axioma->tipo) - 1] == indice[0])
    (ld_axioma->tipo)[strlen (ld_axioma->tipo) - 1] = '\0';

  proyecte_args_tdi (ld_axioma, nom_tad1, indice);
  if (!strcmp (s->codominio, ult_tad->nombre))
    {
      busque_argum_var_tdi (plantilla_select->argumentos, ult_tad->nombre,
			    &arg_tdi);
      copie_lista_argum (arg_tdi, &nuevo_arg_tdi, "", NULL,"");
      /*      nuevo_arg_tdi->sig = NULL; 6/May/1998. VTP */
      exp = (nodo_expresion *) malloc (sizeof (nodo_expresion));
      HayMem (exp, sizeof (nodo_expresion));	/* VTP. 5/May/1997 */
      sprintf (exp->nombre, "PROY_%s", ult_tad->nombre);
      if (!strcmp (indice, "1"))
	strcat (exp->nombre, "2");
      else
	strcat (exp->nombre, "1")
	  ;
      strcpy (exp->tipo, nom_tad2);
      strcpy (exp->clase, "SELECT");
      exp->argumentos = nuevo_arg_tdi;
      /* VTP. 7/Jun/1998 PROY_.. must have just one parameter */
      if (exp->argumentos->sig!=NULL)
	{
	  elimLArgumento(&(exp->argumentos->sig));
	  exp->argumentos->sig=NULL;
	}
      arg = (nodo_argum *) malloc (sizeof (nodo_argum));
      HayMem (arg, sizeof (nodo_argum));	/* VTP. 5/May/1997 */
      arg->expr = exp;
      arg->sig = NULL;		/* VTP. 6/May/1997 */
      exp = (nodo_expresion *) malloc (sizeof (nodo_expresion));
      HayMem (exp, sizeof (nodo_expresion));	/* VTP. 5/May/1997 */
      strcpy (exp->nombre, ult_tad->iniciales->nombre);
      strcpy (exp->tipo, ult_tad->nombre);
      strcpy (exp->clase, "INIC");
      exp->argumentos = (nodo_argum *) malloc (sizeof (nodo_argum));
      HayMem (exp->argumentos, sizeof (nodo_argum));	/* VTP. 5/May/1997 */
      exp->argumentos->sig = NULL;	/* VTP. 6/May/1997 */
      if (!strcmp (indice, "1"))
	{
	  exp->argumentos->expr = ld_axioma;
	  exp->argumentos->sig = arg;
	  arg->sig = NULL;
	  ld_axioma = exp;
	}
      else
	{
	  exp->argumentos = arg;
	  arg->sig = (nodo_argum *) malloc (sizeof (nodo_argum));
	  HayMem (arg->sig, sizeof (nodo_argum));	/* VTP. 5/May/1997 */
	  arg->sig->expr = ld_axioma;
	  arg->sig->sig = NULL;
	  ld_axioma = exp;
	}
    }
  ax = (nodo_igualdad *) malloc (sizeof (nodo_igualdad));
  HayMem (ax, sizeof (nodo_igualdad));	/* VTP. 5/May/1997 */
  ax->sig = NULL;
  ax->lado_izq = plantilla_select;
  ax->lado_der = ld_axioma;
  ax->sig = NULL;
  transf_estruct_iguald (ax);
  s->axiomas = ax;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_inicial_pc_implic(nodo_tad *p, nodo_tad *q)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar la inicial de un tad de tipo PROD_CARTESIANO 	  *
*	       implicito.						  *
* DESCRIPCION: El ult_tad tiene como unica inicial una constructora de pa-*
*	 jas de elementos de <p> y de <q>:			 	  *
*	 PAIR_<ult_tad> : <p> x <q> -> <ult_tad>.			  *
* PARAMETROS:                                                             *
*   nodo_tad	*p	Ap. al primer tad padre de la PROD_CARTESIANO.    *
*   nodo_tad	*q	Ap. al segundo tad padre de la PROD_CARTESIANO.   *
* V.GLOBALES:                                                             *
*    nodo_tad	*ult_tad     	Apunt. al tad de que se esta generando por*
*			  	PROD_CARTESIANO				  *
* PRE!:  { p-> T1, q-> T2, ult_tad-> TC = T1 x T2 			} *
* POST!: { TC.iniciales = <PAIR_TC>					} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void
genere_inicial_pc_implic (nodo_tad * p, nodo_tad * q)
{
/*----------------------------------------------------------------------*/

  nodo_generadora *n;
  nodo_nombre *d;

  assert (p != NULL && q != NULL);	/* VTP. 5/May/1997 */
  n = (nodo_generadora *) malloc (sizeof (nodo_generadora));
  HayMem (n, sizeof (nodo_generadora));		/* VTP. 5/May/1997 */
  n->sig = NULL;
  ult_tad->iniciales = n;
  sprintf (n->nombre, "PAIR_%s", ult_tad->nombre);
  strcpy (n->nombre_fte, n->nombre);
  d = (nodo_nombre *) malloc (sizeof (nodo_nombre));
  HayMem (d, sizeof (nodo_nombre));	/* VTP. 5/May/1997 */
  strcpy (d->nombre, p->nombre);
  n->dominio = d;
  d->sig = (nodo_nombre *) malloc (sizeof (nodo_nombre));
  HayMem (d->sig, sizeof (nodo_nombre));	/* VTP. 5/May/1997 */
  d = d->sig;
  strcpy (d->nombre, q->nombre);
  d->sig = NULL;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_selectoras_pc_implic(nodo_tad *p, nodo_tad *q)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar las selectoras de un tad de tipo PROD_CARTESIANO   *
*	       implicito.						  *
* DESCRIPCION: El conjunto de las selectoras de <p> x <q> es la union dis-*
*	 yunta de los siguientes conjuntos:				  *
*        - constructoras de <p>.			  		  *
*        - constructoras de <q>.			  		  *
*        - selectoras de <p>.			  		  	  *
*        - selectoras de <q>.			  			  *
* PARAMETROS:                                                             *
*   nodo_tad	*p	Ap. al primer tad padre de la PROD_CARTESIANO.    *
*   nodo_tad	*q	Ap. al segundo tad padre de la PROD_CARTESIANO.   *
* V.GLOBALES:                                                             *
*    nodo_tad	*ult_tad     	Apunt. al tad de que se esta generando por*
*			  	PROD_CARTESIANO				  *
* PRE!:  { p-> T1, q-> T2, ult_tad-> TC = T1 x T2, 	  	 	  *
*	   T1.constructoras = <c11,...,c1m1>, m1 >= 0,                    *
*	   T2.constructoras = <c21,...,c2m2>, m2 >= 0,			} *
*	   T1.selectoras =    <s11,...,s1n1>, n1 >= 0,                    *
*	   T2.selectoras =    <s21,...,s2n2>, n2 >= 0,			} *
* POST!: { TC.selectoras =    <c11'1',...,c1m1'1',s11'1',...,s1n1'1',     *
*			       c21'2',...,c2m2'2',s21'2',...,s2n2'2'>     *
* LLAMA A: 								  *
*   void  copie_lista_selectoras 	void  convierta_constrs_selects   *
*   short es_prefijo 			void  genere_axs_select_implic    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void
genere_selectoras_pc_implic (nodo_tad * p, nodo_tad * q)
{
/*----------------------------------------------------------------------*/


  static nodo_select *cabeza_lista = NULL,	/* VTP. 5/May/1997 */
   *ult_lista = NULL,		/* VTP. 5/May/1997 */
   *ult_select = NULL,		/* VTP. 5/May/1997 */
   *prim2 = NULL,		/* VTP. 5/May/1997 */
   *s = NULL;			/* VTP. 5/May/1997 */
  assert (p != NULL && q != NULL && ult_tad != NULL);

  convierta_constrs_selects (p, &cabeza_lista, &ult_lista, "1");
  ult_tad->selectoras = cabeza_lista;
  ult_select = ult_lista;
  copie_lista_selectoras (p->selectoras, &cabeza_lista, &ult_lista,
			  p->nombre, ult_tad->nombre, "1", p,"");
  if (ult_select == NULL)
    {
      ult_tad->selectoras = cabeza_lista;
    }
  else
    {
      ult_select->sig = cabeza_lista;
    }
  if (ult_lista != NULL)
    ult_select = ult_lista;
  convierta_constrs_selects (q, &cabeza_lista, &ult_lista, "2");
  prim2 = cabeza_lista;
  if (ult_select == NULL)
    {
      ult_tad->selectoras = cabeza_lista;
    }
  else
    {
      ult_select->sig = cabeza_lista;
    }
  if (ult_lista != NULL)
    ult_select = ult_lista;
  copie_lista_selectoras (q->selectoras, &cabeza_lista, &ult_lista,
			  q->nombre, ult_tad->nombre, "2", q,"");
  if (ult_select == NULL)
    {
      ult_tad->selectoras = cabeza_lista;
    }
  else
    {
      ult_select->sig = cabeza_lista;
    }

  if (prim2 == NULL)
    prim2 = cabeza_lista
      ;

  /* Suprimir las igualdades de cada uno de los componentes */
  if (ult_tad->selectoras != NULL)
    {
      s = ult_tad->selectoras;
      while ((s != NULL) && (s->sig != NULL))
	{
	  if (es_prefijo ("EQ_", s->sig->nombre))
	    {
	      if (s->sig == prim2)
		{
		  prim2 = prim2->sig;
		}
	      s->sig = s->sig->sig;
	    }
	  else
	    {
	      s = s->sig;
	    }
	}
      if ((ult_tad->selectoras != NULL) &&
	  es_prefijo ("EQ_", ult_tad->selectoras->nombre))
	{
	  if (ult_tad->selectoras == prim2)
	    {
	      prim2 = prim2->sig;
	    }
	  ult_tad->selectoras = ult_tad->selectoras->sig;
	}
    }
  for (s = ult_tad->selectoras; (s != NULL) && (s != prim2);
       s = s->sig)
    {
      genere_axs_select_implic (s, "1", p->nombre, q->nombre);
    }
  for (; (s != NULL); s = s->sig)
    {
      genere_axs_select_implic (s, "2", q->nombre, p->nombre);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void prod_cart_implicito(cad12 nom_pc)				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar un tad de tipo PROD_CARTESIANO implicito.	  *
* DESCRIPCION: Pide los nombre de los tads T1 y T2 que se van a componer, *
*	 genera automatica/. la signatura y los axiomas del nuevo tad 	  *
*	 <nom_pc>.							  *
* PARAMETROS:                                                             *
*    cad12	nom_pc	Nombre del nuevo tad de tipo PROD_CARTESIANO.	  *
* V.GLOBALES:                                                             *
*    nodo_tad	*ult_tad     	Apunt. al tad de que se esta generando por*
*			  	PROD_CARTESIANO				  *
* PRE!:  { nom_pc = NT,                                                	  *
*          tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2.					} *
* POST!: { tad -> <t1,...,tn,t(n+1)> <- ult_tad , donde		  	  *
*	        p-> ti, q-> tj,  1 <= i,j <= n,				  *
*		t(n+1) = <p> x <q>, t(n+1).nombre = NT, o  		  *
*          tad -> <t1,...,tn> <- ult_tad, Si la generacion del nuevo tad  *
*	   	no fue exitosa.						  *
* LLAMA A:nodo_tad  *pida_tad_ambiente					  *
*   void  cree_nodo_tad 		void  informacion_tipo_prod 	  *
*   void  genere_param_pc    		void  genere_inicial_pc_implic	  *
*   void  genere_selectoras_pc_implic                                     *
*   void  adic_proyecciones_implic	void  adic_igualdad		  *
*   short pida_status 			void  esc_nom_tad_actual	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void
prod_cart_implicito (cad12 nom_pc)
{
/*----------------------------------------------------------------------*/
  nodo_tad *p, *q;

  assert (nom_pc != NULL);	/* VTP. 5/May/1997 */
  p = pida_tad_ambiente (mensajes[52]);
  if (p != NULL)
    {
      if (p->parametros!=NULL ||
	  (p->iniciales==NULL && p->constructoras==NULL))
	{
	  escriba_mensaje(179);
	  return;
	}
      q = pida_tad_ambiente (mensajes[51]);
      if (q != NULL)
	{
	  if (q->parametros!=NULL ||
	  (q->iniciales==NULL && q->constructoras==NULL))
	    {
	      escriba_mensaje(179);
	      return;
	    }
	  cree_nodo_tad ();
	  strcpy (ult_tad->nombre, nom_pc);
	  informacion_tipo_prod (ult_tad, p->nombre, q->nombre);
	  genere_param_pc (p, q);
	  genere_inicial_pc_implic (p, q);
	  ult_tad->constructoras = NULL;
	  genere_selectoras_pc_implic (p, q);
	  adic_proyecciones_implic (p, q);
	  adic_igualdad (p, q);
	  ult_tad->status = pida_status ();
	  esc_nom_tad_actual ();
	}
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void proyecte_args_tdi(nodo_expresion *l_axioma,cad12 nom_tad1,	  *
*			 cad5 indice)					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Proyectar todos los argumentos del TDI de una expresion    *
* DESCRIPCION: Proyecta todos los argumentos del TDI que aparecen en la ex*
*	 presion apuntada por <l_axioma> sobre la <indice> componente     *
* PARAMETROS:                                                             *
*   nodo_expresion 	*l_axioma	Ap. a la estructura que contiene  *
*					una expresion.			  *
*   cad12 		nom_tad1	Nombre del tad codominio de la pro*
*					ccion que se aplica.		  *
*   cad5 		indice		"1", Si es la 1a. proyeccion.	  *
*					"2", Si es la 2a. proyeccion.	  *
* V.GLOBALES:                                                             *
*    nodo_tad	*ult_tad     	Apunt. al tad de que se esta generando por*
*			  	PROD_CARTESIANO				  *
* PRE!:  { l_axioma = EXP, nom_tad1 = Ti, indice = I, I = 1 o 2.        } *
* POST!: { l_axioma = PROY(EXP) que resulta de reemplazar cada subexpre-  *
*	   sion t del TDI de EXP por PROY<indice>(t) en EXP.		} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 04-Jul-90 *
**************************************************************************/

void
proyecte_args_tdi (nodo_expresion * l_axioma, cad12 nom_tad1, cad5 indice)
{
/*----------------------------------------------------------------------*/

  nodo_expresion *exp;
  nodo_argum *arg;
  assert (l_axioma != NULL && nom_tad1 != NULL && indice != NULL);	/* VTP. 5/May/1997 */
  for (arg = l_axioma->argumentos; arg != NULL; arg = arg->sig)
    {
      if (!strcmp (arg->expr->tipo, ult_tad->nombre))
	{
	  exp = (nodo_expresion *) malloc (sizeof (nodo_expresion));
	  HayMem (exp, sizeof (nodo_expresion));	/* VTP. 5/May/1997 */
	  sprintf (exp->nombre, "PROY_%s%s", ult_tad->nombre, indice);
	  strcpy (exp->tipo, nom_tad1);
	  strcpy (exp->clase, "SELECT");
	  exp->argumentos = (nodo_argum *) malloc (sizeof (nodo_argum));
	  HayMem (exp->argumentos, sizeof (nodo_argum));	/* VTP. 5/May/1997 */
	  exp->argumentos->sig = NULL;
	  exp->argumentos->expr = arg->expr;
	  arg->expr = exp;
	}
    }
/*----------------------------------------------------------------------*/
}
