
/**
 * Compiler of ManTa 3.1 files
 * Version: 3.1b4 - 1999
 * Author: Vladimir Tamara
 * See http://wwwisc.uniandes.edu.co/~manta
 **/

#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include "apimanta.h"
#include "os_dep.h"


/****************
  PROTOTYPES
 ****************/

/* Loads a type written in a text file (not in environment)  */
nodo_tad *loadTextType(char *name);


/*****************
    MAIN   
 *****************/

int main(int argc,char *argv[])
{

  nodo_tad *et;
  int rc,i,error;
  FILE *out;
  char configFile[200];
  char inFile[200];
  char outFile[200];
  char workDir[1000];

  printf(" ManTa Compiler  v 3.1b4\n");
  printf(" Universidad de los Andes.  \n"
         " Software Construction Group.   1999\n"
         " See http://wwwisc.uniandes.edu.co/~manta\n\n");

  strcpy(configFile,"manta.ini\0");
  for (i=1,error=0;!error && i<argc && argv[i][0]=='-';i++)
    {
      switch (argv[i][1]) 
	{
	case 'd': 
	  if (i+1<argc)
	    { 
	      strcpy(workDir,argv[i+1]); 
	      transArch(workDir);
	      if (chdir(workDir)!=0) {
		printf("Could'nt change to directory %s\n",workDir);
		error=1;
	      }
	      i++;
	    }
	  else
	    { error=1; }
	  break;
	case 'i': 
	  if (i+1<argc)
	    { 
	      strcpy(configFile,argv[i+1]); 
	      transArch(configFile);  
	      i++;
	    }
	  else
	    { error=1; }
	  break;
	default:
	  error=1;
	  break;
	}
    }

  if (error || i!=argc-2) 
    {
      printf("Syntax is: \n"
	     " mancomp [-d <dir>] [-i <confFile>] <inFile> <outFile>\n"
	     "<dir> is the working directory\n"
	     "<confFile> is the configuration file (by default manta.ini)\n"
             "<inFile> is a text with a ManTa specification\n"
	     "<outFile> is the name of the generated bytecode\n");
      return 1;
    }
  strcpy(inFile,argv[i]);
  strcpy(outFile,argv[i+1]);
  
  transArch(configFile);  
  transArch(inFile);
  transArch(outFile);
  
  if (iniMantaAPI(configFile)<0)
    {
      printf("Initialization Failed (sure you have configuration file %s ?)\n",
	     configFile);
      return 1;
    }
  
  if ((et=loadTextType(inFile))==NULL)
    {
      rc=2;
      goto exit;
    }
  if ((out=fopen(outFile,"wt"))==NULL)
    {
      printf("The output file couldn't be written\n");
      rc=3;
      goto exit;
    }      
  if ((rc=salvaTipo(out,et))!=0)
    {
      printf("The output file couldn't be written\n");
      rc=3;
      goto exit;
    }
  fclose(out);
  printf("Succesfull compilation!\n");
      
 exit:
  if (et!=NULL)
    { elimNodoTipo(&et); }
  finMantaAPI();
  return rc;
}



/**
 * Loads a type written in a text file (not in environment)
 * @param name is the file name
 **/
nodo_tad *loadTextType(char *name)
{

  FILE *f;
  texpr msg;
  nodo_tad *tad=NULL;
  assert(name!=NULL);

  transArch(name);
  f=fopen(name,"rb");
  if (f==NULL)
    {
      printf("File not found. \n");
      return NULL;
    }
  msg[0]='\0';
  if (cargaTipoTexto(f,&tad,msg))
    {
      printf("Error loading file. Message:\n");
      printf("%s\n",msg);
    }
  fclose(f);
  return tad;
}

