/***********************************************************/
/* MODULO CON FUNCIONES DEPENDIENTES DEL SISTEMA OPERATIVO */
/* Por Vladimir Tamara                                     */
/* Inicio: 13/Mar/1997                                     */
/* Modificacion mas reciente: 13/Mar/1997                  */
/***********************************************************/
/* Los simbolos de compilacion empleados son:              */
/*  DJGPP - DJGPP es GNU C++ para DOS 32 bits              */
/*  __CYGWIN32__ -- CYGNUS es GNU C++ para Windows 95      */
/*  __BORLANDC__ - Borland C++ 5.0 para Windows 95/        */
/*                 Windows 3.1      y DOS                  */
/***********************************************************/


#include "os_dep.h"
#include "assert.h"
/*** VTP. 15/Ene/97.  Se incluyen funciones para hacer:
    - copia (copy, cp)
    - touch
    - directorio (dir, ls)
***/

/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* j-tamara 29/May/1997 */

#if defined(DJGPP) || defined(__BORLANDC__)
#include <time.h>
#include <dos.h>
#include <dir.h>
#endif

#ifdef __BORLANDC__
#include <io.h>
/* En BC 5.0 la funcion setftime esta en io.h */
#endif

/**************************************************************************
* int copia_archivo(char *fuente,char *destino)                          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Copiar el archivo fuente en el destino                     *
* DESCRIPCION:                                                            *
*   [1] Localiza un bloque lo ms grande posible                       *
*   [2] Abre archivo fuente para lectura y destino para escritura (b)     *
*   [3] Copia bloques del archivo fuente en el destino                    *
*   [4] Cierra archivos abiertos y libera memoria                         *
* R.R.  Puede emplear compilacion condicional para hacerla mas eficiente  *
*       en diversos SO                                                    *
* PRE!:  fuente y destino son cadenas                                     *
* POST!: el archivo fuente es copiado en destino o se retorna un codigo   *
*        de error (numero negativo)                                       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Vladimir Tamara Patino (VTP)                    FECHA: 13-Ene-97 *
**************************************************************************/
int
copia_archivo (char *fuente, char *destino)
{
/*----------------------------------------------------------------------*/

  char *buf;			/* Buffer de copia */
  long tambuf = 0x80000;	/* Aspiramos a casi 500KB para el buffer */
  long ocubuf;			/* Lo ocupado del buffer */
  FILE *in, *out;

  assert (fuente != NULL && destino != NULL);

  /* [1] Localizamos memoria */
  for (buf = malloc (sizeof (char) * tambuf);
       buf == NULL && tambuf > 0;
       tambuf -= 0x1000)
    {
      buf = malloc (sizeof (char) * tambuf);
    }

  if (tambuf <= 0)
    {
      return -1;
    }
  /* [2] Abrimos archivos */
  assert (buf != NULL);
  if ((in = fopen (fuente, "rb")) == NULL)
    {
      return -1;
    }
  if ((out = fopen (destino, "wb")) == NULL)
    {
      return -1;
    }

  /* [3] Copiamos bloques hasta que no haya ms por copiar */
  assert (in != NULL && out != NULL);

  do
    {
      ocubuf = fread (buf, sizeof (char), tambuf, in);
      if (ocubuf > 0)
	{
	  fwrite (buf, sizeof (char), ocubuf, out);
	}
    }
  while (ocubuf > 0);

  /* [4] Cerramos archivos y liberamos memoria */

  fclose (in);
  fclose (out);

  free (buf);

  return 0;

/*----------------------------------------------------------------------*/ 
}


/**************************************************************************
* nodo_nombre *lista_dir(cad20 dir, cad20 ext)                          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Retorna una lista con los nombres de los archivos del      *
*              directorio dir, cuya extension sea ext.                    *
* MODIFICACIONES:
* Hernan Aponte
* VTP 18/Dic/96
*         Esta funcion empleaba system para pedir el directorio a UNIX,
*       tal metodo no funciona en DOS.   Tuvo que emplearse compilacion
*       condicional y las instrucciones findfirst y findnext ofrecidas
*       con tal fin por djgpp
*         La compilacion condicional emplea el simbolo DJGPP que solo
*         funcionara con tal compilador
* VTP. 3/Jun/1997 Se agrego el parametro conext, que indica si
*      deben retornarse o no las extensiones
**************************************************************************/

nodo_nombre *
lista_dir (cad20 dir, cad20 ext, int conext)
{
/*----------------------------------------------------------------------*/

#if !defined(DJGPP) && !defined(__BORLANDC__)
  FILE *fp;
#endif
  texpr comando;
  texpr nombre;
  nodo_nombre *l;

#if defined(DJGPP) || defined(__BORLANDC__)
  struct ffblk entdir;
/*** VTP. Agregado para obtener directorios en DOS */
  int done;
#endif

  l = NULL;

/*** VTP. 18/Dic/96 y 13/Ene/96  Modificado para que corra tambien en DOS ***/
#if defined(DJGPP) || defined(__BORLANDC__)
  sprintf (comando, "%s/*.%s", dir, ext);
  transArch(comando);
/*  for (ci = 0; ci < strlen (comando); ci++)
    {
      if (comando[ci] == '/')
	{
	  comando[ci] = '\\';
	}
    } */

  done = findfirst (comando, &entdir, FA_RDONLY | FA_HIDDEN | FA_SYSTEM | FA_ARCH);
  while (!done)
    {
      formatear_nombre (entdir.ff_name, nombre,conext);
      insertar_nombre_ord (&l, nombre);
      done = findnext (&entdir);
    }
#else
  /* Debe ser UNIX o Cygnus C++ */
  sprintf (comando, "ls %s/*.%s > dir ", dir, ext); /* VTP. 15/Jun/1997  */
  system (comando);
/***VTP.OK***/
  fp = fopen ("dir", "r");
  FileOpen(fp,"dir");                   /* j-tamara 29/May/1997 */
  while (fscanf (fp, "%s", comando) != EOF)
    {
      formatear_nombre (comando, nombre,conext);
      insertar_nombre_ord (&l, nombre);
    }
  fclose (fp);
  system ("rm -f dir");
/*** VTP. OK ***/
  system ("rm -f errores");
/*** VTP. OK ***/
#endif

  return (l);
/*----------------------------------------------------------------------*/
}


/**************************************************************************
* void formatear_nombre(texpr lin1, texpr lin2,int conext)                           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE : lin1 es un nombre de archivo con camino y extension.              *
* POS : contiene unicamente el nombre del archivo que aparece en lin1,    *
*       es decir sin camino ni comentarios.                               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Cesar Munoz                                     FECHA: 28-Nov-91 *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* MODIFICACIONES:
* VTP. 3/Jun/1997.  Agregada a este modulo, con la opcion de dejar o no la
*                   extension del archivo
**************************************************************************/

void formatear_nombre(texpr lin1, texpr lin2,int conext)
{/*----------------------------------------------------------------------*/
	 int prim,ult; /* Primer y ultimo caracter imprimible de lin */
	 int j,i;
	 assert(lin1!=NULL && lin2!=NULL && strlen(lin1)<MAX_TEXPR
				&& strlen(lin2)<MAX_TEXPR); /* VTP. 23/Abr/1997 */

	 for (prim = strlen(lin1); prim > 0 && lin1[prim] != '/'
                                   && lin1[prim]!= '\\';
              prim--);
	 if (lin1[prim] == '/' || lin1[prim] == '\\')
		  prim++;
         if (conext)
         {
             ult=strlen(lin1)-1;
         }
         else
         {
             for (ult = prim; lin1[ult] != '.' && lin1[ult] != '\0'; ult++);
             if (lin1[ult] == '.')
                      ult--;
         }

	 for (j=prim,i=0; j<=ult; lin2[i]=lin1[j],i++,j++);
	 lin2[i]='\0';
/*----------------------------------------------------------------------*/}


/**************************************************************************
* void nombre_autor(texpr nombre)                                         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE : True                                                              *
* POS : nombre == Nombre del usuario del sistema                          *
* MODIFICACIONES:             
*   VTP. 13/Mar/1997
*   En DOS no hay nombres de usuario, se retornara vacio
**************************************************************************/

void 
nombre_autor (texpr nombre)
{
/*----------------------------------------------------------------------*/

#if defined(DJGPP) || defined(__BORLANDC__) || defined(__CYGWIN32__)
  nombre[0]=' ';
  nombre[1] = '\0';
  return;
#else
  FILE *f_who_iam;
  FILE *f_passwd;
  texpr nom_iam;
  texpr linea;

  strcpy (nombre, "");
  system ("awk -F: \'{printf \"%s\\n.%s.\\n\",$1,$5}\'</etc/passwd>usuarios");
  system ("env|awk -F= \'$1==\"LOGNAME\" {print $2}\'>iam");

  f_who_iam = fopen ("iam", "r");
  f_passwd = fopen ("usuarios", "r");
  if (f_who_iam == NULL || f_passwd == NULL)
    return;
  fscanf (f_who_iam, "%s", nom_iam);
  fscanf (f_passwd, "%s\n", linea);
  while (!feof (f_passwd) && strcmp (linea, nom_iam))
    {
      fgets (linea, sizeof (texpr), f_passwd);
      fscanf (f_passwd, "%s\n", linea);
    }
  if (!feof (f_passwd))
    {
      fgets (linea, sizeof (texpr), f_passwd);
      formatear_linea (linea, nombre);
    }
  fclose (f_passwd);
  fclose (f_who_iam);
  system ("rm  -f iam");
  system ("rm -f usuarios");

#endif
/*----------------------------------------------------------------------*/ 
}


/**************************************************************************
* int touch(char *nomarc)                                                 *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   A los archivos especificados en nomarc (pueden usarse      *
*              wildcards *, ?), les cambia la fecha y hora a la actual    *
* DESCRIPCION:                                                            *
*   Si es UNIX emplea el touch del sistema                                *
*   Si es DOS emplea el siguiente algoritmo                               *
*    [1]  Se obtiene hora y fecha actual                                  *
*    [2] Se obtiene la via de los archivos                                *
*    [3] A cada archivo se le cambia fecha y hora por la actual           *
* PRE!:  nomarc es cadena                                                 *
* POST!: Cambio fecha de archivos que "unificara" con nomarc              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Vladimir Tamara Patino (VTP)                    FECHA: 15-Ene-97 *
**************************************************************************/
int 
touch (char *nomarc)
{
/*----------------------------------------------------------------------*/

  texpr comando;

#if defined(DJGPP) || defined(__BORLANDC__)

  time_t tnow;			/* Hora actual */
  struct tm tmnow;		/* Hora actual */
  struct ftime ft;		/* Fecha actual */
  FILE *f;			/* Archivo por tocar */
  struct ffblk entdir;		/* Entrada de directorio */
  texpr nombre;			/* Cadena para almacenar nombres de archivos */
  int done;			/* Indica si se termino de procesar directorio */
  int ci;			/* Contador */
  char drive[MAXDRIVE], path[MAXDIR], file[MAXFILE], ext[MAXEXT];
  int rspl;			/* Resultado retornado por funcion fnsplit */
  texpr nomini;			/* Via por agregar a los nombres de archivos */

  /* [1]  Se obtiene hora y fecha actual */

  tnow = time (NULL);
  tmnow = *localtime (&tnow);

  ft.ft_year = tmnow.tm_year - 80;
  ft.ft_month = tmnow.tm_mon + 1;
  ft.ft_day = tmnow.tm_mday;
  ft.ft_hour = tmnow.tm_hour;
  ft.ft_min = tmnow.tm_min;
  ft.ft_tsec = tmnow.tm_sec / 2;


  /* El nombre de directorio debe cambiarse de UNIX a DOS */
  sprintf (comando, "%s", nomarc);
  for (ci = 0; ci < strlen (comando); ci++)
    {
      if (comando[ci] == '/')
	{
	  comando[ci] = '\\';
	}
    }

  /* [2] Se obtiene la via de los archivos */
  drive[0] = path[0] = file[0] = ext[0] = nomini[0] = '\0';
  rspl = fnsplit (comando, drive, path, file, ext);
  if (rspl | DRIVE)
    {
      sprintf (nomini, "%s:", drive);
    }
  if (rspl | DIRECTORY)
    {
      sprintf (nomini, "%s%s", nomini, path);
    }

  /* [3] A cada archivo se le cambia fecha y hora por la actual */
  done = findfirst (comando, &entdir, FA_RDONLY | FA_HIDDEN | FA_SYSTEM | FA_ARCH);
  if (done)
    {				/* Touch aplicado a un archivo que no existe lo crea */
      f = fopen (comando, "w");
      /* No se verifica porque el nombre podria ser con Wildcards */
      if (f != NULL)
	{
	  fclose (f);
	}
    }

  while (!done)
    {
      sprintf (nombre, "%s\\%s", nomini, entdir.ff_name);
      if ((f = fopen (nombre, "r+b")) != NULL)
	{
	  setftime (fileno (f), &ft);
	  fclose (f);
	}
      done = findnext (&entdir);
    }

#else
  /* CYGNUS y UNIX usan touch */
  sprintf (comando, "touch %s", nomarc);
  system (comando);

#endif

  return 0;


/*----------------------------------------------------------------------*/
}


/**************************************************************************
* void transArch(char *nomarc)                                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   El nombre de archivo nomarc es transformado del formato de *
*              UNIX al del sistema operativo que se este usando           *
* PRE!:  nomarc es cadena                                                 *
* POST!: Convierte nomarc al formato del s.o empleado                     *
*        Por ahora si el simbolo DJGPP existe cambia / por \\             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Vladimir Tamara Patino (VTP)                    FECHA: 15-Ene-97 *
**************************************************************************/
void
transArch (char *nomarc)
{
/*----------------------------------------------------------------------*/

#if defined(DJGPP) || defined(__BORLANDC__)
  char *c;
  for (c = nomarc; *c != '\0'; c++)
    {
      *c = (*c == '/') ? (char) '\\' : *c;
    }
#endif
/*----------------------------------------------------------------------*/
}


/**************************************************************************
* void transArch2(char *nomarc)                                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   En Cygnus pone nomarc entre comillas "
*              Usado para editar bien archivos con edit
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Vladimir Tamara Patino (VTP)                    FECHA: 24-Jun-97 *
**************************************************************************/
void
transArch2 (char *nomarc)
{
/*----------------------------------------------------------------------*/
#if defined(__CYGWIN32__)
  char com[200];
  char *c;
  assert(strlen(nomarc)<200);
  sprintf(com,"\"%s\"",nomarc);
  strcpy(nomarc,com);
  nomarc[strlen(com)]='\0';
  for (c = nomarc; *c != '\0'; c++)
    {
      *c = (*c == '/') ? (char) '\\' : *c;
    }

#endif
  assert(nomarc!=NULL);

/*----------------------------------------------------------------------*/
}
