/* Test for ManTa Kernel 3.1 I */
/* By Vladimir Tamara P. */
/* 1998 */

#include <stdio.h>
#include "definic.h"
#include "apimanta.h"
#include "os_dep.h"
#include <assert.h>
#include <stdlib.h>

/***************************** PROTOTYPES *******************************/

/* Prints the contents of a file in stdout */
int imprimeArchivo(char *aname);

/* Loadas a type written in a text file (not in environment) */
nodo_tad *loadTextType(char *name, char *desc);

/* Prints an ADT */
int imprimeT(nodo_tad *nt,char *inf);

int testAxioms(void);

int testSignature(void);

int testGrammar(void);


/*****************************    MAIN    *******************************/

int main(int argc,char *argv[])
{

  nodo_tad *nat,*cola=NULL,*TBool,*nogen;
  char merr[2000]; /* Error messages */
  char arcnam[500];
  FILE *fp;

  char workDir[1000],configFile[200];
  int error,i;

  printf("# Test for  ManTa's Kernel 3.1. Part I\n"); 
  printf("# The testing environment is:\n");
  printf("#  - The Manta directory structure \n"
	 "#  - A manta.ini, configuring everything in english\n"
	 "#  - The test files (*.mta) \n"
	 "#  - The abstract data types cola and List\n");
  printf("# -----------------------------------------------------\n\n");

  workDir[0]='\0';
  strcpy(configFile,"manta.ini\0");
  for (i=1,error=0;!error && i<argc && argv[i][0]=='-';i++)
    {
      switch (argv[i][1]) 
	{
	case 'd': 
	  if (i+1<argc)
	    { 
	      strcpy(workDir,argv[i+1]); 
	      transArch(workDir);
	      if (chdir(workDir)!=0) {
		printf("Could'nt change to directory %s\n",workDir);
		error=1;
	      }
	      i++;
	    }
	  else
	    { error=1; }
	  break;
	case 'i': 
	  if (i+1<argc)
	    { 
	      strcpy(configFile,argv[i+1]); 
	      transArch(configFile);  
	      i++;
	    }
	  else
	    { error=1; }
	  break;
	default:
	  error=1;
	  break;
	}
    }

  if (error) 
    {
      printf("Syntax is: \n"
	     " mantst1 [-d <dir>] [-i <confFile>]\n"
	     "<dir> is the working directory\n"
	     "<confFile> is the configuration file (by default manta.ini)\n");
      return 1;
    }


  printf("Initializing kernel\n");
  if (iniMantaAPI(configFile)<0)
    {
      printf("Failed (sure you hava manta.ini??)\n");
      return 1;
    }
  printf("Succesfull initialization\n");
  printf("Using NAT\n");    
  if ((nat=buscaNombreLTipo(tad,"NAT"))==NULL)
    {
      printf("Failed. (Sure you have directory with types??) \n");
      return 1;
    }
  
  printf("Using bool\n");
  if ((TBool=buscaNombreLTipo(tad,"BOOL"))==NULL)
    {
      printf("Failed. (Sure you have directory with types??) \n");
      return 1;
    }
  printf("Loading cola\n");
  sprintf(arcnam,"%s/cola.adt",direct[2]);
  printf("Opening %s\n",arcnam);
  fp=fopen(arcnam,"rt");
  if (fp==NULL)
    {  printf("File not found %s\n",arcnam);  }
  else
    {
      merr[0]='\0';
      cola=NULL;
      if (cargaTipo(fp,&cola,merr)!=0)
	{
	  printf("Can't load type %s\n",arcnam); 
	  return 1;
	}
      else
	{  imprimeT(cola,"TAD Cola"); }
      fclose(fp);
    }

  sprintf(arcnam,"nogen1.mta");
  printf("Loading type withot generators (nogen1)\n");
  nogen=loadTextType(arcnam,"No generators");
  if (nogen==NULL || adicTipoAmbiente(&tad,&symtable,nogen,merr)!=0)
    {
      printf("Error adding type nogen1\n");
      return 1;
    }

  printf("\n# I. TESTING TYPES\n");

  printf("# 1. Testing Grammar\n");
  testGrammar();
  testSignature();
  testAxioms();

  return finMantaAPI();
}


/***************************** FUNCTIONS DEFINITION *************************/

/* Prints the contents of a file in stdout */
int imprimeArchivo(char *aname)
{
  FILE *fp;
  char lin[1000];

  printf("Printing file: %s\n",aname);
  fp=fopen(aname,"rt");
  if (fp==NULL)
    {
      printf("Couldn't read from %s\n",aname);
      return -1;
    }
  fgets(lin,1000,fp);
  while (!feof(fp))
    {
      printf("%s",lin);
      fgets(lin,1000,fp);
    }
  fclose(fp);

  return 0;
}

/* Loadas a type written in a text file (not in environment) */
nodo_tad *loadTextType(char *namerec, char *desc)
{

  FILE *f;
  texpr msg;
  nodo_tad *tad=NULL;
  char name[200];
  assert(name!=NULL && strlen(namerec)<200);

  strcpy(name,namerec);
  transArch(name);
  printf("# File:%s: %s\n",name,desc);
  f=fopen(name,"rb");
  if (f==NULL)
    {
      printf("File not found. \n");
      return NULL;
    }
  msg[0]='\0';
  if (cargaTipoTexto(f,&tad,msg))
    {
      printf("Error loading file. Message:\n");
      printf("%s\n",msg);
    }
  fclose(f);
  return tad;
}



/* Prints an ADT */
int imprimeT(nodo_tad *nt,char *inf)
{

  FILE *fp;
  char aname[1000];

  assert(nt!=NULL);
  sprintf(aname,"%s.mta",nt->nombre);
  printf("# Writing file: %s.   %s\n",aname,inf);
  fp=fopen(aname,"wt");
  if (fp==NULL)
    {
      printf("Couldn't write file %s\n",aname);
      return -1;
    }
  salvaTipoTexto(fp,nt);
  fclose(fp);
  imprimeArchivo(aname);
  loadTextType(aname,"Testing generated type");
  return 0;
}


int testAxioms()
{
  nodo_tad *t;
  char arcnam[200];

  printf("# 1.2. Testing axioms\n");
  sprintf(arcnam,"taxiom1.mta");
  t=loadTextType(arcnam,"Axioms definitions");
  elimNodoTipo(&t);
  sprintf(arcnam,"taxiom2.mta");
  t=loadTextType(arcnam,"Axioms definitions");
  elimNodoTipo(&t);
  sprintf(arcnam,"taxiom3.mta");
  t=loadTextType(arcnam,"Axioms definitions");
  elimNodoTipo(&t);
  sprintf(arcnam,"llaxiom1.mta");
  t=    loadTextType(arcnam,"Axioms should be Left-Linear"); 
  elimNodoTipo(&t);
  sprintf(arcnam,"llaxiom2.mta");
  t=    loadTextType(arcnam,"Axioms should be Left-Linear"); 
  elimNodoTipo(&t);
  sprintf(arcnam,"llaxiom3.mta");
  t=    loadTextType(arcnam,"Axioms should be Left-Linear"); 
  elimNodoTipo(&t);
  sprintf(arcnam,"llaxiom4.mta");
  t=    loadTextType(arcnam,"Axioms should be Left-Linear"); 
  elimNodoTipo(&t);
  sprintf(arcnam,"caxiom1.mta");
  t=loadTextType(arcnam,"Total covering");
  elimNodoTipo(&t);
  sprintf(arcnam,"caxiom2.mta");
  t=loadTextType(arcnam,"Total covering");
  elimNodoTipo(&t);
  sprintf(arcnam,"caxiom3.mta");
  t=loadTextType(arcnam,"Total covering");
  elimNodoTipo(&t);
  sprintf(arcnam,"caxiom4.mta");
  t=loadTextType(arcnam,"Total covering");
  elimNodoTipo(&t);
  sprintf(arcnam,"aaxiom1.mta");
  t=loadTextType(arcnam,"Ambiguous definitions");
  elimNodoTipo(&t);
  sprintf(arcnam,"aaxiom2.mta");
  t=loadTextType(arcnam,"Ambiguous definitions");
  elimNodoTipo(&t);
  sprintf(arcnam,"aaxiom3.mta");
  t=loadTextType(arcnam,"Ambiguous definitions");
  elimNodoTipo(&t);
  sprintf(arcnam,"aaxiom4.mta");
  t=loadTextType(arcnam,"Ambiguous definitions");
  elimNodoTipo(&t);
  sprintf(arcnam,"ifaxiom1.mta");
  t=loadTextType(arcnam,"Axiom with if");
  elimNodoTipo(&t);
  sprintf(arcnam,"ifaxiom2.mta");
  t=loadTextType(arcnam,"Axiom with if");
  elimNodoTipo(&t);
  sprintf(arcnam,"ifaxiom3.mta");
  t=loadTextType(arcnam,"Axiom with if");
  elimNodoTipo(&t);
  sprintf(arcnam,"ifaxiom4.mta");
  t=loadTextType(arcnam,"Axiom with if");
  elimNodoTipo(&t);
  sprintf(arcnam,"eraxiom1.mta");
  t=loadTextType(arcnam,"ERROR not allowed at left side");
  elimNodoTipo(&t);
  
  /* Chequear que todos tengan EQUAL ??? */
  sprintf(arcnam,"eq1.mta");
  t=loadTextType(arcnam,"Types with generators have to have EQ");
  elimNodoTipo(&t);
  sprintf(arcnam,"eq2.mta");
  t=loadTextType(arcnam,"Types without generators can't have EQ");
  elimNodoTipo(&t);
  sprintf(arcnam,"eq3.mta");
  t=loadTextType(arcnam,"The signature of a valid EQ is unique");
  elimNodoTipo(&t);
  sprintf(arcnam,"eq4.mta");
  t=loadTextType(arcnam,"The signature of a valid EQ is unique");
  elimNodoTipo(&t);
  
  
  /* Chequear nomenclatura.  Que una funcion definida en un TAD
     no sea: el nombre de una funcion definida en otro o el mismo TAD,
     el nombre de otro TAD, el nombre de un parametro del TAD 
     Una funcion no debe tener el mismo nombre del TAD (habra problemas
     en el momento de implementar)*/
  sprintf(arcnam,"nomenc1.mta");
  t=loadTextType(arcnam,"Overloading is not allowed in initials");
  elimNodoTipo(&t);
  sprintf(arcnam,"nomenc2.mta");
  t=loadTextType(arcnam,"Overloading is not allowed in initials");
  elimNodoTipo(&t);
  sprintf(arcnam,"nomenc3.mta");
  t=loadTextType(arcnam,"Overloading is not allowed for selectors");
  elimNodoTipo(&t);
  sprintf(arcnam,"nomenc4.mta");
  t=loadTextType(arcnam,"A function cannot have an ADT name");
  elimNodoTipo(&t);
  sprintf(arcnam,"nomenc5.mta");
  t=loadTextType(arcnam,"A function cannot have an ADT name");
  elimNodoTipo(&t);
  sprintf(arcnam,"nomenc6.mta");
  t=loadTextType(arcnam,"An ADT doesn't support overloading");
  elimNodoTipo(&t);
  sprintf(arcnam,"nomenc7.mta");
  t=loadTextType(arcnam,"An ADT doesn't support overloading");
  elimNodoTipo(&t);
  sprintf(arcnam,"nomenc8.mta");
  t=loadTextType(arcnam,"Functions names and Parameters different");
  elimNodoTipo(&t);
  sprintf(arcnam,"ADD.mta");
  t=loadTextType(arcnam,"An ADT can't have an already defined name");
  elimNodoTipo(&t);
  sprintf(arcnam,"nomenc9.mta");
  t=loadTextType(arcnam,"An ADT Parameter can't be a defined name");
  elimNodoTipo(&t);
  sprintf(arcnam,"nomenc10.mta");
  t=loadTextType(arcnam,"All the ADT's Parameter should differ");
  elimNodoTipo(&t);
  sprintf(arcnam,"nomenc11.mta");
  t=loadTextType(arcnam,"The EQ_x names are reserved");
  elimNodoTipo(&t);
  sprintf(arcnam,"nomenc12.mta");
  t=loadTextType(arcnam,"The EQ_x names are reserved");
  elimNodoTipo(&t);
  sprintf(arcnam,"nomenc13.mta");
  t=loadTextType(arcnam,"The EQ_x names are reserved");
  elimNodoTipo(&t);

  
  /* Tipos con parametros */
  sprintf(arcnam,"partype1.mta");
  t=loadTextType(arcnam,"Type with Parameters");
  elimNodoTipo(&t);
  sprintf(arcnam,"partype2.mta");
  t=loadTextType(arcnam,"Type with two Parameters");
  elimNodoTipo(&t);
  sprintf(arcnam,"partype3.mta");
  t=loadTextType(arcnam,"Type with Parameters  that will produce an extension");
  elimNodoTipo(&t);
  sprintf(arcnam,"partype4.mta");
  t=loadTextType(arcnam,"Type with Parameters without generators with domain invalid (itself)");
  elimNodoTipo(&t);
  sprintf(arcnam,"partype5.mta");
  t=loadTextType(arcnam,"Type with Parameters without generators with equality");
  elimNodoTipo(&t);
  sprintf(arcnam,"partype6.mta");
  t=loadTextType(arcnam,"Type with Parameters with generators and with equality");
  elimNodoTipo(&t);
  sprintf(arcnam,"partype7.mta");
  t=loadTextType(arcnam,"Type with Parameters with generators and without equality");
  elimNodoTipo(&t);

  return 0;
}


int testSignature()
{
  texpr nom;
  int k;
  nodo_tad *t;
  char arcnam[200];

  printf("# 1.1. Testing signatures\n");
  strcpy(nom,"domgen0.mta");
  for (k='1';k<'8';k++)
    {
      nom[6]=(char)k;
      t=loadTextType(nom,"Domain & Codomain of generators");
      elimNodoTipo(&t);
    }

  strcpy(arcnam,"domgen8.mta");
  t=loadTextType(arcnam,"Domain of initial can't be an extension");
  elimNodoTipo(&t);
  strcpy(arcnam,"domgen9.mta"); t=loadTextType(arcnam,"Domain of selector can't be an extension");
  elimNodoTipo(&t);
  strcpy(arcnam,"domgen10.mta"); t=loadTextType(arcnam,"Codomain of selector can't be an ext.");
  elimNodoTipo(&t);
  strcpy(arcnam,"domgen11.mta"); t=loadTextType(arcnam,"Domain of a initial can't have the ADT");
  elimNodoTipo(&t);
  strcpy(arcnam,"domgen12.mta"); t=loadTextType(arcnam,"Domain of a constructor has to have ADT");
  elimNodoTipo(&t);
  strcpy(nom,"domsel0.mta");
  for (k='1';k<'9';k++)
    {
      nom[6]=(char)k;
      t=loadTextType(nom,"Domain & Codomain of selectors");
      elimNodoTipo(&t);
    }
  strcpy(arcnam,"domsel9.mta"); t=loadTextType(arcnam,"The domain of a selector in an extension can't have the defined ADT");
  strcpy(arcnam,"domsel10.mta"); t=loadTextType(arcnam,"The codomain of a selector in an extension can't be the defined ADT");
  return 0;
}


int testGrammar()
{
  texpr nom;
  int k;
  nodo_tad *t;
  char arcnam[200];

  strcpy(nom,"empty0.mta");
  for (k='1';k<'7';k++)
    {
      nom[5]=(char)k;
      t=loadTextType(nom,"Empty case");
      elimNodoTipo(&t);
    }
  strcpy(arcnam,"noinit1.mta"); t=loadTextType(arcnam,"No initializators");
  elimNodoTipo(&t);
  strcpy(arcnam,"nocons1.mta"); t=loadTextType(arcnam,"No constructors");
  elimNodoTipo(&t);
  strcpy(arcnam,"nosel1.mta"); t=loadTextType(arcnam,"No selectors");
  elimNodoTipo(&t);
  strcpy(arcnam,"noaxiom1.mta"); t=loadTextType(arcnam, "No axioms (no word AXIOMS)");
  elimNodoTipo(&t);
  strcpy(arcnam,"noaxiom2.mta"); t=loadTextType(arcnam, "No axioms (with word AXIOMS)");
  elimNodoTipo(&t);
  strcpy(arcnam,"noaxiom3.mta"); t=loadTextType(arcnam,"Missing axiom in AXIOMS");
  elimNodoTipo(&t);
  strcpy(arcnam,"noaxiom4.mta"); t=loadTextType(arcnam,"Missing axiom in SELECTORS");
  elimNodoTipo(&t);
  strcpy(arcnam,"daxiom1.mta"); t=loadTextType(arcnam,"Different number of parameters in declaration and definition");
  elimNodoTipo(&t);
  strcpy(arcnam,"daxiom2.mta"); t=loadTextType(arcnam,"Different codomain type in declaration and definition");
  elimNodoTipo(&t);
  strcpy(arcnam,"daxiom3.mta"); t=loadTextType(arcnam,"Different domain type in declaration and definition");
  elimNodoTipo(&t);
  strcpy(arcnam,"ident1.mta"); t=loadTextType(arcnam,"Identifiers");
  elimNodoTipo(&t);
  strcpy(arcnam,"IDENT2.mta"); t=loadTextType(arcnam,"Identifiers case sensitive");
  elimNodoTipo(&t);
  strcpy(arcnam,"ident3.mta"); t=loadTextType(arcnam,"Identifiers case sensitive and length");
  elimNodoTipo(&t);
  strcpy(arcnam,"comen1.mta"); t=loadTextType(arcnam,"Comentaries");
  elimNodoTipo(&t);
  return 0;
}
