/* Test for ManTa Kernel (3.1 and 3.2) I */
/* By Vladimir Tamara P. */
/* 1998 */

#include <stdio.h>
#include "definic.h"
#include "apimanta.h"
#include "os_dep.h"
#include <assert.h>
#include <stdlib.h>

/***************************** PROTOTYPES *******************************/

/* Prints the contents of a file in stdout */
int imprimeArchivo(char *aname);

/* Loadas a type written in a text file (not in environment) */
nodo_tad *loadTextType(char *name, char *desc);

/* Prints an ADT */
int imprimeT(nodo_tad *nt,char *inf);

int testCopy(nodo_tad *nat,nodo_tad *TBool,nodo_tad *cola,nodo_tad *nogen);

int testProduct(nodo_tad *nat,nodo_tad *TBool, nodo_tad *cola,nodo_tad *nogen);

int testRestriction(nodo_tad *nat,nodo_tad *TBool, nodo_tad *cola,
		    nodo_tad *nogen);

int testDirectSum(nodo_tad *nat,nodo_tad *TBool,nodo_tad *cola,nodo_tad *nogen);

int testCompletation(nodo_tad *nat,nodo_tad *TBool,
		     nodo_tad *cola,nodo_tad *nogen);

int testVariation(nodo_tad *nat,nodo_tad *TBool,nodo_tad *cola, 
		  nodo_tad *nogen);

int testInstance(nodo_tad *nat,nodo_tad *TBool,nodo_tad *cola,nodo_tad *nogen);

int testRepresentations(nodo_tad *nat, nodo_tad *TBool,nodo_tad *cola,
			nodo_tad *nogen);


/*****************************    MAIN    *******************************/

int main(int argc,char *argv[])
{

  nodo_tad *nat,*cola=NULL,*TBool,*nogen;
  char merr[2000]; /* Error messages */
  char arcnam[500];
  FILE *fp;

  char workDir[1000],configFile[200];
  int error,i;


  printf("# Test for  ManTa's Kernel 3.1. Part II\n"); 
  printf("# The testing environment is:\n");
  printf("#  - The Manta directory structure \n"
	 "#  - A manta.ini, configuring everything in english\n"
	 "#  - The test files (*.mta) \n"
	 "#  - The abstract data types cola and List\n");
  printf("# -----------------------------------------------------\n\n");

  workDir[0]='\0';
  strcpy(configFile,"manta.ini\0");
  for (i=1,error=0;!error && i<argc && argv[i][0]=='-';i++)
    {
      switch (argv[i][1]) 
	{
	case 'd': 
	  if (i+1<argc)
	    { 
	      strcpy(workDir,argv[i+1]); 
	      transArch(workDir);
	      if (chdir(workDir)!=0) {
		printf("Could'nt change to directory %s\n",workDir);
		error=1;
	      }
	      i++;
	    }
	  else
	    { error=1; }
	  break;
	case 'i': 
	  if (i+1<argc)
	    { 
	      strcpy(configFile,argv[i+1]); 
	      transArch(configFile);  
	      i++;
	    }
	  else
	    { error=1; }
	  break;
	default:
	  error=1;
	  break;
	}
    }

  if (error) 
    {
      printf("Syntax is: \n"
	     " mantst2 [-d <dir>] [-i <confFile>]\n"
	     "<dir> is the working directory\n"
	     "<confFile> is the configuration file (by default manta.ini)\n");
      return 1;
    }


  printf("Initializing kernel\n");
  if (iniMantaAPI(configFile)<0)
    {
      printf("Failed (sure you hava manta.ini??)\n");
      return 1;
    }
  printf("Succesfull initialization\n");
  printf("Using NAT\n");    
  if ((nat=buscaNombreLTipo(tad,"NAT"))==NULL)
    {
      printf("Failed. (Sure you have directory with types??) \n");
      return 1;
    }
  
  printf("Using bool\n");
  if ((TBool=buscaNombreLTipo(tad,"BOOL"))==NULL)
    {
      printf("Failed. (Sure you have directory with types??) \n");
      return 1;
    }
  printf("Loading cola\n");
  sprintf(arcnam,"%s/cola.adt",direct[2]);
  printf("Opening %s\n",arcnam);
  fp=fopen(arcnam,"rt");
  if (fp==NULL)
    {  printf("File not found %s\n",arcnam);  }
  else
    {
      merr[0]='\0';
      cola=NULL;
      if (cargaTipo(fp,&cola,merr)!=0)
	{
	  printf("Can't load type %s\n",arcnam); 
	  return 1;
	}
      fclose(fp);
    }

  printf("Loading type withot generators (nogen1)\n");
  strcpy(arcnam,"nogen1.mta");
  nogen=loadTextType(arcnam,"No generators");
  if (nogen==NULL || adicTipoAmbiente(&tad,&symtable,nogen,merr)!=0)
    {
      printf("Error adding type nogen1\n");
      return 1;
    }

  printf("\n# I. TESTING TYPES\n");

  printf("# 2. Testing definition of new types\n");
  testInstance(nat,TBool,cola,nogen);
  testCopy(nat,TBool,cola,nogen);
  testProduct(nat,TBool,cola,nogen);
  testRestriction(nat,TBool,cola,nogen);
  testDirectSum(nat,TBool,cola,nogen);
  testCompletation(nat,TBool,cola,nogen);
  testVariation(nat,TBool,cola,nogen);
  testRepresentations(nat,TBool,cola,nogen);

  return finMantaAPI();
}


/***************************** FUNCTIONS DEFINITION *************************/

/* Prints the contents of a file in stdout */
int imprimeArchivo(char *aname)
{
  FILE *fp;
  char lin[1000];

  printf("Printing file: %s\n",aname);
  fp=fopen(aname,"rt");
  if (fp==NULL)
    {
      printf("Couldn't read from %s\n",aname);
      return -1;
    }
  fgets(lin,1000,fp);
  while (!feof(fp))
    {
      printf("%s",lin);
      fgets(lin,1000,fp);
    }
  fclose(fp);

  return 0;
}

/* Loadas a type written in a text file (not in environment) */
nodo_tad *loadTextType(char *namerec, char *desc)
{

  FILE *f;
  texpr msg;
  nodo_tad *tad=NULL;
  char name[200];
  assert(name!=NULL && strlen(namerec)<200);

  strcpy(name,namerec);
  transArch(name);
  printf("# File:%s: %s\n",name,desc);
  f=fopen(name,"rb");
  if (f==NULL)
    {
      printf("File not found. \n");
      return NULL;
    }
  msg[0]='\0';
  if (cargaTipoTexto(f,&tad,msg))
    {
      printf("Error loading file. Message:\n");
      printf("%s\n",msg);
    }
  fclose(f);
  return tad;
}



/* Prints an ADT */
int imprimeT(nodo_tad *nt,char *inf)
{

  FILE *fp;
  char aname[1000];

  assert(nt!=NULL);
  sprintf(aname,"%s.mta",nt->nombre);
  printf("# Writing file: %s.   %s\n",aname,inf);
  fp=fopen(aname,"wt");
  if (fp==NULL)
    {
      printf("Couldn't write file %s\n",aname);
      return -1;
    }
  salvaTipoTexto(fp,nt);
  fclose(fp);
  imprimeArchivo(aname);
  loadTextType(aname,"Testing generated type");
  return 0;
}


int testCopy(nodo_tad *nat,nodo_tad *TBool,nodo_tad *cola,nodo_tad *nogen)
{
  texpr merr;
  nodo_tad *t3;

  assert(nat!=NULL);
  assert(TBool!=NULL);
  assert(cola!=NULL);
  assert(nogen!=NULL);
  printf("# 2.2. Testing copying\n");
  merr[0]='\0';
  printf("# Basic Type\n");
  if (nuevoCopiaT("CNAT",nat,&t3,merr)!=0)
    printf(" %s",merr);
  else
    {
      imprimeT(t3,"Copy of NAT");
      elimNodoTipo(&t3);
    }
  printf("# Extension\n");
  if (nuevoCopiaT("cnogen",nogen,&t3,merr)!=0)
    printf(" %s",merr);
  else
    {
      imprimeT(t3,"Copy of extension");
      elimNodoTipo(&t3);
    }
  printf("# Type with Parameters\n");
  if (nuevoCopiaT("Ccola",cola,&t3,merr)!=0)
    printf(" %s",merr);
  else
    {
      imprimeT(t3,"Copy of cola[x]");
      elimNodoTipo(&t3);
    }
  return 0;
}



int testProduct(nodo_tad *nat,nodo_tad *TBool, nodo_tad *cola,
		    nodo_tad *nogen)
{
  nodo_tad *t3;
  texpr msg;

  assert(nat!=NULL);
  assert(TBool!=NULL);
  assert(cola!=NULL);
  assert(nogen!=NULL);
  printf("# 2.3. Testing product\n");
  msg[0]='\0';
  if (nuevoProdT("npn",nat,nat,&t3,msg)!=0)
    {
      printf("Couldn't make product NAT x NAT\n%s\n",msg);
    }
  else
    {
      imprimeT(t3,"Product NAT x NAT");
      elimNodoTipo(&t3);
    }
  msg[0]='\0';
  if (nuevoProdT("npb",nat,TBool,&t3,msg)!=0)
    {
      printf("Couldn't make product NAT x TBool\n%s\n",msg);
    }
  else
    {
      imprimeT(t3,"Product NAT x TBool");
      elimNodoTipo(&t3);
    }
  msg[0]='\0';
  if (nuevoProdT("bpn",TBool,nat,&t3,msg)!=0)
    {
      printf("Couldn't make product TBool x NAT\n%s\n",msg);
    }
  else
    {
      imprimeT(t3,"Product TBool x  NAT");
      elimNodoTipo(&t3);
    }
  msg[0]='\0';
  if (nuevoProdT("cpn",cola,nat,&t3,msg)!=0)
    {
      printf("Couldn't make product cola x nat\n%s\n",msg);
    }
  else
    {
      imprimeT(t3,"Product cola x nat");
      elimNodoTipo(&t3);
    }
  msg[0]='\0';
  if (nuevoProdT("npc",nat,cola,&t3,msg)!=0)
    {
      printf("Couldn't make product nat x cola\n%s\n",msg);
    }
  else
    {
      imprimeT(t3,"Product nat x cola");
      elimNodoTipo(&t3);
    }
  msg[0]='\0';
  if (nuevoProdT("cpc",cola,cola,&t3,msg)!=0)
    {
      printf("Couldn't make product cola x cola\n%s\n",msg);
    }
  else
    {
      imprimeT(t3,"Product cola x cola");
      elimNodoTipo(&t3);
    }
  msg[0]='\0';
  if (nuevoProdT("npg",nat,nogen,&t3,msg)!=0)
    {
      printf("Couldn't make product nat x nogen\n%s\n",msg);
    }
  else
    {
      imprimeT(t3,"Product nat x nogen1");
      elimNodoTipo(&t3);
    }
  msg[0]='\0';
  if (nuevoProdT("gpn",nogen,nat,&t3,msg)!=0)
    {
      printf("Couldn't make product nogen x nat\n%s\n",msg);
    }
  else
    {
      imprimeT(t3,"Product nogen1 x nat");
      elimNodoTipo(&t3);
    }
  return 0;
}


int testRestriction(nodo_tad *nat,nodo_tad *TBool, nodo_tad *cola,
		    nodo_tad *nogen)
{

  FILE *fp;
  nodo_tad *t3;
  texpr msg;

  assert(nat!=NULL);
  assert(TBool!=NULL);
  assert(cola!=NULL);
  assert(nogen!=NULL);
  printf("# 2.4. Testing restriction\n");
  fp=fopen("res1.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to res1.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoResT("rnat",nat,"PAR",
		    "PAR(ZERO)=TRUE PAR(SUC(X))=NOT(PAR(X))",
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make restriction of NAT to even\n%s\n",msg);
	}
      else
	{
	  imprimeT(t3,"Restriction of NAT to even (Generating res1.mta)");
	  elimNodoTipo(&t3);
	}
      fclose(fp);
    }
  
  fp=fopen("res2.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to res2.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoResT("rcola",cola,"COLAPAR",
		    "COLAPAR(vacia)=TRUE "
		    "COLAPAR(inserte(C1,X1))=NOT(COLAPAR(C1))",
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make restriction of Cola to even\n%s\n",msg);
	}
      else
	{
	  imprimeT(t3,"Restriction of Cola to even (Generating res2.mta)");
	  elimNodoTipo(&t3);
	}
      fclose(fp);
    }
  fp=fopen("res3.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to res3.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoResT("rnogen",nogen,"TRIVIAL",
		    "TRIVIAL(X)=TRUE ",
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make restriction of nogen1 to trivial\n%s\n",msg);
	}
      else
	{
	  imprimeT(t3,"Restriction of nogen to pred (Generated res3.mta)");
	  elimNodoTipo(&t3);
	}
      fclose(fp);
    }
  return 0;
}


int testDirectSum(nodo_tad *nat,nodo_tad *TBool,nodo_tad *cola,nodo_tad *nogen)
{
  FILE *fp;
  nodo_tad *t3;
  texpr msg;

  assert(nat!=NULL);
  assert(TBool!=NULL);
  assert(cola!=NULL);
  assert(nogen!=NULL);
  printf("# 2.5. Testing Direct Sum\n");
  fp=fopen("sum1.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to sum1.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoSumT("sum1",nat,TBool,
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make direct sum of NAT with TBool\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("sum1.mta");
	  elimNodoTipo(&t3);
	}
    }
  
  fp=fopen("sum2.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to sum2.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoSumT("sum2",nat,cola,
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make direct sum of NAT with cola\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("sum2.mta");
	  elimNodoTipo(&t3);
	}
    }
  
  fp=fopen("sum3.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to sum3.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoSumT("sum3",nogen,nat,
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make direct sum of nogen with NAT\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("sum3.mta");
	  elimNodoTipo(&t3);
	}
    }
  
  fp=fopen("sum4.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to sum4.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoSumT("sum4",nat,nat,
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make direct sum of NAT with NAT\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("sum4.mta");
	  elimNodoTipo(&t3);
	}
    }
  return 0;
}


int testCompletation(nodo_tad *nat,nodo_tad *TBool,
		     nodo_tad *cola,nodo_tad *nogen)
{
  FILE *fp;
  nodo_tad *t3;
  texpr msg;

  assert(nat!=NULL);
  assert(TBool!=NULL);
  assert(cola!=NULL);
  assert(nogen!=NULL);
  printf("# 2.6. Testing Completation\n");
  fp=fopen("comp1.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to comp1.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoComT("comp1",nat,"INF",
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make completation of NAT with INF\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("comp1.mta");
	  elimNodoTipo(&t3);
	}
    }

  fp=fopen("comp2.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to comp2.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoComT("comp2",cola,"INF",
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make completation of cola with INF\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("comp2.mta");
	  elimNodoTipo(&t3);
	}
    }
  
  fp=fopen("comp3.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to comp3.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoComT("comp3",nogen,"INF",
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make completation of nogen with INF\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("comp3.mta");
	  elimNodoTipo(&t3);
	}
    }
  
  fp=fopen("comp4.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to comp4.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoComT("comp4",TBool,"ZERO",
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make completation of TBool with ZERO\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("comp4.mta");
	  elimNodoTipo(&t3);
	}
    }
  return 0;
}


int testVariation(nodo_tad *nat,nodo_tad *TBool,nodo_tad *cola, nodo_tad *nogen)
{
  FILE *fp;
  nodo_tad *t3;
  texpr msg;

  assert(nat!=NULL);
  assert(TBool!=NULL);
  assert(cola!=NULL);
  assert(nogen!=NULL);
  printf("# 2.7. Testing Variation\n");
  
  fp=fopen("var1.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to var1.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoVarT("var1",TBool,nat,
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make variation of TBool and NAT\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("var1.mta");
	  elimNodoTipo(&t3);
	}
    }

  fp=fopen("var2.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to var2.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoVarT("var2",TBool,TBool,
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make variation of TBool and TBool\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("var2.mta");
	  elimNodoTipo(&t3);
	}
    }
  
  fp=fopen("var3.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to var3.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoVarT("var3",cola,TBool,
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make variation of cola and TBool\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("var3.mta");
	  elimNodoTipo(&t3);
	}
    }
  
  fp=fopen("var4.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to var4.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoVarT("var4",nat,nogen,
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make variation of nogen and NAT\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("var4.mta");
	  elimNodoTipo(&t3);
	}
    }
  return 0;
}


int testInstance(nodo_tad *nat,nodo_tad *TBool,nodo_tad *cola,nodo_tad *nogen)
{
  nodo_tad *t;
  texpr merr;
  nodo_tad *t3;
  char arcnam[200];

  assert(nat!=NULL);
  assert(TBool!=NULL);
  assert(cola!=NULL);
  assert(nogen!=NULL);

  printf("# 2.1. Testing instancing\n");
  strcpy(arcnam,"partype1.mta"); t=loadTextType(arcnam,"Instance");
  printf("# Creating instance type\n");
  merr[0]='\0';
  if (nuevoInstanciacionT("inst1",t,"X",nat,&t3,merr)!=0)
    printf(" %s",merr);
  else
    {
      imprimeT(t3,"One Parameter instanced with NAT");
      elimNodoTipo(&t3);
    }
  printf("# Creating instance type\n");
  merr[0]='\0';
  if (nuevoInstanciacionT("inst2",t,"X",t,&t3,merr)!=0)
    printf(" %s",merr);
  else
    {
      imprimeT(t3,"One Parameter instanced with cola");
      elimNodoTipo(&t3);
    }
  elimNodoTipo(&t);
  strcpy(arcnam,"partype2.mta"); t=loadTextType(arcnam,"Instance");
  merr[0]='\0';
  if (nuevoInstanciacionT("inst3",t,"Y",nat,&t3,merr)!=0)
    printf(" %s",merr);
  else
    {
      imprimeT(t3,"One of two Parameters instanced with NAT");
      elimNodoTipo(&t3);
    }
  elimNodoTipo(&t);
  merr[0]='\0';
  if (nuevoInstanciacionT("inst3",cola,"X",nat,&t3,merr)!=0)
    printf(" %s",merr);
  else
    {
      imprimeT(t3,"cola[NAT]");
      elimNodoTipo(&t3);
    }

  strcpy(arcnam,"tins1.mta"); t=loadTextType(arcnam,"Type with Parameter and difficult EQ");
  merr[0]='\0';
  if (nuevoInstanciacionT("inst4",t,"X",nat,&t3,merr)!=0)
    printf(" %s",merr);
  else
    {
      imprimeT(t3,"Parameter of difficult type instanced with NAT");
      elimNodoTipo(&t3);
    }
  elimNodoTipo(&t);
  return 0;
}


int testRepresentations(nodo_tad *nat, nodo_tad *TBool,nodo_tad *cola,
			nodo_tad *nogen)
{
  FILE *fp;
  nodo_tad *lb,*bn,*nlb,*blb,*t;
  texpr msg,arcnam;

  assert(nat!=NULL);
  assert(TBool!=NULL);
  assert(cola!=NULL);
  assert(nogen!=NULL);
  printf("# Testing Inductive Representations\n");

  strcpy(arcnam,"rep1.mta");
  fp=fopen(arcnam,"wt");
  if (fp==NULL)
    {
      printf("Couldn't write to %s",arcnam);
      return -1;
    }
  msg[0]='\0';
  if (nuevaRep("rep1",TBool,nat,
	       fp,&bn,msg)!=0)
    {
      printf("Couldn't make representation of Bool with NAT\n%s\n",msg);
      fclose(fp);
      return -1;
    }
  fclose(fp);
  imprimeArchivo(arcnam);
  strcpy(arcnam,"rep1com.mta");
  fp=fopen(arcnam,"rt");
  if (fp==NULL)
    {
      printf("Couldn't read from %s",arcnam);
      return -1;
    }
  msg[0]='\0';
  if (cambiosRep(&bn,fp,msg)!=0)
    {
      printf("Couldn't change representation\n"
	     "%s\n",msg);
      fclose(fp);
      return -1;
    }
  fclose(fp);
  imprimeArchivo(arcnam);
  strcpy(arcnam,"conjrep1.thm");
  fp=fopen(arcnam,"wt");
  if (fp==NULL)
    {
      printf("Couldn't write to %s",arcnam);
      return -1;
    }
  msg[0]='\0';
  if (conjeturasRep(fp,bn,msg)!=0)
    {
      printf("Couldn't construct conjetures for representation\n"
	     "%s\n",msg);
      fclose(fp);
      return -1;
    }
  fclose(fp);
  imprimeArchivo(arcnam);
  unlink("conjrep1.thm");
  
  strcpy(arcnam,"lb.mta");
  lb=loadTextType(arcnam,"List of Booleans\n");
  if (lb==NULL)
    {
      printf("File not found lb.mta");
      return -1;
    }
  if (adicTipoAmbiente(&tad,&symtable,lb,msg)!=0)
    {
      printf("Couldn't add lb to environment\n%s",msg);
      return -1;
    }

  fp=fopen("nlb.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to nlb.mta");
      return -1;
    }
  msg[0]='\0';
  if (nuevaRep("nlb",nat,lb,
	       fp,&nlb,msg)!=0)
    {
      printf("Couldn't make representation of NAT with lb\n%s\n",msg);
      fclose(fp);
      return -1;
    }
  fclose(fp);
  imprimeArchivo("nlb.mta");
  strcpy(arcnam,"nlbcom.mta");
  fp=fopen(arcnam,"rt");
  if (fp==NULL)
    {
      printf("Couldn't read from %s",arcnam);
      return -1;
    }
  msg[0]='\0';
  if (cambiosRep(&nlb,fp,msg)!=0)
    {
      printf("Couldn't change representation\n"
	     "%s\n",msg);
      fclose(fp);
      return -1;
    }
  fclose(fp);
  imprimeArchivo(arcnam);
  if (adicTipoAmbiente(&tad,&symtable,nlb, msg)!=0)
    {
      printf("Couldn't add nlb to environment\n%s",msg);
      return -1;
    }
  

  printf("\n# Composition of Representations\n");      

  msg[0]='\0';
  if (componeRep("blb",bn,nlb,&blb,msg)!=0)
    {
      printf("Couldn't make composition of bn with nlb\n");
      return -1;
    }
  fp=fopen("blb.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write blb.mta\n");
      return -1;
    }
  salvaTipoTexto(fp,blb);
  fclose(fp);
  strcpy(arcnam,"blb.mta"); t=loadTextType(arcnam,"Booleans represented with list of Booleans\n");

  blb->sig=tad;
  tad=blb;

  elimNodoTipo(&t);

  return 0;
}





