/* Tests for ManTa Kernel (3.1 and 3.2) Part II */
/* By Vladimir Tamara P. */
/* 1998 */

#include <stdio.h>
#include "definic.h"
#include "apimanta.h"
#include "os_dep.h"
#include <assert.h>
#include <stdlib.h>

/***************************** PROTOTYPES *******************************/

/* Prints the contents of a file in stdout */
int imprimeArchivo(char *aname);

/* Loadas a type written in a text file (not in environment) */
nodo_tad *loadTextType(char *name, char *desc);

/* Prints an ADT */
int imprimeT(nodo_tad *nt,char *inf);

/* This function finds the normal form of the expression it receives */
int normaliza(char *expr);

int testNormalization(void);

int testTheorems(void);


/*****************************    MAIN    *******************************/

int main(int argc, char *argv[])
{

  nodo_tad *nat,*cola=NULL,*TBool,*nogen;
  char merr[2000]; /* Error messages */
  char arcnam[500];
  FILE *fp;

  char workDir[1000],configFile[200];
  int error,i;

  printf("# Test for  ManTa's Kernel 3.1.  Part III\n");
  printf("# The testing environment is:\n");
  printf("#  - The Manta directory structure \n"
	 "#  - A manta.ini, configuring everything in english\n"
	 "#  - The test files (*.mta) \n"
	 "#  - The abstract data types cola and List\n");
  printf("# -----------------------------------------------------\n\n");

  workDir[0]='\0';
  strcpy(configFile,"manta.ini\0");
  for (i=1,error=0;!error && i<argc && argv[i][0]=='-';i++)
    {
      switch (argv[i][1]) 
	{
	case 'd': 
	  if (i+1<argc)
	    { 
	      strcpy(workDir,argv[i+1]); 
	      transArch(workDir);
	      if (chdir(workDir)!=0) {
		printf("Could'nt change to directory %s\n",workDir);
		error=1;
	      }
	      i++;
	    }
	  else
	    { error=1; }
	  break;
	case 'i': 
	  if (i+1<argc)
	    { 
	      strcpy(configFile,argv[i+1]); 
	      transArch(configFile);  
	      i++;
	    }
	  else
	    { error=1; }
	  break;
	default:
	  error=1;
	  break;
	}
    }

  if (error) 
    {
      printf("Syntax is: \n"
	     " mantst3 [-d <dir>] [-i <confFile>]\n"
	     "<dir> is the working directory\n"
	     "<confFile> is the configuration file (by default manta.ini)\n");
      return 1;
    }


  printf("Initializing kernel\n");
  if (iniMantaAPI(configFile)<0)
    {
      printf("Failed (sure you hava manta.ini??)\n");
      return 1;
    }
  printf("Succesfull initialization\n");
  printf("Using NAT\n");    
  if ((nat=buscaNombreLTipo(tad,"NAT"))==NULL)
    {
      printf("Failed. (Sure you have directory with types??) \n");
      return 1;
    }
  
  printf("Using bool\n");
  if ((TBool=buscaNombreLTipo(tad,"BOOL"))==NULL)
    {
      printf("Failed. (Sure you have directory with types??) \n");
      return 1;
    }
  printf("Loading cola\n");
  sprintf(arcnam,"%s/cola.adt",direct[2]);
  printf("Opening %s\n",arcnam);
  fp=fopen(arcnam,"rt");
  if (fp==NULL)
    {  printf("File not found %s\n",arcnam);  }
  else
    {
      merr[0]='\0';
      cola=NULL;
      if (cargaTipo(fp,&cola,merr)!=0)
 	{
	  printf("Can't load type %s\n",arcnam); 
	  return 1;
	}
      fclose(fp);
    }

  printf("Loading type withot generators (nogen1)\n");
  nogen=loadTextType("nogen1.mta","No generators");
  if (nogen==NULL || adicTipoAmbiente(&tad,&symtable,nogen,merr)!=0)
    {
      printf("Error adding type nogen1\n");
      return 1;
    }

  printf("\n# II. TESTING NORMALIZATION AND THEOREMS\n");
  testNormalization();
  testTheorems();

  return finMantaAPI();
}


/***************************** FUNCTIONS DEFINITION *************************/

/* Prints the contents of a file in stdout */
int imprimeArchivo(char *aname)
{
  FILE *fp;
  char lin[1000];

  printf("Printing file: %s\n",aname);
  fp=fopen(aname,"rt");
  if (fp==NULL)
    {
      printf("Couldn't read from %s\n",aname);
      return -1;
    }
  fgets(lin,1000,fp);
  while (!feof(fp))
    {
      printf("%s",lin);
      fgets(lin,1000,fp);
    }
  fclose(fp);

  return 0;
}

/* Loadas a type written in a text file (not in environment) */
nodo_tad *loadTextType(char *namerec, char *desc)
{

  FILE *f;
  texpr msg;
  nodo_tad *tad=NULL;
  char name[200];
  assert(name!=NULL && strlen(namerec)<200);

  strcpy(name,namerec);
  transArch(name);
  printf("# File:%s: %s\n",name,desc);
  f=fopen(name,"rb");
  if (f==NULL)
    {
      printf("File not found. \n");
      return NULL;
    }
  msg[0]='\0';
  if (cargaTipoTexto(f,&tad,msg))
    {
      printf("Error loading file. Message:\n");
      printf("%s\n",msg);
    }
  fclose(f);
  return tad;
}



/* Prints an ADT */
int imprimeT(nodo_tad *nt,char *inf)
{

  FILE *fp;
  char aname[1000];

  assert(nt!=NULL);
  sprintf(aname,"%s.mta",nt->nombre);
  printf("# Writing file: %s.   %s\n",aname,inf);
  fp=fopen(aname,"wt");
  if (fp==NULL)
    {
      printf("Couldn't write file %s\n",aname);
      return -1;
    }
  salvaTipoTexto(fp,nt);
  fclose(fp);
  imprimeArchivo(aname);
  loadTextType(aname,"Testing generated type");
  return 0;
}

/* This function finds the normal form of the expression it receives */
int normaliza(char *expr)
{
  FILE *fp;
  nodo_argum *arg = NULL;
  nodo_igualdad *conj = NULL;
  texpr mens_error;

  fp=fopen("normaliz","wt");
  if (fp==NULL)
    {
      printf("Couldn't write file normaliz\n");
      return -1;
    }
  fprintf(fp,"%s",expr);
  fclose(fp);
  fp=fopen("normaliz","rt");
  if (fp==NULL)
    {
      printf("Couldn't read file normaliz\n");
      return -1;
    }
  if ((arg = analice_lista_exp (fp, mens_error))==NULL)
    {
      printf("Wrong expression:\n%s\n",mens_error);
      fclose (fp);
      return -1;
    }
  fclose(fp);
  normalizar_lista (&arg, &conj);
  fp = fopen ("normaliz", "w");
  if (fp==NULL)
    {
      printf("Couldn't write file normaliz\n");
      return -1;
    }
  FileOpen (fp, "normaliz");
  escriba_arch_ig (fp, conj, mensajes[79]);
  fclose (fp);
  imprimeArchivo("normaliz");
  unlink("normaliz");
  
  return 0;
}

/* This function finds the normal form of the expression it receives */
int normaliza_file(char *archrec,char *msg)
{
  FILE *fp;
  nodo_argum *arg = NULL;
  nodo_igualdad *conj = NULL;
  texpr mens_error;
  char arch[200];

  assert(archrec!=NULL && strlen(archrec)<200);
  strcpy(arch,archrec);

  printf("# Normalizing %s: %s\n",arch,msg);
  transArch(arch);

  fp=fopen(arch,"rt");
  if (fp==NULL)
    {
      printf("Couldn't read file %s\n",arch);
      return -1;
    }
  if ((arg = analice_lista_exp (fp, mens_error))==NULL)
    {
      printf("Wrong expression:\n%s\n",mens_error);
      fclose (fp);
      return -1;
    }
  fclose(fp);
  normalizar_lista (&arg, &conj);
  fp = fopen ("normaliz", "w");
  if (fp==NULL)
    {
      printf("Couldn't write file normaliz\n");
      return -1;
    }
  escriba_arch_ig (fp, conj, mensajes[79]);
  fclose (fp);
  imprimeArchivo("normaliz");
  unlink("normaliz");
  
  return 0;
}



int testNormalization()
{

  printf("# Testing Normalization\n");
  normaliza_file("trivial.nor","Trivial cases");
  normaliza_file("natnor1.nor","Natural numbers");

  return 0;
}

int proveConj(char *arcnamrec,char *msg)
{
  int nnop,np; /* Numero de probadas y no probadas */
  nodo_demost *p,*nop;
  FILE *fp;
  texpr merr;
  char arcnam[200];
  
  assert(arcnamrec!=NULL && strlen(arcnamrec)<200);
  strcpy(arcnam,arcnamrec);

  printf("%s\n",msg);
  transArch(arcnam);
  printf("Proving conjetures of file %s\n",arcnam);
  fp=fopen(arcnam,"rt");
  if (fp==NULL)
    {
      printf("Error opening conjetures file %s",arcnam);
      return -1;
    }
  merr[0]='\0';
  if (probarConjeturas(fp,&np,&p,&nnop,&nop,merr)!=0)
    {
      printf("Problems trying to prove conjetures in %s",arcnam);
      fclose(fp);
      return -1;
    }
  fclose(fp);
  printf("Proved: %i\nUnproved %i\n",np,nnop);
  
  liberar_demost(&p);
  liberar_demost(&nop);
  return 0;
}


int testConj(char *arcnamrec)
{
  FILE *fp;
  texpr merr;
  nodo_igualdad *conj;
  char arcnam[200];
  
  assert(arcnamrec!=NULL && strlen(arcnamrec)<200);
  strcpy(arcnam,arcnamrec);

  printf("Testing conjetures file: %s\n",arcnam);
  fp=fopen(arcnam,"rt");
  if (fp==NULL)
    {
      printf("Conjetures file \"%s\" not found\n",arcnam);
      return 1;
    }
  conj = analice_conjeturas(fp,merr);
  if (conj==NULL)
    {
      printf("Couldn't read conjeture\n%s\n",merr);
      return -1;
    }
  fclose(fp);
  elimLIgualdad(&conj);
  return 0;
}

int testTheorems()
{
  printf("# Testing Theorems\n");
  printf("# Testing Induction, Cases, Generalization and Normalization\n");

  if (testConj("prove1.con")!=0)
    {
      printf("Error in conjectures file prove1.con\n");
      return 1;
    }
  if (proveConj("prove1.con","All cases")!=0)
    {
      printf("Error testing prove1.con\n");
      return 1;
    }

  return 0;
}


