/* Test for ManTa Kernel (3.1 and 3.2) */
/* By Vladimir Tamara P. */
/* 1998 */

#include <stdio.h>
#include "definic.h"
#include "apimanta.h"
#include "os_dep.h"
#include <assert.h>
#include <stdlib.h>

/***************************** PROTOTYPES *******************************/

/* Prints the contents of a file in stdout */
int imprimeArchivo(char *aname);

/* Loadas a type written in a text file (not in environment) */
nodo_tad *loadTextType(char *name, char *desc);

/* Prints an ADT */
int imprimeT(nodo_tad *nt,char *inf);

int testPrototypes(nodo_tad *nat,nodo_tad *TBool, nodo_tad *cola,
		    nodo_tad *nogen);


/*****************************    MAIN    *******************************/

int main(int argc,char *argv[])
{

  nodo_tad *nat,*cola=NULL,*TBool,*nogen;
  char merr[2000]; /* Error messages */
  char arcnam[500];
  FILE *fp;

  char workDir[1000],configFile[200];
  int error,i;

  printf("# Test for  ManTa's Kernel 3.1.  Part IV\n");
  printf("# The testing environment is:\n");
  printf("#  - The Manta directory structure \n"
	 "#  - A manta.ini, configuring everything in english\n"
	 "#  - The test files (*.mta) \n"
	 "#  - The abstract data types cola and List\n");
  printf("# -----------------------------------------------------\n\n");

  workDir[0]='\0';
  strcpy(configFile,"manta.ini\0");
  for (i=1,error=0;!error && i<argc && argv[i][0]=='-';i++)
    {
      switch (argv[i][1]) 
	{
	case 'd': 
	  if (i+1<argc)
	    { 
	      strcpy(workDir,argv[i+1]); 
	      transArch(workDir);
	      if (chdir(workDir)!=0) {
		printf("Could'nt change to directory %s\n",workDir);
		error=1;
	      }
	      i++;
	    }
	  else
	    { error=1; }
	  break;
	case 'i': 
	  if (i+1<argc)
	    { 
	      strcpy(configFile,argv[i+1]); 
	      transArch(configFile);  
	      i++;
	    }
	  else
	    { error=1; }
	  break;
	default:
	  error=1;
	  break;
	}
    }

  if (error) 
    {
      printf("Syntax is: \n"
	     " mantst4 [-d <dir>] [-i <confFile>]\n"
	     "<dir> is the working directory\n"
	     "<confFile> is the configuration file (by default manta.ini)\n");
      return 1;
    }


  printf("Initializing kernel\n");
  if (iniMantaAPI(configFile)<0)
    {
      printf("Failed (sure you hava manta.ini??)\n");
      return 1;
    }
  printf("Succesfull initialization\n");
  printf("Using NAT\n");    
  if ((nat=buscaNombreLTipo(tad,"NAT"))==NULL)
    {
      printf("Failed. (Sure you have directory with types??) \n");
      return 1;
    }
  
  printf("Using bool\n");
  if ((TBool=buscaNombreLTipo(tad,"BOOL"))==NULL)
    {
      printf("Failed. (Sure you have directory with types??) \n");
      return 1;
    }
  printf("Loading cola\n");
  sprintf(arcnam,"%s/cola.adt",direct[2]);
  printf("Opening %s\n",arcnam);
  fp=fopen(arcnam,"rt");
  if (fp==NULL)
    {  printf("File not found %s\n",arcnam);  }
  else
    {
      merr[0]='\0';
      cola=NULL;
      if (cargaTipo(fp,&cola,merr)!=0)
	{
	  printf("Can't load type %s\n",arcnam); 
	  return 1;
	}
      fclose(fp);
    }

  printf("Loading type withot generators (nogen1)\n");
  nogen=loadTextType("nogen1.mta","No generators");
  if (nogen==NULL || adicTipoAmbiente(&tad,&symtable,nogen,merr)!=0)
    {
      printf("Error adding type nogen1\n");
      return 1;
    }

  printf("\n# III. TESTING PROTOTYPES\n");
  testPrototypes(nat,TBool,cola,nogen);

  return finMantaAPI();
}


/***************************** FUNCTIONS DEFINITION *************************/

/* Prints the contents of a file in stdout */
int imprimeArchivo(char *aname)
{
  FILE *fp;
  char lin[1000];

  printf("Printing file: %s\n",aname);
  fp=fopen(aname,"rt");
  if (fp==NULL)
    {
      printf("Couldn't read from %s\n",aname);
      return -1;
    }
  fgets(lin,1000,fp);
  while (!feof(fp))
    {
      printf("%s",lin);
      fgets(lin,1000,fp);
    }
  fclose(fp);

  return 0;
}

/* Loadas a type written in a text file (not in environment) */
nodo_tad *loadTextType(char *namerec, char *desc)
{

  FILE *f;
  texpr msg;
  nodo_tad *tad=NULL;
  char name[200];
  assert(name!=NULL && strlen(namerec)<200);

  strcpy(name,namerec);
  transArch(name);
  printf("# File:%s: %s\n",name,desc);
  f=fopen(name,"rb");
  if (f==NULL)
    {
      printf("File not found. \n");
      return NULL;
    }
  msg[0]='\0';
  if (cargaTipoTexto(f,&tad,msg))
    {
      printf("Error loading file. Message:\n");
      printf("%s\n",msg);
    }
  fclose(f);
  return tad;
}



/* Prints an ADT */
int imprimeT(nodo_tad *nt,char *inf)
{

  FILE *fp;
  char aname[1000];

  assert(nt!=NULL);
  sprintf(aname,"%s.mta",nt->nombre);
  printf("# Writing file: %s.   %s\n",aname,inf);
  fp=fopen(aname,"wt");
  if (fp==NULL)
    {
      printf("Couldn't write file %s\n",aname);
      return -1;
    }
  salvaTipoTexto(fp,nt);
  fclose(fp);
  imprimeArchivo(aname);
  loadTextType(aname,"Testing generated type");
  return 0;
}


/* Genera el tipo del archivo d_prp/nom.adt (con descripcion desc), genera
   los archivo apropiados y copia la implementacion (sin interfaz)
   en d_prp */
void copiaFuentesPRP(nodo_tad **t,char *nom,char *desc)
{

  texpr merr,s,d;

  sprintf(s,"d_prp/%s.adt",nom);
  *t=loadTextType(s,desc);
  if (*t==NULL || adicTipoAmbiente(&tad,&symtable,*t,merr)!=0)
    { printf("Error adding type %s\n",nom); }
  if (*t==NULL || generaCProt(*t,CP_TODO)!=0)
    { printf("No pudo generar prototipos de %s\n",nom); }

  sprintf(s,"D_PROT/%s.c",nom);
  sprintf(d,"d_prp/%s.c",nom);
  if (copia_archivo (s,d)!=0)
    { printf("No pudo copiar archivo %s\n",s); }
  sprintf(s,"D_PROT/%s.h",nom);
  sprintf(d,"d_prp/%s.h",nom);
  if (copia_archivo (s,d)!=0)
    { printf("No pudo copiar archivo %s\n",s); }
  sprintf(s,"D_PROT/%s_co.inc",nom);
  sprintf(d,"d_prp/%s_co.inc",nom);
  if (copia_archivo (s,d)!=0)
    { printf("No pudo copiar archivo %s\n",s); }
  sprintf(s,"D_PROT/%s_de.h",nom);
  sprintf(d,"d_prp/%s_de.h",nom);
  if (copia_archivo (s,d)!=0)
    { printf("No pudo copiar archivo %s\n",s); }
  sprintf(s,"D_PROT/%s_in.h",nom);
  sprintf(d,"d_prp/%s_in.h",nom);
  if (copia_archivo (s,d)!=0)
    { printf("No pudo copiar archivo %s\n",s); }
  sprintf(s,"D_PROT/%s_in.c",nom);
  sprintf(d,"d_prp/%s_in.c",nom);
  if (copia_archivo (s,d)!=0)
    { printf("No pudo copiar archivo %s\n",s); }
}


int testPrototypes(nodo_tad *nat,nodo_tad *TBool, nodo_tad *cola,
		    nodo_tad *nogen)
{
  nodo_tad *cn,
    *Char,*String,
    *Parameter,*LParameter,
    *Domain,*LDomain,
    *Expr,*LExpr,
    *Axiom,*LAxiom,
    *Constructor,*LConstructor,
    *Selector,*LSelector,
    *AbsType;
  texpr merr;
  char s[100],nom[100];

  printf("# Testing Prototypes\n");
  printf("# \n");

  printf("Generating ADT Cola\n");
  if (generaCProt(cola,CP_TODO)!=0)
    { printf("No pudo generar prototipos\n"); }
  printf("Generating ADT cn=Cola[NAT]\n");
  if (nuevoInstanciacionT("cn",cola,"X",nat,&cn,merr)!=0)
    printf(" %s",merr);
  else
    {
      generaCProt(cn,CP_ENC);
      imprimeArchivo("D_PROT/cn.h");
      generaCProt(cn,CP_CON);
      imprimeArchivo("D_PROT/cn_de.h");
      imprimeArchivo("D_PROT/cn_co.inc");
      generaCProt(cn,CP_SEL);
      imprimeArchivo("D_PROT/cn.c");
      generaCProt(cn,CP_INT);
      imprimeArchivo("D_PROT/cn_in.h");
      imprimeArchivo("D_PROT/cn_in.c");
      generaCProt(cn,CP_MAIN);
      imprimeArchivo("D_PROT/cn_ma.c");
      generaCProt(cn,CP_MAKE);
      imprimeArchivo("D_PROT/cn.mak");
      elimNodoTipo(&cn);
    }
  printf("Generating a program to read and print ManTa files\n");
  copiaFuentesPRP(&Char,"Char","Characters");

  strcpy(nom,"String");
  sprintf(s,"d_prp/%s.adt",nom);
  String=loadTextType(s,"String");
  if (String==NULL || adicTipoAmbiente(&tad,&symtable,String,merr)!=0)
    { printf("Error adding type %s\n",nom); }
  if (String==NULL || generaCProt(String,CP_TODO)!=0)
    { printf("No pudo generar prototipos de %s\n",nom); }
  printf("  No se empelaran los prototipos de este TAD\n");

  copiaFuentesPRP(&Parameter,"Parameter","Parameter");
  copiaFuentesPRP(&LParameter,"LParameter","List of Parameters");
  copiaFuentesPRP(&Domain,"Domain","Domain");
  copiaFuentesPRP(&LDomain,"LDomain","List of Domains");
  LExpr=loadTextType("d_prp/LExprTemp.adt","Stupid list of expressions");
  if (LExpr==NULL || adicTipoAmbiente(&tad,&symtable,LExpr,merr)!=0)
    { printf("Error adding type LExprTemp\n"); }
  copiaFuentesPRP(&Expr,"Expr","Expression");
  copiaFuentesPRP(&LExpr,"LExpr","List of Expressions");
  copiaFuentesPRP(&Axiom,"Axiom","Axiom");
  copiaFuentesPRP(&LAxiom,"LAxiom","List of Axioms");
  copiaFuentesPRP(&Constructor,"Constructor","Constructor");

  strcpy(nom,"LConstructor");
  sprintf(s,"d_prp/%s.adt",nom);
  LConstructor=loadTextType(s,"List of Constructors");
  if (LConstructor==NULL || 
      adicTipoAmbiente(&tad,&symtable,LConstructor,merr)!=0)
    { printf("Error adding type %s\n",nom); }
  if (LConstructor==NULL || generaCProt(LConstructor,CP_TODO)!=0)
    { printf("No pudo generar prototipos de %s\n",nom); }
  printf("  No se empelaran los prototipos de este TAD\n");

  copiaFuentesPRP(&Selector,"Selector","Selector");
  copiaFuentesPRP(&LSelector,"LSelector","List of Selectors");
  copiaFuentesPRP(&AbsType,"AbsType","Abstract Data Type");
  if (copia_archivo ("D_PROT/manta.c","d_prp/manta.c")!=0)
    { printf("No pudo copiar archivo manta.c\n"); }
  if (copia_archivo ("D_PROT/manta.h","d_prp/manta.h")!=0)
    { printf("No pudo copiar archivo manta.h\n"); }
  if (copia_archivo ("D_PROT/manta_in.h","d_prp/manta_in.h")!=0)
    { printf("No pudo copiar archivo manta_in.h\n"); }
  if (copia_archivo ("D_PROT/manta_in.c","d_prp/manta_in.c")!=0)
    { printf("No pudo copiar archivo manta_in.c\n"); }
  if (copia_archivo ("D_PROT/manta.msg","d_prp/manta.msg")!=0)
    { printf("No pudo copiar archivo manta.msg\n"); }
  return 0;

}




