/* NQTHM 92 Benchmark for ManTa 3.1 */
/* By Vladimir Tamara P. */
/* 1998 */

#include <stdio.h>
#include "definic.h"
#include "apimanta.h"
#include "os_dep.h"
#include <assert.h>
#include <stdlib.h>
#include <time.h>

/***************************** PROTOTYPES *******************************/

/* Prints the contents of a file in stdout */
int imprimeArchivo(char *aname);

/* Loadas a type written in a text file (not in environment) */
nodo_tad *loadTextType(char *name, char *desc);

/* Prints an ADT */
int imprimeT(nodo_tad *nt,char *inf);

int testTheoremsNQTHM(char *file);

/*****************************    MAIN    *******************************/

int main(int argc, char *argv[])
{

  int pr;
  if (argc!=2)
    {
      printf("Syntax is:\n   nq92 <file>\n\n");
      return -1;
    }
  /* Initializing kernel */
  if (iniMantaAPI("manta.ini")<0)
    {
      printf("Failed (sure you hava manta.ini??)\n");
      return -1;
    }
  pr=testTheoremsNQTHM(argv[1]);
  finMantaAPI();
  return pr;
}


/***************************** FUNCTIONS DEFINITION *************************/


/* Loadas a type written in a text file (not in environment) */
nodo_tad *loadTextType(char *namerec, char *desc)
{

  FILE *f;
  texpr msg;
  nodo_tad *tad=NULL;
  char name[200];
  assert(namerec!=NULL);

  strcpy(name,namerec);
  transArch(name);
  f=fopen(name,"rb");
  if (f==NULL)
    {
      printf("File not found. \n");
      return NULL;
    }
  msg[0]='\0';
  if (cargaTipoTexto(f,&tad,msg))
    {
      printf("Error loading file. Message:\n");
      printf("%s\n",msg);
    }
  fclose(f);
  return tad;
}



int proveConj(char *arcnam,
	      char *msg /*BUG de gcc 2.7.2.3 no puede quitarse */
	      )
{
  int nnop,np; /* Numero de probadas y no probadas */
  nodo_demost *p,*nop;
  FILE *fp;
  texpr merr;
  clock_t start, end;

  transArch(arcnam);
  fp=fopen(arcnam,"rt");
  if (fp==NULL)
    {
      printf("Error opening conjetures file %s",arcnam);
      return -1;
    }
  merr[0]='\0';

  start = clock();
  if (probarConjeturas(fp,&np,&p,&nnop,&nop,merr)!=0)
    {
      printf("Problems trying to prove conjetures in %s",arcnam);
      fclose(fp);
      return -1;
    }
  end = clock();
  fclose(fp);
  printf("Proved: %i\n",np);
  printf("Unproved: %i\n",nnop);
  printf("Time: %f\n", (float)(end - start)/CLOCKS_PER_SEC);

  liberar_demost(&p);
  liberar_demost(&nop);
  return np;
}


int testConj(char *arcnam)
{
  FILE *fp;
  texpr merr;
  nodo_igualdad *conj;

  fp=fopen(arcnam,"rt");
  if (fp==NULL)
    {
      printf("Conjetures file \"%s\" not found\n",arcnam);
      return -1;
    }
  conj = analice_conjeturas(fp,merr);
  if (conj==NULL)
    {
      printf("Couldn't read conjeture\n%s\n",merr);
      return -1;
    }
  fclose(fp);
  elimLIgualdad(&conj);
  return 0;
}

int testTheoremsNQTHM(char *file)
{
  texpr merr;
  nodo_tad *t,*list;

  merr[0]='\0';
  list=loadTextType("nqthm92/ListNat.mta",
		    "ADT List");
  if (list==NULL || adicElemLTipo(&tad,list,ADIC_REP,merr)!=0 ||
      updateSymbols(&symtable,list,merr)!=0)
    {
      printf("Error adding type List\n");
      return -1;
    }

  t=loadTextType("nqthm92/nqthmboot.mta",
		 "Basic definitions");
  if (t==NULL || adicElemLTipo(&tad,t,ADIC_REP,merr)!=0 ||
      updateSymbols(&symtable,t,merr)!=0)
    {
      printf("Error adding type nqthmboot\n");
      return -1;
    }

  t=loadTextType("nqthm92/proveall.mta",
		 "More basic definitions");
  if (t==NULL || adicElemLTipo(&tad,t,ADIC_REP,merr)!=0 ||
      updateSymbols(&symtable,t,merr)!=0)
    {
      printf("Error adding type proveall\n");
      return -1;
    }

  t=loadTextType("nqthm92/async18.mta",
		 "Theorems about asynchronous communications");
  if (t==NULL || adicElemLTipo(&tad,t,ADIC_REP,merr)!=0 ||
      updateSymbols(&symtable,t,merr)!=0)
    {
      printf("Error adding type async18\n");
      return -1;
    }

  t=loadTextType("nqthm92/fibon.mta",
		 "Fibonacci");
  if (t==NULL || adicElemLTipo(&tad,t,ADIC_REP,merr)!=0 ||
      updateSymbols(&symtable,t,merr)!=0)
    {
      printf("Error adding type fibon\n");
      return -1;
    }

  t=loadTextType("nqthm92/rsa.mta",
		 "RSA");
  if (t==NULL || adicElemLTipo(&tad,t,ADIC_REP,merr)!=0 ||
      updateSymbols(&symtable,t,merr)!=0)
    {
      printf("Error adding type rsa\n");
      return -1;
    }

  t=loadTextType("nqthm92/Wilson.mta",
		 "Wilson");
  if (t==NULL || adicElemLTipo(&tad,t,ADIC_REP,merr)!=0 ||
      updateSymbols(&symtable,t,merr)!=0)
    {
      printf("Error adding type Wilson\n");
      return -1;
    }

  merr[0]='\0';
  t=loadTextType("nqthm92/Gauss.mta",
		 "Gauss");
  if (t==NULL || adicElemLTipo(&tad,t,ADIC_REP,merr)!=0 ||
      updateSymbols(&symtable,t,merr)!=0)
    {
      printf("Error adding type Gauss\n");
      return -1;
    }

  if (testConj(file)!=0)
    { return -1; }

  return proveConj(file,"");
}


