
/**
 * Translator from ManTa 3.1 Abstract Data Types from ManTa 3.1 to AST 
 * Version: 1.0 - 1998
 * Author: Vladimir Tamara and ManTa System
 **/


#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include "os_dep.h"
#include "est2abs.h"

#include "AbsType.h"
#include "Axiom.h"
#include "Char.h"
#include "Generator.h"
#include "Domain.h"
#include "Expr.h"
#include "LAxiom.h"
#include "LExpr.h"
#include "LDomain.h"
#include "LGenerator.h"
#include "LParameter.h"
#include "LSelector.h"
#include "LString.h"
#include "Parameter.h"
#include "Selector.h"
#include "String.h"


/***************************** FUNCTIONS DEFINITION *************************/


/* Translates c strings to ManTa strings */
int chars2String(String *l,char *n)
{
  Char c=NULL;
  String t=NULL;

  if (*n=='\0')
    { *l=nil_String(); }
  else
    {
      if (chars2String(&t,n+1)!=0)
        {goto badexit;}
      if ((c=charOfNAT((int)*n))==NULL)
	{ goto badexit; }
      if ((*l=cons_String(c,t))==NULL)
        { goto badexit; }
    }
  return 0;

 badexit:
    if (c!=NULL)
      { borrar_obj_Char(&c); }
    if (t!=NULL)
      { borrar_obj_String(&t); }
    return -1;
}

/* Translates a nodo_nombre structure to a LString */
int nodo_nombre2LString(LString *l,nodo_nombre *n)
{
  LString c=NULL;
  String s=NULL;

  if (n==NULL)
    { *l=nil_LString(); }
  else
    {
      if (nodo_nombre2LString(&c,n->sig)!=0)
        { goto badexit;}
      if (chars2String(&s,n->nombre)!=0)
        { goto badexit; }
      if ((*l=cons_LString(s,c))==NULL)
        { goto badexit; }
    }
  return 0;

 badexit:
    if (c!=NULL)
      { borrar_obj_LString(&c); }
    if (s!=NULL)
      { borrar_obj_String(&s); }
    return -1;
}


/* Translates a nodo_nombre structure to a LParameter */
int nodo_nombre2LParameter(LParameter *l,nodo_nombre *n)
{
  LParameter c=NULL;
  Parameter p=NULL;
  String s=NULL;

  if (n==NULL)
    { *l=nil_LParameter(); }
  else
    {
      if (nodo_nombre2LParameter(&c,n->sig)!=0)
        { goto badexit;}
      if (chars2String(&s,n->nombre)!=0)
        { goto badexit; }
      if ((p=parameter(s))==NULL)
        { goto badexit; }
      if ((*l=cons_LParameter(p,c))==NULL)
        { goto badexit; }
    }
  return 0;

 badexit:
    if (c!=NULL)
      { borrar_obj_LParameter(&c); }
    if (p!=NULL)
      { borrar_obj_Parameter(&p); }
    if (s!=NULL)
      { borrar_obj_String(&s); }
    return -1;
}

/* Translates a nodo_dom (product of domains) to a List of Domains */
int nodo_dom2LDomain(LDomain *l,nodo_dom *n)
{
  LDomain c=NULL;
  Domain d=NULL;
  String s=NULL;
  BOOL ind;

  if (n==NULL)
    { *l=nil_LDomain(); }
  else
    {
      if (nodo_dom2LDomain(&c,n->sig)!=0)
        {goto badexit; }
      if (chars2String(&s,n->nombre)!=0)
        { goto badexit; }
      if (n->induccion)
        { ind=TRUE; }
      else
        { ind=TRUE; }
      if ((d=domain(s,ind))==NULL)
        { goto badexit; }
      if ((*l=cons_LDomain(d,c))==NULL)
        { goto badexit; }
    }
  return 0;

 badexit:
    if (c!=NULL)
      { borrar_obj_LDomain(&c); }
    if (d!=NULL)
      { borrar_obj_Domain(&d); }
    if (s!=NULL)
      { borrar_obj_String(&s); }
    return -1;
}

/* Translates a nodo_nombre (list of names) to a List of Domains */
int nodo_nombre2LDomain(LDomain *l,nodo_nombre *n)
{
  LDomain c=NULL;
  Domain d=NULL;
  String s=NULL;

  if (n==NULL)
    { *l=nil_LDomain(); }
  else
    {
      if (nodo_nombre2LDomain(&c,n->sig)!=0)
        {goto badexit; }
      if (chars2String(&s,n->nombre)!=0)
        { goto badexit; }
      if ((d=domain(s,FALSE))==NULL)
        { goto badexit; }
      if ((*l=cons_LDomain(d,c))==NULL)
        { goto badexit; }
    }
  return 0;

 badexit:
    if (c!=NULL)
      { borrar_obj_LDomain(&c); }
    if (d!=NULL)
      { borrar_obj_Domain(&d); }
    if (s!=NULL)
      { borrar_obj_String(&s); }
    return -1;
}


/* Translates a nodo_argum (list of arguments of an expression) to a 
   LExpr (List of Expressions) */
int nodo_argum2LExpr(LExpr *l,nodo_argum *n)
{
  LExpr le=NULL;
  Expr e=NULL;

  if (n==NULL)
    { *l=nil_LExpr(); }
  else
    {
      if (nodo_argum2LExpr(&le,n->sig)!=0)
        {goto badexit; }
      if (nodo_expresion2Expr(&e,n->expr)!=0)
        { goto badexit; }
      if ((*l=cons_LExpr(e,le))==NULL)
        { goto badexit; }
    }
  return 0;

 badexit:
    if (le!=NULL)
      { borrar_obj_LExpr(&le); }
    if (e!=NULL)
      { borrar_obj_Expr(&e); }
    return -1;
}

/* Translates an nodo_expression (expression) to a 
   Expr (Expressions in AST) */
int nodo_expresion2Expr(Expr *e,nodo_expresion *n)
{
  LExpr arg=NULL;
  String nam=NULL;
  Expr e1=NULL,e2=NULL,e3=NULL;

  if (strcasecmp(n->clase,"ERROR")==0)
    {
      if ((*e=errorExpr())==NULL)
	{ goto badexit; }
    }
  else if (strcasecmp(n->clase,"VAR")==0)
    {
      if (chars2String(&nam,n->nombre)!=0)
	{ goto badexit; }
      if ((*e=varExpr(nam))==NULL)
	{ goto badexit; }
    }
  else if (strcasecmp(n->clase,"INIC")==0 || strcasecmp(n->clase,"CONSTR")==0)
    {
      if (chars2String(&nam,n->nombre)!=0)
	{ goto badexit; }
      if (nodo_argum2LExpr(&arg,n->argumentos)!=0)
	{ goto badexit; }
      if ((*e=genExpr(nam,arg))==NULL)
	{ goto badexit; }
    }
  else if (strcasecmp(n->clase,"SELECT")==0)
    {
      if (chars2String(&nam,n->nombre)!=0)
	{ goto badexit; }
      if (nodo_argum2LExpr(&arg,n->argumentos)!=0)
	{ goto badexit; }
      if ((*e=selExpr(nam,arg))==NULL)
	{ goto badexit; }
    }
  else if (strcasecmp(n->clase,"EQUAL")==0)
    {
      if (nodo_expresion2Expr(&e1,n->argumentos->expr)!=0)
	{ goto badexit; }
      if (nodo_expresion2Expr(&e2,n->argumentos->sig->expr)!=0)
	{ goto badexit; }
      if ((*e=eqExpr(e1,e2))==NULL)
	{ goto badexit; }
    }
  else if (strcasecmp(n->clase,"IF")==0 || strcmp(n->nombre,"if")==0)
    {
      if (nodo_expresion2Expr(&e1,n->argumentos->expr)!=0)
	{ goto badexit; }
      if (nodo_expresion2Expr(&e2,n->argumentos->sig->expr)!=0)
	{ goto badexit; }
      if (nodo_expresion2Expr(&e3,n->argumentos->sig->sig->expr)!=0)
	{ goto badexit; }
      if ((*e=ifExpr(e1,e2,e3))==NULL)
	{ goto badexit; }
    }
  
  return 0;

 badexit:
    if (arg!=NULL)
      { borrar_obj_LExpr(&arg); }
    if (nam!=NULL)
      { borrar_obj_String(&nam); }
    if (e1!=NULL)
      { borrar_obj_Expr(&e1); }
    if (e2!=NULL)
      { borrar_obj_Expr(&e2); }
    if (e3!=NULL)
      { borrar_obj_Expr(&e3); }

    return -1;

}

/* Translates a nodo_igualdad (two expressions) to a 
   LAxiom (List of Axioms) */
int nodo_igualdad2LAxiom(LAxiom *l,nodo_igualdad *n)
{
  LAxiom la=NULL;
  Axiom a=NULL;
  Expr li=NULL,ld=NULL;

  if (n==NULL)
    { *l=nil_LAxiom(); }
  else
    {
      if (nodo_igualdad2LAxiom(&la,n->sig)!=0)
        {goto badexit; }
      if (nodo_expresion2Expr(&li,n->lado_izq)!=0)
        { goto badexit; }
      if (nodo_expresion2Expr(&ld,n->lado_der)!=0)
        { goto badexit; }
      if ((a=axiom(li,ld))==NULL)
        { goto badexit; }
      if ((*l=cons_LAxiom(a,la))==NULL)
        { goto badexit; }
    }
  return 0;

 badexit:
    if (la!=NULL)
      { borrar_obj_LAxiom(&la); }
    if (a!=NULL)
      { borrar_obj_Axiom(&a); }
    if (li!=NULL)
      { borrar_obj_Expr(&li); }
    if (ld!=NULL)
      { borrar_obj_Expr(&ld); }
    return -1;
}

/* Translates a nodo_select (list of selectors) to a 
   LSelector (List of Selectors in AST) */
int nodo_select2LSelector(LSelector *l,nodo_select *n)
{
  LSelector ls=NULL;
  Selector s=NULL;
  String nom=NULL,cod=NULL;
  LDomain ld=NULL;
  LAxiom lax=NULL;
  BOOL carac;

  if (n==NULL)
    { *l=nil_LSelector(); }
  else
    {
      if (nodo_select2LSelector(&ls,n->sig)!=0)
        {goto badexit; }
      if (chars2String(&nom,n->nombre)!=0)
        { goto badexit; }
      if (chars2String(&cod,n->codominio)!=0)
        { goto badexit; }
      if (nodo_dom2LDomain(&ld,n->dominio)!=0)
        { goto badexit; }
      if (n->caracterizadora)
        { carac=TRUE; }
      else
        { carac=FALSE; }
      if (nodo_igualdad2LAxiom(&lax,n->axiomas)!=0)
        { goto badexit; }

      if ((s=selector(nom,ld,cod,carac,lax))==NULL)
        { goto badexit; }
      if ((*l=cons_LSelector(s,ls))==NULL)
        { goto badexit; }
    }
  return 0;

 badexit:
    if (ls!=NULL)
      { borrar_obj_LSelector(&ls); }
    if (s!=NULL)
      { borrar_obj_Selector(&s); }
    if (nom!=NULL)
      { borrar_obj_String(&nom); }
    if (ld!=NULL)
      { borrar_obj_LDomain(&ld); }
    if (cod!=NULL)
      { borrar_obj_String(&cod); }
    if (lax!=NULL)
      {	borrar_obj_LAxiom(&lax); }
    return -1;

}


/* Translates a nodo_generadora (list of constructors or initials) to a 
   LGenerator (List of Generators) */
int nodo_generadora2LGenerator(char *nomTad, LGenerator *l,
				 nodo_generadora *n)
{
  LGenerator lg=NULL;
  Generator g=NULL;
  String nom=NULL;
  LDomain ld=NULL;

  if (n==NULL)
    { *l=nil_LGenerator(); }
  else
    {
      if (nodo_generadora2LGenerator(nomTad,&lg,n->sig)!=0)
        {goto badexit;}
      if (chars2String(&nom,n->nombre)!=0)
        { goto badexit; }
      if (nodo_nombre2LDomain(&ld,n->dominio)!=0)
        { goto badexit; }
/*    if (chars2String(&sNomTad,nomTad)!=0)
      { goto badexit; } */
      if ((g=generator(nom,ld))==NULL)
        { goto badexit; }
      if ((*l=cons_LGenerator(g,lg))==NULL)
        { goto badexit; }
    }
  return 0;

 badexit:
    if (lg!=NULL)
      { borrar_obj_LGenerator(&lg); }
    if (g!=NULL)
      { borrar_obj_Generator(&g); }
    if (nom!=NULL)
      { borrar_obj_String(&nom); }
    if (ld!=NULL)
      {	borrar_obj_LDomain(&ld); }
/*    if (sNomTad!=NULL)
      { borrar_obj_String(&sNomTad); } */
    return -1;
}


/* Translates a nodo_tad (TAD) to an AbsType (AST of an ADT) */
int nodo_tad2AbsType(AbsType *ty,nodo_tad *t)
{
    String	S0=NULL;
    LParameter	L2=NULL;
    LGenerator L3=NULL,CI=NULL,CC=NULL;
    LSelector	L4=NULL;

    if (chars2String(&S0,t->nombre)!=0)
      { goto badexit; }
    if (nodo_nombre2LParameter(&L2,t->parametros)!=0)
      { goto badexit; }
    if (nodo_generadora2LGenerator(t->nombre,&CI,t->iniciales)!=0)
      { goto badexit; }
    if (nodo_generadora2LGenerator(t->nombre,&CC,t->constructoras)!=0)
      { goto badexit; }
    if ((L3=concat_LGenerator(CI,CC))==NULL)
      { goto badexit; }
/*    borrar_obj_LGenerator(&CI);
    borrar_obj_LGenerator(&CC); */
    
    if (nodo_select2LSelector(&L4,t->selectoras)!=0)
      { goto badexit; }
    if ((*ty = absType(S0, L2, L3, L4))==NULL)
      { goto badexit; }

    return 0;

 badexit:
    if (S0!=NULL)
      { borrar_obj_String(&S0); }
    if (L2!=NULL)
      { borrar_obj_LParameter(&L2); }
    if (L3!=NULL)
      { borrar_obj_LGenerator(&L3); }
    if (CI!=NULL)
      { borrar_obj_LGenerator(&CI); }
    if (CC!=NULL)
      { borrar_obj_LGenerator(&CC); }
    if (L4!=NULL)
      {	borrar_obj_LSelector(&L4); }
    return -1;
}







