
/**
 * ManTa 3.1 to OCaml Translator v1.1b1
 * Version: 1.0 - 1999
 * Author: Vladimir Tamara and ManTa System
 * See http://wwwisc.uniandes.edu.co/~manta
 **/

#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include "est2abs.h"
#include "apimanta.h"
#include "os_dep.h"

#include "AbsType.h"
#include "String.h"
#include "LString.h"
#include "M2Caml.h"
#include "ImplCaml.h"
#include "InterCaml.h"

#define maxInFiles 100

/****************
  PROTOTYPES
 ****************/

/* Print header of a Generated file */
void printHeader(FILE *out);

/* Prints a String */
void printString(String cadt,FILE *out);

/* Loads a type written in a text file (not in environment)  */
nodo_tad *loadTextType(char *name);

/* Print syntax */
void usage(void);

/*****************
    MAIN
 *****************/

int main(int argc,char *argv[])
{

  nodo_tad *et=NULL;
  nodo_nombre *req=NULL;
  char merr[2000]; /* Error messages */
  char outdir[500];
  char outname[500];
  AbsType ast=NULL;
  LString ls=NULL; /* Required types */
  ImplCaml ic=NULL; /* Translated program */
  InterCaml tc=NULL; /* Translated interface */
  String prgc=NULL; /* Caml Code */
  String intc=NULL;
  int rc=0,i,error;
  FILE *out;
  char configFile[200];
  char workDir[1000];
  char inFile[maxInFiles][100];
  int numInFiles=0;

  printf(" m2ocaml - ManTa (3.1) to OCaml translator v 1.1b1\n"
         " Part of this program has been generated by the ManTa system\n"
         " Universidad de los Andes\n"
         " Software Construction Group. 1999\n"
         " See http://wwwisc.uniandes.edu.co/~manta\n\n");

  strcpy(outdir,".");
  strcpy(configFile,"manta.ini");
  error=0;

  for (i=1;!error && i<argc;i++)
    {
      if (argv[i][0]=='-')
	{
	  switch (argv[i][1]) 
	    {
	    case 'd': 
	      if (i+1<argc)
		{ 
		  strcpy(workDir,argv[i+1]); 
		  transArch(workDir);
		  if (chdir(workDir)!=0) {
		    printf("Could'nt change to directory %s\n",workDir);
		    error=1;
		  }
		  i++;
		}
	      else
		{ error=1; }
	      break;
	    case 'i': 
	      if (i+2<argc)
		{ 
		  strcpy(configFile,argv[i+1]); 
		  transArch(configFile);  
		  i++;
		}
	      else
		{ error=1; }
	      break;
	    case 'o': 
	      if (i+2<argc)
		{ 
		  strcpy(outdir,argv[i+1]); 
		  transArch(outdir);  
		  i++;
		}
	      else
		{ error=1; }
	      break;
	    default:
	      error=1;
	      break;
	    }
	}
      else /* It must be an input file */
	{
	  if (numInFiles>maxInFiles)
	    { 
	      printf("Can't translate more than %i files\n",maxInFiles);
	      rc=3;
	      goto exit;
	    }
	  strcpy(inFile[numInFiles],argv[i]);
	  transArch(inFile[numInFiles]);
	  numInFiles++;
	}
    }
  if (numInFiles==0)
    { error=1; }
  
  if (error)
    {
      printf("Syntax is: \n"
             " m2ocaml [-i <confFile>]  [-d <dir>] [-o <outDir>]"
	     " <inFile> [<inFile>...]\n"
	     "<confFile> is the configuration file (by default manta.ini)\n"
	     "<dir> is the working directory\n"
	     "<outDir> is the output directory\n"
             "<inFile> is a text file with an ADT to generate\n");
      return 1;
    }

  if (iniMantaAPI(configFile)<0)
    {
      printf("Initialization Failed (sure you hava %s\?)\n",configFile);
      return 1;
    }

  for (i=0;i<numInFiles;i++)
    {
      printf("  Processing file %s\n",inFile[i]);
      et=loadTextType(inFile[i]);
      if (et==NULL || adicElemLTipo(&tad,et,ADIC_REP,merr)!=0)
	{
	  printf("Error adding type %s\n",inFile[i]);
	  rc=2;
	  goto exit;
	}
      if (nodo_tad2AbsType(&ast,et)!=0)
	{
	  printf("Error creating AST. Probably ther isn't enough memory\n");
	  rc=3;
	  goto exit;
	}
      if (related_types_list(et,&req,merr)!=0)
	{
	  printf("Problems using environment\n");
	  rc=4;
	  goto exit;
	}
      if (nodo_nombre2LString(&ls,req)!=0)
	{
	  printf("Not enough memory\n");
	  rc=4;
	  goto exit;
	}
      
      sprintf(outname,"%s\\%c%s.mli",outdir,tolower(et->nombre[0]),
	      et->nombre+1);
      transArch(outname);
      out=fopen(outname,"wt");
      if (out==NULL)
	{
	  printf("Can't write file %s.\n",outname);
	  rc=8;
	  goto exit;
	}
      printf("  Writing interface in file %s\n",outname);
      if ((tc=AbsType_InterCaml(ast))==NULL)
	{
	  printf("Error translating AST to interface."
		 "Probably ther isn't enough memory\n");
	  rc=7;
	  goto exit;
	}
      if ((intc=StringOfInterCaml(tc))==NULL)
	{
	  printf("Error generating OCaml Code (Interface)."
		 "Probably ther isn't enough memory\n");
	  rc=8;
	  goto exit;
	}
      printHeader(out);
      printString(intc,out);
      fclose(out);

      sprintf(outname,"%s\\%c%s.ml",outdir,tolower(et->nombre[0]),
	      et->nombre+1);
      transArch(outname);
      out=fopen(outname,"wt");
      if (out==NULL)
	{
	  printf("Can't write file %s.\n",outname);
	  rc=8;
	  goto exit;
	}
      printf("  Writing implementation in file %s\n",outname);
      if ((ic=AbsType_ImplCaml(ast, ls))==NULL)
	{
	  printf("Error translating AST to implementation."
		 "Probably there isn't enough memory\n");
	  rc=5;
	  goto exit;
	}
      if ((prgc=StringOfImplCaml(ic))==NULL)
	{
	  printf("Error generating OCaml Code (Implementation)."
		 "Probably there isn't enough memory\n");
	  rc=6;
	  goto exit;
	}
      printHeader(out);
      printString(prgc,out);
      fclose(out);
      borrar_obj_AbsType(&ast);
      ast=NULL;
      borrar_obj_LString(&ls);
      ls=NULL;
      elimLNombre(&req);
      req=NULL;
      elimElemLTipo(&tad,et->nombre);
      et=NULL;
      printf("  Succesful generation!\n");
    }
  
  
    exit:
  if (ast!=NULL)
    { borrar_obj_AbsType(&ast); }
  if (ls!=NULL)
    { borrar_obj_LString(&ls); }
  if (req!=NULL)
    { elimLNombre(&req); }
  if (et!=NULL)
    { elimElemLTipo(&tad,et->nombre); }
  finMantaAPI();
  return rc;
}


/* Print syntax */
void usage(void)
{
  printf("Syntax is: \n"
         " m2ocaml [-D<dir>]<in_file> \n"
         "<dir> is a directory to place the generated files\n"
         "<in_file> is a text file with a specification to translate\n");
}

void printHeader(FILE *out)
{
  fprintf(out,"(**\n"
              " * This file has been generated by the ManTa 3.1 System\n"
              " * Universidad de los Andes.  \n"
              " * Software Construction Group.   1999\n"
              " * See http://wwwisc.uniandes.edu.co/~manta\n"
              " **)\n\n");
}

void printString(String cadt,FILE *out)
{
  if (!isEmptyList_String(cadt))
    {
      fprintf(out,"%c",pr0charOfNAT(head_String(cadt)));
      printString(tail_String(cadt),out);
    }
}

/* Loads a type written in a text file (not in environment)  */
nodo_tad *loadTextType(char *name)
{

  FILE *f;
  texpr msg;
  nodo_tad *tad=NULL;
  assert(name!=NULL);

  transArch(name);
  f=fopen(name,"rb");
  if (f==NULL)
    {
      printf("File not found. \n");
      return NULL;
    }
  msg[0]='\0';
  if (cargaTipoTexto(f,&tad,msg))
    {
      printf("Error loading file. Message:\n");
      printf("%s\n",msg);
    }
  fclose(f);
  return tad;
}


